/* 
 * $COPYRIGHT$
 * $Id: Concept.java,v 1.27 2002/04/28 23:30:47 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;

import org.media.mn8.concepts.StringConcept;
import org.media.mn8.concepts.SeriesConcept;
import org.media.mn8.concepts.AttributeSeries;
import org.media.mn8.concepts.LogicalConcept;
import org.media.mn8.concepts.AttributeConcept;
import org.media.mn8.concepts.NilConcept;
import org.media.mn8.concepts.Attribute;
import org.media.mn8.concepts.Element;
import org.media.mn8.concepts.Series;
import org.media.mn8.concepts.TypedSeries;
import org.media.mn8.concepts.ElementConcept;
import org.media.mn8.concepts.TransparentElement;
import org.media.mn8.event.*;
import org.media.mn8.util.ConceptNavPatterns;

import java.util.Vector;

/** 
 * being from an mn8 script based concept or a Java one, is 
 * registered in {@link org.media.mn8.ConceptRegister}, where it is 
 * taken by the {@link org.media.mn8.mn8Loader} and instantiated.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.27 $ $Date: 2002/04/28 23:30:47 $
 */
public abstract class Concept {


    protected String _conceptLabel = null;
    protected boolean _isHidden = false;
    protected boolean _showEmpty = true;
    private Concept _errorHandler = null;


    public Concept getConceptNamed( StringConcept name ) {
        return mn8Loader.instantiateConcept( name.toString() );
    }


    public StringConcept getConceptLabel() {
        if( _conceptLabel == null ) {
            _conceptLabel = getConceptDefinition().getConceptLabel().toString();
        }
        return new StringConcept(_conceptLabel);        
    }


    public void setConceptLabel( StringConcept label ) {
        _conceptLabel = label.toString();
    }


    public  StringConcept getConceptType() {
        if( getConceptDefinition() != null )
            return getConceptDefinition().getConceptType();
        else 
            return new StringConcept("Concept");
    }


    public LogicalConcept isHidden () {
        return new LogicalConcept(_isHidden);
    }
    

    public void setHidden(LogicalConcept isHidden) {
        _isHidden = isHidden.getValue();
    }


    public LogicalConcept showEmpty () {
        return new LogicalConcept(_showEmpty);
    }
    

    public void setShowEmpty(LogicalConcept showEmpty) {
        _showEmpty = showEmpty.getValue();
    }


    public  SeriesConcept getConceptAttributes(){
        AttributeSeries attributes = new AttributeSeries();
        Vector attributeFields = getConceptAttributeFields().getVector();
        for( int i = 0 ; i < attributeFields.size() ; i++ ) {
            attributes.add( getConceptAttribute( ((FieldDefinition)attributeFields.elementAt(i)).getName()));
        }
        return attributes;        
    }


    public  LogicalConcept hasConceptAttribute( StringConcept attributeName ){
        return new LogicalConcept(hasConceptAttribute( attributeName.toString() ));
    }


    public boolean hasConceptAttribute( String attributeName ) {
        return getConceptDefinition().hasConceptAttribute( attributeName );
    }


    public  Concept getConceptAttribute( StringConcept attributeName ){
        return getConceptAttribute(attributeName.toString());        
    }


    public Concept getConceptAttribute( String attributeName ) {

        if( hasConceptAttribute( attributeName )) {
            FieldDefinition field = getConceptAttributeField( attributeName );
            if( field != null ) {
                return field.getValue( this );
            }
        } 

        throw new AttributeNotFoundException(getConceptType().toString(), attributeName);        
    }


    public  FieldDefinition getConceptAttributeField( StringConcept attributeName ){
        return getConceptAttributeField( attributeName.toString() );        
    }


    public  FieldDefinition getConceptAttributeField( String attributeName ){
        return getConceptDefinition().getConceptAttributeField(attributeName );        
    }

    
    public SeriesConcept getConceptAttributeFields() {
        return getConceptDefinition().getConceptAttributeFields();
    }


    public LogicalConcept hasConceptMethod( StringConcept methodSignature ) {
        if( getConceptDefinition() != null)
            return getConceptDefinition().hasConceptMethod( methodSignature);
        else 
            return LogicalConcept.FALSE;
    }


    public  SeriesConcept getConceptElements(){
        SeriesConcept elements = new SeriesConcept();
        Vector elementFields = getConceptElementFields().getVector();
        for( int i =0 ; i <  elementFields.size(); i++ ) {
            elements.add( getConceptElement(((FieldDefinition)elementFields.elementAt(i)).getName() ));
        }
        return elements;        
    }


    public  LogicalConcept hasConceptElement( StringConcept elementName ){
        return new LogicalConcept ( hasConceptElement( elementName.toString() ));        
    }


    public boolean hasConceptElement( String elementName ) {
        return getConceptDefinition().hasConceptElement( elementName);
    }


    public  Concept getConceptElement( StringConcept elementName ){
        return getConceptElement(elementName.toString() );        
    }


    public Concept getConceptElement( String elementName ) {
        if( hasConceptElement( elementName )) {
            FieldDefinition field = getConceptElementField( elementName );
            return field.getValue( this );
        } else {
            throw new ElementNotFoundException(getConceptType().toString(), elementName);
        }
    }


    public  FieldDefinition getConceptElementField( String elementName ){
        return getConceptDefinition().getConceptElementField(elementName );        
    }


    public  SeriesConcept getConceptElementFields(){
        return getConceptDefinition().getConceptElementFields();        
    }

    
    public  SeriesConcept getConceptMethods() {
        return getConceptDefinition().getConceptMethods();
    }

    public  SeriesConcept getConceptConstructors(){
        return getConceptDefinition().getConceptConstructors();
    }


    public  SeriesConcept getConceptOperators(){
        return getConceptDefinition().getConceptOperators();
    }


    public  Concept getConceptInstance(){
        return getConceptDefinition().getConceptInstance();
    }


    public SeriesConcept getInheritedConcepts() {
        return getConceptDefinition().getInheritedConcepts();
    }


    public SeriesConcept getAllInheritedConcepts() {
        return getConceptDefinition().getAllInheritedConcepts();
    }


    public final LogicalConcept extendsConcept( StringConcept conceptType ) {
        return getConceptDefinition().extendsConcept( conceptType );
    }


    public final mn8Method getConceptMethod( StringConcept methodSignature ) {
        return getConceptDefinition().getConceptMethod( methodSignature );
    }
    


    public abstract StringConcept toXML();
    

    public abstract StringConcept toTXT();
    

    public  ConceptDefinition getConceptDefinition() {
        return ConceptConceptDefinition.getReference();
    }


    public void loadContent( Concept from ) {
        if( from == null ) return;
        if( mn8RuntimeFlags.isDebug() ) {
            System.err.println("--> in concept:" + Helper.getConceptInfo(this) + "->" + Helper.getConceptInfo(from));
        }

        Helper.loadAttributes(from, this);

        Vector elementes = from.getConceptElements().getVector();
        ElementConcept elem = null;

        Vector localElements = getConceptElements().getVector();
        ElementConcept localElem = null;

        outerElem:
        for(int i = 0; i < elementes.size(); i ++ ) {
            elem = (ElementConcept)elementes.elementAt(i);

            innerElem:
            for( int j = 0 ; j < localElements.size(); j++) {              
                localElem = (ElementConcept)localElements.elementAt(j);

                if( localElem.getLabel().toString().equalsIgnoreCase( elem.getLabel().getValue() )) {
                    localElem.loadContent(elem);
                    localElements.set(j, localElem);
                    continue outerElem;
                }                                         
            }
        }        
    }


    public LogicalConcept hasPath( StringConcept path ) {
        return hasPath( path.toString() );
    }


    public SeriesConcept getConceptsAtPath( StringConcept path ) {
        return getConceptsAtPath(new SeriesConcept(), path.toString());
    }


    
    public SeriesConcept getConceptsAtPath( SeriesConcept holder, String fullpath) {

        String firstItem = ConceptNavPatterns.getFirstItem( fullpath );
        String remainingItems = ConceptNavPatterns.getRemainingItems( fullpath );

        if( ConceptNavPatterns.isAbsolutePath( firstItem ) ) {

            if( ConceptNavPatterns.isAttribute( firstItem )) {
                if( hasConceptAttribute(ConceptNavPatterns.getAttributeName(firstItem))){

                    if( remainingItems == null )
                        holder.add( getAttributeValue( ConceptNavPatterns.getAttributeName(firstItem)));
                    else 
                        holder= subAttributeGetConcept( holder, 
                                                        ConceptNavPatterns.getAttributeName(firstItem), 
                                                        remainingItems);
                }
            }                                                                            // is attribute
            else { 
                if( areThereMoreElemenents( ConceptNavPatterns.getElementName(firstItem) ) ) {

                    Vector elm = getMeAllElements(ConceptNavPatterns.getElementName(firstItem));
                    if( remainingItems == null ) {
                        for ( int i=0; i < elm.size(); i ++ ) {
                            holder.add( (Concept)  elm.elementAt(i));
                        }
                    } else {

                        for ( int i=0; i < elm.size(); i ++ ) {
                            holder = ((Concept)elm.elementAt(i)).getConceptsAtPath(holder, (remainingItems.indexOf("@") == 0 ? "/" : "") + remainingItems);
                        }
                    }
                } 
                return holder;
            }   
        }
        else {
            if ( ConceptNavPatterns.isAttribute( firstItem )) {
                if( hasConceptAttribute(ConceptNavPatterns.getAttributeName(firstItem))){
                    if( remainingItems == null ) {
                        holder.add( getAttributeValue( ConceptNavPatterns.getAttributeName(firstItem)));
                    } else { 
                        holder= subAttributeGetConcept( holder, 
                                                        ConceptNavPatterns.getAttributeName(firstItem), 
                                                        remainingItems);
                    }
                } else {
                    holder = searchDeeperForPath(holder, fullpath);
                }
            }else {
                if( areThereMoreElemenents( ConceptNavPatterns.getElementName(firstItem) )) {

                    Vector elm = getMeAllElements(firstItem);

                    if( remainingItems == null ) {
                        for ( int i=0; i < elm.size(); i ++ ) {
                            holder.add( (Concept)  elm.elementAt(i));
                        }
                    } else {
                        for ( int i=0; i < elm.size(); i ++ ) {
                            holder = ((Concept)elm.elementAt(i)).getConceptsAtPath(holder, (remainingItems.indexOf("@") == 0 ? "/" : "") + remainingItems);
                        }
                    }
                } else {
                    holder = searchDeeperForPath(holder, fullpath);
                }
                
            }
        }
        return holder;        
    }


    public SeriesConcept searchDeeperForPath(SeriesConcept holder, String path) {

        SeriesConcept elms = getConceptElements();
        for( int i =0; i < elms.getVector().size(); i++) {
            holder = ((Concept)elms.getVector().elementAt(i)).getConceptsAtPath( holder, path);
        }
        return holder;
    }


    public SeriesConcept subElementGetConcepts(SeriesConcept holder, String  elementName, String subPath) {
        return getConceptElement( elementName).getConceptsAtPath(holder, subPath );
    }


    public SeriesConcept subAttributeGetConcept(SeriesConcept holder, String  attributeName, String subPath) {
        return getAttributeValue( attributeName).getConceptsAtPath(holder, subPath );
    }


    /**
     * Returns true if the concept has the specified path or not.
     * By overriding getConceptElements or even this method elements and
     * attributes can dynamically be added to concepts, in fact this is 
     * the reason why it is also duplicate here.
     * <p> The path can be given using standard mn8 navigational 
     * patterns <tt>(/)? (elementName)? (/elementName)* (@ attributeName)?</tt>.
     * @param fullpath the path to be looked for.
     * @return true if it exists false otherway.
     */
    public LogicalConcept hasPath( String fullpath ) {

        String firstItem = ConceptNavPatterns.getFirstItem( fullpath );
        String remainingItems = ConceptNavPatterns.getRemainingItems( fullpath );

        if( ConceptNavPatterns.isAbsolutePath( firstItem ) ) {

            if( ConceptNavPatterns.isAttribute( firstItem )) {
                if( !hasConceptAttribute(new StringConcept(ConceptNavPatterns.getAttributeName(firstItem))).getValue())
                return LogicalConcept.FALSE;

                if( remainingItems == null ) return LogicalConcept.TRUE;
                else return subAttributeHasPath( ConceptNavPatterns.getAttributeName(firstItem),
                                                 remainingItems);
            }                                                                            // is attribute
            else { 
                if ( !hasConceptElement(ConceptNavPatterns.getElementName(firstItem))){
                    return LogicalConcept.FALSE;
                }
                if( remainingItems == null ) {
                    return LogicalConcept.TRUE;
                }
                else {
                    return subElementHasPath( ConceptNavPatterns.getElementName(firstItem),
                                              remainingItems);
                }
            }                                                                             // is element
        }                                                                                 // is absolute path
        else {
            if ( ConceptNavPatterns.isAttribute( firstItem )) {
                if( hasConceptAttribute(new StringConcept(ConceptNavPatterns.getAttributeName(firstItem))).getValue())
                    if( remainingItems == null ) return LogicalConcept.TRUE;            
                return searchSubElements(fullpath); 
            }
            else {
                if( hasConceptElement(ConceptNavPatterns.getElementName(firstItem))) {
                    if( remainingItems == null ){
                        return LogicalConcept.TRUE;            
                    } else { 
                        return subElementHasPath( ConceptNavPatterns.getElementName(firstItem), remainingItems);
                    }
                }
                //we still don't have it but maybe some element does                            
                return searchSubElements( fullpath );
            }
        }
    }


    
    /**
     * Searches for the respective path in all the elements of this concept.
     * @param path The path to be looked for corresponding to {@link org.media.mn8.util.ConceptNavPatterns}
     * @return A {@link LogicalConcept} true if the path was found, false otherway.
     * @see org.media.mn8.util.ConceptNavPatterns
     */
    protected LogicalConcept searchSubElements(String path) {
        SeriesConcept elements  = getConceptElements();
        LogicalConcept result = LogicalConcept.FALSE;
        ElementConcept elem = null;
        for( int i = 0 ; i < elements.getVector().size() ; i ++ ){
                result = subElementHasPath( ((ElementConcept)elements.getVector().elementAt(i)).getName().toString() , path );
                if( result.getValue() ) return result;
        }                 
        return result;
    }
    

    /**
     * Returns if an element of this concept has the subpath specified as parameter
     * @param elementName the name of the element
     * @param subPath the subpath to be looked for
     * @return true if it has it false otherway
     */
    public LogicalConcept subElementHasPath(String elementName, String subPath) {

        ElementConcept elem;
        if( hasConceptElement( elementName ) ) {
            elem = (ElementConcept) getConceptElement( elementName );
        } else {
            return LogicalConcept.FALSE;        
        }
        return elem.hasPath(subPath);        
    }


    /**
     * Returns if an attribute of this concept has the subpath specified as parameter
     * @param elementName the name of the element
     * @param subPath the subpath to be looked for
     * @return true if it has it false otherway
     */
    protected LogicalConcept subAttributeHasPath(String attribName, String subPath) {
        return getConceptAttribute( attribName).hasPath( subPath );
    }

    
    protected Concept getAttributeValue( String name ) {
        Concept concept =  getConceptAttribute( name );
        if (concept instanceof AttributeConcept) {
            return concept;
        }
        return new NilConcept();
    }


    public boolean areThereMoreElemenents(String path ) {
        return hasConceptElement(new StringConcept(path)).getValue();
    }

    
    public Vector getMeAllElements( String path ) {
        Concept result = getConceptElement(path);
        SeriesConcept elements ;

        if( result instanceof TransparentElement && ((TransparentElement)result).getValue() instanceof SeriesConcept ) {
            elements = (SeriesConcept) ((TransparentElement)result).getValue();
        } else {
            elements = new SeriesConcept();
            elements.add(result);
        }

        return elements.getVector();
    }


    public abstract Object clone();

    
    public Concept cloneConcept() {
        return (Concept) clone();
    }


    public void setErrorHandler( Concept handler ) {
        _errorHandler = handler;
    }


    public Concept getErrorHandler() {
        return _errorHandler;
    }



    public void printFields() {
        printFields("", this);
    }

    public void printFields( String pre, Concept con ) {
        pre = "  " + pre;
        FieldConcept field = null;
        ElementConcept elem = null;
        ElementConcept innerElem = null;
        for( int i = 0 ; i < con.getConceptElementFields().getVector().size(); i++) {
            
            field = (FieldConcept)con.getConceptElementFields().getVector().elementAt(i);
            elem =  (ElementConcept)con.getConceptElements().getVector().elementAt(i);

            String multi = "";

            if( elem.isMulti().getValue() ) {
                multi = "*";
            }

            if ( elem instanceof TransparentElement && !elem.getValueType().toString().equalsIgnoreCase("element")) {
                System.err.println(pre + field.toTXT().toString() + multi  + "(" 
                                   + elem.getClass().getName()
                                   + "|" + elem.getName() + "|" + elem.getLabel() + "|" + elem.getValueType()
                                   + "|" + elem.getValue().getClass().getName()
                                   + ( elem.getValue() instanceof TypedSeries ? ((TypedSeries)elem.getValue()).getType().toString() : "")
                                   );
            } else if ( elem.getValue() instanceof ElementConcept ) {
                innerElem = (ElementConcept) elem.getValue();
                System.err.println(pre + field.toTXT().toString()  +  multi + "(" 
                                   + innerElem.getClass().getName()
                                   + "|" + innerElem.getName() + "|" + innerElem.getLabel() + "|" + innerElem.getValueType()
                                   + "|" + innerElem.getValue().getClass().getName()
                                   );
            }
            
            printFields(pre, elem.getValue());
        }
    }

}
