/* 
 * $copyright$
 * $Id: ConceptDefinition.java,v 1.34 2002/06/05 14:55:26 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.concepts.*;
import org.media.mn8.event.*;
import org.media.mn8.util.ConceptNavPatterns;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

/** 
 * being from an mn8 script based concept or a Java one, is 
 * registered in {@link org.media.mn8.ConceptRegister}, where it is 
 * taken by the {@link org.media.mn8.mn8Loader} and instantiated.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.34 $ $Date: 2002/06/05 14:55:26 $
 */
public abstract class ConceptDefinition {

    private Vector _inheritance = new Vector();

    private Hashtable _methodTable = new Hashtable();
    private Hashtable _cameleonMethodTable;
    private Hashtable _constructorTable = new Hashtable();
    private Hashtable _operationTable = new Hashtable();

    private Vector _elementTable = new Vector();
    private Vector _elementNameTable = new Vector();

    private Hashtable _attributeTable = new Hashtable();
   

    public abstract StringConcept getConceptLabel();

    public abstract  StringConcept getConceptType();

    public abstract SeriesConcept getInheritedConcepts();



    /**
     * A method also representing the static method Concept.hasConceptAttribute:String
     * which checks if an concept has an attribute named attributeName.
     * @param attributeName the name of the attribute to be looked for.
     * @return true if exists false otherway.
     */
    public LogicalConcept hasConceptAttribute(StringConcept attributeName) {
       return new LogicalConcept( hasConceptAttribute(attributeName.toString()) ) ;
    }


    /**
     * Same as #hasConceptAttribute(StringConcept) in functionality but used internally where
     * a possible conversion to StringConcept might not be wanted.
     * @param attributeName the name of the attribute to be looked for.
     * @return true if exists false otherway.
     */
    public boolean hasConceptAttribute(String attributeName) {
        attributeName = attributeName.toLowerCase();
        if( attributeName.trim().startsWith("@"))
            attributeName = attributeName.substring(1);
        return _attributeTable.containsKey( attributeName ) ;
    }


    public Concept getConceptAttribute(Concept on, StringConcept attributeName) {
        attributeName = attributeName.toLowerCase();
        if ( _attributeTable.containsKey( attributeName.getValue() )) {
            FieldConcept attribute = (FieldConcept)_attributeTable.get( attributeName.getValue() );
            AttributeConcept attr = (AttributeConcept) attribute.getValue( on);
            return attr ;
        }
        else {
            throw new AttributeNotFoundException(on.getConceptType().toString(), attributeName.toString());
        }
    }


    public SeriesConcept getConceptElementFields() {
        return new SeriesConcept(_elementTable);
    }


    public Vector getConceptElementsFieldName() {
        return _elementNameTable;
    }


    public SeriesConcept getConceptAttributeFields() {
        return new SeriesConcept( new Vector( _attributeTable.values() ));
    }


    public Hashtable getConceptAttributeFieldsAsHashtable() {
        return _attributeTable;
    }


    public FieldDefinition getConceptAttributeField( StringConcept attributeName) {
        attributeName = attributeName.toLowerCase();
        return getConceptAttributeField( attributeName.toString() );
    }


    public FieldDefinition getConceptAttributeField( String attributeName ) {
        attributeName = attributeName.toLowerCase();
        if( _attributeTable.containsKey(attributeName) ) {
            return (FieldDefinition) _attributeTable.get( attributeName );
        } else {
            throw new FieldNotFoundException( getConceptType().toString(), attributeName);
        }
    }


    /**
     * Checks if the current concept has an element with the requested name.
     * @param elementName a <code>StringConcept</code> representing the name of the element.
     * @return True if the element us available, false otherway.
     */
    public LogicalConcept hasConceptElement(StringConcept elementName) {
       return new LogicalConcept( hasConceptElement( elementName.toString())) ;
    }


    /**
     * Performs the same function as {@link #hasConceptElement(StringConcept)}
     * but is used for internal operation where a conversion to StringConcept might
     * not be wanted.
     * @param elementName the name of the element to be looked for.
     * @return true if the element is available, false otherways.
     */
    public boolean hasConceptElement(String elementName) {
       return  _elementNameTable.contains(elementName.toLowerCase()) ;
    }


    public Concept getConceptElement(Concept on, StringConcept elementName) {
        if ( _elementNameTable.contains( elementName.getValue().toLowerCase() )) {
            FieldConcept element = (FieldConcept)_elementTable.elementAt( _elementNameTable.indexOf(elementName.getValue().toLowerCase()));
            Element elem = (Element) element.getValue( on);
            return elem.getElementValue();
        }
        else {
            throw new ElementNotFoundException( on.getConceptType().toString(), elementName.toString());
        }
    }


    public SeriesConcept getConceptMethods() {
        return new SeriesConcept( new Vector( _methodTable.values() ));
    }


    public SeriesConcept getCameleonConceptMethods() {
        if( _cameleonMethodTable == null) 
            initializeCameleonMethodTable();
        return new SeriesConcept( new Vector( _cameleonMethodTable.values() ));
    }


    public SeriesConcept getConceptConstructors() {
        return new SeriesConcept( new Vector( _constructorTable.values() ));
    }


    public SeriesConcept getConceptOperators() {
        return new SeriesConcept( new Vector( _operationTable.values() ));
    }


    public Concept getConceptInstance() {
        return new JavaConcept();
    }


    public Vector getInheritanceList() {
        return _inheritance;
    }


    public final void putOperator( mn8Method method) {
        _operationTable.remove(method.getSignature());
        _operationTable.put( method.getSignature(), method );

    }
    

    public final void putMethod( mn8Method method) {
        if ( method.getName().getValue().toLowerCase().equals("create") ){
            _constructorTable.remove(method.getSignature().toLowerCase());
            _constructorTable.put( method.getSignature().toLowerCase(), method );            
        }
        else{
            _methodTable.remove(method.getSignature().toLowerCase());
            _methodTable.put( method.getSignature().toLowerCase(), method );
        }
    }


    public final void removeMethod( String methodSignature) {
        if ( methodSignature.startsWith("create:") )
            _constructorTable.remove( methodSignature );
        else
            if ( methodSignature.startsWith("op") )
                _operationTable.remove( methodSignature );
            else
                _methodTable.remove( methodSignature );
    }

    
    public mn8Method getConstructor( String methodSignature ) {
        return (mn8Method) _constructorTable.get( methodSignature.toLowerCase() );
    }


    public Hashtable getInheritanceMethods() {
        Hashtable result = new Hashtable( _operationTable );
        result.putAll( _methodTable );
        return result;
    }


    public void putAttribute( FieldConcept attributeDef ) {
        _attributeTable.put( attributeDef.getName().toLowerCase(), attributeDef );
    }


    public void removeAttribute ( String attribName ) {
        attribName = attribName.toLowerCase();
        _attributeTable.remove( attribName );
    }

    // aicea vine modificat
    public FieldDefinition getAttribute( String attribName ) {
        attribName = attribName.toLowerCase();
        return (FieldDefinition) _attributeTable.get( attribName );
    }


    public FieldDefinition getAttributeField( String attribName ) {
        attribName = attribName.toLowerCase();
        return (FieldDefinition) _attributeTable.get( attribName );
    }

    
    public void putElement( FieldConcept elementDef ) {
        _elementTable.add(elementDef );
        _elementNameTable.add(elementDef.getName().toLowerCase());
    }
        

    public FieldDefinition getElement( String elementName ) {
        int idx = _elementNameTable.indexOf(elementName.toLowerCase());
        if( idx != -1 ) {
            return (FieldDefinition) _elementTable.elementAt( idx );
        }
        else 
            return null;
    }


    public FieldDefinition getConceptElementField( StringConcept elementName) {
        return getConceptElementField( elementName.toString() );
    }


    public FieldDefinition getConceptElementField( String elementName ) {
        int idx = _elementNameTable.indexOf(elementName.toLowerCase());
        if( idx != -1 ) {
            return (FieldDefinition) _elementTable.elementAt( idx );
        }
        else {
            throw new FieldNotFoundException(getConceptType().toString(), elementName);
        }            
    }


    public final mn8Method getConceptOperator(StringConcept methodSignature) {
        return getConceptOperator( methodSignature.getValue() );
    }


    public final mn8Method getConceptOperator(String methodSignature) {
        mn8Method result = null;
        if( ( result = (mn8Method)_operationTable.get(methodSignature)) != null ){
            return result;
        }
        else {
            result = mn8JavaMethod.getMethodUsingInheritance( methodSignature, 
                                                              getConceptOperators().getValue());
            if( result != null )
                return result;
        }
        throw new RuntimeException("Unable to find operator with signature: " + 
                                   methodSignature + 
                                   " in concept type: " + getConceptType());
    }


    public final mn8Method getConceptMethod(StringConcept methodSignature) {
        return getConceptMethod( methodSignature.getValue() );
    }


    public LogicalConcept hasConceptMethod( StringConcept methodSignature ) {
        try {
            if( getConceptMethod(methodSignature) != null )
                return new LogicalConcept(true);
            else
                return new LogicalConcept(false);
        } catch (RuntimeException ex) {
            return new LogicalConcept(false);
        }
    }


    public final mn8Method getConceptMethod(String methodSignature) {
        methodSignature = methodSignature.toLowerCase();
        mn8Method result = null;
        if( (result = (mn8Method) _methodTable.get(methodSignature)) != null) {
            return result ;
        }
        else {
            result = mn8JavaMethod.getMethodUsingInheritance( methodSignature, 
                                                              getConceptMethods().getValue());
            if( result != null )
                return result;
        }

        throw new MethodNotFoundException(getConceptType().toString(), methodSignature);
    }



    public final mn8Method getConceptConstructor(StringConcept methodSignature) {
        return getConceptConstructor(methodSignature.toString());
    }



    public final mn8Method getConceptConstructor(String methodSignature) {
        mn8Method result = null;
        if( (result = (mn8Method) _constructorTable.get(methodSignature)) != null) {
            return result ;
        } else {
            result = mn8JavaMethod.getMethodUsingInheritance( methodSignature,
                                                              getConceptConstructors().getValue());
            if( result != null )
                return result;
        }

        throw new RuntimeException("Unable to find constructor with signature: " + 
                                   methodSignature);
    }


    public final LogicalConcept extendsConcept( StringConcept conceptType ) {
        if( conceptType.getValue().toLowerCase().equals("concept"))
            return new LogicalConcept(true);
        else {
            if( conceptType.getValue().toLowerCase().equals(getConceptType().getValue().toLowerCase())) {
                return new LogicalConcept(true);       
            }
            else {
                SeriesConcept select = getInheritedConcepts().selectSIMEXP( conceptType.getValue() );
                if( select.getValue().size() > 0 )
                    return new LogicalConcept(true);
                else 
                    return new LogicalConcept(false);                    
            }
        }
    }



    public static void main (String[] args) {
        try {
            Class _class = Class.forName(args[0]);
            java.lang.reflect.Method reference = _class.getDeclaredMethod("getReference", null);
            ConceptConceptDefinition instance = (ConceptConceptDefinition)
                reference.invoke(reference, null);
            System.out.println("--" + instance.getClass().getName());
            System.out.println(instance.getConceptMethods().toTXT().getValue());

        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }


    public SeriesConcept getAllInheritedConcepts() {
        Vector allInheritedConcepts = new Vector();
        Vector newSet = null;
        allInheritedConcepts.addAll(getInheritedConcepts().getVector());
        allInheritedConcepts.remove( new StringConcept("Concept"));

        for(int cnt = 0; cnt < allInheritedConcepts.size() ; cnt ++) {
            newSet = addInheritedConcepts((StringConcept) allInheritedConcepts.elementAt(cnt));

            if( newSet != null && newSet.size() > 0 ) {
                newSet.remove( new StringConcept("Concept"));
                allInheritedConcepts.addAll(newSet);
            }
        }
        allInheritedConcepts.add(new StringConcept("Concept"));
        return new SeriesConcept( allInheritedConcepts );
    }

    
    private Vector addInheritedConcepts(StringConcept ofConcept) {
        Vector result =  mn8Loader.getConceptDefinition(ofConcept.toString()).getInheritedConcepts().getVector();
        if( result != null && result.size() > 0 ) {
            return result;
        } else {
            return null;
        }
    }


    protected void initializeCameleonMethodTable() {
        Enumeration methods = _methodTable.elements();
        mn8Method method = null;
        _cameleonMethodTable = new Hashtable();

        while( methods.hasMoreElements() ) {
            method = (mn8Method) methods.nextElement();

            if( method instanceof mn8JavaMethod &&
                method.getDeclaringConcept().toString().equals( getConceptType().toString()) )
                _cameleonMethodTable.put( method.getSignature(), method );
            
        }
    }


}
