/* 
 * $COPYRIGHT$
 * $Id: ConceptRegister.java,v 1.7 2002/06/05 14:55:35 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.conf.SystemConceptsTable;
import java.util.Enumeration;
import java.util.Hashtable;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.7 $ $Date: 2002/06/05 14:55:35 $
 */
public class ConceptRegister {

    private static ConceptRegister _instance = new ConceptRegister();

    private static Hashtable _register;


    private ConceptRegister() {
        if( mn8RuntimeFlags.isDebug() )
            System.err.println("Making ConceptRegister");

        _register = new Hashtable(SystemConceptsTable.getSystemConcepts());

        if( mn8RuntimeFlags.isDebug() )
            System.err.println("Loaded " + _register.size() + " system concepts into ConceptRegister");
    }


    public static void initialize() {
        _instance = new ConceptRegister();
    }


    public static void cleanAll() {
        _register.clear();
    }


    /**
     * Describe 'getReference' method here.
     *
     * @param nil a value of type ''
     * @return a value of type 'ConceptRegister'
     */
    public static ConceptRegister getReference() {
        return _instance;
    }

    
    /**
     * Describe constructor here.
     *
     * @param name a value of type 'String'
     * @param concept a value of type 'Concept'
     */
    public void register( String name, Object concept ){
        _register.put( name.toLowerCase(), concept );
    }


    /**
     * Describe constructor here.
     *
     * @param name a value of type 'String'
     */
    public void unregister( String name ) {
        _register.remove( name.toLowerCase() );
    }


    public Object lookup ( String name ) {
        return _register.get( name.toLowerCase() );
    }
    

    public Enumeration listConceptsName() {
        return _register.keys();
    }


    public Enumeration listConcepts() {
        return _register.elements();
    }

    
    public boolean isConceptNamed( String name ) {
        return  _register.containsKey( name.toLowerCase() ) ;
    }


    /**
     * Given a class name, tries to find if there is a concept
     * associated with it.
     * <p> It does this by:
     * <ul>
     * <li> parsing all the register and comparing the given class
     * name with the concept's class.
     * <li> if it is none available, that probably means that the 
     * class is an interface, so we get the assumed concept name 
     * from the interface by stripping the package name, we find 
     * the registered concept with that name, after which we compare
     * the two classes to see if there is indeed an inheritance 
     * relationship.
     * </ul>
     * @param className the class name as given by Class.getName();
     * @return the concept name if it is one associated with this class
     * or null if it is none.
     */
    public String resolveClass( String className ) {

        Enumeration keys = _register.keys();
        Object obj = null;
        String conceptName = null;
        Class classInst = null;
        String assumedConceptName = null;

        while( keys.hasMoreElements() ) {
            conceptName = (String) keys.nextElement();

            if( _register.get(conceptName).getClass().getName().equals( className ) )
                return ((Concept)_register.get(conceptName)).getConceptType().getValue();
        }

        try {
            classInst = Class.forName( className );
        } catch (ClassNotFoundException ex) {

            if( className.equals("long") ||
                className.equals("float") |
                className.equals("boolean") ||
                className.equals("char") ||
                className.equals("int") ) return null;

            ex.printStackTrace();
            return null;
        }

        assumedConceptName = className.substring(className.lastIndexOf(".") + 1);
        if( (obj = lookup( assumedConceptName )) != null ) {
            if( classInst.isAssignableFrom( obj.getClass() ) )
                return assumedConceptName;
        }

        return null;
    }
}
