/* 
 * $COPYRIGHT$
 * $Id: FieldConcept.java,v 1.11 2002/06/05 14:55:42 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.event.*;
import org.media.mn8.concepts.*;

import java.lang.reflect.Field;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.11 $ $Date: 2002/06/05 14:55:42 $
 */
public class FieldConcept extends JavaConcept implements FieldDefinition {

    protected String _name = null;

    protected boolean _isStatic = false;

    protected String _label = null;

    protected String _conceptType = null;

    protected Concept _initialValue = null;

    protected Field _field = null;

    
    public FieldConcept() {}


    public FieldConcept ( String name,
                          Field field,
                          boolean isStatic,
                          String label,
                          String type,
                          Concept initial ) {
        _name = name;
        _field = field;
        _isStatic = isStatic;
        _label = label;
        _conceptType = type;
        _initialValue = initial;
    }


    public FieldConcept ( String name,
                          Field field,
                          boolean isStatic,
                          String label,
                          String type,
                          Concept initial,
                          boolean isHidden ) {
        _name = name;
        _field = field;
        _isStatic = isStatic;
        _label = label;
        _conceptType = type;
        _initialValue = initial;
        _isHidden = isHidden;
    }


    public String getName() {
        return _name;
    }

    
    public String getLabel() {
        return _label;
    }
        

    public boolean isStatic() {
        return _isStatic;
    }


    public String getType() {
        return _conceptType;
    }


    public Concept newInstance() {
        if( _initialValue != null )
            return (Concept)_initialValue.clone();
        else
           return  mn8Loader.instantiateConcept( _conceptType );
    }


    public StringConcept toXML() {
        StringBuffer buffer = new StringBuffer();
        if( isAttribute() ) {
            buffer.append("<attribute ");
        } else {
            buffer.append("<element ");
        }

        buffer.append("name=\"");
        buffer.append(_name);

        buffer.append("\" label=\"");
        buffer.append(_label);

        buffer.append("\" type=\"");
        buffer.append(getType());

        buffer.append("\" isStatic=\"");
        buffer.append(isStatic());

        buffer.append("\" isHidden=\"");
        buffer.append(_isHidden);

        buffer.append("\" showEmpty=\"");
        buffer.append(_showEmpty);

        if( isAttribute() ) {
            buffer.append("\" >" + mn8Interpreter.EOL + "</attribute>");
        } else {
            buffer.append("\" isMulti=\"");
            buffer.append(isMulti());
            buffer.append("\" >" + mn8Interpreter.EOL + "</element>");
        }        
        return new StringConcept(buffer.toString());
    }

    
    public boolean isMulti() {
        return false;
    }


    public StringConcept toTXT() {
        StringBuffer buffer = new StringBuffer();

        if( _isStatic ) 
            buffer.append("STATIC ");
        else
            buffer.append("       ");

        if( isAttribute() )
            buffer.append("@");

        buffer.append(_name);

        if( _conceptType != null ) {
            buffer.append(" TYPEOF ");
            buffer.append(_conceptType);
        }

        if( _label != null ) {
            buffer.append(" LABEL \"");
            buffer.append(_label + "\"");
        }

        return new StringConcept(buffer.toString());
    }


    public boolean isAttribute() {
        return  _field.getType().isAssignableFrom(org.media.mn8.concepts.AttributeConcept.class);
    }
    

    public boolean isElement() {
        return  _field.getType().isAssignableFrom(org.media.mn8.concepts.ElementConcept.class);
    }


    public Concept getInitialValue() {
        return _initialValue;
    }


    public Concept getValue( Concept on ) {
        Object result;
        try {
            result = _field.get(on);
            return (Concept) result; //(Concept) ((Concept)result).clone();
        } catch (Exception ex) {
            ex.printStackTrace();
            return new NilConcept();
        }
    }


     public void setValue( Concept on, Concept value) {
        try {
            _field.set(on, value);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
     }

    
    public StringConcept getFieldName () {
        return new StringConcept( _name );
    }


    public StringConcept getFieldLabel () {
        if (_label != null) {
            return new StringConcept( _label );             
        } // end of if (_label != null)
        return new StringConcept("");
    }

    
    public ConceptDefinition getConceptDefinition() {
        return FieldConceptDefinition.getReference();
    }


    Field getField() {
        return _field;
    }

    
    public Object clone() {
        return this;
    }

}
