/* 
 * $COPYRIGHT$
 * $Id: Helper.java,v 1.11 2002/06/03 11:32:26 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.concepts.*;
import org.media.mn8.event.*;
import java.util.Vector;
import java.util.StringTokenizer;
import javax.naming.NamingException;
import java.io.File;

/** 
 * A simple class the holds various methods useful in general.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.11 $ $Date: 2002/06/03 11:32:26 $
 */
public class Helper {


    static public StringConcept getUserHome() {
        String home = System.getProperty("user.home");
        if( home.charAt( home.length() -1 ) != File.separatorChar) 
            home += File.separatorChar;
        return new StringConcept(home);
    }

    
    public static boolean isElement( Concept concept ) {
        if( concept instanceof Element || concept.getAllInheritedConcepts().getVector().contains(new StringConcept("Element"))) {
            return true;
	}
        else 
            return false;
    }


    public static boolean isElement( ConceptDefinition conceptDef ) {
        if( conceptDef instanceof ElementDefinition || conceptDef.getAllInheritedConcepts().getVector().contains(new StringConcept("Element"))) {
            return true;
		}
        else 
            return false;
    }


    public static StringConcept extractName(Concept concept) {
        if( concept.hasConceptMethod(new StringConcept("getName:")).getValue() ) {
            mn8Method metod = concept.getConceptMethod(new StringConcept("getName:"));
            Concept result = metod.invoke(concept, null);
            if( result != null && result instanceof StringConcept )
                return (StringConcept)result;
        }
        return concept.getConceptType();
    }


    public static String getConceptInfo( Concept concept ) {
        String result = "";
        result = concept.getClass().getName();
        result += "(" + concept.getConceptType().toString() + "," +  concept.getConceptLabel().toString() + ")";
        if( concept instanceof TransparentElement  || concept instanceof Element ) {
            ElementConcept el = (ElementConcept) concept;
            result += "(" + el.getValue().getConceptType().toString() + ","  + el.getValue().getConceptLabel().toString() ;
                if( concept instanceof ElementConcept ) {
                    if( ((ElementConcept)concept).isMulti().getValue()) {
                        result += "*";
                    }
                }                
                result += ")";
            result += el.getValue().getClass().getName();
        }
        return result;
    }


    public static Concept loadAttributes(Concept from , Concept in) {
        Vector attributes;
        if (from != null && from.getConceptAttributes()  != null ) {
            attributes = from.getConceptAttributes().getVector();
        } else {
            return in;
        }

        AttributeConcept attr = null;
        Vector localAttributes = in.getConceptAttributes().getVector();
        AttributeConcept localAttr = null;

        outerAttr:
        for( int i = 0; i < attributes.size(); i ++ ) {
            attr = (AttributeConcept)attributes.elementAt(i);
            for( int j = 0 ; j < localAttributes.size(); j++) {
                localAttr = (AttributeConcept)localAttributes.elementAt(j);
                if( localAttr.getName().equalsIgnoreCase( attr.getName()).getValue() ||
                    localAttr.getLabel().equalsIgnoreCase( attr.getLabel()).getValue()) {
                    localAttr.setValue( attr.getValue());
                    continue outerAttr;
                }
            }
        }

        return in;
    }


    public static String fileNameFix(String fileName) {
        String result;
        File f = new File( fileName );
        result = f.toString();
        if( result.toLowerCase().startsWith("file:\\")) {
            result = result.substring( result.indexOf("\\") + 1);
        } else if ( result.toLowerCase().startsWith("file:/")) {
            result = result.substring( result.indexOf(":") + 1);
        }
        return result;
    }


    public static boolean hasEOL(String value) {
        return ( value.indexOf("\r") != -1 || value.indexOf("\n") != -1 );
    }


    public static boolean startsWithEOL(String value) {
        return ( value.startsWith("\r")  || value.startsWith("\n") );
    }


    public static boolean endsWithEOL(String value) {
        return ( value.endsWith("\r") || value.endsWith("\n") );
    }


    
    public static String fillWithValues(String template) {
        StringTokenizer tokenizer = new StringTokenizer(template, "{}", true);
        String token;
        String value;
        boolean templStart = false;

        StringBuffer result = new StringBuffer();
        String val;

        if( template == null ) {
            return null;
        }

        while( tokenizer.hasMoreTokens() ) {
            token = tokenizer.nextToken();
            if( token.equals("{")) {
                templStart = true;
            } else if( token.equals("}")) {
                templStart = false;
            }

            if( token.startsWith("$") && templStart ) {
                val = System.getProperty( token.substring(1));
                if( val == null ) {
                    try {
                        val = ((StringConcept)NamingManager.getReference().lookup( "env:/system/properties/" + token.substring(1))).toString();
                    } catch (NamingException nex) {
                    }
                }
                if( val != null ) {
                    result.append(val);
                } else {
                    result.append("{");
                    result.append(token);
                    result.append("}");
                }
            } else if( !token.equals("{") && !token.equals("}") ) {
                result.append(token);
            }            
        }
        
        if( result.length() > 0 ) {
            return result.toString();
        } else {
            return template;
        }
    }

}
