/* 
 * $COPYRIGHT$
 * $Id: LogManager.java,v 1.5 2001/09/27 00:23:16 neuro Exp $
 *
 * Date        Author            Changes 
 * May 08 2001 Remus Pereni      Created
 */
package org.media.mn8;


import java.util.Properties;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.PrintStream;
import java.io.OutputStream;


/**
 * Class responsible for managing logging messages. Loading them from the resource 
 * bundles and properly displaying them on the requested writer.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.5 $ $Date: 2001/09/27 00:23:16 $
 */
public class LogManager {

    private static PrintStream _out = System.out;


    public static final int VERBOSE = 1;

    public static final int DEBUG = 2;

    public static final int ERROR = 3;

    /**
     * Permits another output stream (a file output stream as example) to be the 
     * default error output stream.
     * @param writer The {@link PrintStream} which will become the default error 
     * writter.
     */
    public static void setOutputStream (PrintStream out) {
        if ( out == null )
            throw new RuntimeException ("Please don't put a null in it !!");
        _out = out;
    }


    /**
     * Returns the default error output stream .
     * @return The {@link java.io.PrintStream} which is  default error output stream.
     */
    public static OutputStream getOutputStream () {
        return _out;
    }


    /**
     * The main method for displaying system messages for mn8.
     * @param type_id The type of the messages (error, warning, ...). This is
     * actually an key in the resource bundle where the actual value which will be
     * displayed is hold. mn8.messages.type. has to be inserted before the key in the resource 
     * bundle to store such messages types.
     * <pre>
     *   mn8.messages.type.warning=[WARNING]
     * </pre>
     * @param message_id The id of the messages. This is
     * actually an key in the resource bundle where the actual value which will be
     * displayed is hold. mn8.messages. has to be inserted before the key in the resource 
     * bundle to store such messages.
     * <pre>
     *   mn8.messages.file_missing=Sorry the file: xxx.jar \n is mising.
     * </pre>
     */
    public static void message(String type_id, String message_id ) {
        message(type_id, message_id, "");
    }

    /**
     * The main method for displaying system messages for mn8.
     * @param type_id The type of the messages (error, warning, ...). This is
     * actually an key in the resource bundle where the actual value which will be
     * displayed is hold. mn8.messages.type. has to be inserted before the key in the resource 
     * bundle to store such messages types.
     * <pre>
     *   mn8.messages.type.warning=[WARNING]
     * </pre>
     * @param message_id The id of the messages. This is
     * actually an key in the resource bundle where the actual value which will be
     * displayed is hold. mn8.messages. has to be inserted before the key in the resource 
     * bundle to store such messages.
     * <pre>
     *   mn8.messages.file_missing=Sorry the file: xxx.jar \n is mising.
     * </pre>
     * @param optional Another text to be added to the message. Usualy displayed on the same line.
     */
    public static void message(String type_id, String message_id, String optional ) {
       sysMessage(  MessageFormatter.getReference().getMessage( type_id, message_id, optional, "" ) );
    }


    /**
     * The main method for displaying system messages for mn8.
     * @param type_id The type of the messages (error, warning, ...). This is
     * actually an key in the resource bundle where the actual value which will be
     * displayed is hold. mn8.messages.type. has to be inserted before the key in the resource 
     * bundle to store such messages types.
     * <pre>
     *   mn8.messages.type.warning=[WARNING]
     * </pre>
     * @param message_id The id of the messages. This is
     * actually an key in the resource bundle where the actual value which will be
     * displayed is hold. mn8.messages. has to be inserted before the key in the resource 
     * bundle to store such messages.
     * <pre>
     *   mn8.messages.file_missing=Sorry the file: xxx.jar \n is mising.
     * </pre>
     * @param optional Another text to be added to the message. Usualy displayed on the same line.
     * @param suplement Another text to be added to the message. Usualy displayed on another line.
     */
    public static void message(String type_id, String message_id, String optional, String suplement ) {
       sysMessage(  MessageFormatter.getReference().getMessage( type_id, message_id, optional, suplement ) );
    }


    /**
     * Prints an message, formated previously probably by MessageFormatter.
     * @param message The message to be displayed.
     */
    public static void message( String message ) {
        sysMessage( message );
    }


    public static void message( int severity, String message ) {
        if( severity <= DEBUG && mn8RuntimeFlags.isDebug() )
            message( message );
        else
            if( severity == VERBOSE && mn8RuntimeFlags.isVerbose() )
                message( message );

        if( severity == ERROR )
            message( message );       
    }


    /**
     * Prints an message previously formated probably by MessageFormatter. 
     * The generator exception information will be printed depending on the
     * verbosity level choose by the user -v (--verbose) and -d (--debug).
     * @param generatorException The exception who triggered this message.
     * @param message The message to be displayed.
     */
    public static void message( Exception generatorException, String message ) {
        sysMessage( message, generatorException);
    }


    protected static void sysMessage(String message) {
        _out.println( message );
    }


    protected static void sysMessage(String message, Exception ex) {
        _out.println( message );
        if ( isVerbose() && ex != null ) 
            _out.println( ex );
        if ( isDebug() && ex != null )
            ex.printStackTrace( _out );
    }


    protected static boolean isVerbose() {
        return false;
    }


    protected static boolean isDebug() {
        return false;
    }
}
