/* 
 * $COPYRIGHT$
 * $Id: Main.java,v 1.21 2002/06/03 11:35:47 neuro Exp $
 *
 * Date        Author            Changes 
 * Apr 22 2001 Remus Pereni      Created
 */

package org.media.mn8;

import org.media.mn8.parser.*;
import org.media.mn8.conf.*;
import org.media.mn8.event.*;
import org.media.mn8.concepts.*;
import org.media.mn8.util.CLIHelper;
import org.media.mn8.util.ArgParser;

import java.io.*;
import java.net.URL;
import java.util.Vector;
import java.util.Enumeration;
import antlr.collections.AST;
import antlr.collections.impl.*;
import antlr.debug.misc.*;
import antlr.*;


/** 
 * The main class for mn8.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.21 $ $Date: 2002/06/03 11:35:47 $
 */

class Main extends CLIHelper {
    
    protected final String usage = getUsage("conf/res/mn8Usage_EN.txt");
    protected String  file    = "";
    protected String  command = "";
    protected boolean execCommand = false;
    protected boolean exec = false;
    protected boolean version = false;
    protected boolean help = false;
    protected String args  = opArg + "-d;--debug;-v;--verbose;-V;--version;--trace;-c:;--showtree;";

    protected void setArgs( ArgParser parser ) {
        super.setArgs( parser );
        String arg = "";

        if ( "".equals( parser.getValue("-d") ) || 
             "".equals( parser.getValue("--debug") ) ) mn8RuntimeFlags.setDebug(true);

        if ( "".equals( parser.getValue("-v") ) || 
             "".equals( parser.getValue("--verbose") ) ) mn8RuntimeFlags.setVerbose(true);

        if ( "".equals( parser.getValue("--showtree") ) ) mn8RuntimeFlags.setShowTree(true); 

        if ( "".equals( parser.getValue("--trace") ) ) mn8RuntimeFlags.setTrace(true); 

        if ( "".equals( parser.getValue("-V") ) || 
             "".equals( parser.getValue("--version") ) ) {
            version = true;
        }
        else if ( parser.getValue("-c:") != null ) { 
            execCommand = true;
            StringBuffer sb = new StringBuffer(parser.getValue("-c:") + " ");
            
            for ( Enumeration e = parser.getUnparsedArg().elements(); e.hasMoreElements(); ) {
                String uarg = (String)e.nextElement();
                if ( "\\n".equals( uarg ) ) sb.append("\n"); else sb.append( uarg + " ");
            }
            //parser.getUnparsedArg().remove(0);
            command = sb.toString();
        }
        else if( parser.getUnparsedArg().size() > 0 ){
            exec = true;
            file = (String) parser.getUnparsedArg().elementAt(0);
            parser.getUnparsedArg().remove(0);
        }
    }
    

    protected String validate( ArgParser parser ) {
        String ret = "";

        ret = super.validate( usage );

        if ( !ret.equals( "" ) ) 
            return ret;

        if ( !( execCommand || version || help || exec ) ) 
            return usage;

        if ( version ) 
            return  mn8Interpreter.VERSION;

        if ( execCommand && command == null ) 
            return "You must suply a command to be executed! \n" + usage; 
        else if ( execCommand ) interpret( command );

        if( exec && file == null) {
            return "You must suply a script file to be executed! \n" + usage; 
        } 
        else {
            Vector params = parser.getUnparsedArg();
            SeriesConcept mn8Params = new SeriesConcept();
            SeriesConcept args = new SeriesConcept();
            for( int i = 0 ; i < params.size(); i ++ )
                args.add( new StringConcept((String) params.elementAt(i)));
            try {
            if( System.in.available() > 0 ) {
                BufferedReader inRead = new BufferedReader( new InputStreamReader(System.in));
                StringBuffer strbuf = null;
                try {
                    strbuf = new StringBuffer();
                    String line = null;
                    while( (line = inRead.readLine()) != null ) {
                        strbuf.append( line );
                        strbuf.append("\n");
                    }
                } catch( IOException ioex ){}
                               
                if( strbuf.length() > 0 )
                    args.add( new StringConcept( strbuf.toString() ));
            }
            } catch (IOException ioex) {}
            mn8Params.add( args );
            execute(file, mn8Params);
            
        }        
        
        return ret;
    }


    protected String parseArgs( String[] argv ) {
        ArgParser parser = new ArgParser( args, ";");
  
        if ( argv.length == 1 ) argv = getArguments( argv[0] ); 
        if ( !parser.parse( argv ) ) return "Invalid argument in command line !";
        setArgs( parser );
        
        String ret = validate( parser ).trim();
        if ( ret.length() > 0 ) return ret;  
         
        return "";
    }
    

    public void execute( String file, SeriesConcept arguments ) {
        try {            
            File f = new File(file);
            if( !f.exists() ) {
                String fileName = (new mn8Loader()).findConcept(file);                
                URL url = new URL(fileName);
                fileName = url.getPath();
                // probably is a concept name and not an actuall file
                mn8Interpreter.executeURL( (new File(fileName)).toURL().toString(), file,  arguments );
            } else {
                mn8Interpreter.executeURL( (new File(file)).toURL().toString(),  arguments );
            }            
            System.exit(0);        
        } catch( mn8ExitEvent ev) {
            if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose())  {
                System.err.println("Exit request received, exiting...");
            }
        } catch( Exception ex ) {
            System.out.println( ex.getMessage() );
            if( mn8RuntimeFlags.isDebug() ) {
                System.out.println( "/---: The stack trace is :----------------------------------\\");
                ex.printStackTrace();
                System.out.println( "\\------------------------------------------------------------/");
            }
        }
    }


    public void interpret( String command ) {
        try {
            if( mn8RuntimeFlags.isDebug()) {
                System.err.println("Executing command:" + command);
            }
            mn8Interpreter.interpret( command );
            System.exit(0);
        } catch( mn8ExitEvent ev) {
            if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose())  {
                System.err.println("Exit request received, exiting...");
            }
        } catch( RuntimeException ex ) {
            System.err.println("exception: " + ex);
            if( mn8RuntimeFlags.isDebug() ) {
                System.out.println( "/---: The stack trace is :----------------------------------\\");
                ex.printStackTrace();
                System.out.println( "\\------------------------------------------------------------/");
            }
        }
    }
    

    public static void main( String[] args ) {
        Vector scriptArgumens = new Vector();
        Main main = new Main();
        String arg[] = args;
        try {
            mn8Interpreter.initialize();
            StringBuffer sis = new StringBuffer();
            BufferedReader     in = new BufferedReader( new InputStreamReader( System.in ) );
            String line;    
            if( System.in.available() > 0 ) {
                while ( ( line = in.readLine() ) != null ) sis.append( line +"\n" );
            }
            if( sis.toString().length() != 0 ) {
                arg = new String[args.length + 1];
                for( int i = 0 ; i < args.length; i++) {
                    arg[i] = args[i];
                }
                arg[args.length] = sis.toString();
            }
        } catch (Exception ex) {

        }
        String argParseResult = main.parseArgs( arg );
        
        if( !argParseResult.equals("") ) {
            System.err.println( argParseResult );
            System.exit( 0 );
        }
    }
}
