/* 
 * $COPYRIGHT$
 * $Id: Scope.java,v 1.14 2002/06/05 14:31:36 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;

import java.util.Hashtable;
import java.util.Enumeration;

/**
 * <p>
 * The base implementation of a Scope.
 * </p>
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.14 $ $Date: 2002/06/05 14:31:36 $
 */
public class Scope implements Cloneable {

    /** The upper (in hierarchy) scope. */
    protected Scope _prev;

    /** The Symbol Table, where the variables are stored. */
    Hashtable _instanceTable;

    Hashtable _forcedTypeTable;

    /**
     * The default constructor. It is protected since there will
     * be specific implementations of this class.
     * @param prev The outer scope, in which this one is situated
     */
    public Scope( Scope prev ){
        _prev=prev;
        _instanceTable = new Hashtable();
        _forcedTypeTable = new Hashtable();
    }

	
    Scope( Hashtable instanceTable, Hashtable forcedTypeTable) {
        _instanceTable = instanceTable;
        _forcedTypeTable = forcedTypeTable;
    }


    public void clear() {
        _instanceTable.clear();
        _forcedTypeTable.clear();
        _prev = null;
    }
    
    /**
     * Inserts a variable into the local scope.
     * @param variable_name The name of the variable
     * @param variable The variable
     */   
    public void insertInstance( String variable_name, Concept variable ){
        _instanceTable.put(variable_name.toLowerCase(), variable );
    }
	

    /**
     * Returns a variable from the local scope.
     * @param variable_name The name of the wanted variable
     * @return the variable
     */
    public Concept getInstance( String variable_name ){
        Concept result = (Concept)_instanceTable.get(variable_name.toLowerCase());
        return result;
    }
	
   
    public boolean isInstance( String instanceName ) {
        if( _instanceTable.containsKey( instanceName.toLowerCase() ))
            return true;
        else 
            return false;        
    }
  

    public boolean isForcedInstance( String instanceName ) {
        if( _forcedTypeTable.containsKey( instanceName.toLowerCase() )) {
            return true;
        } else {
            return false;
        }
    }
   

    public String getForcedType( String instanceName ) {
        return (String) _forcedTypeTable.get(instanceName.toLowerCase());
    }


    public void setForcedType( String instanceName, String forcedType ) {
        if( forcedType.toLowerCase().equals("nil") ) {
            if( _forcedTypeTable.containsKey(instanceName.toLowerCase())) {
                _forcedTypeTable.remove(instanceName.toLowerCase());
            }
        } else {
            _forcedTypeTable.put(instanceName.toLowerCase(), forcedType);
        }
    }
    

    /**
     * Returns the previous (upper in hierarchy) scope.
     * @return The upper scope.
     */
    public Scope getPrevScope(){
        return _prev;
    }


    void setPrevScope(Scope prevScope) {
        _prev = prevScope;
    }

    
    public Object clone() {
        Hashtable instance = new Hashtable();
        Hashtable forced = new Hashtable();

        Enumeration keys = _instanceTable.keys();
        String key = null;
        while( keys.hasMoreElements() ) {
            key = (String) keys.nextElement();
            instance.put(key, ((Concept)_instanceTable.get(key)).clone());
        }


        keys = _forcedTypeTable.keys();
        key = null;
        while( keys.hasMoreElements() ) {
            key = (String) keys.nextElement();
            forced.put(key, _forcedTypeTable.get(key));
        }


        return new Scope( instance, forced );
    }
}
