/* 
 * $COPYRIGHT$
 * $Id: ScopeStack.java,v 1.17 2002/06/05 14:55:51 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.event.ForcedInstance;

import java.util.Hashtable;
import java.util.Stack;
import java.util.EmptyStackException;

/**
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.17 $ $Date: 2002/06/05 14:55:51 $
 */
public class ScopeStack implements Cloneable {
    
    private mn8ConceptDefinition _conceptDef = null;

    private Stack _scopeStack = null;

    private GlobalScope _globalScope = GlobalScope.getReference();

    private String _conceptName = "";

    private Scope _conceptGlobalScope = null;


    public ScopeStack(mn8ConceptDefinition conceptDef) {
        _conceptDef = conceptDef;
        _scopeStack = new Stack();
        _conceptGlobalScope = new Scope(null);
        _scopeStack.push( _conceptGlobalScope );
        _conceptName = _conceptDef.getConceptType().getValue();
    }


    public ScopeStack( mn8ConceptDefinition conceptDef,
                       Stack scopeStack ) {
        _conceptDef = conceptDef;
        _scopeStack = scopeStack;
        _conceptGlobalScope = (Scope) _scopeStack.elementAt(0);
        _conceptName = _conceptDef.getConceptType().getValue();
    }
    

    public void clear() {
        _conceptDef = null;
        _scopeStack.clear();
        _globalScope.cleanAll();
        _conceptGlobalScope.clear();
    }


    public void pushScope( Scope newScope ) {
        _scopeStack.push( newScope );
    }


    public Scope getConceptGlobalScope() {
        return _conceptGlobalScope;
    }


    public int scopeStackSize() {
        return _scopeStack.size() + 1;
    }


    public Scope popScope() {
        try {
           return (Scope) _scopeStack.pop();
        }
        catch (EmptyStackException ex) {
            return _globalScope;
        }
    }

    
    public Scope getCurrentScope() {
        try {
            return (Scope)_scopeStack.peek();
        }
        catch (EmptyStackException ex) {
            return _globalScope;
        }
    }


    public Scope getGlobalScope() {
        return _globalScope;
    }


    public void insertGlobalInstance(String name, Concept instance ) {
        _globalScope.insertInstance( _conceptName + ":" + name, instance );
    }


    public void insertInstance( String name, Concept instance ) {
        if( mn8RuntimeFlags.isDebug() )
            System.err.println("  ->" + name + " of type " + instance.getConceptType().getValue() +
                               " inserted in the stack");
        if( isInstance( name ) ) {
            updateInstance( name, instance );
	}
        else {
            newInstance( name, instance);
	}
    }


    public Concept getInstance( String name ) {
        Scope scope = null;
        scope = getInstanceScope( name );
        if( scope == null )
            return null;
        else
            return scope.getInstance( name );
    }

    
    public boolean isInstance(String name) {
        if ( getInstanceScope(name) != null ) 
            return true;
        else 
            return false;
    }


    public void setForcedType(String name, String type) {
        getInstanceScope(name).setForcedType(name, type);

    }


    private void newInstance( String name, Concept instance ) {
        Scope scope = getCurrentScope();
        scope.insertInstance(name, instance);
    }


    private void updateInstance( String name, Concept instance) {
        Scope scope = getInstanceScope( name );
        if ( scope.isForcedInstance( name ) ) {
            String type = scope.getForcedType( name );
            if( !instance.getConceptType().getValue().equalsIgnoreCase(type) ) 
                throw new ForcedInstance(name, type);        
        }
        scope.insertInstance(name, instance);
    }
    
    
    private Scope getInstanceScope( String name ) {
        Scope scope = getCurrentScope();
        boolean found = false ;

        while ( scope != null && !found ) {
            if ( scope.isInstance(name) )
                return scope;
            else scope = scope.getPrevScope();
        }

        if( scope == null && _globalScope.isInstance( _conceptName + ":" + name )) 
            return _globalScope;

        return null;
    }


    public Object clone() {
        Stack newStack = new Stack();
        Scope curScope = null;

        for( int i = 0 ; i < _scopeStack.size(); i++) {
            curScope = (Scope) _scopeStack.elementAt(i);
            newStack.push( curScope.clone());
            if( i > 0 ) {
                ((Scope)newStack.elementAt(i)).setPrevScope((Scope) newStack.elementAt(i-1));
            }
        }
        return new ScopeStack( _conceptDef, newStack);
    }


    public void printDebugInfo() {
        System.err.println("Stack:" + _conceptDef.getConceptType().toString());
    }
}
