/* 
 * $COPYRIGHT$
 * $Id: AttributeConcept.java,v 1.18 2002/04/17 16:21:22 atech Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.util.Normalizer;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.18 $ $Date: 2002/04/17 16:21:22 $
 */
public class AttributeConcept extends JavaConcept implements Attribute {

    Concept _value = null;
    private StringConcept _name = null;
    private StringConcept _label = null;

    public AttributeConcept(){
        _value = new NilConcept();
        _name = new StringConcept("");
        _label = new StringConcept("");
        _value = new NilConcept();
        _showEmpty = false;
    }
    

    public AttributeConcept( String name, String label, Concept value ) {
        if ( value == null)             
            _value = new NilConcept();
        else 
            _value = value;

        _label = (label==null) ? new StringConcept(name) : new StringConcept(label);
        _name = new StringConcept( name );
        _showEmpty = false;
    }


    public AttributeConcept create(StringConcept label, Concept value ) {
        _name = _label = label;
        _value = value;
        return this;
    }


    public String toString() {
        return _value.toString();
    }

    
    public void setValue(Concept value) {
        _value = value;
    }


    public Concept getValue() {
        if( _value == null)
            return new NilConcept();
        return _value;
    }


    public StringConcept getName() {
        return _name;
    }


    public StringConcept getLabel() {
        return _label;
    }


    public void setLabel( StringConcept label) {
        _label = label;
        if( _name == null || _name.toString().trim() == "" ) {
            _name = _label;
        }
    }


    public StringConcept getValueType() {
        return _value.getConceptType();
    }

    
    public String getAttributeName( ) {
        return _name.getValue();
    }


    public void setAttributeName( String name ) {
        _name = new StringConcept(name) ;
    }


    public void setAttributeLabel( String label ) {
        _label = new StringConcept(label) ;
    }


    public String getAttributeLabel( ) {
        return _label.getValue();
    }

    
    public void setAttributeValue( Concept concept ) {
        setValue( concept );
    }


    public Concept getAttributeValue( ) {
        return getValue();
    }


    public String getAttributeType( ) {
        return _value.getConceptType().getValue();
    }


    public StringConcept toTXT() {

        if( isHidden().getValue() ) {
            return new StringConcept();
        }


        if( _value != null ) {

            if( showEmpty().getValue() == false && _value.toTXT().toString().trim().equals("")) {
                return new StringConcept();
            }

            return new StringConcept(_label.toString() + ": " + _value.toTXT());
        }
        else return new StringConcept("");
    }


    public int hashCode() {
        return _name.toString().hashCode();
    }

    
    public boolean equals(Object obj) {
        if( obj instanceof org.media.mn8.concepts.AttributeConcept ) {
		return (equals( (AttributeConcept)obj  )).getValue();
        }
        return false;
    }


    public StringConcept toXML() {
        if( isHidden().getValue() ) {
            return new StringConcept();
        }

        if ( _value != null) {
            if( showEmpty().getValue() == false && _value.toTXT().toString().trim().equals("")) {
                return new StringConcept();
            }
            String result = "";
            if( _label != null )
                result += _label.toString();
            else
                result += _name.toString();
            result += "=\"";
            result += Normalizer.normalize(_value.toTXT().toString()) + "\"";
            return new StringConcept( result );
        }
        else
            return new StringConcept();
    }

    
    public ConceptDefinition getConceptDefinition() {
        return AttributeDefinition.getReference();
    }


    public Object clone() {
       AttributeConcept attr =  new AttributeConcept( _name.toString(), _label.toString(), (Concept) _value.clone());
       attr.setShowEmpty(showEmpty());
       attr.setHidden(isHidden());
       return attr;
    }

    public LogicalConcept opEquals( AttributeConcept attr ) {  
        return equals( attr );
    }

    public LogicalConcept equals( AttributeConcept attr ) {  
        return new LogicalConcept( _label.toString().equalsIgnoreCase(attr.getAttributeLabel()) && 
                                               _value.equals(attr.getAttributeValue()) ); 
    }
   
}
