/*
 * $COPYRIGHT$
 * $Id: BEEPClientConcept.java,v 1.4 2002/03/20 17:25:19 atech Exp $
 *
 * Date          Author            Changes
 * IAN 24 2002   Szabo Csaba       Created
 */
       
package org.media.mn8.concepts;

import org.media.mn8.*;
import java.net.URL;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import org.media.datastore.sepengine.driver.*; 

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.4 $ $Date: 2002/03/20 17:25:19 $
 */

public class BEEPClientConcept extends JavaConcept {

    private static final String   DEFAULT_HOST = "localhost";
    private static final int      DEFAULT_PORT = 10288;
    private static final String   DEFAULT_USER = "guest";
    private static final String DEFAULT_PASSWD = "guest";

    protected URL _url;
    protected SepStoreConnection _con;

    
    public BEEPClientConcept() {
    }


    public BEEPClientConcept( StringConcept url ) {
        System.setProperty( "sep.drivers", "org.media.datastore.sepengine.driver.beepDriver.BeepDriver");
        initialize( url );
    }
    				 
		 
    public BEEPClientConcept create( StringConcept url ) {
        System.setProperty( "sep.drivers", "org.media.datastore.sepengine.driver.beepDriver.BeepDriver");
        initialize( url );
        return this;
    }
    
    protected boolean initialize( StringConcept url ) {
        URL givenURL = null;
	
        try {
            givenURL = new URL( url.toString() );
            if ( givenURL == null || givenURL.getProtocol().equalsIgnoreCase("beep") ) {
                if ( givenURL != null ) {
                    String   _host = givenURL.getHost();
                    int      _port = givenURL.getPort();
                    String      ui = givenURL.getUserInfo();
                    String _dbName = givenURL.getFile();
                    String _ref = givenURL.getRef() == null ? "" : "#" +givenURL.getRef() ;
                    String   _user = null;
                    String    _pwd = null;
                    
                    _url = givenURL;
                    
                    if ( _dbName.trim().equals("/") || _dbName.trim().length() < 1 )
                        new ErrorConcept( "error", "badURL", "XML DataBase name is missing. It is mandatory." );
                    
                    if ( ui != null ) 
                        if ( ui.indexOf(":") != -1 ) {
                            _user = ui.substring( 0, ui.indexOf(":") );
                            _pwd  = ui.substring( ui.indexOf(":")+1 );
                        }
                        else _user = ui;
                    
                    if ( _host == null || _host.length() < 1 ) _host = DEFAULT_HOST;
                    if ( _port == -1 )  _port = DEFAULT_PORT;
                    if ( _user != null && _user.length() < 1 ) _user = DEFAULT_USER;
                    if ( _pwd != null &&  _pwd.length()  < 1 )  _pwd = DEFAULT_PASSWD;
                    String myURL = "sep:beep://" + _host + ":" + _port + _dbName + _ref;
                    _con = SepStoreDriverManager.getConnection( myURL, _user, _pwd ); 
                }
            } else if ( givenURL != null ) {
                new ErrorConcept( "error", "unsupportedProtocol", "'" + givenURL.getProtocol() + 
                                  "' protocol not supported by BEEPClient" );	
            }
        }
        catch ( Exception ex ) {
            new ErrorConcept("error", "badURL", ex.getMessage() );
            return false;
        }
        
        return true;
    }
    
    
    public StringConcept execute( StringConcept sep ) {
        StringBuffer sb = new StringBuffer();
        String     line = null;
	
        try {
            BufferedReader reader = new BufferedReader( 
                                        new InputStreamReader( _con.executeSEP( sep.toString() ) ) );
            while ( ( line = reader.readLine() ) != null ) sb.append( line + "\n" );
            return new StringConcept( sb.toString() );
        } 
        catch ( Exception ex ) {
            new ErrorConcept( "error", "executeFailed", ex.getMessage() );
            return new StringConcept( "" );
        }
    }


    public ConceptDefinition getConceptDefinition() {
        return BEEPClientConceptDefinition.getReference();
    }
    
    
    public Object clone() {
        return this;
    }
}
