/* 
 * $COPYRIGHT$
 * $Id: BEEPHandler.java,v 1.4 2002/05/23 15:16:09 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 24 2002 Szabo Csaba       Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.util.Vector;
import java.util.StringTokenizer;
import java.net.URL;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.4 $ $Date: 2002/05/23 15:16:09 $
 */

public class BEEPHandler extends URIHandler {

    private String   _url = null;
    private String _uname = null;
    private String   _pwd = null;
    private MapConcept        _options;
    private BEEPClientConcept _client;


    public BEEPHandler( ){
    }


    public BEEPHandler( StringConcept url ) {
        _url = url.toString();
    }
    

    public URIHandler create( StringConcept url ) {
        _url = url.toString();
        return this;
    }

    
    public LogicalConcept acceptsURI( StringConcept uri ) {
        return ( (URIHandlerDefinition)getConceptDefinition() ).acceptsURI( uri );
    }
    
    
    public LogicalConcept initialize( MapConcept options ){
        URL givenURL = null;

        _options = options;
        
        if( options.containsKey(new StringConcept("user")).getValue() ) {
            _uname = options.getValue(new StringConcept("user")).toTXT().toString();
        }

        if( options.containsKey(new StringConcept("pwd")).getValue() ) {
            _pwd = options.getValue(new StringConcept("pwd")).toTXT().toString();
        }
	 
        try {
            givenURL = new URL( _url );
            if ( givenURL != null && givenURL.getProtocol().equalsIgnoreCase("beep") ) {
                if ( _uname != null && _uname.trim().length() > 0 && _pwd != null && _pwd.trim().length() > 0 ) {
                    _url  = givenURL.getProtocol() + "://";
                    _url += _uname + ":" + _pwd + "@";
                    if ( givenURL.getHost()  != null ) _url += givenURL.getHost();
                    if ( givenURL.getPort()  != -1   ) _url += ":" + givenURL.getPort();
                    if ( givenURL.getPath()  != null ) _url += givenURL.getPath();
                    if ( givenURL.getQuery() != null ) _url += "?" + givenURL.getQuery();
                    if ( givenURL.getRef()   != null ) _url += "#" + givenURL.getRef();
                }
            }
        }
        catch ( Exception ex ) {
            new ErrorConcept("error", "badURL", ex.getMessage() );
            return new LogicalConcept( false );
        }
        
        return new LogicalConcept( true );
    }


    public LogicalConcept closeConnection() {
        return new LogicalConcept( true );
    }

    
    public LogicalConcept openConnection() {
        _client = new BEEPClientConcept( new StringConcept( _url ) );
        return new LogicalConcept( true );
    }

 
    public Concept getContent() {
        try {
            URL     url = new URL( _url );
            String   qu = url.getQuery();
            
            if ( qu.trim().length() != 0 )
                return DSResponseConcept.getResponse( 
                   ((StringConcept)_client.execute( new StringConcept( qu.substring( qu.indexOf("=") + 1 ) ) )).toString() );
	            
            return new StringConcept( "" );
        } catch ( Exception ex ) {
            new ErrorConcept("error", "executeFailed", ex.getMessage() );
            return new StringConcept( "" );
        }
    }

 
    public LogicalConcept setContent( Concept value ) {
        return new LogicalConcept( false );
    }


    public ConceptDefinition getConceptDefinition() {
        return BEEPHandlerDefinition.getReference();
    }


    public Object clone() {        
        if( _url != null )
            return new BEEPHandler( new StringConcept( _url ) );
        else
            return new URIHandler();
    }
}
