/* 
 * $COPYRIGHT$
 * $Id: CharacterConcept.java,v 1.6 2002/04/28 14:50:05 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;


/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.6 $ $Date: 2002/04/28 14:50:05 $
 */
public class CharacterConcept extends JavaConcept {

    private static final String[] _chType = { "UNASSIGNED",
                                              "UPPERCASE_LETTER",
                                              "LOWERCASE_LETTER",
                                              "TITLECASE_LETTER",
                                              "MODIFIER_LETTER",
                                              "OTHER_LETTER",
                                              "NON_SPACING_MARK",
                                              "ENCLOSING_MARK",
                                              "COMBINING_SPACING_MARK",
                                              "DECIMAL_DIGIT_NUMBER",
                                              "LETTER_NUMBER",
                                              "OTHER_NUMBER",
                                              "SPACE_SEPARATOR",
                                              "LINE_SEPARATOR",
                                              "PARAGRAPH_SEPARATOR",
                                              "CONTROL",
                                              "FORMAT",
                                              "there is no such symbol ;)",
                                              "PRIVATE_USE",
                                              "SURROGATE",
                                              "DASH_PUNCTUATION",
                                              "START_PUNCTUATION",
                                              "END_PUNCTUATION",
                                              "CONNECTOR_PUNCTUATION",
                                              "OTHER_PUNCTUATION",
                                              "MATH_SYMBOL",
                                              "CURRENCY_SYMBOL",
                                              "MODIFIER_SYMBOL",
                                              "OTHER_SYMBOL" };
    
    private char _value = ' ';


    public CharacterConcept(){}
    

    public CharacterConcept( char value ) {
        _value = value;
    }


    public StringConcept toStringConcept() {
        return new StringConcept( "" + _value );
    }

    
    public StringConcept opPlus( CharacterConcept value ) {
        return new StringConcept( "" + _value + value.getValue());
    }


    public LogicalConcept opEquals( CharacterConcept value ) {
        return new LogicalConcept( _value == value.getValue() );            
    }


    public CharacterConcept opPlus( NilConcept value ) {
        return new CharacterConcept( _value );
    }

    
    public LogicalConcept opLT ( CharacterConcept value) {
        return new LogicalConcept( _value < value.getValue() );
    }


    public LogicalConcept opGT ( CharacterConcept value) {
        return new LogicalConcept( _value > value.getValue() );
    }


    public LogicalConcept isDigit() {
        return new LogicalConcept( Character.isDigit( _value ) );
    }


    public LogicalConcept isWhitespace() {
        return new LogicalConcept( Character.isWhitespace( _value ) );
    }


    public LogicalConcept isLetter() {
        return new LogicalConcept( Character.isLetter( _value ) );
    }


    public LogicalConcept isLetterOrDigit() {
        return new LogicalConcept( Character.isLetterOrDigit( _value ) );
    }


    public LogicalConcept isControl() {
        return new LogicalConcept( Character.isISOControl( _value ) );
    }


    public LogicalConcept isLowerCase() {
        return new LogicalConcept( Character.isLowerCase( _value ) );
    }


    public CharacterConcept toLowerCase() {
        return new CharacterConcept( Character.toLowerCase( _value ) );
    }


    public LogicalConcept isUpperCase() {
        return new LogicalConcept( Character.isUpperCase( _value ) );
    }


    public CharacterConcept toUpperCase() {
        return new CharacterConcept( Character.toUpperCase( _value ) );
    }


    public IntegerConcept getUnicodeNumericValue() {
        return new IntegerConcept( Character.getNumericValue( _value ) );
    }


    public StringConcept getType() {
        return new StringConcept( _chType[Character.getType(_value)] );
    }


    public IntegerConcept toIntegerConcept() {
        return new IntegerConcept( (int) _value );
    }


    public StringConcept toTXT() {
        return toStringConcept();
    }
    

    public StringConcept toXML() {
        return toTXT();
    }


	public StringConcept getEOLChar (){
		return new StringConcept(mn8Interpreter.EOL);
	}


    public char getValue() {
        return _value;
    }

    
    public void setValue( char value ) {
        _value = value;
    }


    public boolean equals (Object obj) {
        if( obj instanceof org.media.mn8.concepts.CharacterConcept ) 
            if (( (CharacterConcept)obj).getValue() == _value ) {
                return true;
            }
        return false;
    }


    public void main ( SeriesConcept param ) {
        System.out.println(_value);
    }


    public ConceptDefinition getConceptDefinition() {
        return CharacterDefinition.getReference();
    }


    public LogicalConcept hasPath( StringConcept path ) {
        return new LogicalConcept(false);
    }

    
    public SeriesConcept getConceptsAtPath( StringConcept path ) {
        return new SeriesConcept();
    }


    public String toString() {
        return "" + _value;
    }


    public Object clone() {
        return new CharacterConcept( _value);
    }    
}



