/* 
 * $COPYRIGHT$
 * $Id: CookieConcept.java,v 1.6 2002/05/28 15:58:06 atech Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;

import org.media.mn8.Helper;
import org.media.mn8.Concept;
import org.media.mn8.JavaConcept;
import org.media.mn8.ConceptDefinition;
import java.util.Vector;
import org.media.mn8.protocol.http.*;
import java.io.File;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.6 $ $Date: 2002/05/28 15:58:06 $
 */
public class CookieConcept extends JavaConcept {

    public static String COOKIE_DB =  Helper.getUserHome().getValue() + 
        ".mn8" + File.separatorChar +
        "http" + File.separatorChar +
        "cookies" + File.separatorChar;

    private Cookie _cookie = new Cookie();

    public AttributeConcept name = new AttributeConcept("name", "name", new StringConcept());
    public AttributeConcept value = new AttributeConcept("value", "value", new StringConcept());
    public AttributeConcept domain = new AttributeConcept("domain", "domain", new StringConcept());
    public AttributeConcept path = new AttributeConcept("path", "path", new StringConcept());
    public AttributeConcept expires = new AttributeConcept("expires", "expires", new StringConcept());
    public AttributeConcept secure = new AttributeConcept("secure", "secure", new StringConcept());

    
    public CookieConcept( ){

    }


    public CookieConcept( StringConcept cookieContent, StringConcept domain) {
        setCookie( new Cookie( cookieContent.toString(), domain.toString()));
    }

    
    public Concept create() {
        return this;
    }

    
    public CookieConcept create(StringConcept cookieContent, StringConcept domain) {
        setCookie( new Cookie( cookieContent.toString(), domain.toString()));
        return this;
    }
 

    public CookieConcept( Cookie cookie ) {
        setCookie(cookie);
    }

    
    public void setName(StringConcept name) {
        _cookie.setName(name.toString());
        setCookie();
    }


    public void setValue(StringConcept value) {
        _cookie.setValue(value.toString());
        setCookie();
    }


    public void setDomain(StringConcept domain) {
        _cookie.setDomain(domain.toString());
        setCookie();
    }

    public void setPath(StringConcept path) {
        _cookie.setPath(path.toString());
        setCookie();
    }


    public void setExpires(StringConcept expires) {
        _cookie.setExpires(expires.toString());
        setCookie();
    }


    public void setSecure(StringConcept secure) {
        _cookie.setSecure(secure.toString());
        setCookie();
    }


    public void setCookie(Cookie cookie){
        if( cookie != null) {
            _cookie = cookie;
            setCookie();
        }
    }
            
    public void setCookie() {       
        name.setAttributeValue( new StringConcept(_cookie.getName()));
        value.setAttributeValue( new StringConcept(_cookie.getValue()));
        domain.setAttributeValue( new StringConcept( _cookie.getDomain()));
        path.setAttributeValue( new StringConcept( _cookie.getPath()));
        expires.setAttributeValue( new StringConcept( _cookie.getExpires()));
        secure.setAttributeValue( new StringConcept( _cookie.getSecure()));        
    }
    

    public Cookie getCookie() {
        return _cookie;
    }


    public boolean equals (Object obj) {
        if( obj instanceof CookieConcept ) 
            if (((CookieConcept)obj).getCookie().equals( _cookie ) ) {
                return true;
            }
        return false;
    }


    public Object clone() {
        return new CookieConcept(_cookie);
    }


    public StringConcept toTXT() {
        return toStringConcept();
    }


    public StringConcept toStringConcept() {
        if( _cookie != null )
            return new StringConcept(_cookie.toString());
        return new StringConcept();
    }


    public StringConcept toXML() {
        if( _cookie != null )
            return new StringConcept(_cookie.toXMLString());
        return new StringConcept();
    }


    public ConceptDefinition getConceptDefinition() {
        return CookieDefinition.getReference();
    }



    public void saveCookie(StringConcept dir) {
        String cookiedb = dir.toString();
        if( cookiedb.charAt( cookiedb.length() -1 ) != File.separatorChar) 
            cookiedb += File.separatorChar;
        if( _cookie == null) return;
        Vector cookies = new Vector();
        cookies.add(_cookie);
        _cookie.writeCookiesToXML(_cookie.getDomain(), cookies , cookiedb);
    }


    public void storeCookie() {
        if( _cookie == null) return;
        Vector cookies = new Vector();
        cookies.add(_cookie);
        _cookie.writeCookiesToXML(_cookie.getDomain(), cookies , COOKIE_DB);
    }
}
