/* 
 * $COPYRIGHT$
 * $Id: CronConcept.java,v 1.4 2002/05/21 14:10:56 neuro Exp $
 *
 * Date        Author            Changes 
 * May 16 2002 Antal Attila      Created
 */
package org.media.mn8.concepts;

import java.util.Vector;
import java.util.NoSuchElementException;
import org.media.mn8.*;
import org.media.mn8.util.Cron;
import org.media.mn8.util.CronMultiplier;
import org.media.mn8.util.CronCollector;
import org.media.mn8.util.CronTimeScheduler;

/**
 * @author <a href="mailto:atech@nolimits.ro">Antal Attila</a>
 * @version $Revision: 1.4 $ $Date: 2002/05/21 14:10:56 $
 */

public class CronConcept extends JavaConcept {


    static private int id = 0;
    static private TypedSeries  tasks = new TypedSeries( "task", "task" );  
    static public ElementConcept task = new ElementConcept( "tasks", "tasks", tasks, null );
    static private Vector repository = new Vector();
    static private CronTimeScheduler timeScheduler = new CronTimeScheduler();

    /**
     *  Singleton definition, use the Cron.getInstance to
     *  acces this instance
     */
    private CronConcept() {
    }

    static private CronConcept _instance = null;

    /**
     * @return The unique instance of this class.
     */
    static public CronConcept getInstance() {
        if(null == _instance) {
            _instance = new CronConcept();
        }
        return _instance;
    }



    static public IntegerConcept addTask( StringConcept cron, StringConcept target ) {
        id++;
        
        CronCollector collector = new CronCollector();
        collector.targetName = target.getValue();  
        collector.cronString = cron.getValue();
        repository.add(id-1, collector);
        
        String _id = "" + id;
        ElementConcept ec = new ElementConcept("task", "task", null, null);
        ec.addAttribute( new AttributeConcept( "id", "id", new StringConcept(_id) ));
        ec.addAttribute( new AttributeConcept( "timeString", "timeString", cron ));
        ec.addAttribute( new AttributeConcept( "commandString", "CommandString", target ));
        tasks.add(ec);
        initialize();
        return new IntegerConcept( id );
    } 
    

    static public TypedSeries getTasks() {
        return tasks;
    }
    
    
    static public void removeTask( IntegerConcept rid ) {
        repository.remove( rid.getInt()-1 );
        tasks.remove(rid);
        int sec_id = 0;
        boolean loop = true;
        while (loop) {
            String _id = "" + rid.getInt() + "." + sec_id;
            try {
                timeScheduler.getEntry(_id);
                timeScheduler.resetTrigger(_id);
                timeScheduler.removeTrigger(_id);
            }
            catch (NoSuchElementException e) {
                loop = false;
            }
            sec_id++;
        }
    } 
    
    
    static public void start() {
        try {
            timeScheduler.start();
        }
        catch (Exception e) {
            new ErrorConcept ("error", "errOnStart",
                              "Error on starting the Cron: " + e.getMessage() );
            
        }
	} 
    

    static public void stop() {
        timeScheduler.stop();
    } 
    
    
    static private void initialize() {
        Cron trigger;
        CronMultiplier cronMaker;
        CronCollector collector;
        String tid, target;
        
        for (int cy = 0; cy < repository.size(); cy++) {
            collector = (CronCollector) repository.get(cy);
            target =  collector.targetName;
            cronMaker = new CronMultiplier();
            cronMaker.setMaker( collector.cronString );
            if ( !cronMaker.ignored ) {
                for (int cx = 0; cx < cronMaker.getSize(); cx++) {
                    cronMaker.setCurrent(cx);
                    trigger = new Cron( cronMaker.getMin(),
                                        cronMaker.getHour(),
                                        cronMaker.getDay(),
                                        cronMaker.getMonth(),
                                        -1,
                                        cronMaker.getIsDayOfWeek() );
                    tid = "" + (cy+1) + "." + cx;
                    try {
                        timeScheduler.addTrigger( tid, trigger, target);
                    }
                    catch(Exception e) {
                        new ErrorConcept ("error", "errAddTrigger", e.getMessage());
                    }
                }
            }
            else {
                new ErrorConcept ("error", "invalidCronString",
                                  "Invalid Cron String: " + collector.cronString +
                                  "\n\t\t\t   " + cronMaker.getErrMsg() );
            }
        }
    }
    
    
    /** 
     * Return the CronConceptDefinition object
     */
    public ConceptDefinition getConceptDefinition() {
        return CronDefinition.getReference();
    }
    
    
    /**
     * Clone this object
     */
    public Object clone() {
        return this;
    }
        
}
