/* 
 * $COPYRIGHT$
 * $Id: DateConcept.java,v 1.10 2002/05/21 14:10:22 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 * IAN 16 2002 Szabo Csaba       Added functionality
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.text.SimpleDateFormat;
import java.text.ParsePosition;
import java.util.Date;


/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.10 $ $Date: 2002/05/21 14:10:22 $
 */
public class DateConcept extends JavaConcept {
    
    private long _value;


    public DateConcept( ) {
        _value = System.currentTimeMillis();
    }


    public DateConcept ( long date ) {
        _value = date;
    }
  

    public DateConcept( StringConcept format, StringConcept input_date ) {
        setDate( format, input_date );
    }
    
    
    public DateConcept( IntegerConcept input_date ) {
        setDate( input_date );
    }
    
    
    public Concept create( StringConcept format, StringConcept input_date ) {
        setDate( format, input_date );
        return this;
    }
    
    
    public Concept create( IntegerConcept input_date ) {
        setDate( input_date );
        return this;
    }


    public Concept create() {
        _value = System.currentTimeMillis();
        return this;
    }



    public LogicalConcept opEQ(DateConcept value) {
        return new LogicalConcept( getValue() == value.getValue());
    }


    public LogicalConcept opGT(DateConcept value) {
        return new LogicalConcept( getValue() > value.getValue());
    }


    public LogicalConcept opLT(DateConcept value) {
        return new LogicalConcept( getValue() < value.getValue());
    }


    public StringConcept getDate( ) {
        return new StringConcept( new Date( _value ).toString() );
    }


    public IntegerConcept getTime( ) {
        return new IntegerConcept(_value);
    }
    
    
    public StringConcept getDate( StringConcept format ) {
        SimpleDateFormat formatter = new SimpleDateFormat( format.toString() );
        return new StringConcept( formatter.format( new Date( _value ) ) );
    }
    
    
    public void setDate( StringConcept format, StringConcept input_date ) {
        SimpleDateFormat formatter = new SimpleDateFormat( format.toString() );
        Date myDate = formatter.parse( input_date.toString(), new ParsePosition( 0 ) );
        _value = myDate.getTime();
    }


    public void setDate( IntegerConcept input_date ) {
        _value = input_date.getLong();
    }
    
    
    public ConceptDefinition getConceptDefinition() {
        return DateDefinition.getReference();
    }


    public StringConcept toTXT() {
        return new StringConcept( (new Date(_value)).toString());
    }


    public StringConcept toXML() {
        return toTXT();
    }


    public long getValue () {
        return _value;
    }

    public boolean equals (Object obj) {
        if( obj instanceof org.media.mn8.concepts.DateConcept ) 
            if ( ( (DateConcept)obj).getValue() == _value ) {
                return true;
            }
        return false;
    }

    
    public Object clone() {
        return new DateConcept( new IntegerConcept ( _value ) );
    }
}
