/* 
 * $copyright
 * $Id: ElementConcept.java,v 1.23 2002/04/29 17:13:52 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import java.util.Vector;
import java.lang.reflect.Method;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.23 $ $Date: 2002/04/29 17:13:52 $
 */
public class ElementConcept extends JavaConcept implements Element {

    Concept _value = null;
    protected StringConcept _name = null;
    protected StringConcept _label = null;
    protected AttributeSeries _attribs = new AttributeSeries();
    protected boolean _multi = false;


    public ElementConcept(){
        _name = new StringConcept("");
        _label = _name;
        _value = new NilConcept();
    }
    

    public ElementConcept( String name, String label, Concept value, SeriesConcept attributes ) {
        if ( value == null)             
            _value = new NilConcept();
        else {
            _value = value;
            //_value.setConceptLabel(new StringConcept(label));
        }

        _label = (label==null) ? new StringConcept(name) : new StringConcept(label);
        _label = (label.trim().equals("")) ? new StringConcept(name) : new StringConcept(label);
        _name = new StringConcept( name );
        
        if ( attributes == null )
            _attribs = new AttributeSeries(_value.getConceptAttributes().getVector());
        else
            _attribs = new AttributeSeries(attributes.getVector());
    }

    
    public ElementConcept( String name, String label, Concept value, SeriesConcept attributes, boolean multi, String multiType) {
        _label = (label==null) ? new StringConcept(name) : new StringConcept(label);
        _name = new StringConcept( name );
        
        if ( attributes != null )
            _attribs = new AttributeSeries(attributes.getVector());        

        _multi = multi;
        if( _multi ) {
            _value = new TypedSeries(multiType, _label.toString());
            _value.setConceptLabel(_label);
            if( value != null && !( value instanceof NilConcept ))                
                ((SeriesConcept)_value).add(value);
        } else {
            if( value != null ){
                _value = value;
            } else {
                _value = new NilConcept();
            }
        }        
        if( _multi && value instanceof SeriesConcept ) {
            _value = value;
        }
    }


    public ElementConcept create(StringConcept label, Concept value) {
        _name = _label = label;
        _multi = false;
        _value = value;
        return this;
    }



    public String toString() {
        return _value.toString();
    }

    
    public void addValue(Concept value) {
        if( _value != null && _value instanceof TypedSeries ) {
            ((TypedSeries)_value).add(value);
            if( !_multi && ((TypedSeries)_value).getVector().size()> 1 ) {
                _multi = true;
            }
        } else if ( _value == null || _value instanceof NilConcept ) {
            setValue(value);
        } else {
            TypedSeries newValue = new TypedSeries();
            newValue.setConceptLabel(_label);
            newValue.add( _value );
            newValue.add( value );
            setValue( newValue );
            _multi = true;
        }
    }


    public void setValue(Concept value) {
        _value = value;
    }


    public Concept getValue() {
        return _value;
    }


    public StringConcept getName() {
        return _name;
    }

    
    public SeriesConcept getAttributes() {
        return _attribs;
    }


    public void setAttributes( SeriesConcept attributes ) {
        _attribs = new AttributeSeries(attributes.getVector());
    }


    public void addAttribute( AttributeConcept attr ) {
        if( attr != null ) {
            _attribs.add(attr);
        }
    }

    
    public void addAttribute( StringConcept name, StringConcept label, Concept value ) {
        _attribs.add( new AttributeConcept(name.toString(), label.toString(), value));
    }
    

    public StringConcept getLabel() {
        return _label;
    }


    public int hashCode() {
        return _name.toString().hashCode();
    }


    public boolean equals(Object obj) {
        if( obj instanceof org.media.mn8.concepts.ElementConcept ) {
            return equals( (ElementConcept) obj ).getValue() ;
        }
        return false;
    }

    
    public void setLabel( StringConcept label ) {
        _label = label;
        if( _name != null && _name.toString().trim().equals("")) {
            _name = label;
        }
    }
    

    public StringConcept getValueType() {
        return _value.getConceptType();
    }


    public String getElementName( ) {
        return _name.getValue();
    }

    public void setElementName( String name ) {
        _name = new StringConcept(name);
    }


    public String getElementLabel( ) {
        return _label.getValue();
    }


    public void setElementLabel( String label ) {
        _label = new StringConcept(label);
    }
    

    public void setElementValue( Concept concept ) {
        setValue( concept );
    }


    public Concept getElementValue( ) {
        return getValue();
    }


    public String getElementType( ) {
        return _value.getConceptType().getValue();
    }


    public void setConceptLabel( StringConcept label ) {
        setLabel(label);
    }


    public StringConcept getConceptLabel() {
        return getLabel();
    }


    public StringConcept toTXT() {
        if( _value == null || isHidden().getValue()) 
            return new StringConcept();

        StringBuffer value = new StringBuffer();
        Vector elements = _value.getConceptElements().getVector();

        if( elements.size() <= 0 ) {
            value.append("\t" + getValue().toTXT().toString());
        } else {
            for(int i =0 ; i < elements.size(); i++) 
                value.append( ((Concept) elements.elementAt(i)).toTXT());
        }

        if( !showEmpty().getValue() && value.toString().trim().equals("")
            && getAttributes().toXML().toString().trim().equals("")){
            return new StringConcept();
        }

        String result = "";

        if( _attribs.getVector().size() > 0 )
            result= _label.toString() + mn8Interpreter.EOL +  _attribs.toTXT().toString() + mn8Interpreter.EOL; 
        else
            result= _label.toString() + mn8Interpreter.EOL;

        result = result +  value.toString();
        return new StringConcept(result);
    }


    public StringConcept toXML() {
        if( _value == null || isHidden().getValue()){
            return new StringConcept();
        }
        
        StringBuffer result = new StringBuffer();
        String value  = _value.toXML().toString();

        if( !showEmpty().getValue() && value.trim().equals("") 
            && getAttributes().toXML().toString().trim().equals("")){
            return new StringConcept();
        }

        result.append("<");
        result.append(getConceptLabel().toString());
        result.append(getAttributes().toXML().toString());
        result.append(">"); 
        if( !Helper.startsWithEOL(value) && value.trim().startsWith("<") && Helper.hasEOL(value) ){
            result.append(mn8Interpreter.EOL);
        }

        result.append(value);

        if( Helper.hasEOL(value) && !Helper.endsWithEOL(value) && value.trim().endsWith(">")) {
            result.append(mn8Interpreter.EOL);
        }

        result.append("</" + getConceptLabel().toString() + ">");
        return new StringConcept(result.toString());
    }


    public ConceptDefinition getConceptDefinition() {
        return ElementDefinition.getReference();
    }

    
    public LogicalConcept hasConceptElement(StringConcept name) {
        return new LogicalConcept(hasConceptElement(name.toString()));
    }


    public boolean hasConceptElement(String name) {

        if( _value != null && _value instanceof Element && ((ElementConcept)_value).getName().toString().equalsIgnoreCase(name)) 
            return true;

        if( _value.hasConceptElement(name))
            return true;
        return super.hasConceptElement(name);
    }


    public boolean hasConceptAttribute(String name) {
        if( _attribs.hasConceptElement(name))
            return true;
        return super.hasConceptAttribute(name);
    }


    public void setShowEmpty(LogicalConcept showEmpty) {
        _showEmpty = showEmpty.getValue();
        _value.setShowEmpty(showEmpty);
        for( int i = 0 ; i < _attribs.getVector().size(); i++ ) {
            ((Concept) _attribs.getVector().elementAt(i)).setShowEmpty(showEmpty);
        }
    }
    

    public SeriesConcept getConceptElements() {
        if( _value != null && ! _multi ) {
            if( _value instanceof TypedSeries ) {
                _multi = true;
                return _value.getConceptElements();
            }

            SeriesConcept result = new TypedSeries();
            result.add( new TransparentElement( Helper.extractName(_value).toString(),
                                                _value.getConceptLabel().toString(),
                                                _value,
                                                null ));
            return result;
        }
        return _value.getConceptElements();
    }

    
    public Concept getConceptElement( StringConcept name ) {
        return getConceptElement(name.toString());
    }


    public Concept getConceptElement(String name) {
        if( _value != null && Helper.isElement(_value)  && ((ElementConcept)_value).getName().toString().equalsIgnoreCase(name)) {
            return _value;
        }
        return _value.getConceptElement(name);
    }


    public FieldDefinition getConceptElementField(StringConcept name) {
        return getConceptElementField(name.toString());
    }


    public FieldDefinition getConceptElementField( String name) {

        if( _value != null && Helper.isElement(_value) ) {

            return new DummyFieldConcept( ((Element)_value).getElementName(),
                                          _value,
                                          false,                                          
                                          ((Element)_value).getElementLabel(),
                                          ((Concept)_value).getConceptType().toString());
                
        }
        return _value.getConceptElementField(name);
    }


    public LogicalConcept isMulti() {
        return new LogicalConcept(_multi);
    }


    public SeriesConcept getConceptElementFields( ) {
        if( _value != null && _value instanceof Element ) {
            SeriesConcept result = new TypedSeries();
            result.add( (FieldConcept)getConceptElementField(""));
            return result;                        
        }
        return _value.getConceptElementFields();
    }



    public SeriesConcept getConceptAttributeFields( ) {
        SeriesConcept result = new SeriesConcept();
        for( int i = 0 ; i < _attribs.getVector().size(); i++ ) {
            if( _attribs.getVector().elementAt(i) instanceof Attribute ) {
                result.add( new DummyFieldConcept( 
                                                  ((Attribute)_attribs.getVector().elementAt(i)).getAttributeName(),
                                                  (AttributeConcept)_attribs.getVector().elementAt(i),
                                                  false,
                                                  ((Attribute)_attribs.getVector().elementAt(i)).getAttributeLabel(),
                                                  ((Attribute)_attribs.getVector().elementAt(i)).getAttributeType(),
                                                  false ));
            }
        }
        return result;
    }


    public FieldDefinition getConceptAttributeField( String name) {
        Element hidenElem = (Element)_attribs.getConceptElement(new StringConcept(name));
        AttributeConcept attr = null;

        if( hidenElem.getElementValue() instanceof Attribute ) {
            attr = (AttributeConcept)hidenElem.getElementValue();
        }
        return new DummyFieldConcept( attr.getAttributeName(),
                                      attr,
                                      false,
                                      attr.getAttributeLabel(),
                                      attr.getAttributeType(),
                                      false );
    }


    public Concept getConceptAttribute(String name) {
        return ((TransparentElement)_attribs.getConceptElement(name)).getValue();
    }


    public SeriesConcept getConceptAttributes() {
        SeriesConcept result = new SeriesConcept();
        for( int i = 0 ; i < _attribs.getVector().size(); i++ ) {
            if( _attribs.getVector().elementAt(i) instanceof Attribute ) {
                result.add( (AttributeConcept) _attribs.getVector().elementAt(i));
            }
        }
        return result;
    }


    public Object clone() {
        ElementConcept result =  new ElementConcept( _name.toString(), 
                                                     _label.toString(), 
                                                     _value == null ? null : (Concept)_value.clone(), 
                                                     (SeriesConcept)_attribs.clone(), 
                                                     _multi,
                                                     _value instanceof TypedSeries ? ((TypedSeries)_value).getType().toString() : "Nil");
        result.setShowEmpty(showEmpty());
        result.setHidden(isHidden());
        return result;
    }


    public void loadContent(Concept from) {        
        if( mn8RuntimeFlags.isDebug() ) {
            System.err.println("--> in element:" + Helper.getConceptInfo(this) + "->" + Helper.getConceptInfo(from));
        }

        if( from instanceof TransparentElement ) {
            ElementConcept el = (ElementConcept) from;
            if( getLabel().toString().equalsIgnoreCase( el.getLabel().toString())) {
                if( !_multi && ! el.isMulti().getValue() ) {

                    if( el.getValue() instanceof ElementConcept ) {
                        loadContent(el.getValue());
                    } else {
                        setValue( ((ElementConcept)from).getValue());
                        Concept conc = Helper.loadAttributes(from, this);
                        setAttributes(conc.getConceptAttributes());
                    }
                } else {
                    if( _value instanceof NilConcept ) {
                       setValue(el.getElementValue());
                    } else {
                        _value.loadContent(el.getValue());
                    }
                }
            }
        } else {
            if( from instanceof ElementConcept ) {
                if( getLabel().toString().equalsIgnoreCase( ((ElementConcept)from).getLabel().toString())) {
                    if( !_multi && !((ElementConcept)from).isMulti().getValue() ) {
                        setValue( ((ElementConcept)from).getValue());
                        Concept conc = Helper.loadAttributes(from, this);
                        setAttributes(conc.getConceptAttributes());
                    } else {
                        if( _value instanceof NilConcept ) {
                            setValue( ((ElementConcept)from).getElementValue());
                        } else {
                            _value.loadContent(from);
                        }
                    }
                }
            }
        }   
    }


    public LogicalConcept opEquals( ElementConcept elem ) {
        return equals( elem );
    }


    public LogicalConcept equals( ElementConcept elem ) {
        Vector     myAttrs = getAttributes().getVector();
        Vector targetAttrs = elem.getAttributes().getVector();
        if ( !myAttrs.equals( targetAttrs ) ) return new LogicalConcept(false);
        if ( !_label.toString().equals(elem.getElementLabel()) )  return new LogicalConcept(false);
        if ( !_value.equals(elem.getElementValue()) )  return new LogicalConcept(false);
        return new LogicalConcept( true );
    }


    public LogicalConcept subElementHasPath(String elementName, String subPath) {

        if( !(_value instanceof TransparentElement) && _value instanceof Element &&
            ( ((ElementConcept)_value).getName().toString().equalsIgnoreCase(elementName) ||
              ((ElementConcept)_value).getLabel().toString().equalsIgnoreCase(elementName))) {
            return _value.hasPath(subPath);
        }

        return _value.subElementHasPath(elementName, subPath);
    }


    public boolean areThereMoreElemenents(String path ) {
        if( !(_value instanceof TransparentElement) && _value instanceof Element ) {
            return _value.getConceptLabel().equals((Object) new StringConcept(path)); 
        }

        return _value.areThereMoreElemenents(path);
    }


    public SeriesConcept searchDeeperForPath(SeriesConcept holder, String path) {

        if( !(_value instanceof TransparentElement) && _value instanceof Element ) {
            return _value.getConceptsAtPath(holder, path); 
        }
        return _value.searchDeeperForPath(holder, path);
    }


    public Vector getMeAllElements( String path ) {
        if( !(_value instanceof TransparentElement) && _value instanceof Element ) {
            Vector result = new Vector();
            result.add(_value);
            return result;
        }

        return _value.getMeAllElements(path);
    }
}
