/* 
 * $COPYRIGHT$
 * $Id: EnvHandler.java,v 1.6 2002/05/02 18:28:38 neuro Exp $
 *
 * Date        Author            Changes 
 * FEB 06 2002 Szabo Csaba       Created
 */
package org.media.mn8.concepts;

import javax.naming.NamingException;
import javax.naming.NamingEnumeration;
import javax.naming.NameClassPair;

import org.media.mn8.*;
import org.media.mn8.event.*;


/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.6 $ $Date: 2002/05/02 18:28:38 $
 */

public class EnvHandler extends URIHandler {

    private String _url = null;
    private ExpressionConcept _filter;
    private MapConcept _options;


    public EnvHandler( ){
    }


    public EnvHandler( StringConcept url ) {
        _url = url.toString();
    }
  

    public URIHandler create( StringConcept url ) {
       _url = url.toString();	
       return this;
    }

    
    public LogicalConcept acceptsURI( StringConcept uri ) {
        return ((URIHandlerDefinition)getConceptDefinition()).acceptsURI( uri );
    }
 

    public LogicalConcept initialize( MapConcept options ){
        _options = options;

        if( options.containsKey(new StringConcept("filter")).getValue() )
            _filter = (ExpressionConcept) options.getValue(new StringConcept("filter"));

        return new LogicalConcept(true);
    }


    public LogicalConcept openConnection() {
        return new LogicalConcept(true);
    }


    public LogicalConcept setContent(Concept value) {
        mn8OperationNotSupported ex =  new mn8OperationNotSupported("env:/ does not support setContent, use mem:/ instead");     
        ex.conceptName = "EnvHandler";
        throw ex;
    }


    public Concept getContent() {
        try {
            return (Concept)NamingManager.getReference().lookup( _url );
        } 
        catch ( Exception ex ) {	
            try {
                SeriesConcept result = new SeriesConcept();
                
                for ( NamingEnumeration ne = NamingManager.getReference().list( _url ); ne.hasMore(); ) {
                    Object obj = ne.next();
                    
                    if ( obj instanceof NameClassPair ) {
                        String myPath = ( _url.endsWith("/") ? _url : _url + "/") + 
                            ((NameClassPair)obj).getName();
                        StringConcept _path = new StringConcept( myPath );
                        if ( _filter != null && !_filter.matches( _path ).getValue() ) continue;
                        try {
                            NamingManager.getReference().list( _path.toString() );
                            _path = new StringConcept( _path.toString() + "/" );
                        } catch ( Exception _ex ) {}
                        result.add( _path );
                    }
                    else if ( obj instanceof String )
                        result.add ( new StringConcept( (String)obj ) ); 
                }
                
                return result;
            }
            catch ( NamingException nex ) {
                new ErrorConcept( "warning", "badURL", nex.getMessage() );
                return new StringConcept( "" );
            }
        }
    }


    public LogicalConcept closeConnection() {
        return new LogicalConcept( true );
    }
    

    public ConceptDefinition getConceptDefinition() {
        return EnvHandlerDefinition.getReference();
    }


    public Object clone() {        
        if( _url != null )
            return new EnvHandler( new StringConcept( _url ) );
        else
            return new URIHandler();
    }
}
