/* 
 * $COPYRIGHT$
 * $Id: ErrorConcept.java,v 1.20 2002/06/03 11:34:03 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Antal Attila      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.parser.mn8TreeWalker;
import org.media.mn8.event.*;
import java.util.Stack;

import java.io.PrintStream;
import java.lang.Thread;

/** 
 * @author <a href="mailto:atech@nolimits.ro">Antal Attila</a>
 * @version $Revision: 1.20 $ $Date: 2002/06/03 11:34:03 $
 */
public class ErrorConcept extends JavaConcept {

    private StringConcept _id = new StringConcept("");
    private StringConcept usage = new StringConcept("No error type defined in Error.create!\n" + 
                                                    "\n\tUsage: Error.create( \"error\", <methodName>, <message> )\n" + 
                                                    "\t       Error.create( \"warning\", <methodName>, <message>)\n"); 

    private StringConcept _msg = new StringConcept("");
    private StringConcept _type = new StringConcept("");
    private Concept _concept = null;
    private Concept _setplace = null;
    private String _set_staticplace = null;
    public int errorLine = -1;
    public Concept result = new NilConcept();

    public ErrorConcept() {
        if( mn8TreeWalker.lineNr != -1 ) {
            errorLine = mn8TreeWalker.lineNr;
        }
    }


    public ErrorConcept( String type, String id, String msg ) {
        if( mn8TreeWalker.lineNr != -1 ) {
            errorLine = mn8TreeWalker.lineNr;
        }
        _id   = new StringConcept( id );
        _msg  = new StringConcept( msg );
        _type = new StringConcept( type ); 
        handleTheError();
    }	

    
    public ErrorConcept( StringConcept type, StringConcept id, StringConcept msg ) {
        if( mn8TreeWalker.lineNr != -1 ) {
            errorLine = mn8TreeWalker.lineNr;
        }
        _id = id;
        _msg = msg;
        _type = type;
        handleTheError();
    }



    public ErrorConcept( String type, String id, String msg, int lineNr ) {
        this(new StringConcept(type), new StringConcept(id), new StringConcept(msg), lineNr);
    }


    public ErrorConcept( StringConcept type, StringConcept id, StringConcept msg, int lineNr ) {
        _id = id;
        _msg = msg;
        _type = type;
        errorLine = lineNr;
        handleTheError();
    }


    public ErrorConcept( StringConcept type, StringConcept id, StringConcept msg, Concept con ) {
        if( mn8TreeWalker.lineNr != -1 ) {
            errorLine = mn8TreeWalker.lineNr;
        }
        _id = id;
        _msg = msg;
        _type = type;
        _concept = con;	
        handleTheError();
    }


    public ErrorConcept create( StringConcept type, StringConcept id, StringConcept msg ) {
        if( mn8TreeWalker.lineNr != -1 ) {
            errorLine = mn8TreeWalker.lineNr;
        }
        _id  =  id;
        _msg = msg;
        _type = type;
        handleTheError();
        return this;
    }


    public ErrorConcept create( StringConcept type, StringConcept id, StringConcept msg, Concept con ) {
        if( mn8TreeWalker.lineNr != -1 ) {
            errorLine = mn8TreeWalker.lineNr;
        }
        _id  =  id;
        _msg = msg;
        _type = type;
        _concept = con;
        handleTheError();
        return this;
    }


    public void handleTheError() {
        String _met = null;
        String _con = null;
        try {
            _met = ExecutionStack.getReference().peekMethod();
            _con = ExecutionStack.getReference().peekConcept();
        }
        catch ( RuntimeException e ) {
            _met = "main:Series";
            _con = "_current_";
        }
        Concept con  = mn8Loader.instantiateConcept( _con );
        StringConcept _scriptName = con.getConceptMethod( new StringConcept(_met) ).getDeclaringConcept();
        Concept hConcept = getHandler( makeID( getID() ) );
        try {
            if ( !validType() || ( hConcept == null ) ) throw new Exception();
            invokeHandlerMethod( hConcept, getID() );
        }
        catch ( Exception e ) {
            Concept handler = getHandler( makeID( getType() ) );
            try {
                invokeHandlerMethod( handler, getType() );
            }
            catch( Exception ee) {
                defaultErrorHandler( _scriptName.getValue(), _met );
            }
        }
    }


    private void invokeHandlerMethod( Concept handlerConcept, StringConcept id ) throws Exception {
        StringConcept methodName = makeID( id );
        SeriesConcept _ser = new SeriesConcept( );
        _ser.add( this );
        mn8ScriptMethod handlerMethod = (mn8ScriptMethod) handlerConcept.getConceptMethod( methodName );
        if ( getThePlace() == null )
            result = handlerMethod.invokeErrorHandler( getTheStaticPlace(), _ser );
        else 
            result = handlerMethod.invokeErrorHandler( getThePlace(), _ser );
    }


    private StringConcept makeID( StringConcept id ) {
        return new StringConcept( id.getValue() + ":Error" );
    }
    

    private void setThePlace( Concept pl ) {
        _setplace = pl;
    }

    
    private Concept getThePlace() {
        return _setplace;
    }
    

    private void setTheStaticPlace( String pl ) {
        _set_staticplace = pl;
    }
    

    private String getTheStaticPlace() {
        return _set_staticplace;
    }

    
    public Concept getHandler( StringConcept id ) {
        Stack hStack = (Stack) ExecutionStack.getReference().getHandlerStack().clone();
        Stack cStack = (Stack) ExecutionStack.getReference().getConceptStack().clone();
        Concept con;
        String cstack;
        try {
            con  = (Concept) hStack.peek();
            cstack =  (String) cStack.peek(); 
        }
        catch ( Exception e ) {
            con = null;
            cstack = "_current_";
        }
        if (con == null) {
            setTheStaticPlace( cstack );
            return ConceptConceptDefinition.getErrorHandler();
        }
        Concept handler = null;
        while ( handler == null && hStack.size() > 1 )  {
            con  = (Concept) hStack.peek();
            if (con == null) {
                setTheStaticPlace( (String) cStack.peek() );
                handler = null;
                break;
            }	    
            handler = con.getErrorHandler();
            try {
                if ( handler != null) {
                    mn8ScriptMethod handlerMethod = (mn8ScriptMethod) handler.getConceptMethod( id );
                }
            }
            catch ( Exception e ) {
                handler = null;
            }
            hStack.pop();
            cStack.pop();
        } 
        setThePlace( con );
        setTheStaticPlace( (String) cStack.peek() );
        if (handler == null) return ConceptConceptDefinition.getErrorHandler();
        return handler;
    }
    
    
    public boolean validType() {
        if ( !(_type.getValue().equalsIgnoreCase("error") || _type.getValue().equalsIgnoreCase("warning")) ) {
            if ( !_type.getValue().equalsIgnoreCase("silent") )  _msg = usage;
            return false;
        }
        return true;
    }

    
    public void defaultErrorHandler( String con, String met ) {
        if ( !_type.getValue().equalsIgnoreCase("silent") ) {
            PrintStream _err = mn8Interpreter.getErr( Thread.currentThread());
            if( _type.getValue().equalsIgnoreCase("error") ) {
                _err.println( "---------------- ERROR occured in concept ----------------");
            } else {
                _err.println( "---------------------- MN8 Warning ------------------------");
            }

            String lineInfo = "";
            if( errorLine == -1 && mn8TreeWalker.lineNr != -1 ) {
                errorLine = mn8TreeWalker.lineNr;
            }

            if( errorLine != -1 ) {
                lineInfo = "\tline nr: " + errorLine;
            }

            _err.println( " Concept name: " + con );
            _err.println( "  Method name: " + met  + lineInfo );
            _err.println( "Error message: " + getMessage() );
            _err.println( " Concept path:" + ExecutionStack.getReference().chart() );
            _err.println( "-------------------------------------------------------------");
            if ( _type.getValue().equalsIgnoreCase("error") || !validType() ) {
                mn8Interpreter.interruptInterpret();
            }
        }
    }


    public StringConcept getID() {
        return _id;
    }


    public StringConcept getMSG() {
        return _msg;
    }


    public IntegerConcept getLineNr() {
        return new IntegerConcept(errorLine);
    }


    public String getMessage() {
        return _msg.getValue();
    }


    public Concept getConcept() {
        return _concept;
    }


    public void setMSG( StringConcept msg ) {
        _msg = msg;
    }

    public void setType( StringConcept type ) {
        _type = type;
    }

    public StringConcept getType() {
        return _type;
    }


    public boolean equals (Object obj) {
        if( obj instanceof org.media.mn8.concepts.ErrorConcept ) {
            ErrorConcept _ec = (ErrorConcept)obj;
            if ( _ec.getID()  == _id  &&
                 _ec.getMSG() == _msg &&
                 _ec.getConcept() == _concept ) return true;
        }
        return false;
    }


    public ConceptDefinition getConceptDefinition() {
        return ErrorDefinition.getReference();
    }


    public LogicalConcept hasPath( StringConcept path ) {
        return new LogicalConcept(false);
    }

    
    public SeriesConcept getConceptsAtPath( StringConcept path ) {
        return new SeriesConcept();
    }


    public Object clone() {
        return this;
    }    


}
