/*
 * $COPYRIGHT$
 * $Id: FTPClientConcept.java,v 1.8 2002/02/21 10:45:10 crow Exp $
 *
 * Date          Author            Changes
 * IAN 18 2002   Szabo Csaba       Created
 */
      
package org.media.mn8.concepts;

import org.media.mn8.*;
import java.net.URL;
import java.io.PrintWriter;
import java.io.OutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import com.oroinc.net.ftp.*;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.8 $ $Date: 2002/02/21 10:45:10 $
 */

public class FTPClientConcept extends JavaConcept {
    protected FTPClient _client;
    protected URL       _url;
    
    public AttributeConcept errorMessage = new AttributeConcept("errorMessage", "errorMessage", new NilConcept() );
    public AttributeConcept    errorCode = new AttributeConcept("errorCode", "errorCode", new NilConcept() ); 


    public FTPClientConcept() {
        _client = null;
    }
    
    
    public LogicalConcept openConnection ( StringConcept url ) {
        URL givenURL = null;
	
        try {
            givenURL = new URL( url.toString() );
            if( givenURL == null || givenURL.getProtocol().toLowerCase().equals("ftp") ) {
                if( givenURL != null ) {
                    _url    = givenURL;
                    _client = new FTPClient( );
                    _client.addProtocolCommandListener ( new PrintCommandListener ( new PrintWriter ( System.out ) ) );
                    
                    if ( givenURL.getHost() == null || givenURL.getHost().trim().length() < 1 ) {
                        new ErrorConcept("error", "missingHost", "Missing host name for FTP connection.");
                        return new LogicalConcept( false );
                    } 
                    
                    if ( givenURL.getHost() != null && givenURL.getPort() == -1 )
                        _client.connect( givenURL.getHost() );
					
                    if ( givenURL.getHost() != null && givenURL.getPort() != -1 )
                        _client.connect( givenURL.getHost() , givenURL.getPort() );
    
                    errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
                    errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
					
                    if( !FTPReply.isPositiveCompletion( _client.getReplyCode() ) ) {
                        _client.disconnect();
                        new ErrorConcept("error", "connectionRefuzed" ,"FTP server (" + givenURL.getHost() + 
                                         ") refuzed connection, the reply code is:" + _client.getReplyCode() );
                        return new LogicalConcept( false );
                    }
                }
            } else if ( givenURL != null ) {
                new ErrorConcept( "error", "unsupportedProtocol", "'" + givenURL.getProtocol() + 
                                  "' protocol not supported by FTPClient" );	
            }
        } 
        catch ( Exception ex ) {
            new ErrorConcept( "error", "badURL", ex.getMessage() );
            return new LogicalConcept( false );
        }
        
        return new LogicalConcept( true );
    }
	
    
    public void closeConnection() {
        try {
            _client.disconnect();
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
        }
        catch ( Exception ex ) {			
            new ErrorConcept( "error", "closeConnection", ex.getMessage() );
        }
    }
    
    
    public LogicalConcept login ( StringConcept uname, StringConcept pwd ) {
        try {
            boolean logined = _client.login( uname.toString(), pwd.toString() );
			
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
	     
            if ( logined ) {
                String _path = _url.getFile(); 
				
                while ( !_client.changeWorkingDirectory( _path ) )
                    _path = _path.substring( 0, _path.lastIndexOf("/") );
				
                return new LogicalConcept( true );
            }
            new ErrorConcept( "warning", "loginWarning", _client.getReplyString() );
            return new LogicalConcept( false );
        }
        catch ( FTPConnectionClosedException e ) {
            new ErrorConcept( "error", "loginFailed", e.getMessage() );
            return new LogicalConcept( false );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "loginFailed", ex.getMessage() );
            return new LogicalConcept( false );
        }
    }
    
    
    public LogicalConcept logout() {
        try {
            boolean status = _client.logout();
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            return new LogicalConcept( status );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "logoutFailed", ex.getMessage() );
            return new LogicalConcept( false );
        } 
    }
    

    public StringConcept getWorkingDirectory () {
        try {
            String pwd = _client.printWorkingDirectory();
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            return new StringConcept( pwd );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
            return new StringConcept( "" );
        }
    }


    public LogicalConcept changeWorkingDirectory ( StringConcept path ) {
        try {
            boolean status = _client.changeWorkingDirectory( path.toString() );
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            if ( !status ) new ErrorConcept( "warning", "FTPOperationWarning", _client.getReplyString() );
            return new LogicalConcept( status );
        }
        catch ( Exception ex ) {	
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
            return new LogicalConcept( false );
        }
    }
    
    
    public LogicalConcept mkdir( StringConcept _path ) {
        try {
            boolean status = _client.makeDirectory( _path.toString() );
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            if ( !status ) new ErrorConcept( "warning", "FTPOperationWarning", _client.getReplyString() );
            return new LogicalConcept( status );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
            return new LogicalConcept( false );
        }
    }


    public LogicalConcept delete( StringConcept _path ) {
        try {
            boolean status = false;
            if ( !( status = _client.deleteFile( _path.toString() ) ) )
                status = _client.removeDirectory( _path.toString() );
	    
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            if ( !status ) new ErrorConcept( "warning", "FTPOperationWarning", _client.getReplyString() );
            
            return new LogicalConcept( status );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
            return new LogicalConcept( false );
        }
    }
    
    
    public LogicalConcept rename( StringConcept from, StringConcept to ) {
        try {
            boolean status = _client.rename( from.toString(), to.toString() );
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            if ( !status ) new ErrorConcept( "warning", "FTPOperationWarning", _client.getReplyString() );
            return new LogicalConcept( status );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
            return new LogicalConcept( false );
        }
    }


    public LogicalConcept setBinaryMode( LogicalConcept isBinary ) {
        try {
            boolean status = false;
            if ( isBinary.getValue() )
                status = _client.setFileType(FTP.BINARY_FILE_TYPE);
            else 
                status = _client.setFileType(FTP.ASCII_FILE_TYPE);	
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            if ( !status ) new ErrorConcept( "warning", "FTPOperationWarning", _client.getReplyString() );
            
            return new LogicalConcept( status );

        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
            return new LogicalConcept( false );
        }
    }
    
    
    public LogicalConcept isPositiveCompletion() {
        return new LogicalConcept( FTPReply.isPositiveCompletion( _client.getReplyCode() ) );
    }
    
    
    public ConceptDefinition getConceptDefinition() {
        return FTPClientConceptDefinition.getReference();
    }
    
    
    public SeriesConcept getLinks() {
        SeriesConcept result = new SeriesConcept();
        
        try {
            FTPFile fileNames[] = _client.listFiles();
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            if ( fileNames != null ) {
                for ( int i = 0; i < fileNames.length; i++ )
                    result.add( new StringConcept( fileNames[i].getRawListing() ) );
            }
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
        }
        
        return result;
    }


    public SeriesConcept getLinks( StringConcept path ) {
        SeriesConcept result = new SeriesConcept();
        String _path = path.toString().startsWith("./") ? path.toString().substring(2) : path.toString(); 
        
        try {
            FTPFile fileNames[] = _client.listFiles( _path );
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            if ( fileNames != null ) {
                for ( int i = 0; i < fileNames.length; i++ )
                    result.add( new StringConcept( fileNames[i].getRawListing() ) );
            }
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
        }
        
        return result;
    }
    
    
    public StringConcept getFile( StringConcept fileName ) {
        StringBuffer sb = new StringBuffer();
	
        try {
            BufferedReader reader = new BufferedReader( new InputStreamReader( 
                                       _client.retrieveFileStream( fileName.toString() ) ) );
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            int ch;
            while ( ( ch = reader.read() ) != -1 ) sb.append( (char)ch ); 
            reader.close();
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
        }
        
        return new StringConcept( sb.toString() );
    }
    
    
    public LogicalConcept putFile( StringConcept fName, StringConcept content ) {
        OutputStream out = null;
        
        try {
            _client.allocate( content.toString().getBytes().length );
            out = _client.storeFileStream( fName.toString() );
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            errorCode.setAttributeValue( new IntegerConcept( _client.getReplyCode() ) );
            out.write( content.toString().getBytes(), 0, content.toString().getBytes().length );
            out.flush();
            out.close();
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );	
            return new LogicalConcept( false );
        }
        return new LogicalConcept ( true );
    }


    public FTPClient getClient() {
        return _client;
    }


    public boolean equals ( Object obj ) {
        if( obj instanceof org.media.mn8.concepts.FTPClientConcept ) {
            FTPClient _obj_client = ( (FTPClientConcept)obj ).getClient();
            if ( _obj_client == null && _client == null ) return true;
            if ( _client.equals( _obj_client ) ) return true;
        }
        return false;
    }

    
    public Object clone() {
        return this;
    }
}
