/* 
 * $COPYRIGHT$
 * $Id: FTPFileConcept.java,v 1.8 2002/05/02 18:27:30 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.io.File;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.OutputStream;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.net.URL;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import javax.activation.MimetypesFileTypeMap;
import java.lang.reflect.Method;
import com.oroinc.net.ftp.*;
import com.oroinc.net.*;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.8 $ $Date: 2002/05/02 18:27:30 $
 */
public class FTPFileConcept extends JavaConcept {

    protected FTPClient _client = null;
    protected FTPFile _file =null;
    protected URL _url = null;
    protected boolean _exists = true;

    public AttributeConcept url = new AttributeConcept ("url", "url", new NilConcept());
    public AttributeConcept length = new AttributeConcept ("length", "length", new NilConcept());
    public AttributeConcept contentType = new AttributeConcept ("contentType", "contentType", new NilConcept());
    
    public ElementConcept content = new ElementConcept("content", "content", new NilConcept(), null);


    public FTPFileConcept( ){

    }


    public FTPFileConcept( String url ) {
        initialize( url );
    }


    public FTPFileConcept create( StringConcept url ) {
        initialize( url.toString() );
        return (FTPFileConcept)this;
    }


    public boolean initialize( String url ) {
        URL givenURL = null;

        try {
            givenURL = new URL( url );

            if( givenURL == null || givenURL.getProtocol().toLowerCase().equals("ftp")) {
                if( givenURL != null ) {
                    _url = givenURL;
                    _client = new FTPClient( );
                    _client.addProtocolCommandListener (new PrintCommandListener (new PrintWriter (System.out)));
                    _client.connect( givenURL.getHost());
                    
                    if( !FTPReply.isPositiveCompletion(_client.getReplyCode())) {
                        _client.disconnect();
                        new ErrorConcept("error", "connectionRefuzed" ,"FTP server (" + givenURL.getHost() + 
                                         ") refuzed connection, the reply code is:" + _client.getReplyCode() );
                        return false;
                    }

                    _client.login(givenURL.getUserInfo().substring(0, givenURL.getUserInfo().indexOf(":")),
                                  givenURL.getUserInfo().substring(givenURL.getUserInfo().indexOf(":")+1));

                    FTPFile[] files = _client.listFiles(givenURL.getFile());
                    
                    if( files != null && files.length == 1 ) 
                        _file = files[0];
                    else if( files == null ) {
                        _exists = false;
                        if( givenURL.getFile() != null ) {
                            if ( _file == null ){
                                _file = new FTPFile();
                                _file.setType(FTPFile.FILE_TYPE);
                                _file.setName( givenURL.getFile().substring(givenURL.getFile().lastIndexOf("/")+1));   
                            }
                        }
                    } else if ( files.length > 1 ) {
                                _file = new FTPFile();
                                _file.setType(FTPFile.DIRECTORY_TYPE);
                                _file.setName( givenURL.getFile());   
                    }
                    this.url.setAttributeValue(   new StringConcept(_url.getProtocol() + "://" +
                                                                    _url.getAuthority() +
                                                                    _url.getPath() + _file.getName()));
                    
                    length.setAttributeValue(new IntegerConcept(_file.getSize()));
                    contentType.setAttributeValue( new StringConcept( (new MimetypesFileTypeMap()).getContentType(_file.getName())));
                    this.url.setAttributeValue(new StringConcept( givenURL.toString()));
                }
            } else if ( givenURL != null ) {
                new ErrorConcept( "error", "unsupportedProtocol", "'" + givenURL.getProtocol() + 
                                  "' protocol not supported by FTPFile" );	
            }         
        } catch ( UnknownHostException ex) {
            throw new FTPOperationFailure("Unknown host: " + ex.getMessage());
        } catch ( Exception ex ) {            
            ex.printStackTrace();
            new ErrorConcept( "error", "badURL", ex.getMessage() );
            return false;
         }
        return true;
    }


    public ConceptDefinition getConceptDefinition() {
        return FTPFileConceptDefinition.getReference();
    }


    public LogicalConcept exists() {
        return new LogicalConcept( _exists );
    }


    public LogicalConcept isFile() {
        return new LogicalConcept( _file != null && !_file.isDirectory() );
    }


    public SeriesConcept getLinks() {
        SeriesConcept result = new SeriesConcept();
        if( isFile().getValue() || !exists().getValue()) return result; 

        try {
            _client.changeWorkingDirectory(_url.getPath() != null ? _url.getPath() : "./");
            FTPFile[] files = _client.listFiles();
            FTPFile tmpFile = null;
            if( files != null ) {
                for( int i = 0; i < files.length ; i++ ) {
                    tmpFile = files[i];
                    result.add(new StringConcept(_url.getProtocol() + "://" +
                                                 _url.getAuthority() +
                                                 _url.getPath() + tmpFile.getName()));
                }
            }
        } catch ( Exception ex ) {
            
            new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
        }
        return result;
    }
 
    
    public Concept getContent() { 
        readContent();
        return content.getElementValue();
    }

    
    public LogicalConcept setContent( Concept value ) {
        content.setElementValue( value.toTXT() );
        return writeContent();
    }
 

    protected LogicalConcept readContent() {
        if( _file != null && exists().getValue() && isFile().getValue()) {
	
            length.setAttributeValue( new IntegerConcept( _file.getSize() ) );

            StringBuffer buffer = new StringBuffer();
            try {
                BufferedReader reader = new BufferedReader( new InputStreamReader( _client.retrieveFileStream( _url.getPath() ) ) );
                int character;
                try {
                    while( (character = reader.read()) != -1 ) {
                        buffer.append((char)character);
                    }
                } catch ( IOException ex ) { // end of file 
                    new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
                    return new LogicalConcept( false );
                }
            } catch ( FileNotFoundException fnfex ) {

            } catch ( Exception ex ) {
                new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() ); 
                 return new LogicalConcept( false );
            }
            content.setElementValue( new StringConcept( buffer.toString() ) );
        }
        else if ( _file != null && ! isFile().getValue() ) {
            content.setElementValue( new StringConcept() );
        }
        return new LogicalConcept( true );
    }


    
    protected LogicalConcept writeContent() {
        if( _file != null && isFile().getValue() ) {
            OutputStream writter = null;
            try {
                _client.changeWorkingDirectory(_url.getPath() != null ? _url.getPath() : "./");
                String contentString = content.getElementValue().toTXT().toString();
                _client.allocate( contentString.getBytes().length);
                writter = _client.storeFileStream(_url.getPath());
            
                writter.write( contentString.getBytes(), 0, contentString.getBytes().length);
                writter.flush();
            } catch ( IOException ex ) {
                new ErrorConcept( "error", "FTPOperationFailed", ex.getMessage() );
                return new LogicalConcept(false);
            } finally {
                try {
                    writter.close();
                    if( !_client.completePendingCommand() ) {
                        new ErrorConcept( "warning", "FTPOperationWarning", _client.getReplyString() );
                    }
                    _client.logout();
                    _client.disconnect();
                } catch ( IOException ioex ) {
                   new ErrorConcept( "warning", "FTPOperationWarning", ioex.getMessage() ); 
                }
            }
        } else {
            return new LogicalConcept( false );
        } 
        initialize(_url.toString() );
        readContent();
        return new LogicalConcept( true );
    }


    public FTPFile getFTPFile() {
        return _file;
    }

    public boolean equals (Object obj) {
        if( obj instanceof org.media.mn8.concepts.FTPFileConcept ) {
            FTPFile _obj_client = ( (FTPFileConcept)obj).getFTPFile();
            if ( _obj_client == null && _file == null )    return true;
            if (  _file.equals(_obj_client) )  return true;
        }
        return false;
    }


    public Object clone() {
        return this;
    }
}
