/* 
 * $COPYRIGHT$
 * $Id: FTPHandler.java,v 1.5 2002/02/20 17:52:20 crow Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.util.Vector;
import java.net.URL;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.5 $ $Date: 2002/02/20 17:52:20 $
 */
public class FTPHandler extends URIHandler {
    private String _url = null;
    private boolean _overwrite = true; 
    private FTPFileConcept _file = null;
    private int _requests = 1;
    private int _depth = 0;
    private String _uname = null;
    private String _pwd = null;
    private ExpressionConcept _filter;
    private MapConcept _options;


    public FTPHandler( ){
    }


    public FTPHandler( StringConcept url ) {
        _url = url.toString();
    }
   

    public URIHandler create( StringConcept url ) {
       _url = url.toString();
       return this;
    }

    
    public LogicalConcept acceptsURI(StringConcept uri) {
        return ((URIHandlerDefinition)getConceptDefinition()).acceptsURI(uri);
    }
    

    public LogicalConcept initialize( MapConcept options ){
	
        _options = options;
        
        if( options.containsKey(new StringConcept("action")).getValue() ) {
            String action = options.getValue(new StringConcept("action")).toTXT().toString();
            if( action.toLowerCase().equals("append") ) {
                _overwrite = false;
            }
            else {
                _overwrite = true;
            }
        }

        if( options.containsKey(new StringConcept("requests")).getValue() ) {
            String requests = options.getValue(new StringConcept("requests")).toTXT().toString();
            _requests = (new Integer(requests)).intValue();
        }

        if( options.containsKey(new StringConcept("depth")).getValue() ) {
            String depth = options.getValue(new StringConcept("depth")).toTXT().toString();
            _depth = (new Integer(depth)).intValue();
        }

        if( options.containsKey(new StringConcept("filter")).getValue() ) {
            _filter = (ExpressionConcept) options.getValue(new StringConcept("filter"));
        }

        if( options.containsKey(new StringConcept("user")).getValue() ) {
            _uname = options.getValue(new StringConcept("user")).toTXT().toString();
        }

        if( options.containsKey(new StringConcept("pwd")).getValue() ) {
            _pwd = options.getValue(new StringConcept("pwd")).toTXT().toString();
        }
        
        if( _depth > 0 && _requests ==1 ) 
            _requests = Integer.MAX_VALUE;

        if( _requests > 1 && _depth == 0 )
            _depth = Integer.MAX_VALUE;
	
        URL givenURL = null;
	 
        try {
            givenURL = new URL( _url );
            if ( givenURL != null && givenURL.getProtocol().equalsIgnoreCase("ftp") ) {
                if ( _uname != null && _uname.trim().length() > 0 && _pwd != null && _pwd.trim().length() > 0 ) {
                    _url  = givenURL.getProtocol() + "://";
                    _url += _uname + ":" + _pwd + "@";
                    if ( givenURL.getHost()  != null ) _url += givenURL.getHost();
                    if ( givenURL.getPort()  != -1   ) _url += ":" + givenURL.getPort();
                    if ( givenURL.getPath()  != null ) _url += givenURL.getPath();
                    if ( givenURL.getQuery() != null ) _url += "?" + givenURL.getQuery();
                    if ( givenURL.getRef()   != null ) _url += "#" + givenURL.getRef();
                }
            }
        }
        catch ( Exception ex ) {
            new ErrorConcept("error", "badURL", ex.getMessage() );
            return new LogicalConcept( false );
        }
        
        return new LogicalConcept( true );
    }
    
    
    public LogicalConcept openConnection() {
        _file = new FTPFileConcept( _url );
        if( _file != null ) {
            if( !_overwrite )
                _file.readContent();
            return new LogicalConcept( true );
        }
        else
            return new LogicalConcept( false );
    }
    
    
    public Concept getContent() {
        _file.readContent();
        if( _requests == 1 && _depth == 0 ) {
            return _file;
        }
        // we have to get more files ;)
        Vector result = new Vector();
        result.add( _file );
        SeriesConcept links = null;
        links = _file.getLinks();
        if( links.getVector().size() == 0 )
            return new SeriesConcept(result);
        
        --_depth;
        int cnt = 1;
        Vector tmpVector = links.getVector();
        FTPFileConcept tmpFile;
        for( int i = 0 ; i < tmpVector.size(); i ++ ){

            if( mn8Interpreter.isExitAsked() ) 
                mn8Interpreter.interruptInterpret();

            if( _filter != null && 
                !_filter.matches((StringConcept) tmpVector.elementAt(i)).getValue()) continue;

            tmpFile = new FTPFileConcept( ((StringConcept) tmpVector.elementAt(i)).toString());
            tmpFile.readContent();
            result.add( tmpFile );
        }

        while ( _depth > 0 && cnt <= _requests ) {

          if( mn8Interpreter.isExitAsked() ) 
              mn8Interpreter.interruptInterpret();
            
            tmpVector = getDeepLinks( tmpVector, cnt );
            cnt = cnt + (tmpVector.size() == 0 ? 1 : tmpVector.size());

            for( int i = 0 ; i < tmpVector.size(); i ++ ) {
                if( _filter != null && !_filter.matches((StringConcept) tmpVector.elementAt(i)).getValue()) continue;
                tmpFile = new FTPFileConcept( ((StringConcept) tmpVector.elementAt(i)).toString());
                tmpFile.readContent();
                result.add( tmpFile );
            }
            --_depth;
        }
        
        return new SeriesConcept(result);  
    }


    protected Vector getDeepLinks(Vector links, int cnt) {
        Vector result = new Vector();
        FTPFileConcept tmpFile;
        StringConcept uri;
        for(int i =0; i < links.size(); i++ ) {

            if( mn8Interpreter.isExitAsked() ) 
                mn8Interpreter.interruptInterpret();

            if( cnt >= _requests ) return result;

            uri = (StringConcept) links.elementAt(i); 
            tmpFile = new FTPFileConcept(uri.toString());
            if( !tmpFile.isFile().getValue() ) {
               result.addAll(tmpFile.getLinks().getVector());
            }
            ++cnt;
        }
        return result;
    }


    public LogicalConcept setContent(Concept value) {
        if( _overwrite ) 
            _file.content.setElementValue(value.toTXT());
        else
            _file.content.setElementValue( 
                new StringConcept(_file.content.getElementValue().toTXT().toString()
                                  + value.toTXT().toString()));
        return _file.writeContent();
    }


    public LogicalConcept closeConnection() {
        return new LogicalConcept( true );
    }


    public ConceptDefinition getConceptDefinition() {
        return FTPHandlerDefinition.getReference();
    }


    public Object clone() {        
        if( _url != null )
            return new FTPHandler(new StringConcept(_url));
        else
            return new URIHandler();
    }
}
