/* 
 * $COPYRIGHT$
 * $Id: FileConcept.java,v 1.13 2002/06/05 14:53:44 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.net.URL;
import java.net.MalformedURLException;
import javax.activation.MimetypesFileTypeMap;
import java.lang.reflect.Method;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.13 $ $Date: 2002/06/05 14:53:44 $
 */
public class FileConcept extends JavaConcept {
    protected String _fileUrl = null;
    protected File _file = null;
 
    public AttributeConcept url = new AttributeConcept ("url", "url", new NilConcept());
    public AttributeConcept length = new AttributeConcept ("length", "length", new NilConcept());
    public AttributeConcept contentType = new AttributeConcept ("contentType", "contentType", new NilConcept());
    public ElementConcept content = new ElementConcept("content", "content", new NilConcept(), null);
    

    public FileConcept( ){
        _file = new File("");
        try {
            _fileUrl = _file.toURL().toString();
        } catch( MalformedURLException me) {
            _fileUrl = _file.toString();
        }
        initialize(_fileUrl);
    }


    public FileConcept( String url ) {
        initialize( url );
    }
    

    public FileConcept create( StringConcept url ) {
        initialize( url.toString() );
        return this;
    }


    public ConceptDefinition getConceptDefinition() {
        return FileConceptDefinition.getReference();
    }


    public LogicalConcept exists() {
        return new LogicalConcept( _file.exists() );
    }

    
    public LogicalConcept delete() {
        return new LogicalConcept( _file.delete() );
    }


    public LogicalConcept canRead() {
        return new LogicalConcept( _file.canRead() );
    }


    public LogicalConcept canWrite() {
        return new LogicalConcept( _file.canWrite() );
    }
    
    
    public LogicalConcept isFile() {
        return new LogicalConcept( _file.isFile());
    }


    public LogicalConcept mkdir() {
        return new LogicalConcept( _file.mkdirs() );
    }


    public LogicalConcept isHidden() {
        if( _file != null )
            return new LogicalConcept( _file.isHidden() );
        else {
            return new LogicalConcept(false);
        }
    }


    public LogicalConcept setReadOnly( ) {
        return new LogicalConcept( _file.setReadOnly() );
    }


    public LogicalConcept rename( StringConcept dest ) {
        if ( !_file.renameTo( new File ( dest.toString() ) ) ) 
            return new LogicalConcept( false );
        initialize ( dest.toString() );
        return new LogicalConcept( true );
    }


    public SeriesConcept getLinks() {
        File[] files = _file.listFiles();
        SeriesConcept result = new SeriesConcept();
        if( files != null ) {
            File tmpFile = null;
            for( int i = 0; i < files.length ; i++ ) {
                tmpFile = files[i];
                try {
                    result.add( new StringConcept( tmpFile.toURL().toString( ) ) );
                } catch ( MalformedURLException ex ) {
                    result.add( new StringConcept( tmpFile.toString() ) );
                }
            }
        }
        return result;
    }


    public Concept getContent() { 
        readContent();
        return content.getElementValue();
    }


    public LogicalConcept setContent( Concept value ) {
        content.setElementValue( value.toTXT() );
        return writeContent();
    }

 
    protected LogicalConcept readContent() {
       if( _file != null && _file.exists() && _file.isFile()) {
           StringBuffer buffer = new StringBuffer();
           try {
               BufferedReader reader = new BufferedReader(new FileReader(_file));
               int character;
               try {
                   while( (character = reader.read()) != -1 ) {
                       buffer.append( (char)character );
                   }
               } catch ( IOException ex ) { // end of file
                   new ErrorConcept( "error", "fileOperationFailed", ex.getMessage() );
                   return new LogicalConcept( false );
               }
           } catch ( FileNotFoundException fnfex ) {
           } catch ( Exception ex ) {
               new ErrorConcept( "error", "fileOperationFailed", ex.getMessage() );
               return new LogicalConcept( false );
           }
           content.setElementValue( new StringConcept( buffer.toString() ) );
       }
       else if ( _file != null && ! _file.isFile() ) {
           content.setElementValue( new StringConcept() );                
       }
       return new LogicalConcept( true );
    }

 
    protected LogicalConcept writeContent() {
        if( _file != null && !_file.exists()) {
            try {
                _file.createNewFile();
            } catch ( IOException ioex ) {
                new ErrorConcept("error", "fileOperationFailed", ioex.getMessage() );
            }
        }
        if( _file != null && _file.canWrite()) {
            FileWriter writter = null;
            try {
                writter = new FileWriter( _file );
                char[] contentChars = content.getElementValue().toTXT().toString().toCharArray();
            
                writter.write( contentChars, 0, contentChars.length);
                writter.flush();
            } catch ( IOException ex ) {
                new ErrorConcept("error", "fileOperationFailed", ex.getMessage() );
                return new LogicalConcept(false);
            } finally {
                try {
                    writter.close();
                } catch ( IOException ioex ) {
                    new ErrorConcept("error", "fileOperationFailed", ioex.getMessage() );
                }
            }
        } else {
            return new LogicalConcept( false );
        }
        return new LogicalConcept( true );
    }


    public Object clone() {
        return this;
    }


    private void initialize( String url ) {
        URL givenURL = null;
        try {
            givenURL = new URL( url );
        } catch ( MalformedURLException ex ) {
        }

        if( givenURL == null || givenURL.getProtocol().toLowerCase().equals("file")) {
            if( givenURL != null )
                _file = new File( givenURL.getPath() );
            else
                _file = new File( url );
            _fileUrl = url;
            try {
                this.url.setAttributeValue( new StringConcept( _file.toURL().toString() ) );
            } catch ( MalformedURLException ex){}
            length.setAttributeValue( new IntegerConcept( _file.length() ) );            
            contentType.setAttributeValue( new StringConcept(  MimetypesFileTypeMap.getDefaultFileTypeMap().getContentType( _file ) ) );
        }
        _fileUrl = url;
    }
}
