/* 
 * $COPYRIGHT$
 * $Id: FileHandler.java,v 1.3 2002/01/31 20:27:07 crow Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.util.Vector;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.3 $ $Date: 2002/01/31 20:27:07 $
 */
public class FileHandler extends URIHandler {
    private String _url = null;
    private boolean _overwrite = true; 
    private FileConcept _file = null;
    private int _requests = 1;
    private int _depth = 0;
    private ExpressionConcept _filter;
    private MapConcept _options;


    public FileHandler( ){
    }


    public FileHandler( StringConcept url ) {
        _url = url.toString();
    }
  

    public URIHandler create( StringConcept url ) {
       _url = url.toString();
       return this;
    }

    
    public LogicalConcept acceptsURI(StringConcept uri) {
        return ((URIHandlerDefinition)getConceptDefinition()).acceptsURI(uri);
    }
    

    public LogicalConcept initialize( MapConcept options ){
        _options = options;

        if( options.containsKey(new StringConcept("action")).getValue() ) {
            String action = options.getValue(new StringConcept("action")).toTXT().toString();
            if( action.toLowerCase().equals("append") ) {
                _overwrite = false;
            }
            else {
                _overwrite = true;
            }
        }


        if( options.containsKey(new StringConcept("requests")).getValue() ) {
            String requests = options.getValue(new StringConcept("requests")).toTXT().toString();
            _requests = (new Integer(requests)).intValue();
        }

        if( options.containsKey(new StringConcept("filter")).getValue() ) {
            _filter = (ExpressionConcept) options.getValue(new StringConcept("filter"));
        }

        if( options.containsKey(new StringConcept("depth")).getValue() ) {
            String depth = options.getValue(new StringConcept("depth")).toTXT().toString();
            _depth = (new Integer(depth)).intValue();
        }
        
        if( _depth > 0 && _requests ==1 ) 
            _requests = Integer.MAX_VALUE;

        if( _requests > 1 && _depth == 0 )
            _depth = Integer.MAX_VALUE;

        return new LogicalConcept(true);
    }
    

    public LogicalConcept openConnection() {
        _file = new FileConcept(  _url );
        if( _file != null ) {
            if( !_overwrite )
                _file.readContent();
            return new LogicalConcept(true);
        }
        else
            return new LogicalConcept(false);
    }

    
    public Concept getContent() {
        _file.readContent();
        if( _requests == 1 && _depth == 0 ) {
            return _file;
        }
        // we have to get more files ;)
        Vector result = new Vector();
        result.add( _file );
        SeriesConcept links = null;
        links = _file.getLinks();
        if( links.getVector().size() == 0 )
            return new SeriesConcept(result);
        
        --_depth;
        int cnt = 1;
        Vector tmpVector = links.getVector();
        FileConcept tmpFile;
        for( int i = 0 ; i < tmpVector.size(); i ++ ){
            if( _filter != null && !_filter.matches((StringConcept) tmpVector.elementAt(i)).getValue()) continue;

            tmpFile = new FileConcept( ((StringConcept) tmpVector.elementAt(i)).toString());
            tmpFile.readContent();
            result.add( tmpFile );
        }

        while ( _depth > 0 && cnt <= _requests ) {

            tmpVector = getDeepLinks( tmpVector, cnt );
            cnt = cnt + (tmpVector.size() == 0 ? 1 : tmpVector.size());

            for( int i = 0 ; i < tmpVector.size(); i ++ ) {
                if( _filter != null && !_filter.matches((StringConcept) tmpVector.elementAt(i)).getValue()) continue;
                tmpFile = new FileConcept( ((StringConcept) tmpVector.elementAt(i)).toString());
                tmpFile.readContent();
                result.add( tmpFile );
            }
            --_depth;
        }
        
        return new SeriesConcept(result);  
    }


    protected Vector getDeepLinks(Vector links, int cnt) {
        Vector result = new Vector();
        FileConcept tmpFile;
        StringConcept uri;
        for(int i =0; i < links.size(); i++ ) {

            if( cnt >= _requests ) return result;

            uri = (StringConcept) links.elementAt(i); 
            tmpFile = new FileConcept(uri.toString());
            if( !tmpFile.isFile().getValue() ) {
               result.addAll(tmpFile.getLinks().getVector());
            }
            ++cnt;
        }
        return result;
    }

    public LogicalConcept setContent(Concept value) {
        if( _overwrite ) 
            _file.content.setElementValue(value.toTXT());
        else
            _file.content.setElementValue( new StringConcept(_file.content.getElementValue().toTXT().toString() + value.toTXT().toString()));
        return _file.writeContent();
    }


    public LogicalConcept closeConnection() {
        return new LogicalConcept( true );
    }


    public ConceptDefinition getConceptDefinition() {
        return FileHandlerDefinition.getReference();
    }


    public Object clone() {        
        if( _url != null )
            return new FileHandler(new StringConcept(_url));
        else
            return new URIHandler();
    }
}
