/* 
 * $COPYRIGHT$
 * $Id: HTMLFormConcept.java,v 1.8 2002/05/28 15:58:06 atech Exp $
 *
 * Date        Author            Changes 
 * IAN 29 2002 Szabo Casba       Created
 * May 27 2002 Antal Attila      Added methods: storeForm, getStoredForms, getPath, getHost
 */

package org.media.mn8.concepts;

import org.media.mn8.Concept;
import org.media.mn8.ConceptConceptDefinition;

import org.media.mn8.Helper;
import org.media.mn8.mn8RuntimeFlags;
import org.media.mn8.JavaConcept;
import org.media.mn8.ConceptDefinition;
import java.util.Vector;
import java.util.Enumeration;
import java.net.URLEncoder;
import org.media.mn8.protocol.http.*;
import java.io.File;
import java.net.URL;
import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.FileInputStream;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.8 $ $Date: 2002/05/28 15:58:06 $
 */

public class HTMLFormConcept extends MapConcept {

    public static String FORM_DB = Helper.getUserHome().getValue() + 
        ".mn8" + File.separatorChar +
        "http" + File.separatorChar +
        "forms" + File.separatorChar;
    
    private Form _form = null;

    public AttributeConcept id = new AttributeConcept( "id", "id", new StringConcept() );
    public AttributeConcept url = new AttributeConcept( "url", "url", new StringConcept() );
    public AttributeConcept method = new AttributeConcept( "method", "method", new StringConcept("GET") );
    public AttributeConcept name = new AttributeConcept( "name", "name", new StringConcept() );
    public AttributeConcept target = new AttributeConcept( "target", "target", new StringConcept() );
    public AttributeConcept encoding = new AttributeConcept( "encoding", "encoding", new StringConcept() );
    
    public HTMLFormConcept( ) {
        _form = new Form(null, null);
    } 

    
    public HTMLFormConcept( Form form ) {
        setForm ( form );
    }
   
 

    public void setURL( StringConcept url ) {
        _form.setAction( url.toString() );
        setForm();
    }
    

    public StringConcept getAsEncodedURL() {
        StringBuffer result = new StringBuffer();
        result.append(url.getValue().toString());
        if( !name.getValue().toString().equals("")) {
            result.append("?name=");
            result.append(URLEncoder.encode( name.getValue().toString()));
        } 
        for( int i=0 ; i< _keys.size(); i++ ) {
            if( i == 0 && result.toString().indexOf("?") == -1 ) {
                result.append("?");
                result.append( ((Concept)_keys.elementAt(i)).toTXT().toString());
                result.append("=");
                result.append( URLEncoder.encode(((Concept)_values.elementAt(i)).toTXT().toString()));
            }else {
                result.append("&");
                result.append(((Concept)_keys.elementAt(i)).toTXT().toString());
                result.append("=");
                result.append( URLEncoder.encode(((Concept)_values.elementAt(i)).toTXT().toString()));                
            }        
        }
        return new StringConcept(result.toString());
    }

    public void setMethod( StringConcept method ) {
        _form.setMethod( method.toString() );
        setForm();
    }


    public void setName( StringConcept name ) {
        _form.setName( name.toString() );
        setForm();
    }
    

    public void setTarget( StringConcept target ) {
        _form.setTarget( target.toString() );
        setForm();
    }


    public void setEncoding( StringConcept enc ) {
        _form.setEncoding( enc.toString() );
        setForm();
    }


    public String getID() {
        return ((StringConcept)id.getValue()).getValue();
    }


    public Object clone() {
        return this;
    }


    public ConceptDefinition getConceptDefinition() {
        return HTMLFormDefinition.getReference();
    }

    
    private void setForm( Form form ){
        if( form != null) {
            _form = form; setForm();
            id.setAttributeValue( new StringConcept( "" + _form.toXMLString().hashCode() ) );
            addControls();
        }
    }
            

    private void setForm() {
        url.setAttributeValue( new StringConcept( _form.getAction() ) );
        method.setAttributeValue( new StringConcept( _form.getMethod() ) );
        name.setAttributeValue( new StringConcept( _form.getName() ) );
        target.setAttributeValue( new StringConcept( _form.getTarget() ) );
        encoding.setAttributeValue( new StringConcept( _form.getEncoding() ) );
    }


    private void addControls() {
        for ( Enumeration e = _form.getControls().elements(); e.hasMoreElements(); ) {
            DefaultControl _control = (DefaultControl)e.nextElement();
            
            if ( _control.isSendable() && _control.getControlType().equalsIgnoreCase("INPUT") && 
                 ((InputControl)_control).getType().equalsIgnoreCase("file") )
                for ( Enumeration e1 = _control.getValues().elements(); e1.hasMoreElements(); )
                    add( new StringConcept( _control.getName() ), new FileConcept( (String)e1.nextElement() ) );
            else if ( _control.isSendable() )
                for ( Enumeration e1 = _control.getValues().elements(); e1.hasMoreElements(); )
                    add( new StringConcept( _control.getName() ), new StringConcept( (String)e1.nextElement() ) );
        }
    }


    public static Concept fromXML( StringConcept content ) {
        return  ((ConceptConceptDefinition) HTMLFormDefinition.getReference()).fromXML( content );
    }


    public void loadContent( Concept from ) {
        if( from == null ) return;
        if( mn8RuntimeFlags.isDebug() ) {
            System.err.println("--> in concept:" + Helper.getConceptInfo(this) +
                               "->" + Helper.getConceptInfo(from) );
        }
        Helper.loadAttributes(from, this);

        Vector elementes = from.getConceptElements().getVector();
        ElementConcept elem = null;

        for(int i = 0; i < elementes.size(); i ++ ) {
            elem = (ElementConcept)elementes.elementAt(i);
            StringConcept name = new StringConcept( elem.getName().toString() );
            add( name, new StringConcept( elem.getValue().toString() ) );
        }
    }


    public void storeForm() {
        String domain = getHost((StringConcept)url.getValue());
        if( _form == null) return;
        String fName = FORM_DB + File.separator + domain;
        if ( !( new File( FORM_DB ) ).exists() ) new File( FORM_DB ).mkdirs();
        int formNr = getFormNr( FORM_DB, domain );
        try {
            PrintWriter writer =
                new PrintWriter( new BufferedWriter(
                                 new FileWriter( fName + "[" + formNr + "].xml" ) ) ); 
            writer.println( toXML() );
            writer.close();
        }
        catch ( Exception e ) {
            new ErrorConcept("error", "storeFormURL",
                             "Error durring when save XML: " + e.getMessage() );
        }

    }


    private int getFormNr( String dest, String domain ) {
        int       nr = 1;
        String fName = dest + File.separator + domain;
        while ( new File( fName + "[" + nr + "].xml").exists() ) nr++;
        return nr;
    }


    static public SeriesConcept getStoredForms ( StringConcept durl ) {
        SeriesConcept ser = new SeriesConcept();
        if ( !( new File( FORM_DB ) ).exists() ) return ser;
        File formFiles[] =
            ( new File( FORM_DB ) ).listFiles( new DomainNameFilter ( getHost(durl) ));
        for ( int i = 0; i < formFiles.length; i++ ) {
            FileConcept file = new FileConcept(FORM_DB + formFiles[ i ].getName());
            HTMLFormConcept con =
                (HTMLFormConcept)HTMLFormConcept.fromXML((StringConcept)file.getContent());
            AttributeConcept url_attr =
                (AttributeConcept)con.getConceptAttribute(new StringConcept("url"));
            StringConcept url_str = (StringConcept)url_attr.getValue();
            if (getHost(durl).equals( getHost(url_str) ) && 
                getPath(durl).equals( getPath(url_str) ) ) {
                ser.add(con);
            }
        }
        return ser;
    }


    static public SeriesConcept getStoredForms ( ExpressionConcept expr ) {
        SeriesConcept ser = new SeriesConcept();
        if ( !( new File( FORM_DB ) ).exists() ) return ser;
        File formFiles[] = ( new File( FORM_DB ) ).listFiles();
        for ( int i = 0; i < formFiles.length; i++ ) {
            FileConcept file = new FileConcept(FORM_DB + formFiles[ i ].getName());
            HTMLFormConcept con =
                (HTMLFormConcept)HTMLFormConcept.fromXML((StringConcept)file.getContent());
            AttributeConcept url_attr =
                (AttributeConcept)con.getConceptAttribute(new StringConcept("url"));
            StringConcept url_str = (StringConcept)url_attr.getValue();
            if ( (expr.matches(url_str)).getValue() ) {
                ser.add(con);
            }
        }
        return ser;
    }
    

    static private String getPath( StringConcept durl ) {
        String _path = "";
        try {
            URL _url = new URL( durl.getValue() );
            _path = _url.getPath();
        }
        catch (Exception e) {
            new ErrorConcept("error", "getStoredFormsURL", e.getMessage());
        }
        return _path;
    }


    static private String getHost( StringConcept durl ) {
        String _host = "";
        try {
            URL _url = new URL( durl.getValue() );
            _host = _url.getHost();
        }
        catch (Exception e) {
            new ErrorConcept("error", "getStoredFormsURL", e.getMessage());
        }
        return _host;
    }

    
}
