/* 
 * $COPYRIGHT$
 * $Id: HTMLPageConcept.java,v 1.13 2002/05/29 15:52:28 atech Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 * May 28 2002 Antal Attila      Added methods: getResponseCode, getResponseMessage
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import org.media.mn8.protocol.http.*;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.util.Enumeration;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.DataOutputStream;


/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.13 $ $Date: 2002/05/29 15:52:28 $
 */
public class HTMLPageConcept extends JavaConcept {

    private URL _url = null;
    private StringConcept _content = null;
    private MapConcept _headers = null;
    private SeriesConcept _cookies = new SeriesConcept();
    private int resp_code = 0;
    private String resp_msg = "";

    public AttributeConcept url = new AttributeConcept ("url", "url", new StringConcept());
    public AttributeConcept length =
        new AttributeConcept ("length", "length", new IntegerConcept(-1));
    public AttributeConcept contentType =
        new AttributeConcept ("contentType", "contentType", new StringConcept());
    public AttributeConcept lastModified =
        new AttributeConcept ("lastModified", "lastModified", new IntegerConcept(-1));
    public AttributeConcept title =
        new AttributeConcept ("titile", "title", new StringConcept());
    public ElementConcept content =
        new ElementConcept("content", "content", new NilConcept(), null);


    public HTMLPageConcept( ){
    }


    public HTMLPageConcept( String url ) {
        initialize( url );
    }
  

    public HTMLPageConcept( HTMLFormConcept form ) {
        initialize( form );
    }


    public HTMLPageConcept create( StringConcept url ) {
        initialize( url.toString() );
        return this;
    }


    public HTMLPageConcept create( HTMLFormConcept form ) {
        initialize( form );
        return this;
    }


    public boolean initialize( String url ) {
        URL givenURL = null;
        try {
            givenURL = new URL( url );
            
            if( givenURL != null ) {
                _url = givenURL;
                this.url.setAttributeValue( new StringConcept( givenURL.toString() ) );
                getConnection();
                getContent();
            }
            
        } 
        catch ( Exception ex ) {
            new ErrorConcept( "error", "badURL", ex.getMessage( ) );
            return false;
        }
        return true;
    }


    public boolean initialize( HTMLFormConcept form ) {
        URL  givenURL = null;
        String _myURL = form.url.getValue().toString() + "?";
        String method = form.method.getValue().toString();


        if ( method.equalsIgnoreCase("get") )
            for ( Enumeration e = form.getKeyVector().elements(); e.hasMoreElements(); ) {
                StringConcept   _key = (StringConcept)e.nextElement();
                StringConcept _value = null;
		
                if ( form.getValue( _key ) instanceof StringConcept )
                    _value = (StringConcept)form.getValue( _key );
		
                if ( _value != null && _value.toString().trim().length() > 0 ) 
                    _myURL += _key.toString() + "=" + URLEncoder.encode( _value.toString() ) + "&"; 
            }
        
        if ( _myURL.trim().endsWith("&") || _myURL.trim().endsWith("?") ) 
            _myURL = _myURL.substring(0, _myURL.length() - 1 );
        
        try {
            givenURL = new URL( _myURL );
            
            if( givenURL != null ) {
                _url = givenURL;
                this.url.setAttributeValue( new StringConcept( givenURL.toString() ) );
                if ( method.equalsIgnoreCase("get") ) getConnection();
                else {
                    getPostConnection( form );
                }
                getContent();
            }
        } 
        catch ( Exception ex ) { 
            new ErrorConcept( "error", "badURL", ex.getMessage( ) );
            return false;
        }
        
        return true;
    }


    private URLConnection getPostConnection( HTMLFormConcept form ) throws Exception {
        String encType = form.encoding.getValue().toString();

        if ( encType == null || encType.trim().length() < 1 ) 
            encType = "application/x-www-form-urlencoded";
	
        try {
            URLConnection connection = _url.openConnection();
	    
            if( _url.getProtocol().equalsIgnoreCase("http") ) {
                HttpURLConnection httpcon = (HttpURLConnection)connection;
                httpcon.setRequestMethod( form.method.getValue().toString().toUpperCase() );
                httpcon.setDoInput(true);
                httpcon.setDoOutput(true);
                httpcon.setUseCaches(false);
                httpcon.setRequestProperty("Connection","Keep-Alive");
                httpcon.setRequestProperty("User-Agent","Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; DigExt)");
                httpcon.setRequestProperty("Accept-Language","en-us");
                httpcon.setRequestProperty("Content-Type", encType );
                httpcon.setAllowUserInteraction(false);

                String cookiesString = ((CookieDefinition)CookieDefinition.getReference()).getCookieString( new StringConcept(_url.toString())).toString();
                if( cookiesString != null ) {
                    httpcon.setRequestProperty("Cookie", cookiesString.trim());
                }

                String  content = "";
                for(Enumeration e=form.getKeyVector().elements(); e.hasMoreElements();) {
                    StringConcept   _key = (StringConcept)e.nextElement();
                    String        _value = null;
                    
                    if ( form.getValue( _key ) instanceof StringConcept )
                        _value = ((StringConcept)form.getValue( _key )).toString();
		
                    if ( form.getValue( _key ) instanceof FileConcept ) 
                        _value = ((StringConcept)((FileConcept)form.getValue( _key )).url.getValue()).toString().substring(5);
                    
                    if ( _value != null && _value.trim().length() > 0 ) 
                        content += _key.toString() + "=" +
                                  URLEncoder.encode( _value ) + "&"; 
                }

                DataOutputStream out = new DataOutputStream( httpcon.getOutputStream() );
                out.writeBytes( content.length() > 0 ? content.substring(0,
                                                              content.length() - 1) : "" );
                out.flush(); 
                out.close();

                loadHeaders( httpcon );
                connection = httpcon;
            }
            else {        
                connection.setUseCaches( false );
                connection.setRequestProperty("Connection","Keep-Alive");
            }
	    
            contentType.setAttributeValue(new StringConcept( connection.getContentType()));
            length.setAttributeValue(new IntegerConcept( connection.getContentLength()));
            lastModified.setAttributeValue( new IntegerConcept( connection.getLastModified()));        
            return connection;
        } 
        catch ( Exception ex ) {
            throw ex;
        }
    }


    private URLConnection getConnection() throws Exception {
        try {
            URLConnection connection = _url.openConnection();
            if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose())
                System.err.println("http:connecting to: " + _url.toString());
            
            if( _url.getProtocol().equalsIgnoreCase("http")) {
                HttpURLConnection httpcon = (HttpURLConnection) connection;

                httpcon.setDoInput(true);
                httpcon.setUseCaches(false);
                httpcon.setRequestProperty("Connection","Keep-Alive");
                httpcon.setRequestProperty("User-Agent",
                                 "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; DigExt)");
                httpcon.setRequestProperty("Accept-Language","en-us");
                httpcon.setAllowUserInteraction(false);
                
                String cookiesString =
                    ((CookieDefinition)CookieDefinition.getReference()).getCookieString( 
                    new StringConcept(_url.toString())).toString();
                if(cookiesString != null ) httpcon.setRequestProperty("Cookie",
                                                                      cookiesString.trim());
                if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose())
                    System.err.println("http:sent cookie: " + cookiesString);
                
                if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose())
                    System.err.println("http:" + httpcon.getResponseCode() +
                                       ":" +httpcon.getResponseMessage());
                httpcon.connect();
                if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose())
                    System.err.println("http:" + httpcon.getResponseCode() +
                                       ":" +httpcon.getResponseMessage());
                loadHeaders(httpcon);                
                connection = httpcon;
            }else {
                connection.setUseCaches(false);
                connection.setRequestProperty("Connection","Keep-Alive");
            }
                
            contentType.setAttributeValue(new StringConcept( connection.getContentType()));
            length.setAttributeValue(new IntegerConcept( connection.getContentLength()));
            lastModified.setAttributeValue( new IntegerConcept( connection.getLastModified()));
	    return connection;
        } catch ( Exception ex ) {
            throw ex;
        }
    }


    public StringConcept getContent() {
        if( _content == null ) {
            _content = new StringConcept( getPageContent());
            content.setElementValue(_content);
        }
        return _content;
    }

    
    public SeriesConcept getURIs() {
        String cType = contentType.getAttributeValue().toString();
	
        if ( cType == null || !cType.trim().equalsIgnoreCase("text/html") )
            return new SeriesConcept();
	    
        URIExtractor extractor = new URIExtractor();
        extractor.baseURL = _url;
        return new SeriesConcept( extractor.getURIs( getContent().toString() ) );
    }
    

    public SeriesConcept getForms() {
        String cType = contentType.getAttributeValue().toString();
        
        if ( cType == null || !cType.trim().equalsIgnoreCase("text/html") )
            return new SeriesConcept();
        
        FormExtractor extractor = new FormExtractor();
        extractor.baseURL = _url;
        return new SeriesConcept( extractor.getForms( getContent().toString() ) );
    }

    
    public SeriesConcept getLinks() {
        String cType = contentType.getAttributeValue().toString();
        
        if ( cType == null || !cType.trim().equalsIgnoreCase("text/html") )
            return new SeriesConcept();
	
        LinkExtractor extractor = new LinkExtractor();
        extractor.baseURL = _url;
        return new SeriesConcept(extractor.getLinks(getContent().toString()));
    }

    
    
    public SeriesConcept getTags() {
        TagExtractor extractor = new TagExtractor(null, false);
        return new SeriesConcept(extractor.parsePage(getContent().toString()));
    }


    public SeriesConcept getTagsWithText() {
        TagExtractor extractor = new TagExtractor(null, true);
        return new SeriesConcept(extractor.parsePage(getContent().toString()));
    }


    public SeriesConcept getCookies() {
        return _cookies;
    }


    public MapConcept getHeaders() {
        if( _headers == null ) return new MapConcept();
        return _headers;
    }


    public Object clone() {
        return this;
    }


    public ConceptDefinition getConceptDefinition() {
        return HTMLPageConceptDefinition.getReference();
    }


    public StringConcept toTXT() {
        return new StringConcept( TagStriper.strip_tags(getPageContent(), true));
    }

    
    public StringConcept getResponseMessage(){
        return new StringConcept(resp_msg);
    }


    public IntegerConcept getResponseCode() {
        return new IntegerConcept(resp_code);
    }


    static public void setFollowRedirects(LogicalConcept bol) {
        HttpURLConnection.setFollowRedirects(bol.getValue());         
    }


    static public LogicalConcept isFollowRedirects() {
        boolean bol = HttpURLConnection.getFollowRedirects();
        return new LogicalConcept(bol);
    }


    private String getPageContent() {
        StringBuffer buffer = new StringBuffer();
        try {
            if( _url == null ) return "";
            
            URLConnection connection  = getConnection();
            if( connection == null) return "";

            BufferedReader reader = new BufferedReader(
                                    new InputStreamReader( connection.getInputStream()));
            int character;
            try {
                while( (character = reader.read()) != -1 ) {
                    buffer.append((char)character);
                }
            } catch ( IOException ex ) {
                new ErrorConcept("error", "httpOperationFailed", ex.getMessage() );
            }
        } catch (Exception ex) { 
            new ErrorConcept("error", "httpOperationFailed", ex.getMessage() );
        }
        return buffer.toString();
    }


    private void loadHeaders(HttpURLConnection connection) {
        try {
            _headers = new MapConcept();
            int i = 1;
            String key;
            resp_code = connection.getResponseCode();
            resp_msg = connection.getResponseMessage();

            while( (key = connection.getHeaderFieldKey(i)) != null ) {
                _headers.add( new StringConcept( key),
                              new StringConcept( connection.getHeaderField(i)));
                if( key.equalsIgnoreCase("set-cookie") ||
                    key.equalsIgnoreCase("set-cookie2")) {
                    CookieConcept cookie = 
                        new CookieConcept( new StringConcept( connection.getHeaderField(i)),
                                           new StringConcept(connection.getURL().getHost()));
                    _cookies.add( cookie );
                }
                i++;
            }
        } catch ( Exception ex ) {
            new ErrorConcept("error", "httpOperationFailed", ex.getMessage() );
        }
    }
}
