/* 
 * $COPYRIGHT$
 * $Id: HttpHandler.java,v 1.6 2002/02/21 10:12:49 crow Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.util.Vector;
import java.net.*;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.6 $ $Date: 2002/02/21 10:12:49 $
 */
public class HttpHandler extends URIHandler {
    private String _url = null;
    private int _requests = 1;
    private int _depth = 0;
    private ExpressionConcept _filter;
    private MapConcept _options;
    private HTMLPageConcept _page;


    public HttpHandler( ){
    }


    public HttpHandler( StringConcept url ) {
        _url = url.toString();
    }
  

    public URIHandler create( StringConcept url ) {
        _url = url.toString();
        return this;
    }

    
    public LogicalConcept acceptsURI(StringConcept uri) {
        return ((URIHandlerDefinition)getConceptDefinition()).acceptsURI(uri);
    }
    

    public LogicalConcept initialize( MapConcept options ){
        _options = options;

        if( options.containsKey(new StringConcept("requests")).getValue() ) {
            String requests = options.getValue(new StringConcept("requests")).toTXT().toString();
            _requests = (new Integer(requests)).intValue();
        }

        if( options.containsKey(new StringConcept("filter")).getValue() ) {
            _filter = (ExpressionConcept) options.getValue(new StringConcept("filter"));
        }

        if( options.containsKey(new StringConcept("depth")).getValue() ) {
            String depth = options.getValue(new StringConcept("depth")).toTXT().toString();
            _depth = (new Integer(depth)).intValue();
        }
        
        if( _depth > 0 && _requests == 0 ) 
            _requests = Integer.MAX_VALUE;

        if( _requests > 1 && _depth == 0 )
            _depth = Integer.MAX_VALUE;

        if( mn8RuntimeFlags.isVerbose() || mn8RuntimeFlags.isDebug() ) {
            System.err.println("requests=" + _requests + "/" + "depth=" + _depth);
        }

        return new LogicalConcept(true);
    }
    

    public LogicalConcept openConnection() {
        _page = new HTMLPageConcept(_url);
        return new LogicalConcept(true);
    }

    
    public Concept getContent() {
        if( _requests == 1 && _depth == 0 ) {
            return _page;
        }
        // we have to get more pages ;)
        Vector result = new Vector();
        result.add( _page );
        SeriesConcept links = null;
        links = _page.getURIs();
        if( links.getVector().size() == 0 ) return new SeriesConcept( result );
        
        --_depth;
        int cnt = 1;
        Vector tmpVector = links.getVector();
        HTMLPageConcept tmpPage;

        for( int i = 0 ; i < tmpVector.size() && cnt < _requests; i++ ) {
	    
            if( mn8Interpreter.isExitAsked() ) 
                mn8Interpreter.interruptInterpret();
    	    
            if( _filter != null ) {
                StringConcept _path = null;
		
                if ( tmpVector.elementAt(i) instanceof StringConcept )
                    _path = (StringConcept)tmpVector.elementAt(i);
                else if ( tmpVector.elementAt(i) instanceof HTMLFormConcept )
                    _path = (StringConcept)(((HTMLFormConcept)tmpVector.elementAt(i)).url).getValue();
          
                if ( _path != null && !_filter.matches( _path ).getValue() ) {
                    tmpVector.remove(i);
                    i = i - 1;
                    continue;
                }
            } 
      
            tmpPage = null;
	    
            if ( tmpVector.elementAt(i) instanceof StringConcept )
                tmpPage = new HTMLPageConcept( ((StringConcept)tmpVector.elementAt(i)).toString() );
            else if ( tmpVector.elementAt(i) instanceof HTMLFormConcept )
                tmpPage = new HTMLPageConcept( (HTMLFormConcept)tmpVector.elementAt(i) );
      
            if ( tmpPage != null ) { result.add( tmpPage );cnt++; }
        } 

        while ( _depth > 0 && cnt < _requests ) {

            if( mn8Interpreter.isExitAsked() ) 
                mn8Interpreter.interruptInterpret();
            
            if( mn8RuntimeFlags.isVerbose() || mn8RuntimeFlags.isDebug() ) {
                System.err.println("requests=" + cnt + "/" + "depth=" + _depth);
            }

            tmpVector = getDeepLinks( tmpVector, cnt );

            for( int i = 0 ; i < tmpVector.size() && cnt < _requests; i ++ ) {
                if( _filter != null ) {
                    StringConcept _path = null;
		
                    if ( tmpVector.elementAt(i) instanceof StringConcept )
                        _path = (StringConcept)tmpVector.elementAt(i);
                    else if ( tmpVector.elementAt(i) instanceof HTMLFormConcept )
                        _path = (StringConcept)(((HTMLFormConcept)tmpVector.elementAt(i)).url).getValue();
                    
                    if ( _path != null && !_filter.matches( _path ).getValue() ) {
                        tmpVector.remove(i);
                        i = i - 1;
                        continue;
                    }
                } 
                
                tmpPage = null;
                
                if ( tmpVector.elementAt(i) instanceof StringConcept )
                    tmpPage = new HTMLPageConcept( ((StringConcept)tmpVector.elementAt(i) ).toString() );
                else if ( tmpVector.elementAt(i) instanceof HTMLFormConcept )
                    tmpPage = new HTMLPageConcept( (HTMLFormConcept)tmpVector.elementAt(i) );
            
                if ( tmpPage != null ) { result.add( tmpPage );cnt++; }
            }
            --_depth;
        }
        
        return new SeriesConcept( result );  
    }


    protected Vector getDeepLinks(Vector links, int cnt) {
        Vector result = new Vector();
        HTMLPageConcept tmpPage;
        StringConcept   uri;
	
        for( int i = 0; i < links.size(); i++ ) {

            if( cnt >= _requests ) return result;

            if( mn8Interpreter.isExitAsked() ) 
                mn8Interpreter.interruptInterpret();
            
            tmpPage = null;

            if ( links.elementAt(i) instanceof StringConcept ) 
                tmpPage = new HTMLPageConcept( ((StringConcept)links.elementAt(i)).toString() ); 
            else if ( links.elementAt(i) instanceof HTMLFormConcept )
                tmpPage = new HTMLPageConcept( (HTMLFormConcept)links.elementAt(i) );
            
            if ( tmpPage != null ) {
                Vector tmpVector = tmpPage.getURIs().getVector();
                result.addAll(  tmpVector );
                cnt += tmpVector.size();
            }
        }
        
        return result;
    }


    public LogicalConcept setContent(Concept value) {
        return new LogicalConcept( false );
    }

    
    public LogicalConcept closeConnection() {
        return new LogicalConcept( true );
    }
    
    
    public ConceptDefinition getConceptDefinition() {
        return HttpHandlerDefinition.getReference();
    }
    

    public Object clone() {        
        if( _url != null )
            return new HttpHandler( new StringConcept( _url ) );
        else
            return new HttpHandler();
    }
}
