/* 
 * $COPYRIGHT$
 * $Id: LocaleDefinition.java,v 1.1 2002/05/23 13:58:03 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.util.Locale;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.1 $ $Date: 2002/05/23 13:58:03 $
 */

public class LocaleDefinition extends ConceptConceptDefinition {
    
    static private SeriesConcept _inheritedConcepts = new SeriesConcept("Concept");
    static private LocaleDefinition _reference = new LocaleDefinition();
    static private SeriesConcept _availableLocales = null;
    static private SeriesConcept _isoCountries = null;
    static private SeriesConcept _isoLanguages = null;

    private LocaleDefinition() {
        super();
        try {            
            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "create:",
                                        "create:", "Locale", "Locale"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "create:org.media.mn8.concepts.StringConcept,org.media.mn8.concepts.StringConcept",
                                        "create:String,String", "Locale", "Locale"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "create:org.media.mn8.concepts.StringConcept,org.media.mn8.concepts.StringConcept,org.media.mn8.concepts.StringConcept",
                                        "create:String,String,String", "Locale", "Locale"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleDefinition", true,
                                        "getDefault:",
                                        "getDefault:", "Locale", "Locale"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleDefinition", true,
                                        "getAvailableLocales:",
                                        "getAvailableLocales:", "Locale", "Series"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleDefinition", true,
                                        "getISOCountries:",
                                        "getISOCountries:", "Locale", "Series"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleDefinition", true,
                                        "getISOLanguages:",
                                        "getISOLanguages:", "Locale", "Series"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleDefinition", true,
                                        "setDefault:org.media.mn8.concepts.LocaleConcept",
                                        "setDefault:Locale:", "Locale", null));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getCountry:",
                                        "getCountry:", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getLanguage:",
                                        "getLanguage:", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getVariant:",
                                        "getVariant:", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayCountry:",
                                        "getDisplayCountry:", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayName:",
                                        "getDisplayName:", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayLanguage:",
                                        "getDisplayLanguage:", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayVariant:",
                                        "getDisplayVariant:", "Locale", "String"));


            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayCountry:org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayCountry:Locale", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayName:org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayName:Locale", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayLanguage:org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayLanguage:Locale", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayVariant:org.media.mn8.concepts.LocaleConcept",
                                        "getDisplayVariant:Locale", "Locale", "String"));


            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getISO3Language:",
                                        "getISO3Language:", "Locale", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.LocaleConcept",
                                        "getISO3Country:",
                                        "getISO3Country:", "Locale", "String"));

        }catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public static ConceptDefinition getReference() {
        return _reference;
    }


    public static LocaleConcept getDefault() {
        return new LocaleConcept( Locale.getDefault());
    }


    public static void setDefault( LocaleConcept value) {
        if( value != null && value.getValue() != null ) {
            Locale.setDefault(value.getValue());
        }
    }


    public static SeriesConcept getAvailableLocales() {
        if( _availableLocales != null ) {
            return _availableLocales;
        }

        SeriesConcept result = new SeriesConcept();
        Locale[] locales = Locale.getAvailableLocales();
        if( locales != null ) {
            for( int i = 0 ; i < locales.length ; i++ ) {
                result.add( new LocaleConcept( locales[i]));
            }
        }
        _availableLocales = result;
        return result;
    }



    public static SeriesConcept getISOCountries() {
        if( _isoCountries != null ) {
            return _isoCountries;
        }

        SeriesConcept result = new SeriesConcept();
        String[] countries = Locale.getISOCountries();
        if( countries != null ) {
            for( int i = 0 ; i < countries.length ; i++ ) {
                result.add( new StringConcept( countries[i]));
            }
        }
        _isoCountries = result;
        return result;
    }


    public static SeriesConcept getISOLanguages() {
        if( _isoLanguages != null ) {
            return _isoLanguages;
        }

        SeriesConcept result = new SeriesConcept();
        String[] languages = Locale.getISOLanguages();
        if( languages != null ) {
            for( int i = 0 ; i < languages.length ; i++ ) {
                result.add( new StringConcept( languages[i]));
            }
        }
        _isoLanguages = result;
        return result;
    }


    public Concept getConceptInstance() {
        return  getDefault();
    }


    public StringConcept getConceptLabel() {
        return new StringConcept("Locale");
    }


    public StringConcept getConceptType() {
        return new StringConcept("Locale");
    }


    public SeriesConcept getInheritedConcepts() {
        return _inheritedConcepts;
    }
}



