/* 
 * $COPYRIGHT$
 * $Id: MapConcept.java,v 1.11 2002/06/05 14:54:25 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import org.media.mn8.util.SimpleExpression;

import java.util.Vector;
import java.util.Enumeration;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.11 $ $Date: 2002/06/05 14:54:25 $
 */
public class MapConcept extends JavaConcept implements Map {

    protected Vector _keys = new Vector();
    protected Vector _values = new Vector();
    protected Vector _nameCache = new Vector();

    protected IntegerConcept _index = new IntegerConcept(0);
    protected IntegerConcept _length = new IntegerConcept(0);
    protected LogicalConcept _hasNext = new LogicalConcept(false);
    protected LogicalConcept _hasPrevious = new LogicalConcept(false);

    public AttributeConcept length = new AttributeConcept( "length", "length", _length);
    public AttributeConcept index  = new AttributeConcept( "index", "index", _index);
    public AttributeConcept hasNext= new AttributeConcept( "hasNext", "hasNext", _hasNext);
    public AttributeConcept hasPrevious = new AttributeConcept( "hasPrevious", 
                                                                "hasPrevious", 
                                                                _hasPrevious);

    public MapConcept() {
        LogicalConcept true_value = new LogicalConcept(true);
        length.setHidden(true_value);
        index.setHidden(true_value);
        hasPrevious.setHidden(true_value);
        hasNext.setHidden(true_value);
    }


    public MapConcept( Vector keys, Vector values ) {
        super();
        _values = values;
        _keys = keys;
        _length.setValue( keys.size() );        
        updateNameCache();
        updateNavigationFans();
    }

    
    public void add( Concept key, Concept newValue) {
        _keys.add( key );
        _values.add( newValue);
        _length.setValue(_keys.size());
        _nameCache.add( key.toTXT().getValue() );
        updateNavigationFans();
    }


    public void remove(Concept key) {
        int ind = _keys.indexOf(key);
        if( ind != -1 ) {
            _keys.remove( ind );
            _values.remove( ind );
            _nameCache.remove( ind  );
            updateNavigationFans();
        }
    }


    public Concept nextValue() {
        if ( _hasNext.getValue() ) {
            _index.opIntegerINC();
            Concept result = (Concept)_values.get( _index.getInt() -1 );
            updateNavigationFans();
            return result;
        } 
        else return new NilConcept();        
    }


    public Concept nextKey() {
        if ( _hasNext.getValue() ) {
            _index.opIntegerINC();
            Concept result = (Concept)_keys.get( _index.getInt() -1 );
            updateNavigationFans();
            return result;
        } 
        else return new NilConcept();        
    }
    

    public void clear() {
        _keys.clear();
        _values.clear();
        updateNameCache();
        updateNavigationFans();
    }


    public Concept getValue( Concept key ) {
        if( _keys.contains( key ) ) {
            return (Concept) _values.elementAt( _keys.indexOf(key));

        } else {
            return new NilConcept();
        }

    }

    public Concept currentKey() {
        if ( _index.getLong() >= 1 && _index.getLong() <= _keys.size()   ) {
            Concept result = (Concept)_keys.get( _index.getInt() -1 );
            return result;
        } 
        else return new NilConcept();        
    }


    public Concept previousKey() {
        if ( _hasPrevious.getValue() ) {
            _index.opIntegerDEC();
            Concept result = (Concept)_keys.get( _index.getInt() -1 );
            updateNavigationFans();
            return (Concept) result;
        } 
        else return new NilConcept();        
    }
    

    public void insert( Concept newKey, Concept newValue ) {
        insert( new IntegerConcept( _index.getInt() +1 ), newKey, newValue);
    }


    public void insert(IntegerConcept position, Concept newKey,  Concept newValue ) {
        try {
            _keys.add( position.getInt()-1, newKey );
            _values.add( position.getInt()-1, newValue );
            _nameCache.add( position.getInt()-1, newKey.toTXT().getValue() );
            _length.setValue(_keys.size());
            updateNavigationFans();
        } catch (ArrayIndexOutOfBoundsException ex) {
            throw new mn8IndexOutOfBoundsException( position.toString() );
        }
    }


    public LogicalConcept containsKey( Concept key ) {
        return new LogicalConcept( _keys.contains(key));
    }


    public LogicalConcept containsValue( Concept value ) {
        return new LogicalConcept( _values.contains(value));
    }
    

    public IntegerConcept indexOfKey( Concept key ) {
        int idx = _keys.indexOf(key);
        if( idx == -1 ) return new IntegerConcept( idx);
        else return new IntegerConcept( idx +1);
    }


    public IntegerConcept indexOfValue( Concept value ) {
        int idx = _values.indexOf(value);
        if( idx == -1 ) return new IntegerConcept( idx);
        else return new IntegerConcept( idx +1);
    }


    public void sort( Concept sorter ) {

    }


    public SeriesConcept getKeys() {
        return new SeriesConcept( _keys );
    }


    public SeriesConcept getValues() {
        return new SeriesConcept( _values );
    }


    public Vector getKeyVector() {
        return _keys;
    }


    public Vector getValuesVector() {
        return _values;
    }

    
    public void setVectors( Vector keys, Vector contents) {
        _values = contents;
        _keys = keys;        
        _length.setValue(_keys.size());
        updateNameCache();
        updateNavigationFans();
    }

     
    public ConceptDefinition getConceptDefinition() {
        return MapDefinition.getReference();
    }


    public boolean hasNext() {
        return _hasNext.getValue();
    }


    public boolean hasPrevious() {
        return _hasPrevious.getValue();
    }


    public Concept getKeyAt( IntegerConcept position ) {
        return (Concept) _keys.elementAt(position.getInt() - 1);
    }


    public Concept getValueAt( IntegerConcept position ) {
        return (Concept) _values.elementAt(position.getInt() - 1);
    }


    public int getLength() {
        return _length.getInt();
    }


    public int getIndex() {
        return _index.getInt();
    }


    public void setIndex( int idx) {
        _index.setValue(idx);
        updateNavigationFans();
    }


    public void setIndex( IntegerConcept idx ) {
        setIndex( idx.getInt() );
    }


    protected void updateNameCache() {
        _nameCache.clear();
        for( int i = 0 ; i < _keys.size() ; i++) {
            _nameCache.add( ((Concept)_keys.elementAt(i)).toTXT().getValue());
        }

    }


    public boolean equals (Object obj) {
        if( obj instanceof org.media.mn8.concepts.MapConcept ) 
            if ( ( (MapConcept)obj).getKeyVector().equals(_keys) && ( (MapConcept)obj).getValuesVector().equals(_values) ) {
                return true;
            }
        return false;
    }


    protected void updateNavigationFans(){
        if( _keys.size() - _index.getInt() > 0 )
            _hasNext.setValue(true);
        else {
            _hasNext.setValue(false);
            _index.setValue( _keys.size() );
        }
        
        if( _index.getInt() >= 1 )
            if( _index.getInt() > 1)
                _hasPrevious.setValue(true);
            else
                _hasPrevious.setValue(false);
        else {
            _hasPrevious.setValue(false);
            _index.setValue( 0 );
        }
        _length.setValue( _keys.size() ); 
    }


    public Concept getConceptElement( StringConcept name ) {
        try {
            Concept result = super.getConceptElement(name);
            if( result != null ) 
                return result;

            if( _keys.indexOf( name ) == -1 ) return new NilConcept();
            int i= _keys.indexOf(name);

            return new ElementConcept( ((Concept)_keys.elementAt(i)).toTXT().toString(),
                                       ((Concept)_keys.elementAt(i)).toTXT().toString(), 
                                       (Concept) _values.elementAt(i), null);
        } catch (Exception ex) {}
        return new NilConcept();
    }


    public boolean hasConceptElement( String name ) {
        boolean result = super.hasConceptElement( name.toString() );
        if( result  == true )
            return result;

        if( _keys.indexOf( new StringConcept(name) ) == -1 ) return false;
        return true;
    }


    public SeriesConcept getConceptElementFields() {
        SeriesConcept result = super.getConceptElementFields();
        for( int i = 0; i < _keys.size(); i++) {
            result.add( new DummyFieldConcept( ((Concept)_keys.elementAt(i)).toTXT().toString(),
                                               new ElementConcept( ((Concept)_keys.elementAt(i)).toTXT().toString(), 
                                                                   ((Concept)_keys.elementAt(i)).toTXT().toString(),
                                                                   (Concept)_values.elementAt(i), null),
                                               false,
                                               ((Concept)_keys.elementAt(i)).toTXT().toString(),
                                               ((Concept)_keys.elementAt(i)).getConceptType().toString()));
        }
        return result;
    }


    public FieldDefinition getConceptElementField( String elementName ) {
        try {
            FieldDefinition result = super.getConceptElementField( elementName );
            if( result != null )
                return result;

            if( _keys.indexOf( new StringConcept(elementName) ) == -1 ) return null;
            int i= _keys.indexOf(new StringConcept(elementName));
            return new DummyFieldConcept(((Concept)_keys.elementAt(i)).toTXT().toString(),
                                         new ElementConcept( ((Concept)_keys.elementAt(i)).toTXT().toString(), 
                                                             ((Concept)_keys.elementAt(i)).toTXT().toString(),
                                                             (Concept)_values.elementAt(i), null),
                                         false,
                                         ((Concept)_keys.elementAt(i)).toTXT().toString(),
                                         ((Concept)_keys.elementAt(i)).getConceptType().toString());
        } catch (Exception ex) {}
        return null;
    }


    public SeriesConcept getConceptElements() {
        SeriesConcept result = new SeriesConcept();
        for( int i = 0; i < _keys.size(); i++) {
            result.add( new ElementConcept( ((Concept)_keys.elementAt(i)).toTXT().toString(), 
                                            ((Concept)_keys.elementAt(i)).toTXT().toString(), 
                                            (Concept)_values.elementAt(i), null));
        }
        return result;
    }

    
    public Object clone() {
        return new MapConcept((Vector)_keys.clone(), (Vector) _values.clone());
    }

}


