/* 
 * $COPYRIGHT$
 * $Id: MemHandler.java,v 1.4 2002/02/21 10:46:25 crow Exp $
 *
 * Date        Author            Changes 
 * FEB 07 2002 Szabo Csaba       Created
 */
package org.media.mn8.concepts;

import javax.naming.NamingException;
import javax.naming.NamingEnumeration;
import javax.naming.NameClassPair;

import org.media.mn8.*;
import org.media.mn8.event.*;


/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.4 $ $Date: 2002/02/21 10:46:25 $
 */

public class MemHandler extends URIHandler {

    private String _url = null;
    private ExpressionConcept _filter;
    private MapConcept _options;


    public MemHandler( ){
    }


    public MemHandler( StringConcept url ) {
        _url = url.toString();
    }
  

    public URIHandler create( StringConcept url ) {
       _url = url.toString();	
       return this;
    }

    
    public LogicalConcept acceptsURI( StringConcept uri ) {
        return ((URIHandlerDefinition)getConceptDefinition()).acceptsURI( uri );
    }
 

    public LogicalConcept initialize( MapConcept options ){
        _options = options;

        if( options.containsKey(new StringConcept("filter")).getValue() )
            _filter = (ExpressionConcept) options.getValue(new StringConcept("filter"));

        return new LogicalConcept(true);
    }


    public LogicalConcept openConnection() {
        return new LogicalConcept(true);
    }


    public Concept getContent() {
        try {
            return (Concept)NamingManager.getReference().lookup( _url );
        } 
        catch ( Exception ex ) {	
            try {
                SeriesConcept result = new SeriesConcept();

                for ( NamingEnumeration ne = NamingManager.getReference().list( _url ); ne.hasMore(); ) {
                    Object obj = ne.next();

                    if ( obj instanceof NameClassPair ) {
                        String myPath = ( _url.endsWith("/") ? _url : _url + "/") + 
                            ((NameClassPair)obj).getName();
                        StringConcept _path = new StringConcept( myPath );
                        if ( _filter != null && !_filter.matches( _path ).getValue() ) continue;
                        try {
                            NamingManager.getReference().list( _path.toString() );
                            _path = new StringConcept( _path.toString() + "/" );
                        } catch ( Exception _ex ) {}
                        result.add( _path );
                    }
                    else if ( obj instanceof String )
                        result.add ( new StringConcept( (String)obj ) ); 
                }
                
                return result;
            }
            catch ( NamingException nex ) { 
                new ErrorConcept( "warning", "badURL", nex.getMessage() );
                return new StringConcept( "" );
            }
        }
    }


    public LogicalConcept setContent( Concept value ) {
        try {
            NamingManager.getReference().rebind( _url, value ); 
            return new LogicalConcept( true );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "bindFailed", ex.getMessage() );
            return new LogicalConcept( false );
        }
    }


    public LogicalConcept closeConnection() {
        return new LogicalConcept( true );
    }


    public ConceptDefinition getConceptDefinition() {
        return MemHandlerDefinition.getReference();
    }


    public Object clone() {        
        if( _url != null )
            return new MemHandler( new StringConcept( _url ) );
        else
            return new URIHandler();
    }
}
