/*
 * $COPYRIGHT$
 * $Id: POP3ClientConcept.java,v 1.2 2002/02/21 10:46:51 crow Exp $
 *
 * Date          Author            Changes
 * IAN 25 2002   Szabo Csaba       Created
 */
      
package org.media.mn8.concepts;

import org.media.mn8.*;
import java.net.URL;
import java.io.PrintWriter;
import java.io.OutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import com.oroinc.net.pop3.*;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.2 $ $Date: 2002/02/21 10:46:51 $
 */

public class POP3ClientConcept extends JavaConcept {
    protected POP3Client _client;
    protected URL        _url;
    
    public AttributeConcept errorMessage = new AttributeConcept("errorMessage", "errorMessage", new NilConcept() );


    public POP3ClientConcept() {
    }
    
    
    public LogicalConcept openConnection ( StringConcept url ) {
        URL givenURL = null;
	
        try {
            givenURL = new URL( url.toString() );
            if( givenURL == null || givenURL.getProtocol().toLowerCase().equals("pop3")) {
                if( givenURL != null ) {
                    _url    = givenURL;
                    _client = new POP3Client( );
                    _client.addProtocolCommandListener ( new PrintCommandListener ( new PrintWriter ( System.out ) ) );
		    
                    if ( givenURL.getHost() == null || givenURL.getHost().trim().length() < 1 ) {
                        new ErrorConcept("error", "missingHost", "Missing host name for POP3 connection.");
                        return new LogicalConcept( false );
                    } 
		    
                    if ( givenURL.getHost() != null && givenURL.getPort() == -1 )
                        _client.connect( givenURL.getHost() );
                    
                    if ( givenURL.getHost() != null && givenURL.getPort() != -1 )
                        _client.connect( givenURL.getHost() , givenURL.getPort() );
    
                    errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
	            
                    if ( POP3.DISCONNECTED_STATE == _client.getState() ) {
                        _client.disconnect();
                        new ErrorConcept("error", "connectionRefuzed", "POP3 server (" + givenURL.getHost() + ") refuzed connection." );
                        return new LogicalConcept( false );
                    }
                }
            } else if ( givenURL != null ) {
                new ErrorConcept( "error", "unsupportedProtocol", "'" + givenURL.getProtocol() + 
                                  "' protocol not supported by POP3Client" );	
            }
        } 
        catch ( Exception ex ) {
            if( mn8RuntimeFlags.isDebug() ) ex.printStackTrace();
            return new LogicalConcept( false );
        }
        
        return new LogicalConcept( true );
    }

    
    public void closeConnection() {
        try {
            _client.disconnect();
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "closeConnection", ex.getMessage() );
        }
    }
    
    
    public LogicalConcept login ( StringConcept uname, StringConcept pwd ) {
        try {
            boolean logined = _client.login( uname.toString(), pwd.toString() );
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            if ( !logined ) new ErrorConcept( "warning", "loginWarning", _client.getReplyString() );
            return new LogicalConcept( logined );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "loginFailed", ex.getMessage() ); 
            return new LogicalConcept( false );
        }
    }
    
    
    public LogicalConcept logout() {
        try {
            boolean status = _client.logout();
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            return new LogicalConcept( status );
        }
        catch ( Exception ex ) { 
            new ErrorConcept( "error", "logoutFailed", ex.getMessage() );
            return new LogicalConcept( false );
        } 
    }
    

    public SeriesConcept getLinks() {
        SeriesConcept    result = new SeriesConcept();
	
        try {
            POP3MessageInfo infos[] = _client.listMessages();
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            if ( infos != null ) {
                for ( int i = 0; i < infos.length; i++ )
                    result.add( new StringConcept( infos[i].number + " (" + infos[i].size + ")" ) );
            }
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "POP3OperationFailed", ex.getMessage() );
         }	
        
        return result;
    }


    public StringConcept getLinks( IntegerConcept messID ) {
        try {
            POP3MessageInfo info = _client.listMessage( messID.getInt() );
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            if ( info != null ) 
                return new StringConcept ( info.number + " (" + info.size + ")" );
            else 
                return new StringConcept ( "" );
        } 
        catch ( Exception ex ) {
            new ErrorConcept( "error", "POP3OperationFailed", ex.getMessage() );
            return new StringConcept ( "" );
        }
    }


    public LogicalConcept delete( IntegerConcept messID ) {
        try {
            boolean status = _client.deleteMessage( messID.getInt() );
            errorMessage.setAttributeValue( new StringConcept( _client.getReplyString() ) );
            return new LogicalConcept( status );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "POP3OperationFailed", ex.getMessage() );
            return new LogicalConcept( false );
        }
    }
     
    
    public StringConcept getMessage( IntegerConcept messID ) {
        try {
            BufferedReader reader = new BufferedReader ( _client.retrieveMessage( messID.getInt() ) );
            StringBuffer   buffer = new StringBuffer();
            String           line = null;

            while ( ( line = reader.readLine() ) != null ) buffer.append( line + "\n" );
            
            reader.close();
            return new StringConcept( buffer.toString() ); 
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "POP3OperationFailed", ex.getMessage() );
            return new StringConcept( "" );
        }    
    }


    public ConceptDefinition getConceptDefinition() {
        return POP3ClientConceptDefinition.getReference();
    }
    
    
    public Object clone() {
        return this;
    }
}
