/* 
 * $COPYRIGHT$
 * $Id: POP3Handler.java,v 1.3 2002/02/21 11:08:16 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 28 2002 Szabo Csaba       Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.net.URL;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.3 $ $Date: 2002/02/21 11:08:16 $
 */

public class POP3Handler extends URIHandler {

    private String   _url = null;
    private String _uname = null;
    private String   _pwd = null;
    private MapConcept _options;
    private POP3ClientConcept _client;

    public POP3Handler( ){
    }


    public POP3Handler( StringConcept url ) {
        _url = url.toString();
    }
    

    public URIHandler create( StringConcept url ) {
       _url = url.toString();
       return this;
    }

    
    public LogicalConcept acceptsURI( StringConcept uri ) {
        return ( (URIHandlerDefinition)getConceptDefinition() ).acceptsURI( uri );
    }
    
    
    public LogicalConcept initialize( MapConcept options ){
        URL givenURL = null;

        _options = options;
        
        if( options.containsKey(new StringConcept("user")).getValue() ) {
            _uname = options.getValue(new StringConcept("user")).toTXT().toString();
        }

        if( options.containsKey(new StringConcept("pwd")).getValue() ) {
            _pwd = options.getValue(new StringConcept("pwd")).toTXT().toString();
        }
	 
        try {
            givenURL = new URL( _url );
            if ( givenURL != null && givenURL.getProtocol().equalsIgnoreCase("pop3") ) {
                if ( _uname != null && _uname.trim().length() > 0 && _pwd != null && _pwd.trim().length() > 0 ) {
                    _url  = givenURL.getProtocol() + "://";
                    _url += _uname + ":" + _pwd + "@";
                    if ( givenURL.getHost()  != null ) _url += givenURL.getHost();
                    if ( givenURL.getPort()  != -1   ) _url += ":" + givenURL.getPort();
                    if ( givenURL.getPath()  != null ) _url += givenURL.getPath();
                    if ( givenURL.getQuery() != null ) _url += "?" + givenURL.getQuery();
                    if ( givenURL.getRef()   != null ) _url += "#" + givenURL.getRef();
                }
            }
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "badURL", ex.getMessage() );
            return new LogicalConcept( false );
        }
        
        return new LogicalConcept( true );
    }


    public LogicalConcept closeConnection() {
        return _client.logout();
    }

    
    public LogicalConcept openConnection() {
        _client = new POP3ClientConcept( ); 
	
        if ( !(_client.openConnection( new StringConcept ( _url ) ) ).getValue() ) 
            return new LogicalConcept( false );

        try {
            URL     myURL = new URL( _url  );
            String     ui = myURL.getUserInfo();
            String   user = null;
            String passwd = null;
            
            if ( ui == null || ui.trim().length() < 1 ) 
                return new LogicalConcept( false );	
            
            if ( ui.indexOf(":") != -1 ) {
                user = ui.substring( 0, ui.indexOf(":") );
                passwd = ui.substring( ui.indexOf(":")+1 );
            }
            else {
                new ErrorConcept( "error", "userInfoMissing", "Username and password to login to POP3 server is missing." );
                return new LogicalConcept( false );
            }
            return _client.login( new StringConcept( user ), new StringConcept ( passwd ) );
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "badURL", ex.getMessage() );
            return new LogicalConcept( false );
        }
    }
    
 
    public Concept getContent() {
        String command = null;
        String  messID = null; 
        
        try {
            URL     url = new URL( _url );
            String file = url.getFile();
            String   qu = url.getQuery();

            if ( file.equals("/") || file.trim().length() < 1 ) {
                command = "retrieve";
            } 
            else {
                if ( file.indexOf("?") < 0 ) command = file.substring(1);
                else command = file.substring( 1, file.indexOf("?") );
            }
            
            if ( qu != null && qu.indexOf("=") > 0 && qu.trim().length() != 0 )
                messID = qu.substring( qu.indexOf("=") + 1 );
            
            if ( command.equalsIgnoreCase("list") ) {
                if ( messID == null ) 
                    return _client.getLinks();
                else 
                    return _client.getLinks ( new IntegerConcept ( messID ) );
            }
            
            if ( command.equalsIgnoreCase("retrieve") ) {
                if ( messID != null ) 
                    return _client.getMessage( new IntegerConcept ( messID ) );
                else {
                    SeriesConcept result = new SeriesConcept();
                    
                    for ( Enumeration e = _client.getLinks().getVector().elements(); e.hasMoreElements(); ) {
                        String id = ( (StringConcept)e.nextElement() ).toString();
                        id = id.substring(0, id.indexOf("(") ).trim();
                        result.add( _client.getMessage( new IntegerConcept( id ) ) );
                    }
                    
                    return result;
                }
            }
            
            if ( command.equalsIgnoreCase("delete") ) {
                if ( messID != null )
                    return _client.delete( new IntegerConcept( messID ) );
                else {
                    boolean allDeleted = true;
                    
                    for ( Enumeration e = _client.getLinks().getVector().elements(); e.hasMoreElements(); ) {
                        String id = ( (StringConcept)e.nextElement() ).toString();
                        id = id.substring(0, id.indexOf("(") ).trim();
                        if ( !_client.delete( new IntegerConcept( id ) ).getValue() ) allDeleted = false;
                    }
                    
                    return new LogicalConcept( allDeleted );
                }
            } 
            
            new ErrorConcept("error", "unknownCommand", "Unknown command '" + command +"'.\n" +
                             "Command may be: list, delete or retrieve.");

        } catch ( Exception ex ) {
            new ErrorConcept( "error", "badURL", ex.getMessage() );
        }
        return new StringConcept( "" );
    }

 
    public LogicalConcept setContent( Concept value ) {
        return new LogicalConcept( false );
    }


    public ConceptDefinition getConceptDefinition() {
        return POP3HandlerDefinition.getReference();
    }


    public Object clone() {        
        if( _url != null )
            return new POP3Handler( new StringConcept( _url ) );
        else
            return new URIHandler();
    }
}
