/* 
 * $COPYRIGHT$
 * $Id: ProcessConcept.java,v 1.1 2002/02/28 20:28:54 crow Exp $
 *
 * Date        Author            Changes 
 * FEB 28 2002 Szabo Csaba       Created
 */

package org.media.mn8.concepts;

import org.media.mn8.Concept;
import org.media.mn8.JavaConcept;
import org.media.mn8.ConceptDefinition;
import java.io.*;
import java.util.Vector;
import java.util.Enumeration;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/02/28 20:28:54 $
 */

public class ProcessConcept extends JavaConcept {
    
    public AttributeConcept exitValue = new AttributeConcept("exitValue", "exitValue", new StringConcept() );

    public ElementConcept  error = new ElementConcept("error", "error", new NilConcept(), null );
    public ElementConcept  input = new ElementConcept("input", "input", new NilConcept(), null );
    public ElementConcept output = new ElementConcept("output", "output", new NilConcept(), null );

    private String inputText = null;


    public ProcessConcept() {
    }


    public void setInput( StringConcept text ) {
        inputText = text.toString();
        input.setElementValue( text ); 
    }

    
    public ProcessConcept exec( StringConcept cmd ) {
        try {
            Process process = Runtime.getRuntime().exec( cmd.toString() );
            if ( inputText != null ) sendTo( process );process.waitFor();inputText = null;
            exitValue.setAttributeValue( new StringConcept( process.exitValue() + "" ) );
            error.setElementValue( new StringConcept( getStringFromIS( process.getErrorStream() ) ) );
            output.setElementValue( new StringConcept( getStringFromIS( process.getInputStream() ) ) );
        } 
        catch ( Exception ex ) {
            new ErrorConcept( "error", "execFailed", "Unable to execute command" );
        }
        return this;
    }

    
    public ProcessConcept exec( StringConcept cmd, StringConcept dir ) {
        try {
            Process process = Runtime.getRuntime().exec( cmd.toString(), null, new File( dir.toString() ) );
            if ( inputText != null ) sendTo( process );process.waitFor();inputText = null;
            exitValue.setAttributeValue( new StringConcept( process.exitValue() + "" ) );
            error.setElementValue( new StringConcept( getStringFromIS( process.getErrorStream() ) ) );
            output.setElementValue( new StringConcept( getStringFromIS( process.getInputStream() ) ) );
        } 
        catch ( Exception ex ) {
            new ErrorConcept( "error", "execFailed", "Unable to execute command" );
        }
        return this;
    }

    
    public ProcessConcept exec( SeriesConcept cmd ) {
        try {
            Process process = Runtime.getRuntime().exec( getArray( cmd.getValue() ) );
            if ( inputText != null ) sendTo( process );process.waitFor();inputText = null;
            exitValue.setAttributeValue( new StringConcept( process.exitValue() + "" ) );
            error.setElementValue( new StringConcept( getStringFromIS( process.getErrorStream() ) ) );
            output.setElementValue( new StringConcept( getStringFromIS( process.getInputStream() ) ) );
        } 
        catch ( Exception ex ) {
            new ErrorConcept( "error", "execFailed", "Unable to execute command" );
        }
        return this;
    }

    
    public ProcessConcept exec( SeriesConcept cmd, StringConcept dir ) {
        try {
            Process process = Runtime.getRuntime().exec( getArray( cmd.getValue() ), null, new File( dir.toString() ) );
            if ( inputText != null ) sendTo( process );process.waitFor();inputText = null;
            exitValue.setAttributeValue( new StringConcept( process.exitValue() + "" ) );
            error.setElementValue( new StringConcept( getStringFromIS( process.getErrorStream() ) ) );
            output.setElementValue( new StringConcept( getStringFromIS( process.getInputStream() ) ) );
        } 
        catch ( Exception ex ) {
            new ErrorConcept( "error", "execFailed", "Unable to execute command" );
        }
        return this;
    }


    public Object clone() {
        return this;
    }


    public ConceptDefinition getConceptDefinition() {
        return ProcessDefinition.getReference();
    }
 
    
    private String getStringFromIS( InputStream is ) throws IOException {
        String          line;
        StringBuffer   result = new StringBuffer();
        BufferedReader reader = new BufferedReader( new InputStreamReader( is ) );
       
        while ( ( line = reader.readLine() ) != null ) result.append( line + "\n" );
        reader.close();
        
        return result.toString();
    }

    
    private String[] getArray( Vector cmd ) {
        String[] cmds = new String[ cmd.size() ];
        int i = 0;

        for ( Enumeration e = cmd.elements(); e.hasMoreElements(); i++ )
            cmds[i] = ( (StringConcept)e.nextElement() ).toString();

        return cmds;
    }
    
   
    private void sendTo( Process process ) throws Exception {
        BufferedWriter writer = new BufferedWriter( new OutputStreamWriter( process.getOutputStream() ) );
        writer.write( inputText, 0, inputText.length() );
        writer.flush(); writer.close();
    }
}
