/* 
 * $COPYRIGHT$
 * $Id: RealConcept.java,v 1.10 2002/05/02 18:25:52 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;


/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.10 $ $Date: 2002/05/02 18:25:52 $
 */
public class RealConcept extends JavaConcept {
    
    protected float _value=0f ;


    public RealConcept( ){
    }


    public RealConcept( long value ){
        _value = value;
    }


    public RealConcept( float value ) {
        _value = value;
    }


    public RealConcept( String value) {
        _value = (new Float(value)).floatValue();
    }


    public Concept create ( StringConcept value ) {
        _value = (new Float(value.toString())).floatValue();
        return this;
    }


    public RealConcept opPlus(RealConcept value) {
        return new RealConcept(getFloat() + value.getFloat());
    }

    
    public void opPlusEqual(RealConcept value) {
        _value = getFloat() + value.getFloat();
    }


    public RealConcept opRealINC() {
        ++_value;
        return new RealConcept( _value );
    }


    public RealConcept opRealDEC() {
        --_value;
        return new RealConcept( _value );
    }


    public RealConcept opMinus(RealConcept value) {
        return new RealConcept(getFloat() - value.getFloat() );
    }


    public RealConcept opSTAR(RealConcept value) {
        return new RealConcept(getFloat() * value.getFloat() );
    }


    public RealConcept opDIV(RealConcept value) {
        return new RealConcept(getFloat() / value.getFloat() );
    }


    public RealConcept sqrt() {
        return new RealConcept( getSqrt() );
    }


    public RealConcept mod(RealConcept mod) {
        return new RealConcept( getFloat() %  mod.getFloat() );
    }


    public Concept abs() {
        return new RealConcept( Math.abs( getFloat() ) );
    }


    public Concept max(RealConcept value) {
        return new RealConcept( Math.max( getFloat(), value.getFloat() ) );
    }

 
    public Concept min(RealConcept value) {
        return new RealConcept( Math.min( getFloat(), value.getFloat() ) );
    }


    public RealConcept acos() {
        return new RealConcept( ((float)Math.acos( (double)getFloat() ) ) );
    }


    public RealConcept asin() {
        return new RealConcept( ((float)Math.asin( (double)getFloat() ) ) );
    }


    public RealConcept atan() {
        return new RealConcept( ((float)Math.atan( (double)getFloat() ) ) );
    }


    public RealConcept ceil() {
        return new RealConcept( ((float)Math.ceil( (double)getFloat() ) ) );
    }


    public RealConcept cos() {
        return new RealConcept( ((float)Math.cos( (double)getFloat() ) ) );
    }


    public RealConcept exp() {
        return new RealConcept( ((float)Math.exp( (double)getFloat() ) ) );
    }


    public RealConcept floor() {
        return new RealConcept( ((float)Math.floor( (double)getFloat() ) ) );
    }


    public RealConcept log() {
        return new RealConcept( ((float)Math.log( (double)getFloat() ) ) );
    }


    public Concept pow(RealConcept value) {
        return new RealConcept( (float)Math.pow( getFloat(), value.getFloat() ) );
    }


    public RealConcept sin() {
        return new RealConcept( ((float)Math.sin( (double)getFloat() ) ) );
    }


    public RealConcept tan() {
        return new RealConcept( ((float)Math.tan( (double)getFloat() ) ) );
    }


    public RealConcept toDegrees() {
        return new RealConcept( ((float)Math.toDegrees( (double)getFloat() ) ) );
    }


    public RealConcept toRadians() {
        return new RealConcept( ((float)Math.toRadians( (double)getFloat() ) ) );
    }


    public boolean equals (Object obj) {
        if( obj instanceof org.media.mn8.concepts.RealConcept ) 
            if ( ( (RealConcept)obj).getFloat() == _value ) {
                return true;
            }
        return false;
    }


    // Relational Operations    
    public LogicalConcept opGT(RealConcept value) {
        return new LogicalConcept( getFloat() > value.getFloat());
    }

    public LogicalConcept opGE(RealConcept value) {
        return new LogicalConcept( getFloat() >= value.getFloat());
    }


    public LogicalConcept opLT(RealConcept value) {
        return new LogicalConcept( getFloat()  < value.getFloat());
    }


    public LogicalConcept opLE(RealConcept value) {
        return new LogicalConcept( getFloat()  <= value.getFloat());
    }


    public LogicalConcept opEqual(RealConcept value) {
        return new LogicalConcept( getFloat() == value.getFloat());
    }


    public IntegerConcept round() {
        return new IntegerConcept( Math.round( _value ));
    }


    public LogicalConcept toLogicalConcept() {
        if ( getFloat() > 0 ) 
            return new LogicalConcept(true);
        else 
            return new LogicalConcept(false);
    }


    public StringConcept toStringConcept() {
        return new StringConcept("" + _value);
    }


    public StringConcept toTXT() {
        return toStringConcept();
    }


    public StringConcept toXML() {
        return toStringConcept();
    }
    

    public float getFloat() {
        return _value;
    }


    public float getSqrt() {
        return (float) Math.sqrt( (double ) getFloat() );
    }


    public void setValue( float value) {
        _value = value;
    }


    public ConceptDefinition getConceptDefinition() {
        return RealDefinition.getReference();
    }


    public void main ( SeriesConcept param ) {
        System.out.println(_value);
    }

    public Object clone() {
        return new RealConcept(getFloat());
    }
}
