/* 
 * $COPYRIGHT$
 * $Id: SMTPHandler.java,v 1.4 2002/02/21 12:15:53 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 28 2002 Szabo Csaba       Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;
import java.io.BufferedReader;
import java.io.StringReader;
import java.io.PrintWriter;
import java.io.IOException;
import java.io.Writer;

import com.oroinc.net.smtp.*;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.4 $ $Date: 2002/02/21 12:15:53 $
 */

public class SMTPHandler extends URIHandler {

    private String        _url = null;
    private SMTPClient _client;

    public SMTPHandler( ){
    }


    public SMTPHandler( StringConcept url ) {
        _url = url.toString();
    }
   

    public URIHandler create( StringConcept url ) {
       	_url = url.toString();
	return this;
    }

    
    public LogicalConcept acceptsURI( StringConcept uri ) {
        return ( (URIHandlerDefinition)getConceptDefinition() ).acceptsURI( uri );
    }
    
    
    public LogicalConcept initialize( MapConcept options ){
        return new LogicalConcept( true );
    }


    public LogicalConcept closeConnection() {
        try {
            return new LogicalConcept( _client.logout() );
        }
        catch ( Exception ex ) {
            return new LogicalConcept( false );
        }
    }

    
    public LogicalConcept openConnection() {
        URL givenURL = null;
        
        try {
            givenURL = new URL( _url );
            if( givenURL == null || givenURL.getProtocol().toLowerCase().equals("smtp") ) {
                if( givenURL != null ) {
                    _client = new SMTPClient( );
                    _client.addProtocolCommandListener ( new PrintCommandListener ( new PrintWriter ( System.out ) ) );
		    
                    if ( givenURL.getHost() == null || givenURL.getHost().trim().length() < 1 ) {
                        new ErrorConcept( "error", "missingHost", "Missing host name for SMTP connection." );
                        return new LogicalConcept( false );
                    } 
		    
                    if ( givenURL.getHost() != null && givenURL.getPort() == -1 )
                        _client.connect( givenURL.getHost() );
                    
                    if ( givenURL.getHost() != null && givenURL.getPort() != -1 )
                        _client.connect( givenURL.getHost() , givenURL.getPort() );
                    
                    if ( !SMTPReply.isPositiveCompletion (_client.getReplyCode() ) ) {
                        _client.disconnect();
                        new ErrorConcept( "error", "connectionRefuzed", "SMTP server (" + givenURL.getHost() + ") refuzed connection." );
                        return new LogicalConcept( false );
                    }
                    
                    if ( !_client.login() ) {
                        new ErrorConcept( "error", "loginFailed", "Login failed to SMTP server." );
                        return new LogicalConcept( false );
                    }

                    return new LogicalConcept( true );
                }
            }
        } 
        catch ( IOException e ) {
            if ( _client.isConnected() ) {
                try {
                    _client.disconnect();
                } catch ( IOException f ) {}
             }
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "badURL", ex.getMessage() );
        } 
        return new LogicalConcept( false );
   }

 
    public Concept getContent() {
        return new LogicalConcept( false );
    }

 
    public LogicalConcept setContent( Concept value ) {
        String content = ( (StringConcept)value ).toString();
        String   field = null;

        try {
            String    from = getEmailAddress( getFromHeader( content , "From:") );
            String      to = getEmailAddress( getFromHeader( content , "To:") );
            String subject = getFromHeader( content , "Subject:");

            if ( from == null || from.length() < 1 ) return new LogicalConcept( false );
            if ( to == null || to.length() < 1 )     return new LogicalConcept( false );
            if ( !_client.setSender( from ) )  return new LogicalConcept( false );
            if ( !_client.addRecipient( to ) ) return new LogicalConcept( false );
            Writer writer = _client.sendMessageData();
            if( writer == null ) return new LogicalConcept( false );
            SimpleSMTPHeader header = new SimpleSMTPHeader(from, to , subject);
            Vector headers = getHeaderFields( content );

            if ( headers != null ) {
                for ( Enumeration e = headers.elements(); e.hasMoreElements(); ) {
                    String _header = (String)e.nextElement();
                    if ( isSetable( _header ) && ( field = getFromHeader( content, _header + ":" ) ) != null )
                        header.addHeaderField( _header, field);
                }
            }
            
            writer.write( header.toString() );
            writer.write( getMessage( content ) );
            writer.close();
            if( !_client.completePendingCommand() ) return new LogicalConcept( false );
        } 
        catch ( Exception ex ) { 
            new ErrorConcept( "error", "sendFailed", ex.getMessage() );
            return new LogicalConcept( false );
        }
        
        return new LogicalConcept( true );
    }
    
    
    private boolean isSetable( String field ) {
        if ( field != null && !field.equalsIgnoreCase("from") && !field.equalsIgnoreCase("to") &&
             !field.equalsIgnoreCase("subject") && !field.equalsIgnoreCase("received") &&
             !field.equalsIgnoreCase("message-id") && !field.equalsIgnoreCase("return-path") &&
             !field.equalsIgnoreCase("delivered-to") ) return true;
        return false;
    }

    
    private Vector getHeaderFields( String content ) throws Exception {
        Vector headers = new Vector();
        String line = null;
        
        try {
            BufferedReader reader = new BufferedReader( new StringReader( content ) );

            while ( ( line = reader.readLine() ) != null ) {
                if ( line.trim().length() < 1 ) break;
                if ( line.indexOf(": ") > 0 && line.lastIndexOf(" ", line.indexOf(": ") ) == -1 ) 
                    headers.add ( line.substring(0, line.indexOf(":" ) ) );
            }
            reader.close();
        } catch ( Exception ex ) {
            throw ex;
        }
        
        return headers;
    }

    
    private String getEmailAddress( String str ) {
        int poz;
        
        if ( ( poz = str.indexOf( "<" ) ) != -1 ) {
            if ( ( poz = str.indexOf(":", poz ) ) == -1 ) poz = str.indexOf( "<" );
            return str.substring( poz + 1, 
                                  str.indexOf( ">", poz ) != -1 ? str.indexOf( ">", poz ) : str.length() ).trim();
        }
        return str;
    }
    
    
    private String getFromHeader( String content, String txt ) throws Exception {
        String     line = null;
        StringBuffer sb = new StringBuffer();
        
        try {
            BufferedReader reader = new BufferedReader( new StringReader( content ) );
            
            while ( ( line = reader.readLine() ) != null ) {
                if ( line.trim().length() < 1 ) break;
                if ( line.toLowerCase().startsWith( txt.toLowerCase() ) ) {
                    sb.append ( line.substring( line.indexOf(":") + 1 ).trim() );
                    while ( ( line = reader.readLine() ) != null ) {
                        if ( line.trim().length() < 1 ) break;
                        if ( line.indexOf(": ") > 0 && line.lastIndexOf(" ", line.indexOf(": ") ) == -1 ) break;
                        sb.append( "\n" + line );
                    }
                    return sb.toString();
                }
            }
            reader.close();
        } catch ( Exception ex ) {
            throw ex;
        }
        
        return sb.toString();
    }
    

    private String getMessage( String content ) throws Exception {
        String      line = null;
        StringBuffer  sb = new StringBuffer();
        boolean isHeader = true;
        
        try {
            BufferedReader reader = new BufferedReader( new StringReader( content ) );
            
            while ( ( line = reader.readLine() ) != null ) {
                if ( !isHeader ) sb.append( line + "\n" );
                if ( line.trim().length() < 1 ) isHeader = false;
            }
            reader.close();
        } catch ( Exception ex ) {
            throw ex;
        }
        
        return sb.toString();
    }


    public ConceptDefinition getConceptDefinition() {
        return SMTPHandlerDefinition.getReference();
    }


    public Object clone() {        
        if( _url != null )
            return new SMTPHandler( new StringConcept( _url ) );
        else
            return new URIHandler();
    }
}
