/* 
 * $COPYRIGHT$
 * $Id: SeriesConcept.java,v 1.41 2002/06/05 14:54:35 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import org.media.mn8.util.SimpleExpression;
import org.media.mn8.util.ConceptNavPatterns;
import java.util.Vector;
import java.util.Enumeration;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.41 $ $Date: 2002/06/05 14:54:35 $
 */
public class SeriesConcept extends JavaConcept implements Series {

    protected Vector _contents = new Vector();
    protected Vector _navCache = new Vector();

    protected IntegerConcept _index = new IntegerConcept(0);
    protected IntegerConcept _length = new IntegerConcept(0);
    protected LogicalConcept _hasNext = new LogicalConcept(false);
    protected LogicalConcept _hasPrevious = new LogicalConcept(true);

    public AttributeConcept length = new AttributeConcept( "length", "length", _length);
    public AttributeConcept index  = new AttributeConcept( "index", "index", _index);
    public AttributeConcept hasNext= new AttributeConcept( "hasNext", "hasNext", _hasNext);
    public AttributeConcept hasPrevious = new AttributeConcept( "hasPrevious", 
                                                                "hasPrevious", 
                                                                _hasPrevious);

    public SeriesConcept() {
        LogicalConcept true_value = new LogicalConcept(true);
        length.setHidden(true_value);
        index.setHidden(true_value);
        hasPrevious.setHidden(true_value);
        hasNext.setHidden(true_value);
    }


    public SeriesConcept( Vector values ) {
        super();
        _contents = values;
        _length.setValue( values.size() );        
        updateNameCache();
        updateNavigationFans();
    }


    public SeriesConcept( String value ) {
        super();
        _contents.add(new StringConcept(value));
        _length.setValue(_contents.size());        
        updateNameCache();
        updateNavigationFans();
    }


    public SeriesConcept(CharacterConcept from, CharacterConcept to, IntegerConcept step) {
        super();
        for( char i = from.getValue(); i <= to.getValue() ; i = (char) (i + step.getInt()) ) {
            _contents.add( new CharacterConcept(i));
            _length.setValue(_length.getLong() +1);            
        }
        updateNameCache();
        updateNavigationFans();
    }


    public SeriesConcept(IntegerConcept from, IntegerConcept to, IntegerConcept step) {
        super();
        for( long i = from.getLong(); i <= to.getLong() ; i= i + step.getLong() ) {
            _contents.add( new IntegerConcept(i));
            _length.setValue(_length.getLong() +1);            
        }
        updateNameCache();
        updateNavigationFans();
    }


    public SeriesConcept(IntegerConcept from, IntegerConcept to, RealConcept step) {
        super();
        for( float i = from.getLong(); i <= to.getLong() ; i= i + step.getFloat() ) {
            _contents.add( new RealConcept(i));
            _length.setValue(_length.getLong() +1);            
        }
        updateNameCache();
        updateNavigationFans();
    }


    public SeriesConcept(RealConcept from, RealConcept to, RealConcept step) {
        super();
        for( float i = from.getFloat(); i <= to.getFloat() ; i= i + step.getFloat() ) {
            _contents.add( new RealConcept(i));
            _length.setValue(_length.getLong() +1);            
        }
        updateNameCache();
        updateNavigationFans();
    }


    public SeriesConcept(RealConcept from, RealConcept to, IntegerConcept step) {
        super();
        for( float i = from.getFloat(); i <= to.getFloat() ; i= i + step.getFloat() ) {
            _contents.add( new RealConcept(i));
            _length.setValue(_length.getLong() +1);            
        }
        updateNameCache();
        updateNavigationFans();
    }


    public void setIndex(IntegerConcept where) {
        setIndex(where.getLong());
    }


    public void add( Concept newValue) {
        _contents.add(newValue);
        _length.setValue(_contents.size());
        _navCache.add( extractNavName(newValue));
        updateNavigationFans();
    }


    public void addSeries( SeriesConcept series) {
        Vector newElemVect = series.getVector();
        for( int i = 0; i < newElemVect.size(); i++) 
            add((Concept) newElemVect.elementAt(i));
        updateNavigationFans();
        updateNameCache();
    }
    
    public LogicalConcept contains( Concept key ) {
        return new LogicalConcept( _contents.contains(key));
    }


    public IntegerConcept indexOf( Concept key ) {
        int idx = _contents.indexOf(key);
        if( idx == -1 ) return new IntegerConcept( idx);
        else return new IntegerConcept( idx +1);
    }


    public Concept remove() {
        int idx;
        if (_index.getInt() == 0) idx=0;
        else idx = _index.getInt()-1;
        Concept result =  (Concept) _contents.remove( idx );
        updateNavigationFans();
        updateNameCache();
        _length.setValue(_contents.size());
        return result;
    }


    public Concept remove( IntegerConcept ind ) {
        int idx;
        if (ind.getInt() == 0) idx=0;
        else idx = ind.getInt()-1;
        Concept result =  (Concept) _contents.remove( idx );
        updateNavigationFans();
        updateNameCache();
        _length.setValue(_contents.size());
        return result;
    }

        
    public Concept next() {
        if ( _hasNext.getValue() ) {
            _index.opIntegerINC();
            Concept result = (Concept)_contents.get( _index.getInt() -1 );
            updateNavigationFans();
            return result;
        } 
        else return new NilConcept();        
    }


    public Concept current() {
        if ( _index.getLong() >= 1 && _index.getLong() <= _contents.size()   ) {
            Concept result = (Concept)_contents.get( _index.getInt() -1 );
            return result;
        } 
        else return new NilConcept();        
    }


    public Concept previous() {
        if ( _hasPrevious.getValue() ) {
            _index.opIntegerDEC();
            Concept result = (Concept)_contents.get( _index.getInt() -1 );
            updateNavigationFans();
            return (Concept) result;
        } 
        else return new NilConcept();        
    }
    

    public void insert( Concept newValue ) {
        insert( _index, newValue);
    }


    public void insert(IntegerConcept position,  Concept newValue ) {
        try {
            _contents.add( position.getInt()-1, newValue );
            _navCache.add( position.getInt()-1, extractNavName(newValue));
            _length.setValue(_contents.size());
            updateNavigationFans();
        } catch (ArrayIndexOutOfBoundsException ex) {
            new ErrorConcept("error", "invalidIndex",
                             "Cannot insert to this position: " + position.getInt());
        }
    }


    public void sort( Concept sorter ) {
    }


    public StringConcept toTXT() {
        StringBuffer buffer = new StringBuffer();
        for( int cnt = 0 ; cnt < _contents.size() ; cnt ++) {
            buffer.append( ((Concept)_contents.elementAt(cnt)).toTXT().getValue());
            if( cnt < _contents.size() -1)
                buffer.append(mn8Interpreter.EOL);
        }

        if( !showEmpty().getValue() && buffer.toString().trim().equals("") ) {
            return new StringConcept();
        }

        return new StringConcept(buffer.toString());
    }



    public StringConcept toXML() {
        StringBuffer buffer = new StringBuffer();
        String current = null;
        for( int cnt = 0 ; cnt < _contents.size() ; cnt ++) {
            current = ((Concept)_contents.elementAt(cnt)).toXML().toString();
            buffer.append(current);
            if( ! Helper.endsWithEOL(current)
                && current.trim().startsWith("<")
                && current.trim().endsWith(">") 
                && cnt < _contents.size() -1) {
                buffer.append(mn8Interpreter.EOL);
            }
        }
        
        if( !showEmpty().getValue() && buffer.toString().trim().equals("") ) {
            return new StringConcept();
        }

        return new StringConcept(buffer.toString());
    }


    public Vector getVector() {
        return _contents;
    }

    
    public void setVector( Vector contents) {
        _contents = contents;        
        _length.setValue(_contents.size());
        updateNameCache();
        updateNavigationFans();
    }


    public Vector getValue() {
        return _contents;
    }


    public void setValue( Vector contents ){
        setVector(contents);
    }

    
    public ConceptDefinition getConceptDefinition() {
        return SeriesDefinition.getReference();
    }


    public boolean hasNext() {
        return _hasNext.getValue();
    }


    public boolean hasPrevious() {        
        return _hasPrevious.getValue();
    }


    public Concept elementAt( IntegerConcept position ) {
        if( getLength() < position.getInt() ) {
            return new NilConcept();
        }
        return elementAt(position.getInt());
        
    }


    public Concept elementAt( int position ) {
        return (Concept) _contents.elementAt(position - 1);
    }


    public int getLength() {
        return _contents.size();
    }


    public long getIndex() {
        return _index.getInt();
    }


    public void setIndex( long idx) {
        _index.setValue(idx);
        updateNavigationFans();
    }


    protected void updateNameCache() {
        _navCache.clear();
        String navName;
        for( int i = 0 ; i < _contents.size() ; i++) {
            _navCache.add( extractNavName((Concept)_contents.elementAt(i)));
        }

    }


    public boolean equals (Object obj) {
        if( obj instanceof org.media.mn8.concepts.SeriesConcept ) {
            if ( ( (SeriesConcept)obj).getVector().equals(_contents) ) {
                return true;
            }
	}
        return false;
    }

 
    public LogicalConcept equals( SeriesConcept target ) {
        return new LogicalConcept ( target.getVector().equals( _contents ) );
    }


    protected void updateNavigationFans(){
        if( _contents.size() - _index.getInt() > 0 )
            _hasNext.setValue(true);
        else {
            _hasNext.setValue(false);
            _index.setValue( _contents.size() );
        }
        
        if( _index.getInt() >= 1 )
            if( _index.getInt() > 1)
                _hasPrevious.setValue(true);
            else
                _hasPrevious.setValue(false);
        else {
            _hasPrevious.setValue(false);
            _index.setValue( 0 );
        }
    }


    public Concept getConceptElement( StringConcept name ) {
        Concept result = null;
        try {
            result = super.getConceptElement(name);
        } catch (Exception ex) {}

            if( result != null ) {
                return result;
            }
            int i = -1;
            i = _navCache.indexOf(new StringConcept(name));

            try {
                if( i == -1 ){
                    i= ( new Integer( name.toString() )).intValue();
                }
            } catch (NumberFormatException ex) {}            

            if( i <= _contents.size() && i > 0) {
                if( Helper.isElement( (Concept) _contents.elementAt(i - 1))) {
                    return (ElementConcept) _contents.elementAt(i - 1);
                } else {
                    return new TransparentElement( ((StringConcept) _navCache.elementAt(i - 1)).toString(), 
                                                   ((Concept)_contents.elementAt(i - 1)).getConceptLabel().toString(), 
                                                   (Concept) _contents.elementAt(i - 1), null);
                }
            }
            throw new ElementNotFoundException("Series", name.toString());
    }

    
    public LogicalConcept hasConceptElement( StringConcept name) {
        return new LogicalConcept( hasConceptElement(name.toString()));
    }


    public LogicalConcept hasPath( String fullpath ) {

        String firstItem = ConceptNavPatterns.getFirstItem( fullpath );
        String remainingItems = ConceptNavPatterns.getRemainingItems( fullpath );

        if( ConceptNavPatterns.isAbsolutePath( firstItem ) ) {

            if( ConceptNavPatterns.isAttribute( firstItem )) {
                if( !hasConceptAttribute(new StringConcept(ConceptNavPatterns.getAttributeName(firstItem))).getValue())
                return LogicalConcept.FALSE;
            } else { 
                StringConcept elmName = new StringConcept(ConceptNavPatterns.getElementName(firstItem));
                int idx = _navCache.indexOf(elmName);

                if ( idx != -1){
                    if( remainingItems == null ) {
                        return LogicalConcept.TRUE;
                    } else {
                        LogicalConcept tmpRes;
                        for( int a = 0 ; a < _navCache.size(); a++ ) {
                            if( _navCache.elementAt(a).equals(elmName)) {
                               tmpRes = ((Concept)_contents.elementAt(a)).hasPath( remainingItems );
                               if( tmpRes.getValue() ) {
                                   return LogicalConcept.TRUE;
                               }
                            }
                        }
                        return LogicalConcept.FALSE;
                    }
                }
                // nope it is not here
                return subElementHasPath( ConceptNavPatterns.getElementName(firstItem),                                              remainingItems);
            }
        } else {
            if( ConceptNavPatterns.isAttribute( firstItem )) {
                StringConcept attribute = new StringConcept(ConceptNavPatterns.getAttributeName(firstItem));
                if( hasConceptAttribute(attribute).getValue()) {
                    return LogicalConcept.TRUE;
                }                
                LogicalConcept result = LogicalConcept.FALSE;
                for(int j = 0 ; j < _contents.size() ; j++) {
                    result = ((Concept)_contents.elementAt(j)).hasPath(firstItem);
                    if( result.getValue() ) { 
                        return result;
                    }
                }
                return result;
            } else {
                StringConcept first = new StringConcept(firstItem);
                int idx = _navCache.indexOf(first);
                if(  idx != -1 ) {
                    if( remainingItems == null ) {
                        return LogicalConcept.TRUE;
                    } else {
                        LogicalConcept result = null;
                        for(int j = 0 ; j < _navCache.size() ; j++) {
                            if( ((StringConcept)_navCache.elementAt(j)).toString().equals(firstItem) ) {
                                result = ((Concept)_contents.elementAt(j)).hasPath(remainingItems);
                                if( result.getValue() ) { 
                                    return result;
                                }
                            }
                        }
                        return result;
                    }
                }else {
                    LogicalConcept result = null;
                    for(int j = 0 ; j < _contents.size() ; j++) {
                        result = ((Concept)_contents.elementAt(j)).hasPath(fullpath);
                        if( result.getValue() ) { 
                            return result;
                        }
                    }                       
                    return result;
                }                
            }
        }
        return LogicalConcept.FALSE;
    }


    public boolean hasConceptElement( String name ) {

        if( _navCache.contains( new StringConcept(name)) ){
            return true;
        }
        
        try {
           int i= ( new Integer( name.toString() )).intValue();
           if( i <= _contents.size() && i > 0)
               return true;
        } catch (Exception ex) {}
        return false;
    }


    public SeriesConcept getConceptElementFields() {
        //SeriesConcept result = super.getConceptElementFields();
        TypedSeries result = new TypedSeries();
        for( int i = 0; i < _contents.size(); i++) {
            result.add( new DummyFieldConcept( ((StringConcept) _navCache.elementAt(i)).toString(),
                                               new TransparentElement( ((StringConcept) _navCache.elementAt(i)).toString(), 
                                                                       ((Concept)_contents.elementAt(i)).getConceptLabel().toString(),
                                                                       (Concept)_contents.elementAt(i), null),
                                               false,
                                               ((Concept)_contents.elementAt(i)).getConceptLabel().toString(),
                                               ((Concept)_contents.elementAt(i)).getConceptType().toString()));
        }
        return result;
    }

    
    public FieldDefinition getConceptElementField( StringConcept elemenName ) {
        return getConceptElementField(elemenName.toString());
    }
    

    public FieldDefinition getConceptElementField( String elementName ) {
        FieldDefinition result = null;
        StringConcept elNameStrCon = new StringConcept(elementName);
        try {
            result = super.getConceptElementField( elementName );
        } catch (Exception ex) {}
             if( result != null )
                 return result;

            int i = -1;
            i = _navCache.indexOf(new StringConcept(elementName)) + 1;
            if( _navCache.indexOf(new StringConcept(elementName), i) != -1 ) {
                TypedSeries multiple = new TypedSeries();
                int j = 0;
                while( (j = _navCache.indexOf(elNameStrCon, j)) != -1 ) {
                    multiple.add((Concept)_contents.elementAt(j));
                    j++;
                }

                return new DummyFieldConcept(elementName,
                                             new TransparentElement( elementName, 
                                                                     elementName,
                                                                     multiple, null),
                                             false,
                                             elementName,
                                             "Series");
                
            }
            try {
                if( i == 0 )               
                    i= ( new Integer( elementName.toString() )).intValue();
            } catch (NumberFormatException ex) {}
            
            if( i <= _contents.size() && i > 0) {
                    return new DummyFieldConcept(((StringConcept) _navCache.elementAt(i - 1)).toString(),
                                                 Helper.isElement((Concept)_contents.elementAt(i - 1)) ? (Concept)_contents.elementAt(i - 1) :
                                                 new TransparentElement( ((StringConcept) _navCache.elementAt(i - 1)).toString(), 
                                                                         ((Concept)_contents.elementAt(i - 1)).getConceptLabel().toString(),
                                                                         (Concept)_contents.elementAt(i - 1), null),
                                                 false,
                                                 ((Concept)_contents.elementAt(i-1)).getConceptLabel().toString(),
                                                 ((Concept)_contents.elementAt(i-1)).getConceptType().toString());
            }
            throw new FieldNotFoundException("Series", elementName);
    }


    public SeriesConcept getConceptElements() {
        TypedSeries result = new TypedSeries();
        for( int i = 0; i < _contents.size(); i++) {
            result.add( new TransparentElement( ((StringConcept) _navCache.elementAt(i)).toString(), 
                                                ((Concept)_contents.elementAt(i)).getConceptLabel().toString(), 
                                                (Concept)_contents.elementAt(i), null));
        }
        return result;
    }

    
    public Object clone() {
        Vector newVect = new Vector( _contents.size());
        for( int i = 0 ; i < _contents.size() ; i ++)
            newVect.add( ((Concept)_contents.elementAt(i)).clone());
        return new SeriesConcept( newVect );
    }


    protected StringConcept extractNavName(Concept concept) {
        return Helper.extractName(concept);
    }

    
    public void loadContent(Concept from) {
        if( from instanceof SeriesConcept ) {
            Vector elements = ((SeriesConcept)from).getVector();
            for( int i = 0 ; i < elements.size(); i++ ) {
                if( elements.elementAt(i) instanceof TransparentElement ) {
                    add( ((TransparentElement) elements.elementAt(i)).getValue());
                } else {
                    add( (Concept) elements.elementAt(i));
                }
            }
        } else {
                add( from );
        }        
    }


    public SeriesConcept selectSIMEXP(String expr) {
        SeriesConcept result = new SeriesConcept();

        for( int i = 0 ; i < _contents.size() ; i++ ) {
            if( SimpleExpression.match( ((Concept)_contents.elementAt(i)).toTXT().toString().trim(), expr))
                result.add((Concept) _contents.elementAt(i));
        }
        return result;
    }



    public SeriesConcept searchDeeperForPath(SeriesConcept holder, String path) {

        for( int i =0; i < _contents.size(); i++) {
            holder = ((Concept)_contents.elementAt(i)).getConceptsAtPath( holder, path);
        }
        return holder;
    }

    
    // it is a multi pattern and we have the first but we have to see if it also has
    // the continuation of the path
    public LogicalConcept subElementHasPath(String elementName, String subPath) {

        int idx = _navCache.indexOf( new StringConcept(elementName));

        if( idx == -1 ) {
            return LogicalConcept.FALSE;
        }
        int cnt = idx;

        Concept con = (Concept)_contents.elementAt(idx);
        LogicalConcept result =  con.hasPath(subPath);

        if( con.hasPath(subPath).getValue() ) {
            return LogicalConcept.TRUE;
        }

        while( (idx = _navCache.indexOf(new StringConcept(elementName), cnt+1)) !=  -1 && cnt < _navCache.size()) {
            con = (Concept) _contents.elementAt(idx);
            result = con.hasPath(subPath);
            if( result.getValue()) {
                return LogicalConcept.TRUE;
            }
            cnt = idx;
        }
        
        return LogicalConcept.FALSE;
    }


    public boolean areThereMoreElemenents(String path ) {
        return _navCache.indexOf(new StringConcept(path)) != -1;
    }


    public Vector getMeAllElements( String path ) {
        int cnt = -1;
        int idx = -1;
        Concept con;
        Vector result = new Vector();
        while( (idx = _navCache.indexOf(new StringConcept(path), cnt+1)) !=  -1 && cnt < _navCache.size()) {
            con = (Concept) _contents.elementAt(idx);
            result.add(con);
            cnt = idx;
        }        
        
        return result;
    }

}


