/* 
 * $COPYRIGHT$
 * $Id: StringConcept.java,v 1.20 2002/04/28 13:11:37 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import org.media.mn8.util.LineWrapper;
import org.media.mn8.util.Normalizer;
import java.util.StringTokenizer;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.20 $ $Date: 2002/04/28 13:11:37 $
 */
public class StringConcept extends JavaConcept {
    
    private String _value = "";


    public StringConcept(){}
    

    public StringConcept( String value ) {
        if( value != null ) {
            _value = value;
        }
    }


    public StringConcept( StringConcept value) {
        new StringConcept( value.getValue() );
    }


    public StringConcept toStringConcept() {
        return this;
    }

    
    public StringConcept opPlus( IntegerConcept value ) {
        return new StringConcept( _value + value.getLong() );
    }


    public StringConcept opPlus( RealConcept value ) {
        return new StringConcept( _value + value.getFloat() );
    }


    public StringConcept opPlus( Concept value ) {
        return new StringConcept( _value + value.toTXT().toString() );
    }


    public StringConcept opPlus( LogicalConcept value ) {
        if ( value.getValue() )
            return new StringConcept( _value + "true" );
        else 
            return new StringConcept( _value + "false" );            
    }


    public LogicalConcept opEquals( StringConcept value ) {
        return new LogicalConcept( _value.equals(value.toString()) );            
    }


    public StringConcept opPlus( StringConcept value ) {
        return new StringConcept( _value + value.getValue());
    }


    public StringConcept opPlus( CharacterConcept value ) {
        return new StringConcept( _value + value.getValue());
    }


    public StringConcept opPlus( NilConcept value ) {
        return new StringConcept( _value );
    }

    
    public LogicalConcept opLT ( StringConcept value) {
        return new LogicalConcept( _value.compareTo(value.getValue()) < 0 );
    }


    public LogicalConcept opGT ( StringConcept value) {
        return new LogicalConcept( _value.compareTo(value.getValue()) > 0 );
    }


    public String getValue() {
        return _value;
    }

    
    public void setValue( String value ) {
        _value = value;
    }


    public void main ( SeriesConcept param ) {
        System.out.println(_value);
    }


    public ConceptDefinition getConceptDefinition() {
        return StringDefinition.getReference();
    }

    
    public SeriesConcept select( ExpressionConcept expression ) {
        return expression.getMatches( toTXT());
    }


    public LogicalConcept matches(ExpressionConcept expression) {
        return expression.matches(toTXT());
    }


    public LogicalConcept contains(ExpressionConcept expression) {
        return expression.contains(toTXT());
    }


    public LogicalConcept contains(StringConcept expression) {
        return new LogicalConcept( _value.indexOf(expression.toString()) != -1);
    }


    public int hashCode() {
        return _value.toLowerCase().hashCode();
    }

    
    public boolean equals (Object obj) {
        if( obj instanceof org.media.mn8.concepts.StringConcept ) 
            if (( (StringConcept)obj).getValue().equalsIgnoreCase(_value)) {
                return true;
            }
            else if( obj instanceof String ) {
                if (( (String)obj).equalsIgnoreCase(_value)) {
                    return true;
                }
            }

        return false;
    }


    public LogicalConcept hasPath( StringConcept path ) {
        return LogicalConcept.FALSE;
    }

    
    public LogicalConcept hasPath(String path) {
        return LogicalConcept.FALSE;
    }
    
    public SeriesConcept getConceptsAtPath( StringConcept path ) {
        return new SeriesConcept();
    }


    public String toString() {
        return _value;
    }

    
    public StringConcept toTXT() {
        return new StringConcept(toString());
    }

    
    public StringConcept toXML() {
        return new StringConcept( Normalizer.normalize( toString() ) );
    }


    public Object clone() {
        return new StringConcept( _value );
    }


    public CharacterConcept charAt( IntegerConcept at ) {
        return new CharacterConcept( _value.charAt( at.getInt() -1 ));
        
    }

    
    public LogicalConcept equalsIgnoreCase( StringConcept value ) {        
        return new LogicalConcept( _value.equalsIgnoreCase( value.toString() ));
    }


    public IntegerConcept compareTo( StringConcept value ) {
        return new IntegerConcept( _value.compareTo( value.toString() ));
    }


    public LogicalConcept startsWith( StringConcept value ) {
        return new LogicalConcept( _value.startsWith(value.toString()));
    }


    public LogicalConcept startsWith( StringConcept value, IntegerConcept offset ) {
        return new LogicalConcept( _value.startsWith(value.toString(), offset.getInt() - 1));
    }

    public LogicalConcept endsWith( StringConcept value) {
        return new LogicalConcept( _value.endsWith( value.toString()));
    }


    public IntegerConcept indexOf( StringConcept value ) {
        if( _value.indexOf( value.toString()) != -1 )
            return new IntegerConcept( _value.indexOf( value.toString()) +1);
        else 
            return new IntegerConcept( -1 );
    }


    public IntegerConcept indexOf( StringConcept value, IntegerConcept startFrom ) {
        int result =  _value.indexOf( value.toString(), startFrom.getInt() - 1);
        if( result == -1 )
            return new IntegerConcept(result);
        else
            return new IntegerConcept( result + 1);
    }


    public IntegerConcept lastIndexOf( StringConcept value) {
        int result = _value.lastIndexOf( value.toString() );
        if( result == -1 )
            return new IntegerConcept(result);
        else
            return new IntegerConcept( result + 1);
    }


    public IntegerConcept lastIndexOf( StringConcept value, IntegerConcept startFrom ) {
        int result = _value.lastIndexOf( value.toString(), startFrom.getInt() -1 );
        if( result == -1 )
            return new IntegerConcept(result);
        else
            return new IntegerConcept( result + 1);
    }


    public StringConcept substring( IntegerConcept from ) {
        return new StringConcept( _value.substring( from.getInt() -1));
    }


    public StringConcept substring( IntegerConcept from, IntegerConcept to ) {
        return new StringConcept( _value.substring( from.getInt() -1, to.getInt() - 1));
    }


    public StringConcept replace( CharacterConcept oldChar, CharacterConcept newChar ) {
        return new StringConcept( _value.replace( oldChar.getValue(), newChar.getValue() ));
    }

    
    public StringConcept trim() {
        return new StringConcept( _value.trim());
    }


    public StringConcept toLowerCase() {
        return new StringConcept( _value.toLowerCase());
    }

    
    public StringConcept toUpperCase() {
        return new StringConcept( _value.toUpperCase());
    }


    public SeriesConcept getLines() {
        return getTokens(new StringConcept("\r\n\r\n\f"));

    }
    

    public SeriesConcept getTokens(StringConcept delimiters) {
        StringTokenizer tokenizer = new StringTokenizer( _value, delimiters.toString());
        SeriesConcept result = new SeriesConcept();
        while( tokenizer.hasMoreTokens() ) {
            result.add( new StringConcept( tokenizer.nextToken() ));
        }
        return result;
    }


    public SeriesConcept getTokensWithDelimiters(StringConcept delimiters) {
        StringTokenizer tokenizer = new StringTokenizer( _value, delimiters.toString(), true);
        SeriesConcept result = new SeriesConcept();
        while( tokenizer.hasMoreTokens() ) {
            result.add( new StringConcept( tokenizer.nextToken() ));
        }
        return result;
    }


    public StringConcept wrap() {
        return new StringConcept( LineWrapper.alignText( _value, 72, "left"));
    }


    public StringConcept wrap(IntegerConcept colon) {
        return new StringConcept( LineWrapper.alignText( _value, colon.getInt(), "left"));
    }


    public StringConcept wrap(IntegerConcept colon, StringConcept align) {
        return new StringConcept( LineWrapper.alignText( _value, colon.getInt(), align.toString()));
    }


    public IntegerConcept getLength() {
        return new IntegerConcept( _value.length() );        
    }


    public void loadContent(Concept from ) {
        if( from instanceof StringConcept ) {
            _value = from.toString();
        } else if (from instanceof ElementConcept && ((ElementConcept)from).getValue() instanceof StringConcept) {
            _value = ((ElementConcept)from).getValue().toString();
        }
    }

}



