/* 
 * $COPYRIGHT$
 * $Id: StyleDefinition.java,v 1.2 2002/03/18 10:52:00 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import org.media.mn8.concepts.ErrorConcept;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import java.io.StringReader;
import java.io.StringWriter;

/*
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.2 $ $Date: 2002/03/18 10:52:00 $
 */

public class StyleDefinition extends ConceptConceptDefinition {
    
    static private SeriesConcept _inheritedConcepts = new SeriesConcept("Concept");
    static private StyleDefinition _reference = new StyleDefinition();
    
    private StyleDefinition() {
        super();
        try {            

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.StyleDefinition", true,
                                        "applyStyle:org.media.mn8.concepts.StringConcept,org.media.mn8.concepts.StringConcept",
                                        "applyStyle:String,String", "Style", "String"));

        }catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    

    public StringConcept applyStyle(StringConcept on, StringConcept style) {
        try {
            StringWriter _result = new StringWriter();
            StreamResult result = new StreamResult(_result);
            StreamSource source = new StreamSource(new StringReader(on.toString()));
            getTransformer(style).transform(source, result);
            return new StringConcept( _result.getBuffer().toString());
        } catch (TransformerConfigurationException tce) {            
            new ErrorConcept( "error", "transformerConfigError", tce.getMessage() );
            if( mn8RuntimeFlags.isVerbose() || mn8RuntimeFlags.isDebug() ) {
                Throwable x = tce;
                if (tce.getException() != null)
                    x = tce.getException();
                x.printStackTrace();
            }
        } catch (TransformerException te) {
            new ErrorConcept( "error", "transformerError", te.getMessage() );
            if( mn8RuntimeFlags.isVerbose() || mn8RuntimeFlags.isDebug() ) {
                Throwable x = te;
                if (te.getException() != null)
                    x = te.getException();
                x.printStackTrace();
            }
        }             
        return new StringConcept("");
    }


    public static Transformer getTransformer(StringConcept style) throws TransformerConfigurationException,
                                                                         TransformerException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        StreamSource stylesource = new StreamSource(new StringReader(style.toString()));
        return tFactory.newTransformer(stylesource);
    }


    public static ConceptDefinition getReference() {
        return _reference;
    }


    public Concept getConceptInstance() {
        return new StyleConcept();
    }


    public StringConcept getConceptLabel() {
        return new StringConcept("Style");
    }


    public StringConcept getConceptType() {
        return new StringConcept("Style");
    }


    public SeriesConcept getInheritedConcepts() {
        return _inheritedConcepts;
    }
}



