/* 
 * $COPYRIGHT$
 * $Id: SystemDefinition.java,v 1.5 2002/06/03 11:28:51 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.lang.Thread;
import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.5 $ $Date: 2002/06/03 11:28:51 $
 */

public class SystemDefinition extends ConceptConceptDefinition {
    
    static private SeriesConcept _inheritedConcepts = new SeriesConcept("Concept");
    static private SystemDefinition      _reference = new SystemDefinition();

    
    private SystemDefinition() {
        super();
        try {            
            // pause:Integer
            putMethod(new mn8JavaMethod("org.media.mn8.concepts.SystemDefinition", true,
                                        "pause:org.media.mn8.concepts.IntegerConcept",
                                        "pause:Integer", "System", null));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.SystemDefinition", true,
                                        "getLoadedConcepts:",
                                        "getLoadedConcepts:", "System", "Series"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.SystemDefinition", true,
                                        "getMN8Version:",
                                        "getMN8Version:", "System", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.SystemDefinition", true,
                                        "getMN8Info:",
                                        "getMN8Info:", "System", "String"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.SystemDefinition", true,
                                        "getCurrentTime:",
                                        "getCurrentTime:", "System", "Integer"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.SystemDefinition", true,
                                        "getFreeMemory:",
                                        "getFreeMemory:", "System", "Integer"));

            putMethod(new mn8JavaMethod("org.media.mn8.concepts.SystemDefinition", true,
                                        "getTotalMemory:",
                                        "getTotalMemory:", "System", "Integer"));

        }
        catch ( Exception ex ) {
            ex.printStackTrace();
        }
    }


    public SeriesConcept getLoadedConcepts() {
        Enumeration enum = ConceptRegister.getReference().listConceptsName();
        SeriesConcept result = new SeriesConcept();
        while( enum.hasMoreElements() ) {
            result.add( new StringConcept( (String)enum.nextElement()));
        }
        return result;
    }


    public final static IntegerConcept getCurrentTime() {
        return new IntegerConcept( System.currentTimeMillis());
    }


    public final static IntegerConcept getFreeMemory() {
        return new IntegerConcept( Runtime.getRuntime().freeMemory());
    }


    public final static IntegerConcept getTotalMemory() {
        return new IntegerConcept( Runtime.getRuntime().totalMemory());
    }


    public final static StringConcept getMN8Version() {
        return new StringConcept(mn8Interpreter.VERSION);
    }


    public final static StringConcept getMN8Info() {
        return new StringConcept(mn8Interpreter.IDENTIFIER);
    }


    public void pause( IntegerConcept time ) {
        try {
            if( time.getLong() == 0 ) {
                InputStream in = System.in;
                while( in.available() <= 0 ){
                    Thread.currentThread().yield();
                }
                while( in.available() > 0 )
                    in.read();
            }
            else {
                Thread.currentThread().sleep( time.getLong());
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    

    public static ConceptDefinition getReference() {
        return _reference;
    }


    public Concept getConceptInstance() {
        return new NilConcept();
    }


    public StringConcept getConceptLabel() {
        return new StringConcept("System");
    }


    public StringConcept getConceptType() {
        return new StringConcept("System");
    }


    public SeriesConcept getInheritedConcepts() {
        return _inheritedConcepts;
    }
}
