/* 
 * $COPYRIGHT$
 * $Id: TarConcept.java,v 1.2 2002/02/28 16:32:04 crow Exp $
 *
 * Date        Author            Changes 
 * FEB 25 2002 Szabo Csaba       Created
 */

package org.media.mn8.concepts;

import org.media.mn8.*;
import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;
import org.media.mn8.util.tar.*;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.2 $ $Date: 2002/02/28 16:32:04 $
 */

public class TarConcept extends JavaConcept {

    //Name of the Tar arhive
    private String     tarName;

    //Flag that determines if IO is GZIP - ed
    private boolean    compressed;


    public TarConcept() {
    }
    
    
    public TarConcept create( StringConcept path, LogicalConcept compress ) {
        tarName    = path.toString();
        compressed = compress.getValue();
        return this;
    }

    
    public void add( StringConcept path ) {
        OutputStream      os = null;
        InputStream       is = null;
        TarInputStream tarIn = null;
        boolean       exists = new File( tarName ).exists();

        try {
            if ( exists ) {
                new File( tarName ).renameTo( new File( tarName + ".tmp") );
                is = new FileInputStream( tarName + ".tmp" );
                if ( compressed ) is = new GZIPInputStream ( is );
                tarIn = new TarInputStream( is );
            }
            os = new FileOutputStream( tarName );
            if ( compressed ) os = new GZIPOutputStream( os );
            TarOutputStream tarOut = new TarOutputStream( os );
            if ( exists && tarIn != null ) readd( tarIn, tarOut );
            addDirectory( tarOut, path.toString() );
            tarOut.close();
            if ( exists ) {
                tarIn.close();
                new File( tarName + ".tmp" ).delete();
            }
        }
        catch ( IOException ioe ) {
            new ErrorConcept( "error", "openTarFailed", "Unable to open '" + tarName + "' tar file." );
        }
    }
    

    public void add( StringConcept text, StringConcept path ) {
        OutputStream      os = null;
        InputStream       is = null;
        TarInputStream tarIn = null;
        boolean       exists = new File( tarName ).exists();

        try {
            if ( exists ) {
                new File( tarName ).renameTo( new File( tarName + ".tmp") );
                is = new FileInputStream( tarName + ".tmp" );
                if ( compressed ) is = new GZIPInputStream ( is );
                tarIn = new TarInputStream( is );
            }
            os = new FileOutputStream( tarName );
            if ( compressed ) os = new GZIPOutputStream( os );
            TarOutputStream tarOut = new TarOutputStream( os );
            if ( exists && tarIn != null ) readd( tarIn, tarOut );
            addIS( tarOut, new ByteArrayInputStream( text.toString().getBytes() ), path.toString(), text.toString().length() );
            tarOut.close();
            if ( exists ) {
                tarIn.close();
                new File( tarName + ".tmp" ).delete();
            }
        }
        catch ( IOException ioe ) {
            new ErrorConcept( "error", "openTarFailed", "Unable to open '" + tarName + "' tar file." );
        }
    }
    

    public SeriesConcept list() {
        SeriesConcept result = new SeriesConcept();
        InputStream        f = null;
        TarEntry          te = null;
        String      longLink = null;

        try {
            f = new FileInputStream( tarName );
            if ( compressed ) f = new GZIPInputStream( f );
            TarInputStream tarIn = new TarInputStream( new BufferedInputStream( f ) );
            while ( ( te = tarIn.getNextEntry() ) != null ) {
                if ( te.getName().equals( "././@LongLink" ) ) {
                    longLink = getLongLink( tarIn );
                } 
                else {
                    if ( longLink == null )
                        result.add( new StringConcept( te.getName() ) );
                    else 
                        result.add( new StringConcept( longLink ) );
                    longLink = null;
                }
            }
            tarIn.close();
        } 
        catch ( IOException ioe ) {
            new ErrorConcept( "error", "openTarFailed", "Unable to open '" + tarName + "' tar file.\n" + ioe.getMessage() );
        }

        return result;
    }
    

    public StringConcept view( StringConcept path ) {
        StringBuffer result = new StringBuffer();
        boolean       found = false; 
        InputStream       f = null;
        TarEntry         te = null;
        String     fileName = null;
        byte[]        _buff = new byte[ 32 * 1024 ];

        try {
             f = new FileInputStream( tarName );
             if ( compressed ) f = new GZIPInputStream( f );
             TarInputStream in = new TarInputStream( new BufferedInputStream( f ) );

             while ( ( te = in.getNextEntry() ) != null && !found ) {
                 if ( te.getName().equals( "././@LongLink" ) ) {
                     fileName = getLongLink( in );
                 } 
                 else {
                     if ( fileName == null ) fileName = te.getName();
                     if ( path.toString().equals( fileName ) ) {
                         while ( true ) {
                             int nb = in.read( _buff );
                             if ( nb == -1 ) break;
                             result.append( new String( _buff, 0, nb ) );
                         } found = true;
                     } fileName = null;
                 } 
             } 
             in.close();
             if ( !found ) new ErrorConcept("warning", "pathNotFound", "File '" + path + "' doesn't exists.");
         } 
         catch ( IOException ioex ) {
             new ErrorConcept( "error", "readFailed", "Could not read '" + path.toString() + 
                               "' from  '" + tarName +"'.\n" + ioex.getMessage() ); 
         }
        
        return new StringConcept( result.toString() );
    }    


    public void delete( StringConcept path ) {
        OutputStream os = null;
        InputStream  is = null;
        TarEntry     te = null;
        byte[]    _buff = new byte[ 32 * 1024 ];
        String fileName = null;

        if ( !new File( tarName ).exists() ) { 
            new ErrorConcept( "error", "openTarFailed", "Unable to open '" + tarName + "' tar file." );
            return;
        }

        try {
            new File( tarName ).renameTo( new File( tarName + ".tmp" ) );
            is = new FileInputStream ( tarName + ".tmp" );
            os = new FileOutputStream( tarName ); 
            if ( compressed ) {
                is = new GZIPInputStream ( is );
                os = new GZIPOutputStream( os );
            }

            TarInputStream  tarIn  = new TarInputStream ( is );
            TarOutputStream tarOut = new TarOutputStream( os );
 
            while ( ( te = tarIn.getNextEntry() ) != null ) {
                if ( te.getName().equals( "././@LongLink" ) ) {
                    fileName = getLongLink( tarIn );
                    if ( fileName.equals( path.toString() ) ) {
                        tarIn.getNextEntry();
                    }
                    else {
                        tarOut.putNextEntry( te );
                        tarOut.write( fileName.getBytes(), 0, fileName.length() );
                        tarOut.closeEntry();
                    }
                }
                else if ( fileName != null || !te.getName().equals( path.toString() ) ) {
                    tarOut.putNextEntry( te );
                    while ( true ) {
                        int nb = tarIn.read( _buff, 0, _buff.length );
                        if ( nb == -1 ) break;
                        tarOut.write( _buff, 0, nb );
                    }
                    tarOut.closeEntry();
                    fileName = null;
                }  
            } 
            tarOut.closeEntry();
            tarOut.close();
            tarIn.close();
        } 
        catch ( IOException ioex ) {
            new ErrorConcept( "error", "removeFailed", "Could not remove '" + path.toString() + 
                              "' from '" + tarName +"'.\n" + ioex.getMessage() ); 
        }
    }

    
    public void extractAll( StringConcept dir ) {
        extract( dir.toString(), null );
    }


    public void extract( StringConcept dir, StringConcept path ) {
        extract( dir.toString(), path.toString() );
    }


    public ConceptDefinition getConceptDefinition() {
        return TarDefinition.getReference();
    }
    

    public Object clone() {
        return this;
    }
 

    private void extract( String directory, String path ) {
        InputStream   f = null;
        TarEntry     te = null;
        String fileName = null;

        try {
            if ( !directory.endsWith( File.separator ) ) 
                directory = directory.concat( File.separator );
            
            File currentDir = new File( directory );
            if ( !currentDir.exists() ) currentDir.mkdirs();
            f = new FileInputStream( tarName );
            if ( compressed ) f = new GZIPInputStream( f );
            TarInputStream in = new TarInputStream( new BufferedInputStream( f ) );

            while ( ( te = in.getNextEntry() ) != null ) {
                
                if ( te.getName().equals( "././@LongLink" ) ) {
                    fileName = getLongLink( in );
                    continue;
                } else if ( fileName == null ) fileName = te.getName();

                if ( path != null && !fileName.equals( path ) ) { 
                    fileName = null;
                    continue;
                }

                String   dest = directory.concat( fileName ); fileName = null;
                File filetemp = new File( dest );

                if ( filetemp.exists() ) 
                    new ErrorConcept( "warning", "fileExist", filetemp.toString() + " already exists");
                
                if ( !te.isDirectory() ) {
                    if ( filetemp.getPath() != null ) {
                        File _dir = new File( filetemp.getParent() );
                        if ( !_dir.exists() ) _dir.mkdirs();
                    }
                    filetemp.createNewFile();
                    FileOutputStream out = new FileOutputStream( new String( dest ) );
                    byte[] _buff = new byte[ 32 * 1024 ];
                    while ( true ) {
                        int nb = in.read( _buff );
                        if ( nb == -1 ) break;
                        out.write( _buff, 0, nb );
                    }
                    out.flush(); out.close();
                } else filetemp.mkdirs();
            }
            in.close();
        }
        catch ( FileNotFoundException e ) {
            new ErrorConcept( "error", "readFailed", "Could not read '" + tarName +"'.\n" + e.getMessage() ); 
        }
        catch ( IOException ioex ) {
            new ErrorConcept( "error", "readFailed", "Could not read '" + tarName +"'.\n" + ioex.getMessage() ); 
        }
    }
    

    private void addDirectory( TarOutputStream out, String path ) {
        File file = new File( path );
        
        if ( file.exists() ) {
            if ( file.isDirectory() ) {
                File [] fileList = file.listFiles();
                for ( int i = 0; i < fileList.length; i++ ) {
                    if ( fileList[i].isDirectory() ) 
                        addDirectory( out, fileList[i].getPath() );
                    else if ( fileList[i].isFile() )
                        addFile( out, fileList[i].getPath() );
                }
            } else if ( file.isFile() ) addFile( out, path );
        } else new ErrorConcept("error", "pathNotFound", "Directory or file '" + path + "' doesn't exists.");
    }


    private void addFile( TarOutputStream out, String path ) {
         try {
             addIS( out, new FileInputStream( path ), path, (int)new File( path ).length() );
         } 
         catch ( IOException ioex ) {
             new ErrorConcept( "warning", "addtoTarFailed", "Could not add '" + path + 
                               "' file to '" + tarName +"'.\n" + ioex.getMessage() ); 
         } 
    }


    private void addIS( TarOutputStream out, InputStream is, String path , int length ) {
        try {
            if ( path.startsWith( File.separator ) ) path = path.substring(1);
            if ( path.length() >= TarHeader.NAMELEN ) {
                TarEntry te = new TarEntry("././@LongLink", path.length() );
                out.putNextEntry( te );
                out.write( path.getBytes(), 0, path.length() );
                out.closeEntry();
                path = path.substring(0, TarHeader.NAMELEN);
            }

            BufferedInputStream in = new BufferedInputStream( is );
            TarEntry te = new TarEntry( path , length );
            
            out.putNextEntry( te );
            byte[] buf = new byte[ 32 * 1024 ];
            while ( true ) {
                int nr = in.read( buf, 0, buf.length );
                if ( nr == -1 ) break;
                out.write( buf, 0, nr );
            }
            in.close();
            out.closeEntry();
        } 
        catch ( IOException ioex ) {
            new ErrorConcept( "warning", "addtoTarFailed", "Could not add '" + path + 
                              "' file to '" + tarName +"'.\n" + ioex.getMessage() ); 
        } 
    }

    private void readd( TarInputStream in, TarOutputStream out ) {
        String fName = "";
        TarEntry  te = null;
        byte[] _buff = new byte[ 32 * 1024 ];
 
        try {
             while ( ( te = in.getNextEntry() ) != null ) {
                 out.putNextEntry( te );
                 while ( true ) {
                     int nb = in.read( _buff, 0, _buff.length );
                     if ( nb == -1 ) break;
                     out.write( _buff, 0, nb );
                 }
                 out.closeEntry(); 
             } 
        } 
        catch ( IOException ioex ) {
            new ErrorConcept( "warning", "addtoTarFailed", "Could not readd '" + fName + 
                              "' file to '" + tarName +"'.\n" + ioex.getMessage() ); 
        }
    }

    
    private String getLongLink( TarInputStream tarIn ) throws IOException {
        StringBuffer fileName = new StringBuffer();
        byte[]          _buff = new byte[ 32 * 1024 ];
         
        while ( true ) {
            int nb = tarIn.read( _buff, 0, _buff.length );
            if ( nb == -1 ) break;
            fileName.append( new String( _buff, 0, nb ) );
        }
        
        return fileName.toString().trim();
    }
}
