/* 
 * $COPYRIGHT$
 * $Id: TypedSeries.java,v 1.7 2002/05/02 22:11:35 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import org.media.mn8.util.SimpleExpression;

import java.util.Vector;
import java.util.Enumeration;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.7 $ $Date: 2002/05/02 22:11:35 $
 */
public class TypedSeries extends SeriesConcept {
    protected StringConcept _type = new StringConcept("Nil");
    protected StringConcept _label = null;

    public TypedSeries() {
        LogicalConcept true_value = new LogicalConcept(true);
        length.setHidden(true_value);
        index.setHidden(true_value);
        hasPrevious.setHidden(true_value);
        hasNext.setHidden(true_value);
    }


    public TypedSeries( String type, String typeLabel ) {
        super();
        _type = new StringConcept(type);
        if( typeLabel != null ) {
            _label = new StringConcept(typeLabel);
        }
    }

    public TypedSeries( Vector values ) {
        super(values);
    }


    public TypedSeries( Vector values, String type ) {
        super(values);
        _type = new StringConcept(type);
    }


    public TypedSeries create(StringConcept type) {
        _type = type;
        return this;
    }


    public TypedSeries create( StringConcept type, SeriesConcept values) {
        addSeries(values);
        _type = type;
        return this;
    }


    public Concept createNewEntry() {
        Concept instance = mn8Loader.instantiateConcept(_type.toString());
        if( _label != null ) instance.setConceptLabel( _label );
        super.add(instance);
        return lastEntry();
    }


    public void addEntry(Concept value) {
        createNewEntry();
        ElementConcept last = (ElementConcept) _contents.elementAt(_contents.size()-1);
        last.setValue( value );
    }


    public Concept lastEntry() {
        if( _contents.size() <= 0 ) {
            return new NilConcept();
        }
        return elementAt( _contents.size());
    }


    public void add( Concept value ) {
        if( !_type.toString().equalsIgnoreCase("nil") && _label != null ) {
            value.setConceptLabel(_label);
            super.add(value);
        } else {
            super.add(value);
        }
    }

    
    public void setType(StringConcept type) {
        _type = type;
    }
        

    public StringConcept getType() {
        return _type;
    }

    
    public Object clone() {
        return new TypedSeries((Vector)_contents.clone(), _type.toString());
    }


    public void loadContent(Concept from) {
        if( mn8RuntimeFlags.isDebug() ) {
            System.err.println("--> in typed:" + Helper.getConceptInfo(this) + "->"+ Helper.getConceptInfo(from));
        }
        if( from instanceof SeriesConcept ) {
            ConceptDefinition def =  mn8Loader.getConceptDefinition(_type.toString());
            Vector elements = ((SeriesConcept)from).getVector();
            Concept el = null;

            for( int i = 0 ; i < elements.size(); i++ ) {
                el = (Concept) elements.elementAt(i);
                if( el.getConceptType().toString().equalsIgnoreCase(_type.toString())) {
                    add(el);
                } else {
                    if( el instanceof Element ) {
                        if( def.getConceptLabel().toString().equalsIgnoreCase(el.getConceptLabel().toString()) 
                            || _type.toString().equalsIgnoreCase("nil")
                            || _type.toString().equalsIgnoreCase("element") ) {
                            createNewEntry();
                            ((Concept)_contents.elementAt(_contents.size()-1)).loadContent(from);
                            updateNameCache();
                            updateNavigationFans();
                        }
                    }
                }
            }
        } else {
            ConceptDefinition def =  mn8Loader.getConceptDefinition(_type.toString());
            if( getConceptLabel().toString().equalsIgnoreCase(from.getConceptLabel().toString()) 
                || _type.toString().equalsIgnoreCase("nil")
                || _type.toString().equalsIgnoreCase("element") ) {
                createNewEntry();
                ((Concept)_contents.elementAt(_contents.size()-1)).loadContent(from);
                updateNameCache();
                updateNavigationFans();
            }
        }        
    }


    public ConceptDefinition getConceptDefinition() {
        return TypedSeriesDefinition.getReference();
    }

}


