/* 
 * $COPYRIGHT$
 * $Id: URI.java,v 1.8 2002/05/20 10:59:13 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.io.IOException;
import javax.naming.NamingException;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.8 $ $Date: 2002/05/20 10:59:13 $
 */
public class URI extends JavaConcept {
    
    public AttributeConcept uri = new AttributeConcept("uri", "uri", new NilConcept());
    public AttributeConcept contentType = new AttributeConcept("contentType", "contentType", new NilConcept());
    public AttributeConcept length = new AttributeConcept("length", "length", new IntegerConcept(-1));
 
    public ElementConcept options = new ElementConcept("opt", "opt", new MapConcept(), null);
    

    public URI( ){
    }


    public URI( StringConcept url ) {
        uri.setAttributeValue(url);
        init(url);
    }


    protected void init( StringConcept url ) {
        try {
            URL _url = new URL( url.toString());
            URLConnection uc = _url.openConnection();
            length.setAttributeValue( new IntegerConcept( uc.getContentLength()));
            String _contentType = uc.getContentType();
            if( _contentType != null ) {
                contentType.setAttributeValue( new StringConcept( _contentType));
            }
        } catch (MalformedURLException ex) {
            
        } catch (IOException ioex ) {
            
        }
    }
    

    public URI create( StringConcept url ) {
        uri.setAttributeValue(url);
        init(url);
        return this;
    }


    public ConceptDefinition getConceptDefinition() {
        return URIDefinition.getReference();
    }


    public void setUri(StringConcept url) {
        length.setAttributeValue( new IntegerConcept(-1));
        contentType.setAttributeValue(new NilConcept());
        uri.setAttributeValue(url);
        init(url);
    }

    
    public Concept getURIHandler() {
        int endpos = -1;
        String protocol = "file";

        if ( uri.getAttributeType().toLowerCase().equals("string") ) {
            if ( ( endpos = uri.getAttributeValue().toString().indexOf(":") ) != -1 )
                protocol = uri.getAttributeValue().toString().substring(0, endpos); 
            else {
                String _uri =  uri.getAttributeValue().toString().trim();
                if ( !_uri.startsWith("/") ) _uri = "/" + _uri;
                uri.setAttributeValue( new StringConcept ( "file:" + _uri ) );
            }
            
            try {
                Object uhandler = NamingManager.getReference().lookup("mem:/system/protocols/" + protocol);
                if( uhandler != null) {
                    String handlerConceptType= ((StringConcept)uhandler).toString();
                    URIHandlerDefinition handlerDef = (URIHandlerDefinition) mn8Loader.getConceptDefinition(handlerConceptType);
                    if( handlerDef.acceptsURI((StringConcept)uri.getAttributeValue()).getValue()) {
                        return ((URIHandler)handlerDef.getConceptInstance()).create((StringConcept)uri.getAttributeValue());
                    }
                }                   
            } catch ( NamingException nex ) {
                // we do nothing here what is important is env not mem.
            }
            
            try {
                Object uhandler = null;
                try {
                    uhandler = NamingManager.getReference().lookup("env:/system/protocols/" + protocol);
                } catch (NamingException ex) {
                    // we don't know that type of protocol, but maybe it is just file

                    if( ((FileHandlerDefinition)FileHandlerDefinition.getReference()).acceptsURI( (StringConcept)uri.getAttributeValue() ).getValue()) {
                        // it's ok 
                        uhandler = FileHandlerDefinition.getReference().getConceptType();
                    } else {
                        throw new NamingException(ex.getMessage());
                    }
                }
                if( uhandler != null) {
                    String handlerConceptType= ((StringConcept)uhandler).toString();
                    ConceptDefinition handlerDef = (ConceptDefinition) mn8Loader.getConceptDefinition(handlerConceptType);
                    StringConcept methodName = new StringConcept("acceptsURI:String");
                    if( handlerDef.hasConceptMethod(methodName).getValue()) {
                        mn8Method method= handlerDef.getConceptMethod(methodName);
                        SeriesConcept param = new SeriesConcept();
                        param.add((StringConcept)uri.getAttributeValue());
                        Concept concept = handlerDef.getConceptInstance();
                        Concept result = method.invoke(concept, param);  

                        if( result instanceof LogicalConcept && ((LogicalConcept)result).getValue()) {
                            method = handlerDef.getConceptConstructor(new StringConcept("create:String"));
                            result =  method.invoke(concept, param);
                            return result;
                        } else {
                            return new NilConcept();
                        }
                    }
                }                   
            } catch (NamingException nex) {
                throw new mn8NoURIHandler(protocol);
            }
        }
        throw new mn8NoURIHandler(protocol);
    }

    
    public Object clone() {
        URI newuri = new URI();
        newuri.uri = (AttributeConcept)this.uri.clone();
        newuri.contentType = (AttributeConcept)this.contentType.clone();
        newuri.length = (AttributeConcept)this.length.clone();
        newuri.options = (ElementConcept)this.options.clone();
        return newuri;
    }
}
