/*
 * $COPYRIGHT$
 * $Id: XMLRPCClientConcept.java,v 1.8 2002/05/02 22:11:00 neuro Exp $
 *
 * Date          Author            Changes
 * IAN 22 2002   Szabo Csaba       Created
 */
       
package org.media.mn8.concepts;

import org.media.mn8.*;
import java.net.URL;
import java.util.Vector;
import java.util.Date;
import java.util.Hashtable;
import java.util.Enumeration;
import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.8 $ $Date: 2002/05/02 22:11:00 $
 */

public class XMLRPCClientConcept extends JavaConcept {

    private static final String DEFAULT_HOST = "localhost";
    private static final int    DEFAULT_PORT = 10300;
    private static final String DEFAULT_USER = "guest";
    private static final String DEFAULT_PWD  = "guest";
    
    protected URL          _url;
    protected XmlRpcClient _client;
    private StringConcept DEFAULT_URL = new StringConcept( "xmlrpc://" + DEFAULT_USER + ":" + DEFAULT_PWD + 
							  "@" + DEFAULT_HOST + ":" + DEFAULT_PORT + "/" );

    
    public XMLRPCClientConcept() {
        initialize( DEFAULT_URL );
    }


    public XMLRPCClientConcept( StringConcept url ) {
        initialize( url );
    }
   				 
		 
    public XMLRPCClientConcept create( StringConcept url ) {
        initialize( url );
        return this;
    }
 
    
    protected boolean initialize( StringConcept url ) {
        URL givenURL = null;
        
        try {
            givenURL = new URL( url.toString() );
            if ( givenURL == null || givenURL.getProtocol().equalsIgnoreCase("xmlrpc") ) {
                if ( givenURL != null ) {
                    _url = givenURL;
                    String _host = givenURL.getHost();
                    int    _port = givenURL.getPort();
                    String    ui = givenURL.getUserInfo();
                    String _user = null;
                    String  _pwd = null;
		    
                    if ( ui != null ) 
                        if ( ui.indexOf(":") != -1 ) {
                            _user = ui.substring( 0, ui.indexOf(":") );
                            _pwd  = ui.substring( ui.indexOf(":")+1 );
                        }
                        else _user = ui;
                    
                    if ( _host == null || _host.length() < 1 ) _host = DEFAULT_HOST;
                    if ( _port == -1 )  _port = DEFAULT_PORT;
                    if ( _user != null && _user.length() < 1 ) _user = null;
                    if ( _pwd != null &&  _pwd.length()  < 1 )  _pwd = null;
                    
                    _client = new XmlRpcClient( "http://" + _host + ":" + _port + "/");
                    
                    if ( _user != null && _pwd != null ) _client.setBasicAuthentication( _user, _pwd ); 
                }
            } else if ( givenURL != null ) {
                _client = new XmlRpcClient(givenURL.toString());
            } else {
                new ErrorConcept( "error", "unsupportedProtocol", "'" + givenURL.getProtocol() + 
                                  "' protocol not supported by XMLRPCClient" );	
            }
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "badURL", ex.getMessage() );
            return false;
        }
        
        return true;
    }
    
    
    public Concept executeCmd( StringConcept method, SeriesConcept params ) {
        try {
            Vector _param = new Vector();
            for ( int i = 0; i < params.getValue().size(); i++ ) {

                if( params.getValue().elementAt(i) instanceof StringConcept ) {
                    StringConcept param = ( StringConcept )params.getValue().elementAt(i);
                    _param.add ( param.toString() );
                    continue;
                }
                if ( params.getValue().elementAt(i) instanceof LogicalConcept ) {
                    _param.add( new Boolean(((LogicalConcept)params.getValue().elementAt(i)).getValue()));
                    continue;
                }
                if ( params.getValue().elementAt(i) instanceof IntegerConcept ) {
                    _param.add( new Integer(((IntegerConcept)params.getValue().elementAt(i)).getInt()));
                    continue;
                }
                if ( params.getValue().elementAt(i) instanceof RealConcept ) {
                    Float fl = new Float( ((RealConcept)params.getValue().elementAt(i)).getFloat() );
                    _param.add( new Double( fl.doubleValue() )) ;
                    continue;
                }
                if ( params.getValue().elementAt(i) instanceof DateConcept ) {
                    _param.add( new Date(((DateConcept)params.getValue().elementAt(i)).getValue()));
                    continue;
                }
                if ( params.getValue().elementAt(i) instanceof MapConcept ) {
                    Hashtable htable = new Hashtable();
                    MapConcept map = (MapConcept)params.getValue().elementAt(i);
                    for ( int j = 0; j < map.getLength(); j++ ) {
                        htable.put( map.getKeyVector().elementAt(j), map.getValuesVector().elementAt(j) );
                    }		    
                    _param.add( htable );
                    continue;
                }
                if ( params.getValue().elementAt(i) instanceof SeriesConcept ) {
                    _param.add( ((SeriesConcept)params.getValue().elementAt(i)).getValue() );
                    continue;
                }
                
            }

            Object obj = _client.execute( method.toString(), _param );
            return getConcept ( obj );
        }
        catch ( XmlRpcException ex ) {
            new ErrorConcept( "error", "executeFailed", "XMLRPC -> " + ex.getMessage() );
            return new StringConcept( "" );
        }
        catch ( Exception _ex ) {
            new ErrorConcept( "error", "executeFailed", _ex.getMessage() );
            return new StringConcept( "" );
        }
    }
    
    
    public Concept getConcept ( Object obj ) {
        if ( obj instanceof String )    return getStringConcept( obj );
        if ( obj instanceof Integer )   return getIntegerConcept( obj );
        if ( obj instanceof Boolean )   return getLogicalConcept( obj ); 
        if ( obj instanceof Double )    return getRealConcept( obj ); 
        if ( obj instanceof Date )      return getDateConcept( obj );
        if ( obj instanceof Hashtable ) return getMapConcept( obj );
        if ( obj instanceof Vector )    return getSeriesConcept( obj );
        if ( obj.getClass().isArray() ) {
            byte[] data = (byte[]) obj;
            return new StringConcept( new String( data));
        }
        return new StringConcept( (String)obj );
    }
    

    public ConceptDefinition getConceptDefinition() {
        return XMLRPCClientConceptDefinition.getReference();
    }
    
    
    public Object clone() {
        return this;
    }


    private StringConcept getStringConcept( Object obj ) {
        return new StringConcept( (String)obj );
    }


    private IntegerConcept getIntegerConcept( Object obj ) {
        return new IntegerConcept( ((Integer)obj).longValue() );
    } 

    
    private LogicalConcept getLogicalConcept( Object obj ) {
        return new LogicalConcept( ((Boolean)obj).booleanValue() );
    } 

    
    private RealConcept getRealConcept( Object obj ) {
        return new RealConcept( ((Double)obj).floatValue() ); 
    }


    private DateConcept getDateConcept( Object obj ) {
        return new DateConcept( new IntegerConcept( ((Date)obj).getTime() ) );
    }

    
    private MapConcept getMapConcept( Object obj ) {
        return new MapConcept( getVector( ((Hashtable)obj).keys() ), 
                               getVector( ((Hashtable)obj).elements() ) );
    }


    private SeriesConcept getSeriesConcept( Object obj ) {
        return new SeriesConcept( getVector( ((Vector)obj).elements() ) );
    } 


    private Vector getVector( Enumeration en ) {
        Vector result = new Vector();
        
        for ( Enumeration e = en; e.hasMoreElements(); ) {
            Object _obj = e.nextElement();
            
            if ( _obj instanceof String ) {
                result.add( getStringConcept( _obj ) );
                continue;
            }
            if ( _obj instanceof Integer ) {
                result.add( getIntegerConcept( _obj ) );
                continue;
            }
            if ( _obj instanceof Boolean ) {
                result.add( getLogicalConcept( _obj ) );
                continue;
            }
            if ( _obj instanceof Double ) {
                result.add( getRealConcept( _obj ) );
                continue;
            }
            if ( _obj instanceof Date ) {
                result.add( getDateConcept( _obj ) );
                continue;
            }
            if ( _obj instanceof Hashtable ) {
                result.add( getMapConcept( _obj ) );
                continue;
            }
            if ( _obj instanceof Vector ) {
                result.add( getSeriesConcept( _obj ) );
                continue;
            }
            result.add( getStringConcept ( _obj ) );
        } 
        
        return result;
    }
}
