/* 
 * $COPYRIGHT$
 * $Id: XMLRPCHandler.java,v 1.4 2002/02/21 13:58:32 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 23 2002 Szabo Csaba       Created
 */
package org.media.mn8.concepts;


import org.media.mn8.*;
import org.media.mn8.event.*;
import java.util.Vector;
import java.util.StringTokenizer;
import java.net.URL;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.4 $ $Date: 2002/02/21 13:58:32 $
 */

public class XMLRPCHandler extends URIHandler {

    private String   _url = null;
    private String _uname = null;
    private String   _pwd = null;
    private MapConcept _options;
    private XMLRPCClientConcept _client;

    public XMLRPCHandler( ){
    }


    public XMLRPCHandler( StringConcept url ) {
        _url = url.toString();
    }
  

    public URIHandler create( StringConcept url ) {
       _url = url.toString();
       return this;
    }

    
    public LogicalConcept acceptsURI( StringConcept uri ) {
        return ( (URIHandlerDefinition)getConceptDefinition() ).acceptsURI( uri );
    }
    
    
    public LogicalConcept initialize( MapConcept options ){
        URL givenURL = null;

        _options = options;
        
        if( options.containsKey(new StringConcept("user")).getValue() ) {
            _uname = options.getValue(new StringConcept("user")).toTXT().toString();
        }

        if( options.containsKey(new StringConcept("pwd")).getValue() ) {
            _pwd = options.getValue(new StringConcept("pwd")).toTXT().toString();
        }
	 
        try {
            givenURL = new URL( _url );
            if ( givenURL != null && givenURL.getProtocol().equalsIgnoreCase("xmlrpc") ) {
                if ( _uname != null && _uname.trim().length() > 0 && _pwd != null && _pwd.trim().length() > 0 ) {
                    _url  = givenURL.getProtocol() + "://";
                    _url += _uname + ":" + _pwd + "@";
                    if ( givenURL.getHost()  != null ) _url += givenURL.getHost();
                    if ( givenURL.getPort()  != -1   ) _url += ":" + givenURL.getPort();
                    if ( givenURL.getPath()  != null ) _url += givenURL.getPath();
                    if ( givenURL.getQuery() != null ) _url += "?" + givenURL.getQuery();
                    if ( givenURL.getRef()   != null ) _url += "#" + givenURL.getRef();
                }
            }
        }
        catch ( Exception ex ) {
            new ErrorConcept( "error", "badURL", ex.getMessage() );
            return new LogicalConcept( false );
        }
        
        return new LogicalConcept( true );
    }


    public LogicalConcept closeConnection() {
        return new LogicalConcept( true );
    }

    
    public LogicalConcept openConnection() {
        _client = new XMLRPCClientConcept( new StringConcept( _url ) );
        return new LogicalConcept( true );
    }

 
    public Concept getContent() {
        String    methodName = "";
        SeriesConcept params = new SeriesConcept();

        try {
            URL     url = new URL( _url );
            String file = url.getFile();
            String   qu = url.getQuery();
            
            if ( file.equals("/") || file.trim().length() < 1 ) { 
                new ErrorConcept( "error", "missingMethod", "Missing method parameter from URL" );
                return new StringConcept("");
            } 
            else {
                if ( file.indexOf("?") < 0 ) methodName = file.substring(1);
                else methodName = file.substring( 1, file.indexOf("?") );
            }
            
            if ( methodName.indexOf(".") < 0 ) {
                new ErrorConcept( "error", "badMathod", "The method name must contain the handler eg.:\n" +
                                  "xmlrpc://user:password@host:port/handler.method?param1=...&param2=...&..." );
                return new StringConcept("");
            }
            
            if ( qu == null || qu.indexOf("=") < 0 ) {
                new ErrorConcept( "error", "badQuery", "The query structure is: ?name1=parameter1&name2=parameter2&...\n" );
                return new StringConcept("");
            }
            
            if ( qu.trim().length() != 0 ) {
                StringTokenizer pt = new StringTokenizer( qu, "&" );
                String t;
                
                while ( pt.hasMoreTokens() ) {
                    t = pt.nextToken();
                    params.add( new StringConcept( t.substring( t.indexOf("=") + 1 )  ) );
                }
            }
            
            return _client.executeCmd( new StringConcept( methodName) , params );
        } catch ( Exception ex ) {
            new ErrorConcept( "error", "badURL", ex.getMessage() );
            return new StringConcept( "" );
        }
    }
    
 
    public LogicalConcept setContent( Concept value ) {
        return new LogicalConcept( false );
    }


    public ConceptDefinition getConceptDefinition() {
        return XMLRPCHandlerDefinition.getReference();
    }


    public Object clone() {        
        if( _url != null )
            return new XMLRPCHandler( new StringConcept( _url ) );
        else
            return new URIHandler();
    }
}
