/* 
 * $COPYRIGHT$
 * $Id: ZipConcept.java,v 1.7 2002/03/19 23:57:01 neuro Exp $
 *
 * Date        Author            Changes 
 * FEB 21 2002 Szabo Csaba       Created
 */

package org.media.mn8.concepts;

import org.media.mn8.*;
import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.zip.*;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.7 $ $Date: 2002/03/19 23:57:01 $
 */

public class ZipConcept extends JavaConcept {

    // Name of the Zip
    private String zipName;
    // Hastable of the ZipEntries of the jar (ZipEntry-Name, ZipEntry)
    private Hashtable  zip;

    private ZipOutputStream zipOut;
    private ZipFile         zipIn;


    public ZipConcept() {
    }
    
    
    public ZipConcept create( StringConcept path ) {
        zipName = path.toString();
        zip     = new Hashtable();
        
        if ( new File( zipName ).exists() ) {
            readZip();
        } else {
            // maybe is an url
            try {
                URL url = new URL( zipName );
		zipName = url.toString();
                readZip();
            } catch (MalformedURLException mex) {
                new ErrorConcept( "error", "openZipFailed", 
                                  "Unable to open '" + zipName + "' zip file." + 
                                  "\nThe error seems to be:" + mex.getMessage());
            }
        }
        return this;
    }

    
    public void add( StringConcept path ) {
        boolean exists = new File( zipName ).exists();

        try {
            if ( exists ) { 
                new File( zipName ).renameTo( new File( zipName+".tmp" ) );
                zipIn = new ZipFile( zipName + ".tmp" );
            }
            zipOut = new ZipOutputStream( new FileOutputStream( zipName ) );
            if ( exists ) readd();
            addDirectory( path.toString() );
            zipOut.close();
            if ( exists ) {
                zipIn.close();
                new File( zipName + ".tmp" ).delete();
            }
        }
        catch ( IOException ioe ) {
            new ErrorConcept( "error", "openZipFailed", "Unable to open '" + zipName + "' zip file." );
        }
    }
    

    public void add( StringConcept text, StringConcept path ) {
        boolean exists = new File( zipName ).exists();

        try {
            if ( exists ) {
                new File( zipName ).renameTo( new File( zipName + ".tmp" ) );
                zipIn = new ZipFile( zipName + ".tmp" );
            }
            zipOut = new ZipOutputStream( new FileOutputStream( zipName ) );
            if ( exists ) readd();
            addIS( new ByteArrayInputStream( text.toString().getBytes() ), path.toString() );
            zipOut.close(); 
            if ( exists ) {
                zipIn.close(); 
                new File( zipName + ".tmp" ).delete();
            }
        }        
        catch ( IOException ioe ) {
            new ErrorConcept( "error", "openZipFailed", "Unable to open '" + zipName + "' zip file." );
        }
    }
    

    public SeriesConcept list() {
        SeriesConcept result = new SeriesConcept();
        for ( Enumeration e = zip.keys(); e.hasMoreElements(); )
            result.add( new StringConcept( (String)e.nextElement() ) );
        return result;
    }
    

    public StringConcept view( StringConcept path ) {
        try {
            ZipInputStream zin = new ZipInputStream( 
                                     new BufferedInputStream( new FileInputStream ( zipName ) ) );
            ZipEntry ze;
            while ( ( ze = zin.getNextEntry() ) != null ) {
                if ( ze.getName().equals( path.toString() ) ) {
                    StringBuffer result = new StringBuffer();
                    int c;
        
                    while ( ( c = zin.read() ) != -1 ) result.append( (char)c );
                    zin.close();
                    return new StringConcept( result.toString() );
                }
            } zin.close();
            new ErrorConcept("error", "pathNotFound", "File '" + path + "' doesn't exists.");
        } 
        catch ( IOException ioex ) {
            new ErrorConcept( "error", "readFailed", "Could not read '" + path.toString() + 
                              "' from  '" + zipName +"'.\n" + ioex.getMessage() ); 
        }
        
        return new StringConcept( "" );
    }    


    public void delete( StringConcept path ) {
        int c;

        if ( !new File( zipName ).exists() ) { 
            new ErrorConcept( "error", "openZipFailed", "Unable to open '" + zipName + "' zip file." );
            return;
        }

        try {
            new File( zipName ).renameTo( new File( zipName+".tmp" ) );
            zipIn  = new ZipFile( zipName + ".tmp" );
            zipOut = new ZipOutputStream( new FileOutputStream( zipName ) );
            for ( Enumeration e = zipIn.entries(); e.hasMoreElements(); ) {
                ZipEntry ze = (ZipEntry)e.nextElement();
                if ( !ze.getName().equals( path.toString() ) ) {
                    BufferedReader in = new BufferedReader( 
                                            new InputStreamReader( zipIn.getInputStream( ze ) ) );
                    zipOut.putNextEntry( ze );
                    while ( ( c = in.read() ) != -1 ) zipOut.write ( c );
                    in.close();
                } else zip.remove( path.toString() ); 
            } 
            zipOut.close();
            zipIn.close();
        } 
        catch ( IOException ioex ) {
            new ErrorConcept( "error", "removeFailed", "Could not remove '" + path.toString() + 
                              "' from '" + zipName +"'.\n" + ioex.getMessage() ); 
        }
    }

    
    public void extractAll( StringConcept dir ) {
        extract( dir.toString(), null );
    }


    public void extract( StringConcept dir, StringConcept path ) {
        extract( dir.toString(), path.toString() );
    }


    public ConceptDefinition getConceptDefinition() {
        return ZipDefinition.getReference();
    }
    

    public Object clone() {
        return this;
    }
 

    private void extract( String directory, String path ) {
        String dst = null;
        try {
            if ( !directory.endsWith( File.separator ) ) 
                directory = directory.concat( File.separator );
            
            File currentDir = new File( directory );
            if ( !currentDir.exists() ) currentDir.mkdirs();

            ZipInputStream zin = new ZipInputStream( new BufferedInputStream( getZipContentAsStream() ) );
            ZipEntry ze;

            while ( ( ze = zin.getNextEntry() ) != null ) {
                
                if ( path != null && !ze.getName().equals( path ) ) continue;

                String   dest = Helper.fileNameFix(directory.concat( ze.getName() ));
                dst = dest; //for debugging purposes only
                File filetemp = new File( dest );

                if ( !ze.isDirectory() ) {
                    if ( filetemp.getPath() != null ) {
                        File _dir = new File( filetemp.getParent() );
                        if ( !_dir.exists() ) _dir.mkdirs();
                    }
                    filetemp.createNewFile();
                    
                    FileOutputStream out = new FileOutputStream(filetemp.toString());

                    int c;
                    while ( (c = zin.read() ) != -1) out.write(c); 
                    out.flush(); out.close();
     
                } else filetemp.mkdirs();
            }
            zin.close();
        }
        catch ( FileNotFoundException e ) {
            if( mn8RuntimeFlags.isDebug() ) {
                e.printStackTrace();
            }
            new ErrorConcept( "error", "readFailed", "Could not read '" + zipName +"'.\n" + e.getMessage() ); 
        }
        catch ( IOException ioex ) {
            if( mn8RuntimeFlags.isDebug() ) {
                ioex.printStackTrace();
            }
            new ErrorConcept( "error", "readFailed", "Error extracting zip file: " + dst + "\n" + ioex.getMessage() ); 
        }
    }


    private void readZip() {
        ZipEntry ze;
        try {
            ZipInputStream in = new ZipInputStream( new BufferedInputStream( getZipContentAsStream() ) );
            while ( ( ze = in.getNextEntry() ) != null ) zip.put( ze.toString(), ze );
            in.close();
        } 
        catch ( IOException ioe ) {
            if( mn8RuntimeFlags.isDebug()) {
                ioe.printStackTrace();
            }
            new ErrorConcept( "error", "openZipFailed", "Unable to open '" + zipName + "' zip file." );
        }
    }


    private InputStream getZipContentAsStream() throws IOException {
       InputStream is = null;
       
       try{
           URL furl = new URL(zipName);
           is = furl.openStream();                
       } catch (Exception ex) {
           is = new FileInputStream(zipName);
       }               
       return is;
    }


    private void addDirectory( String path ) {
        File file = new File( path );
        
        if ( file.exists() ) {
            if ( file.isDirectory() ) {
                File [] fileList = file.listFiles();
                for ( int i = 0; i < fileList.length; i++ ) {
                    if ( fileList[i].isDirectory() ) 
                        addDirectory( fileList[i].getPath() );
                    else if ( fileList[i].isFile() )
                        addFile( fileList[i].getPath() );
                }
            } else if ( file.isFile() ) addFile( path );
        } else new ErrorConcept("error", "pathNotFound", "Directory or file '" + path + "' doesn't exists.");
    }


    private void addFile( String path ) {
        try {
            addIS( new FileInputStream( path ), path );
        } 
        catch ( IOException ioex ) {
            new ErrorConcept( "warning", "addtoZipFailed", "Could not add '" + path + 
                              "' file to '" + zipName +"'.\n" + ioex.getMessage() ); 
        } 
    }


    private void addIS( InputStream is, String path ) {
        int c;
                
        try {
             BufferedReader in = new BufferedReader( new InputStreamReader ( is ) );
             ZipEntry zt = new ZipEntry( path );
             
             if ( zip.get( zt.toString() ) == null ) {
                 zip.put( zt.toString(), zt );
                 zipOut.putNextEntry( zt );
                 while ( ( c = in.read() ) != -1 ) zipOut.write ( c );
             }
             else
                 new ErrorConcept( "warning", "addtoZipFailed", "Could not add '" + path + 
                                   "' file to '" + zipName +"'.Already exists."); 
            
             in.close();
        } 
        catch ( IOException ioex ) {
            new ErrorConcept( "warning", "addtoZipFailed", "Could not add '" + path + 
                              "' file to '" + zipName +"'.\n" + ioex.getMessage() ); 
        } 
    }

    private void readd() {
        int c;
        String fName = "";

        try {
            for ( Enumeration e = zipIn.entries(); e.hasMoreElements(); ) {
                ZipEntry ze = (ZipEntry)e.nextElement();
                BufferedReader in = new BufferedReader( 
                                        new InputStreamReader( zipIn.getInputStream( ze ) ) );
                zipOut.putNextEntry( ze );
                while ( ( c = in.read() ) != -1 ) zipOut.write ( c );
                in.close();
             }
        } 
        catch ( IOException ioex ) {
            new ErrorConcept( "warning", "addtoZipFailed", "Could not readd '" + fName + 
                              "' file to '" + zipName +"'.\n" + ioex.getMessage() ); 
        }
    }
}
