/* 
 * $COPYRIGHT$
 * $Id: NamingHandlerBase.java,v 1.8 2002/02/06 12:01:14 crow Exp $
 *
 * Date        Author         Changes 
 * May 04 2001 Remus Pereni   Created
 * Sep 23 2001 Remus Pereni   Refactoring, moving cond. check to 
 *                            checkValidity
 */
package org.media.mn8.conf;

import java.util.Hashtable;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import java.util.Hashtable;
import java.util.Vector;
import javax.naming.Context;

/**
 * Class who is responsible for parsing the 
 * configuration information stored in /conf/naning.mn8.xml
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.8 $ $Date: 2002/02/06 12:01:14 $
 */
public class NamingHandlerBase extends DefaultHandler{
    
    private Hashtable _initEnv = new Hashtable();

    private Hashtable _sysEnvCmd = new Hashtable();

    private Hashtable _conceptPath = new Hashtable();

    /* variable holding all the patterns specified in env.load <load name="*" /> */
    private Vector _envLoad = new Vector();

    /* variable holding all the patterns specified in env.load <except name="*" /> */
    private Vector _envLoadExcept = new Vector();
    
    private String _envMapping = "";
    
    private String _memMapping = "";

    private String _lastElemCtx= "";

    public void startElement ( String uri, String qname, String name, Attributes attr) 
        throws SAXException {
        
        if ( name.toLowerCase().equals("factory") ) {
            _lastElemCtx = "factory";

            if ( attr.getLength() >= 1 &&
                 attr.getQName(0).toLowerCase().equals("name") &&
                 ( attr.getValue(0) != null ||
                   !attr.getValue(0).equals("")
                   )) {
                String value = attr.getValue(0);
                _initEnv.put( Context.INITIAL_CONTEXT_FACTORY, value);
            }
        }

        if ( name.toLowerCase().equals("context") ) {
            _lastElemCtx = "context";

            if ( attr.getLength() >= 1 &&
                 attr.getQName(0).toLowerCase().equals("map.to") &&
                 ( attr.getValue(0) != null ||
                   !attr.getValue(0).equals("")
                   )) {
                _envMapping = attr.getValue(0);
            }
        }


        if ( name.toLowerCase().equals("mem.context") ) {
            _lastElemCtx = "mem.context";

            if ( attr.getLength() >= 1 &&
                 attr.getQName(0).toLowerCase().equals("map.to") &&
                 ( attr.getValue(0) != null ||
                   !attr.getValue(0).equals("")
                   )) {
                _memMapping = attr.getValue(0);
            }

        }


        if ( name.toLowerCase().equals("load") &&
             _lastElemCtx.equals("context") ) {

            if ( attr.getLength() >= 1 &&
                 attr.getQName(0).toLowerCase().equals("name") &&
                 ( attr.getValue(0) != null ||
                   !attr.getValue(0).equals("")
                   )) {
                _envLoad.add(attr.getValue(0));
            }
        }


        if ( name.toLowerCase().equals("except") &&
             _lastElemCtx.equals("context") ){

            if ( attr.getLength() >= 1 &&
                 attr.getQName(0).toLowerCase().equals("name") &&
                 ( attr.getValue(0) != null ||
                   !attr.getValue(0).equals("")
                   )) {
                _envLoadExcept.add(attr.getValue(0));
            }
        }


        if ( _lastElemCtx.equals("factory") && 
             name.toLowerCase().equals("param") ) {

            if ( attr.getLength() >= 2 &&
                 attr.getQName(0).toLowerCase().equals("name") &&
                 attr.getQName(1).toLowerCase().equals("value") &&
                 attr.getValue(0) != null    &&
                 !attr.getValue(0).equals("") &&
                 attr.getValue(1) != null    &&
                 !attr.getValue(1).equals("") ) {
                _initEnv.put( attr.getValue(0), attr.getValue(1));
            }
        }


        if ( _lastElemCtx.equals("context") && 
             name.toLowerCase().equals("os") ) {
            String attrList[] = {"name", "cmd"};
            if( checkeValidity( attr, attrList ) )
                _sysEnvCmd.put( attr.getValue(0).toLowerCase(), attr.getValue(1));
        }
    }

    
    public void endElement ( String uri, String name, String qname ) 
        throws SAXException {
        
        if ( name.toLowerCase().equals("context") ||
             name.toLowerCase().equals("mem.context") ||
             name.toLowerCase().equals("factory") )
            _lastElemCtx = "";
    }


    public void error ( SAXParseException spe) 
        throws SAXParseException {
        throw spe;
    }

    /**
     * Returns an hashtable loaded with all the data contained in
     * the /conf/mn8configg.xml file in the initial.context section.<br>
     * This hashtable should be enough to create an InitialContext.
     * @returns An hashtable containing the properties needed to 
     * successfully create an JNDI InitialContext.
     */
    Hashtable getInitialContextProperties () {
        if ( _initEnv.isEmpty() )
            return null;
        else
            return _initEnv;
    }


    /**
     * Returns an hashtable loaded with all the data contained in
     * the /conf/mn8config.xml file in the env.command section
     * and containing the operating system and the echivalent set 
     * or env command.
     * @returns An hashtable containing the command to be executed
     * on the key operating system to get the system environment.
     */
    Hashtable getEnvCommands () {
        if ( _sysEnvCmd.isEmpty() )
            return null;
        else
            return _sysEnvCmd;
    }


    /**
     * Returns an vector loaded with all the data contained in
     * the /conf/mn8config.xml file in the env.load section.<br>
     * @returns An vector containing the patterns needed to 
     * load system environment variables in the env:/ context.
     */
    Vector getEnvLoad () {
        if ( _envLoad.isEmpty() )
            return null;
        else
            return _envLoad;
    }


    /**
     * Returns an vector loaded with all the data contained in
     * the /conf/naming.mn8.xml file in the env.load section.<br>
     * @returns An vector containing the patterns not needed to be
     * loaded in the env:/ context.
     */
    Vector getEnvLoadExcept () {
        if ( _envLoadExcept.isEmpty() )
            return null;
        else
            return _envLoadExcept;
    }

    
    String getEnvMapping () {
        return _envMapping;
    }


    String getMemMapping () {
        return _memMapping;
    }


    private boolean checkeValidity( Attributes attr, String[] expectedAttr ){

        if( attr.getLength() < expectedAttr.length )
            return false;

        for( int i=0; i < expectedAttr.length; i++) {
            if ( !attr.getQName(i).toLowerCase().equals( expectedAttr[i] ) ||
                 attr.getValue(i) == null    ||
                 attr.getValue(i).equals("") )
                return false;       
        }
        
        return true;
    }
}
