/* 
 * $COPYRIGHT$
 * $Id: mn8Exception.java,v 1.1 2001/08/06 08:42:06 neuro Exp $
 *
 * Date        Author            Changes 
 * Apr 22 2001 Remus Pereni      Created
 */
package org.media.mn8.event;

import org.media.mn8.MessageFormatter;
import java.io.PrintStream;
import java.io.PrintWriter;
/** 
 * <p>The basic mn8 exception class.
 * <p>In mn8 all signaling of user messages is done through
 * exceptions. All mn8 exceptions are derived from this
 * class, and having separate exception types for different
 * kind of exceptions (general system errors and warnings, 
 * mn8 interpretation specific errors). In this way a user can
 * choose the amount of information wanted, and use it's custom
 * message handling utilities. Ex: general errors go on screen, mn8
 * warning go to email.
 * For normal interpretation, all exceptions are caught by the
 * LogManager and displayed on the system console.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.1 $ $Date: 2001/08/06 08:42:06 $
 * @see org.media.mn8.LogManager
 */

public class mn8Exception extends RuntimeException {
    private String _messageTypeId = null;
    private String _messageId = null; 
    private String _optionalInfo = null;
    private String _suplementaryInfo = null;

    private Exception _generatorException = null;

    public mn8Exception( String message ) {
        super( message );
    }

    
    public mn8Exception( String type_id, String message_id ) {
        _messageTypeId = type_id;
        _messageId = message_id;
    }


    public mn8Exception( String type_id, String message_id, 
                         String optional, String suplement ) {
        _messageTypeId = type_id;
        _messageId = message_id;
        _optionalInfo = optional;
        _suplementaryInfo = suplement;
    }


    public mn8Exception( String type_id, String message_id, 
                         String optional ) {
        _messageTypeId = type_id;
        _messageId = message_id;
        _optionalInfo = optional;
    }


    public void setGeneratorException( Exception initialException ) {
        _generatorException = initialException;
    }


    public Exception getGeneratorException() {
        return _generatorException;
    }


    public String getMessage() {
        if ( _generatorException != null )
            return _generatorException.getMessage();
        else 
            return "";            
    }


    public void setMessageTypeId ( String messageTypeId ) {
        _messageTypeId  = messageTypeId;
    }


    public String getMessageTypeId() {
        return _messageTypeId;
    }


    public void setMessageId ( String messageId ) {
        _messageId = messageId;
    }

    
    public String getMessageId() { 
        return _messageId;
    }


    public void setOptionalInfo( String optionalInfo ) {
        _optionalInfo = optionalInfo;
    }


    public String getOptionalInfo() {
        return _optionalInfo;
    }

    
    public void setSuplementaryInfo( String suplemantaryInfo ){
        _suplementaryInfo =suplemantaryInfo;
    }

    public String getSuplementaryInfo(){
        return _suplementaryInfo;
    }


    public String toString() {
        return MessageFormatter.getReference().getMessage( _messageTypeId,
                                                           _messageId,
                                                           _optionalInfo,
                                                           _suplementaryInfo );
    }
    
    
    public void printStackTrace( PrintStream s) {
        if ( _generatorException != null ) 
            _generatorException.printStackTrace( s );
    }


    public void printStackTrace( PrintWriter w) {
        if ( _generatorException != null ) 
            _generatorException.printStackTrace( w );
    }


    public void printStackTrace() {
        if ( _generatorException != null ) 
            _generatorException.printStackTrace();
    }


}
