/* 
 * $COPYRIGHT$
 * $Id: mn8ConceptDefinition.java,v 1.15 2002/02/26 14:15:01 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.event.*;
import org.media.mn8.concepts.*;
import antlr.collections.AST;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.15 $ $Date: 2002/02/26 14:15:01 $
 */
public class mn8ConceptDefinition extends ConceptConceptDefinition {

    private StringConcept _type  = null;
    private StringConcept _label = null;
    private SeriesConcept _mn8InheritedConcepts = new SeriesConcept("Concept");

    private Vector _cameleonJavaConcepts = null;
    /** only used for debugging purposes in mn8ScriptMethod.interpretAST */
    private static String[] _tokenNames = null;
    

    public mn8ConceptDefinition ( String type,
                                  String label,
                                  Vector inheritance ) {
        super();

        _type = new StringConcept(type);
        _label = new StringConcept(label);
        
        if (inheritance != null) {
            for ( int i =0 ; i < inheritance.size(); i++ ) {
                _mn8InheritedConcepts.add( (StringConcept) inheritance.elementAt( i));
            }
        }
        prepareCameleonJavaConcepts( inheritance );
        copyInheritedMethods();
        copyInheritedAttributes();
         copyInheritedElements();
    }

   
    public StringConcept getConceptLabel() {
        return _label;
    }


    public StringConcept getConceptType() {
        return _type;
    }


    public SeriesConcept getInheritedConcepts() {
        return _mn8InheritedConcepts;
    }


    /**
     * In case we have an concept definition, this method is in charge of getting
     * the mn8ConceptDefinition out of it, processing further the methods an so on.
     * @param conceptASTNode AST Node, probably if it is a concept def, will be the root node.
     * @param tokenNames the token names as returned by the parser.
     * @see antlr.Parser
     * @see antlr.collections.AST
     */
    public mn8ConceptDefinition ( AST conceptASTNode, String[] tokenNames ) {

    }


    public static boolean ismn8ConceptDefinition( AST codeASTNode, String[] tokenNames){
        return ( tokenNames[codeASTNode.getType()].toLowerCase().equals("define") || 
                 tokenNames[codeASTNode.getType()].toLowerCase().equals("\"define\"") );
    }


    public static String getConceptTypeFromAST( AST codeASTNode, String[] tokenNames){
        if( ismn8ConceptDefinition( codeASTNode, tokenNames )) {
            return codeASTNode.getFirstChild().getText();
        } 
        else return null;
    }


    public Concept getConceptInstance() {
        mn8Concept result = null;

        if( _cameleonJavaConcepts != null ) {
            // we have special Java Concepts inherited to take care of

            Concept[] inhConceptsArray = new Concept[ _cameleonJavaConcepts.size() ];
            for ( int i =0 ; i < inhConceptsArray.length ; i ++ ) {
                if( ((String)_cameleonJavaConcepts.elementAt(i)).toLowerCase().equals("concept") )
                    inhConceptsArray[i] = null;
                else
                    inhConceptsArray[i] = mn8Loader.getConceptDefinition(
                                       (String) _cameleonJavaConcepts.elementAt(i)).getConceptInstance();
            }            
            result =  new mn8Concept( this, makeFreshStack(), inhConceptsArray );
            for ( int i =0 ; i < inhConceptsArray.length ; i ++ )
                if( inhConceptsArray[i] == null )
                    inhConceptsArray[i] = result;
            return result;
        }
        else {
            return new mn8Concept( this, makeFreshStack(), null );
        }
    }
    

    /**
     * The scripts made ad-hoc, without encapsulating any concept
     * definition we create a dummy concept <i>_current_</i>, and 
     * the code goes to a main dummy method.
     * @return the value of the <i>_current_</i> concept.
     */
    static mn8ConceptDefinition makeTempConcept () {
        mn8ConceptDefinition result = new mn8ConceptDefinition("_current_", "", null);
        result.putMethod( (mn8Method) mn8ScriptMethod.makeMainMethod(result) );
        return result;
    }

    
    private ScopeStack makeFreshStack() {
        ScopeStack result = new ScopeStack( this );
        Iterator conceptFields = getConceptElementFields().getVector().iterator();
        FieldConcept field = null;
        // Elements
        while( conceptFields.hasNext() ){
            field = (FieldConcept) conceptFields.next();
            if( !field.isStatic() ) {
                result.insertInstance( field.getName(), field.newInstance() );
            }
        }
        // Attributes
        conceptFields = getConceptAttributeFieldsAsHashtable().values().iterator();
        while( conceptFields.hasNext() ){
            field = (FieldConcept) conceptFields.next();
            if( !field.isStatic() && !(field instanceof CameleonJavaAttribute)) {
                result.insertInstance( "@" + field.getName(), field.newInstance() );
            }
        }

        return result;
    }


    mn8ScriptMethod getMainMethod() {
        return (mn8ScriptMethod) getConceptMethod("main:series");
    }


    public String[] getTokenNames() {
        return _tokenNames;
    }


    public void setTokenNames( String[] tokenNames) {
        _tokenNames = tokenNames;
    }

    
    protected void copyInheritedMethods() {
        String conceptType = null;
        int idxOf;
        mn8JavaMethod javaMethod = null;

        for ( int i = 0; i < _mn8InheritedConcepts.getVector().size(); i++ ) {
            conceptType = ((StringConcept)_mn8InheritedConcepts.getVector().elementAt(i)).toString();

            SeriesConcept _methods = mn8Loader.getConceptDefinition( conceptType ).getConceptMethods();
            if( _cameleonJavaConcepts != null && _cameleonJavaConcepts.contains( conceptType )) {
                idxOf = _cameleonJavaConcepts.indexOf( conceptType );
                _methods  = mn8Loader.getConceptDefinition( conceptType ).getCameleonConceptMethods();
                for( int j =0 ; j < _methods.getVector().size() ; j++) {
                    javaMethod = (mn8JavaMethod)_methods.getVector().elementAt(j);

                    putMethod( new CameleonJavaMethod( getConceptType().toString(),
                                                       idxOf,
                                                       javaMethod));
                }
            }
            else {
                for( int j =0 ; j < _methods.getVector().size() ; j++) {
                    putMethod( (mn8JavaMethod)_methods.getVector().elementAt(j));
                }
            }
        }

    }


    protected void copyInheritedAttributes() {
        String conceptType = null;
        int idxOf;
        FieldConcept attribute= null;

        for ( int i = 0; i < _mn8InheritedConcepts.getVector().size(); i++ ) {
            conceptType = ((StringConcept)_mn8InheritedConcepts.getVector().elementAt(i)).toString();

            SeriesConcept _attributes = mn8Loader.getConceptDefinition( conceptType ).getConceptAttributeFields();
            if( _cameleonJavaConcepts != null && _cameleonJavaConcepts.contains( conceptType )) {
                idxOf = _cameleonJavaConcepts.indexOf( conceptType );
                _attributes  = mn8Loader.getConceptDefinition( conceptType ).getConceptAttributeFields();
                for( int j =0 ; j < _attributes.getVector().size() ; j++) {
                    attribute = (FieldConcept)_attributes.getVector().elementAt(j);
                    putAttribute( new CameleonJavaAttribute( getConceptType().toString(),
                                                             idxOf,
                                                             attribute));
                }
            }
            else {
                for( int j =0 ; j < _attributes.getVector().size() ; j++) {
                    putAttribute( (FieldConcept)_attributes.getVector().elementAt(j));
                }
            }
        }

    }


    protected void copyInheritedElements() {       
        String conceptType = null;
        int idxOf;
        FieldConcept element= null;

        for ( int i = 0; i < _mn8InheritedConcepts.getVector().size(); i++ ) {
            conceptType = ((StringConcept)_mn8InheritedConcepts.getVector().elementAt(i)).toString();

            SeriesConcept _elements = mn8Loader.getConceptDefinition( conceptType ).getConceptElementFields();
            if( _cameleonJavaConcepts != null && _cameleonJavaConcepts.contains( conceptType )) {
                idxOf = _cameleonJavaConcepts.indexOf( conceptType );
                _elements  = mn8Loader.getConceptDefinition( conceptType ).getConceptElementFields();
                for( int j =0 ; j < _elements.getVector().size() ; j++) {
                    element = (FieldConcept)_elements.getVector().elementAt(j);
                    putElement( new CameleonJavaElement( getConceptType().toString(),
                                                         idxOf,
                                                         element));
                }
            }
            else {
                for( int j =0 ; j < _elements.getVector().size() ; j++) {
                    putElement( (FieldConcept)_elements.getVector().elementAt(j));
                }
            }
        }        
    }


    private void prepareCameleonJavaConcepts( Vector inheritedConcepts ) {
        if( inheritedConcepts == null || inheritedConcepts.size() < 1 )
            return;
        _cameleonJavaConcepts = new Vector();
        String conceptType = null;
        ConceptDefinition conceptDef = null;
        for ( int i =0 ; i < inheritedConcepts.size(); i++ ) {
            conceptType =((StringConcept) inheritedConcepts.elementAt( i )).getValue();
            conceptDef = mn8Loader.getConceptDefinition( conceptType );
            // the problem is present only at the Java Concepts
            if( !(conceptDef instanceof mn8ConceptDefinition) ) {  
                _cameleonJavaConcepts.add( conceptType );                   
            }
        }       
    }

}
