/* 
 * $COPYRIGHT$
 * $Id: mn8EOLFixupTokenStream.java,v 1.2 2002/03/28 16:34:38 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import antlr.TokenStreamHiddenTokenFilter;
import antlr.TokenStream;
import antlr.Token;
import antlr.CommonToken;
import antlr.TokenStreamException;
import antlr.CommonHiddenStreamToken;
import org.media.mn8.parser.mn8TokenTypes;

/** 
 * mn8 Requires end of line after each operation. Sometimes
 * however it happens that the user forgets to insert one
 * after the last command. We take care of that using this
 * TokenStream implementation which automatically adds and
 * end of line at the end of the script.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.2 $ $Date: 2002/03/28 16:34:38 $
 */
public class mn8EOLFixupTokenStream implements TokenStream {
    Token hasLast = null;
    TokenStream _stream = null;

    public mn8EOLFixupTokenStream (TokenStream input) {
        _stream = input;
    }
        
    public Token nextToken() throws TokenStreamException {   
    
        if( hasLast != null ) {
            hasLast.setLine(hasLast.getLine());
            hasLast.setColumn(0);
            return hasLast;
        }

        Token tok = _stream.nextToken();

        if ( tok.getType() == mn8TokenTypes.EOF ) {
            hasLast = tok;
            CommonHiddenStreamToken ct = new CommonHiddenStreamToken(mn8TokenTypes.EOL, "\r\n");
            ct.setLine(tok.getLine());
            ct.setColumn(tok.getColumn());
            return ct;
        }

        return tok;
    }
}
