/* 
 * $COPYRIGHT$
 * $Id: mn8InterpretThread.java,v 1.1 2002/06/03 11:38:55 neuro Exp $
 *
 * Date        Author            Changes 
 * May 18 2001 Remus Pereni      Created
 */
package org.media.mn8;


import antlr.*;
import antlr.collections.*;
import antlr.debug.misc.*;

import org.media.mn8.concepts.*;
import org.media.mn8.conf.*;
import org.media.mn8.event.*;
import org.media.mn8.parser.*;

import java.io.StringReader;
import java.io.PrintStream;
import java.io.InputStream;
import java.io.Reader;

import java.lang.Thread;
import java.util.Vector;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

/**
 * The main class which holds the threaded interpreter functions.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.1 $ $Date: 2002/06/03 11:38:55 $
 */
public class mn8InterpretThread extends Thread {
    private String _scriptURL;
    private String _concept;
    private SeriesConcept _arguments;
    private mn8TreeWalker _currentTW = null;

    public mn8InterpretThread(String scriptURL, String concept, SeriesConcept arguments) {
        _scriptURL = scriptURL;
        _concept = concept;
        _arguments = arguments;
        mn8Interpreter.addInterpreter(this, Thread.currentThread());
    }


    public void run () {
        String conceptType = mn8Loader.loadConceptFromURL( _scriptURL, _concept );
        mn8ScriptMethod main = (mn8ScriptMethod)mn8Loader.getConceptDefinition( conceptType ).getConceptMethod("main:series");
        if ( main.isStatic().getValue() ){
            main.invoke(conceptType, _arguments);
        }
        mn8Interpreter.delInterpreter(this, Thread.currentThread());
    }


    /**
     * Interprets an portion of code represented by it's AST tree.
     * That piece of code can be the code of an method, or a portion
     * of it as currently inputed by the user in interactive mode.
     * @param tree the portion of code as AST tree.
     * @param on the concept on which to execute the code. Important 
     * because it will use the ScopeStack of the concept.
     * @return if there is a result, the resulting concept.
     */
    public final Concept interpretAST(AST tree, mn8Concept on, mn8ScriptMethod method, ScopeStack scopeStack) {
        if ( tree==null ) return null;

        if ( mn8RuntimeFlags.isShowTree() ) {
            mn8ConceptDefinition mnConcDef= null;

            if( on == null ) { //static method invocation, we don't have an instance
                mnConcDef = (mn8ConceptDefinition) mn8Loader.getConceptDefinition(method.getDeclaringConcept().toString());
            } else {
                mnConcDef = (mn8ConceptDefinition) mn8Loader.getConceptDefinition(on.getConceptType().getValue());
            }

            ((CommonAST)tree).setVerboseStringConversion(true, mnConcDef.getTokenNames());
            
            ASTFactory factory = new ASTFactory();
            AST r = factory.create(0,"AST ROOT");
            r.setFirstChild(tree);
            ASTFrame frame = new ASTFrame("mn8 " + mnConcDef.getConceptType().toString() + " AST", r);
            frame.setVisible(true);
            
        }
        
        try {            
            mn8TreeWalker mn8TreeWalker = new mn8TreeWalker();
            _currentTW = mn8TreeWalker;
            return mn8TreeWalker.interpretationUnit( tree, on, method, scopeStack);
        }
        catch( mn8ExitEvent ev) {
            throw ev;
        }
        catch (RecognitionException  ex) {
            new ErrorConcept("warning", "recognitionWarning", ex.toString());
            return new NilConcept();
        }   
        
    }

}
