/* 
 * $COPYRIGHT$
 * $Id: mn8Interpreter.java,v 1.31 2002/06/04 17:10:37 atech Exp $
 *
 * Date        Author            Changes 
 * May 18 2001 Remus Pereni      Created
 */
package org.media.mn8;


import antlr.*;
import antlr.collections.*;
import antlr.debug.misc.*;

import org.media.mn8.concepts.*;
import org.media.mn8.conf.*;
import org.media.mn8.event.*;
import org.media.mn8.parser.*;

import java.io.StringReader;
import java.io.PrintStream;
import java.io.InputStream;
import java.io.Reader;

import java.lang.Thread;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

/**
 * The main class which holds the interpreter functions.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.31 $ $Date: 2002/06/04 17:10:37 $
 */
public class mn8Interpreter {
    public  static String VERSION = "0.6a";
    public  static String IDENTIFIER = "MN8 Ver. " + VERSION + " [ http://mappa.mundi.net/spacemapper/ ]";
    private static NamingManager _namingManager = NamingManager.getReference();
    private static ConceptRegister _conceptRegister = ConceptRegister.getReference();
    private static mn8TreeWalker _currentTW = null;
    private String _scriptName = "";
    public  static String EOL = System.getProperty("line.separator");

    private static Hashtable _outStreams = new Hashtable();
    private static Hashtable _errStreams = new Hashtable();
    private static Hashtable _inStreams = new Hashtable();
    private static Hashtable _interpreters = new Hashtable();
    public  static PrintStream GLOBAL_OUT_STREAM  = System.out;
    public  static PrintStream GLOBAL_ERR_STREAM  = System.err;
    public  static InputStream GLOBAL_IN_STREAM  = System.in;


    public static void reInitialize() {
        cleanAll();
        System.gc();
        ConceptRegister.initialize();
        StaticSystemSetup.setup();
        findEOLType();
        if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose() ) {
            System.err.println("mn8 Reinitialization performed");
        }
    }


    private static void cleanAll() {
        _currentTW = null;
        GlobalScope.cleanAll();
        ExecutionStack.cleanAll();
        ConceptRegister.getReference().cleanAll();
        StaticSystemSetup.cleanAll();
        mn8ScriptMethod.cleanAll();
        NamingConf.cleanAll();
    }


    public static void initialize() {
        StaticSystemSetup.setup();
        findEOLType();
        if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose() ) {
            System.err.println("mn8 Initialization performed");
        }
    }   


    public static mn8TreeWalker getCurrentWalker() {
        return _currentTW;
    }

    
    public static final void executeURL (String scriptURL, SeriesConcept arguments ){
        executeURL(scriptURL, null, arguments);        
    }


    public static final void threadExecuteURLRun ( String scriptURL, String concept, SeriesConcept arguments) {
        (new mn8InterpretThread( scriptURL, concept, arguments)).run();
    }


    public static final Thread threadExecuteURL ( String scriptURL, String concept, SeriesConcept arguments) {
        Thread interpr = new mn8InterpretThread( scriptURL, concept, arguments);
        return interpr;
    }


    public static final void executeURL (String scriptURL, String concept, SeriesConcept arguments ){
            String conceptType = mn8Loader.loadConceptFromURL( scriptURL, concept );
            mn8ScriptMethod main = (mn8ScriptMethod)mn8Loader.getConceptDefinition( conceptType ).getConceptMethod("main:series");
            if ( main.isStatic().getValue() ){
                main.invoke(conceptType, arguments);
            }
    }


    public static final void interpret (String command){
        try {

            String conceptType = (new mn8Loader()).loadConceptFromReader( new StringReader(command), null );
            mn8ScriptMethod main = (mn8ScriptMethod)mn8Loader.getConceptDefinition( conceptType ).getConceptMethod("main:series");
            if ( main.isStatic().getValue() ){
                SeriesConcept arguments = new SeriesConcept();
                arguments.add(new SeriesConcept());
                main.invoke(conceptType, arguments);
            }
        }
        catch( mn8ExitEvent ev) {
            if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose())  {
                System.err.println("Exit request received, exiting...");
            }
        } 
        catch(Exception ex) {
            System.err.println("exception: "+ex);
            if( mn8RuntimeFlags.isDebug()) {
                System.err.println( 
                                   "/---: The stack trace is :----------------------------------\\");
                ex.printStackTrace();
                System.err.println( 
                                   "\\------------------------------------------------------------/");
            }
        }        
    }


    /**
     * Interprets an portion of code represented by it's AST tree.
     * That piece of code can be the code of an method, or a portion
     * of it as currently inputed by the user in interactive mode.
     * @param tree the portion of code as AST tree.
     * @param on the concept on which to execute the code. Important 
     * because it will use the ScopeStack of the concept.
     * @return if there is a result, the resulting concept.
     */
    static synchronized final Concept interpretAST(AST tree, mn8Concept on, mn8ScriptMethod method, ScopeStack scopeStack) {
        if ( tree==null ) return null;

        if ( mn8RuntimeFlags.isShowTree() ) {
            mn8ConceptDefinition mnConcDef= null;

            if( on == null ) { //static method invocation, we don't have an instance
                mnConcDef = (mn8ConceptDefinition) mn8Loader.getConceptDefinition(method.getDeclaringConcept().toString());
            } else {
                mnConcDef = (mn8ConceptDefinition) mn8Loader.getConceptDefinition(on.getConceptType().getValue());
            }

            ((CommonAST)tree).setVerboseStringConversion(true, mnConcDef.getTokenNames());
            
            ASTFactory factory = new ASTFactory();
            AST r = factory.create(0,"AST ROOT");
            r.setFirstChild(tree);
            ASTFrame frame = new ASTFrame("mn8 " + mnConcDef.getConceptType().toString() + " AST", r);
            frame.setVisible(true);
            
        }
        
        try {            
            mn8TreeWalker mn8TreeWalker = new mn8TreeWalker();
            _currentTW = mn8TreeWalker;
            return mn8TreeWalker.interpretationUnit( tree, on, method, scopeStack);
        }
        catch( mn8ExitEvent ev) {
            throw ev;
        }
        catch (RecognitionException  ex) {
            new ErrorConcept("warning", "recognitionWarning", ex.toString());
            return new NilConcept();
        }   
        
    }


    public static void interruptInterpret() {
        if( _currentTW != null ) {
            _currentTW.exit();
        }
    }


    public static boolean isExitAsked() {
        if( _currentTW != null ) {
            return _currentTW.readyToReturn;
        } else {
            return false;
        }
    }

    
    /**
     * Parses the token stream. It is necessary because the parser
     * creates the AST representation of the mn8 code. The AST is 
     * the separated in methods and assigned to an ConceptDefinition
     * from where it will be interpreted at runtime.
     * @param tokStream a value of type 'TokenStream'
     * @return the parser of the token stream.
     */
    static final mn8Parser parseTokenStream( TokenStream tokStream ) {
        try {
            mn8Parser parser = new mn8Parser(tokStream);
            parser.mn8ScriptFile(null);
            return parser;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }


    /**
     * Parses the token stream. It is necessary because the parser
     * creates the AST representation of the mn8 code. The AST is 
     * the separated in methods and assigned to an ConceptDefinition
     * from where it will be interpreted at runtime.
     * @param tokStream a value of type 'TokenStream'
     * @return the parser of the token stream.
     */
    static final mn8Parser parseTokenStream( TokenStream tokStream, String conceptName ) {
        try {
            mn8Parser parser = new mn8Parser(tokStream);
            parser.mn8ScriptFile(conceptName);
            return parser;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }


    /**
     * Creates a new TokenStream from the content of mn8 source from
     * the stream by running the mn8 lexer over the code. 
     * @param stream The mn8 source as a reader.
     * @return the token stream which is the input to the mn8 parser.
     */
    static final TokenStream getLexer(Reader stream){
        mn8Lexer mn8Lexer = new mn8Lexer(stream);
        mn8Lexer.setTokenObjectClass("antlr.CommonHiddenStreamToken");
        TokenStreamHiddenTokenFilter lexer = new TokenStreamHiddenTokenFilter(new mn8EOLFixupTokenStream(mn8Lexer));
        lexer.hide(mn8Lexer.WS);        
        lexer.hide(mn8Lexer.LCONT);

        if ( mn8RuntimeFlags.isTrace() ) {
            System.out.println( "In trace!" );
            try {
                Token tok = lexer.nextToken();
                while (tok.getText() != null){
                    System.out.print( tok );
                    tok = lexer.nextToken();
                }

            }catch (Exception ex) {
                ex.printStackTrace();
            }
            System.exit(0);
        }
        return lexer;
    }


    
    /**
     * Just for debugging. It provides an Swing frame with the representation
     * of the AST tree.
     * @param tree a value of type 'AST'
     * @param tokenNames a value of type 'String[]'
     */
    static synchronized final void preparingTreeAction(AST tree, String[] tokenNames) {
        if ( tree==null ) return;

        if ( mn8RuntimeFlags.isShowTree() ) {

            ((CommonAST)tree).setVerboseStringConversion(true, tokenNames);
            ASTFactory factory = new ASTFactory();
            AST r = factory.create(0,"AST ROOT");
            r.setFirstChild(tree);
            ASTFrame frame = new ASTFrame("mn8 AST", r);
            frame.setVisible(true);
        }
        
    }


    /**
     * Processes the mn8 code represented as AST tree.
     *<p>
     * If the tree represents an concept definition, the mn8ConceptDefinition
     * was already created by the mn8 Parser and loader in the registry. in
     * this case out only job is to find out what is our concept name.
     *<p>
     * If the AST code does not represent an concept definition but rather
     * just code as random scripts, it will create a _current_ concept
     * definition if it doesn't already exist, and append the code to
     * it's main method.
     * @param codeAST the current mn8 code as AST (already gone through lexer and parser)
     * @param tokenNames the token names from the parser
     * @return the name of the concept.
     */
    static synchronized final String processTree( AST codeAST, String[] tokenNames ) {
        String result;

        if( mn8ConceptDefinition.ismn8ConceptDefinition(codeAST, tokenNames) ) {
            result = mn8ConceptDefinition.getConceptTypeFromAST( codeAST, tokenNames);
            return result;
        }

        mn8ConceptDefinition _currentConcept ;
        result = "_current_";
        if( ConceptRegister.getReference().isConceptNamed( "_current_" ) ) {
            _currentConcept = (mn8ConceptDefinition) ConceptRegister.getReference().lookup( "_current_" );
        } 
        else {
            _currentConcept = mn8ConceptDefinition.makeTempConcept();
            _currentConcept.setTokenNames( tokenNames );
            ConceptRegister.getReference().register("_current_", (ConceptDefinition)_currentConcept);
        }
        ((mn8ScriptMethod)_currentConcept.getConceptMethod("main:series")).setCodeAST( codeAST );
        return result;
    }


    private static void findEOLType () {
        String eol = EOL;
        try {
            Object nen = _namingManager.lookup("env:/system/properties/EOL");
            if( nen != null) {
                eol = ((StringConcept)nen).toString();
            }

        } catch (NamingException nex) {
        }

        
        try {
            Object nen = _namingManager.lookup("mem:/system/properties/EOL");
            if( nen != null && nen instanceof StringConcept ) {
                eol = ((StringConcept)nen).toString();
            }
        } catch (NamingException nex) {
        }

        if( eol.equals("\\r\\n")) {
            eol = "\r\n";
        } else if( eol.equals("\\r")) {
            eol = "\r";
        } else if( eol.equals("\\n")) {
            eol = "\n";
        }
        EOL = eol;
        try {
            _namingManager.envBind("env:/system/properties/EOL", new StringConcept(EOL));
        } catch( Exception ex) {
            if( mn8RuntimeFlags.isDebug()) {
                ex.printStackTrace();
            }
        }
    }


    public synchronized static PrintStream getOut(Thread thread) {
        if( _outStreams.containsKey(thread)) {
            return (PrintStream) _outStreams.get(thread);
        } else {
            return GLOBAL_OUT_STREAM;
        }

    }


    public synchronized static PrintStream getErr(Thread thread) {
        if( _errStreams.containsKey(thread)) {
            return (PrintStream) _errStreams.get(thread);
        } else {
            return GLOBAL_ERR_STREAM;
        }

    }

    public static InputStream getIn(Thread thread) {
        if( _inStreams.containsKey(thread)) {
            return (InputStream) _inStreams.get(thread);
        } else {
            return GLOBAL_IN_STREAM;
        }
    }


    public synchronized static void printTDebug(Thread curr, String message) {
        if( curr != null ) {
            System.err.println("->" + message + " " +  curr.hashCode());
        }
        System.err.println("\t Out: " + _outStreams.size() + " Err: " + _errStreams.size() + " In: " + _inStreams.size());
        System.err.println("\tOut: " + GLOBAL_OUT_STREAM.toString() );
        Enumeration keys = _outStreams.keys();
        while( keys.hasMoreElements() ) {
            Thread key = (Thread) keys.nextElement();
            System.err.println("\t\t" + key.hashCode() +  " - " + _outStreams.get(key).toString());
        }
    }


    public static PrintStream getOut() {
        return getOut( Thread.currentThread());
    }


    public static PrintStream getErr() {
        return getErr( Thread.currentThread());
    }


    public static InputStream getIn() {
        return getIn( Thread.currentThread());
    }


    public static void addInterpreter(  mn8InterpretThread interp, Thread inThread ) {
        _interpreters.put( inThread, interp);
    }


    public static void delInterpreter(  mn8InterpretThread interp, Thread inThread ) {
        _interpreters.remove( inThread );
    }


    public static mn8InterpretThread getInterpreter(  Thread inThread ) {
        return (mn8InterpretThread) _interpreters.get( inThread );
    }


    public static void setOut( PrintStream out, Thread inThread ) {
        _outStreams.put(inThread, out);
    }

    public static void setErr( PrintStream err, Thread inThread ) {
        _errStreams.put(inThread, err);
    }


    public static void setIn( InputStream in, Thread inThread ) {
        _interpreters.put(inThread, in);
    }


    public static void setOut( PrintStream out) {
        setOut(out, Thread.currentThread());
    }


    public static void setErr( PrintStream err ) {
        setErr(err, Thread.currentThread());
    }


    public static void setIn( InputStream in ) {
        setIn(in, Thread.currentThread());
    }


    public static void cleanStreams( Thread inThread ) {
        if( _outStreams.containsKey(inThread)) {
            _outStreams.remove( inThread );
        }

        if( _errStreams.containsKey(inThread)) {
            _errStreams.remove( inThread );
        }

        if( _inStreams.containsKey(inThread)) {
            _inStreams.remove( inThread );
        }
    }
}
