/* 
 * $COPYRIGHT$
 * $Id: mn8JavaMethod.java,v 1.24 2002/06/05 14:55:58 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.event.*;
import org.media.mn8.concepts.*;
import org.media.mn8.util.Normalizer;

import java.util.Vector;
import java.util.StringTokenizer;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.InvocationTargetException;
import org.apache.oro.text.regex.MalformedPatternException;

public class mn8JavaMethod extends JavaConcept implements mn8Method {

    
    public AttributeConcept name = new AttributeConcept( "name", "Name", null );

    protected boolean _isStatic = false;

    protected String _declaringConcept = null;

    protected Vector _paramTypes = null;

    protected Method _method = null;

    protected String _returnType = null;

    private String _signatureChache = null;

    String _javaSignature = null;

    public int lineNr = -1;

    public mn8JavaMethod() {}

    /**
     * One form of the constructor.
     * @param fromClass The Java class where the described method belongs
     * @param javaSig the signature of the java method: like insert:org.media.mn8.Concept,org.media...
     * @param mn8Sig the signature of the mn8 method: like insert:Integer,Concept
     * @param declaringConcept The type of the concept where this method belongs: Concept
     * @param returnType null or the concept type of the returning value
     */
    public mn8JavaMethod ( String fromClass,
                           boolean isStatic,
                           String javaSig,
                           String mn8Sig,
                           String declaringConcept,
                           String returnType ) {                          

        createFromFormalDef( fromClass, isStatic, javaSig, mn8Sig, declaringConcept, returnType);
    }


    /**
     * One form of the constructor.
     * @param fromClass The Java class where the described method belongs
     * @param javaSig the signature of the java method: like insert:org.media.mn8.Concept,org.media...
     * @param mn8Sig the signature of the mn8 method: like insert:Integer,Concept
     * @param declaringConcept The type of the concept where this method belongs: Concept
     * @param returnType null or the concept type of the returning value
     */
    public mn8JavaMethod ( String fromClass,
                           String javaSig,
                           String mn8Sig,
                           String declaringConcept,
                           String returnType ) {                          
        createFromFormalDef( fromClass, false, javaSig, mn8Sig, declaringConcept, returnType);
    }


    public void createFromFormalDef( String fromClass,
                                     boolean isStatic,
                                     String javaSig,
                                     String mn8Sig,
                                     String declaringConcept,
                                     String returnType ) {                          

        try {
            Class _class = Class.forName(fromClass);
            Class[] _params = null;
            String methodName = null;
            
            _javaSignature = javaSig;

            // prepairing from the javaSig the Class[] object needed for method retrieval
            if( javaSig != null ){
                methodName = getMethodNameFromJavaSig( javaSig );
                _params = getMethodParamsFromJavaSig( javaSig );
            }
            
            
            // getting the Java methods
            _method = _class.getDeclaredMethod( methodName, _params);
       
            // making all the necessary data for an mn8Method definition
            methodName = mn8Sig.substring(0, mn8Sig.indexOf(":"));

            this.name.setAttributeValue( new StringConcept(methodName));
            _isStatic = Modifier.isStatic(_method.getModifiers());
            _paramTypes = new Vector();

            if( mn8Sig != null) {
                StringTokenizer _mn8ParamType = new StringTokenizer(mn8Sig.substring(mn8Sig.indexOf(":")+1), ",");
                while( _mn8ParamType.hasMoreTokens() )
                    _paramTypes.add(new StringConcept(_mn8ParamType.nextToken()));
            }
            _declaringConcept = declaringConcept;
            _returnType = returnType;
            _isStatic = isStatic;
        } catch (Exception ex) {
            if( mn8RuntimeFlags.isDebug() )
                ex.printStackTrace();
            else
                System.out.println(ex.getMessage());
        }
    }

    
    public mn8JavaMethod ( Method method,
                           String[] paramTypes,
                           String declaringConcept,
                           String returnType ) {                           
        _method = method;
        this.name.setAttributeValue( new StringConcept(method.getName()) );
        _isStatic = Modifier.isStatic(method.getModifiers());

        _paramTypes = new Vector();
        if( paramTypes != null) 
            for( int i = 0 ; i < paramTypes.length ; i++) {
                _paramTypes.add(new StringConcept((String)paramTypes[i]));
            }
        _declaringConcept = declaringConcept;
        _returnType = returnType;
    }


    public mn8JavaMethod ( String name,
                           Method method,
                           boolean isStatic,
                           Vector paramTypes,
                           String declaringConcept,
                           String returnType ) {
        this.name.setAttributeValue( new StringConcept(name) );
        _method = method;
        _isStatic = isStatic;
        _paramTypes = paramTypes;
        _declaringConcept = declaringConcept;
        _returnType = returnType;
    }


    public StringConcept getName() {
        return (StringConcept)  name.getAttributeValue() ;
    }

    
    public LogicalConcept isStatic() {
        return new LogicalConcept( _isStatic );
    }


    public StringConcept getDeclaringConcept() {
        return new StringConcept( _declaringConcept );
    }


    public LogicalConcept hasReturn() {
        return new LogicalConcept( (_returnType == null) ? false : true );
    }


    public StringConcept getReturnType() {
        if( getName().toString().equalsIgnoreCase("create")) {
            return getDeclaringConcept();
        } 
        return new StringConcept( (_returnType == null) ? "Nil" : _returnType );       
    }

    
    public Concept invoke( Concept on, SeriesConcept parameters ){
        Object result = null;

        try {
            if ( !(_isStatic) || _method.getDeclaringClass().isInstance(on) ){
                result = _method.invoke( on, (parameters == null ? null : parameters.getVector().toArray()) );
            }
            else {
                Method method = findDeepMethod( on, getSignature());
                if( method != null ) {
                    result = method.invoke( on, (parameters == null ? null : parameters.getVector().toArray()));
                } else {
                    result = invoke( on.getConceptType().toString(), parameters );
                }
            }

            // we have an constructor
            if( getName().toString().equalsIgnoreCase("create")) {
                return on;
            }

            if( hasReturn().getValue()) {
                return (Concept) result;
            } else return new NilConcept();
            
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
            return new NilConcept();
        } catch (InvocationTargetException ex) {
            if( ex.getTargetException() instanceof mn8Warning ) {
                throw (mn8Warning) ex.getTargetException();
            } else if( ex.getTargetException() instanceof mn8Error ) {
                throw (mn8Error) ex.getTargetException();
            } else {
                ex.printStackTrace();
                return new NilConcept();
            }
        }
    }


    public Concept invoke( String conceptDef, SeriesConcept parameters ){
        Object result;
        try {
            result = _method.invoke( mn8Loader.getConceptDefinition( conceptDef ), 
                                     (parameters == null ? null : parameters.getVector().toArray()) );
            if( hasReturn().getValue() || getName().toString().equalsIgnoreCase("create")) {
                return (Concept) result;
            } else return new NilConcept();

        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
            return new NilConcept();
        } catch (InvocationTargetException ex) {
            if( ex.getTargetException() instanceof mn8Warning ) {
                throw (mn8Warning) ex.getTargetException();
            } else if( ex.getTargetException() instanceof mn8Error ) {
                throw (mn8Error) ex.getTargetException();
            } else {
                ex.printStackTrace();
                return new NilConcept();
            }
        }            
    }

    
    public SeriesConcept getParameterTypes() {
        if( _paramTypes == null ) 
            return new SeriesConcept();
        else return new SeriesConcept( _paramTypes );
    }
    

    public SeriesConcept getModifiers() {
        throw new mn8NotImplementedException( "mn8JavaMethod", "getModifiers");
    }


    public StringConcept getMethodSignature() {
        return new StringConcept( getSignature());
    }

    
    public String getSignature() {
        String result = null;
        
        if( _signatureChache != null ) return _signatureChache;
        
        if( _paramTypes != null )
            result = mn8ScriptMethod.getSignatureFromVect( 
               ((StringConcept)name.getAttributeValue()).getValue(), _paramTypes);
        else {
            result = mn8ScriptMethod.getSignature( 
               ((StringConcept)name.getAttributeValue()).getValue(), null );
        }
        if( _signatureChache == null )
            _signatureChache = result;

        return result;
    }


    public StringConcept toTXT() {
        StringBuffer buffer = new StringBuffer();
        if( _isStatic ) 
            buffer.append("STATIC ");
        else
            buffer.append("       ");
        
        buffer.append(name.getValue().toTXT().getValue());
        buffer.append(":");

        if( _paramTypes != null ){
            for( int i = 0 ; i < _paramTypes.size(); i++){
                if( i > 0 )             
                    buffer.append(",");
                
                buffer.append(((StringConcept)_paramTypes.elementAt(i)).getValue());
            }
        }
        if( _returnType != null || getName().toString().equalsIgnoreCase("create")) {
            buffer.append(" TYPEOF ");
            buffer.append(getReturnType().toString());
        }
        return new StringConcept(buffer.toString());
    }


    public StringConcept toXML() {
        StringBuffer buffer = new StringBuffer("<Method name=\"");

        buffer.append(Normalizer.normalize(name.toString()));
        buffer.append("\" declaringConcept=\"" + _declaringConcept + "\" ");
    
        if( _isStatic ) 
            buffer.append("isStatic=\"true\" ");
        else
            buffer.append("isStatic=\"false\" ");

        if( _returnType == null || getName().toString().equalsIgnoreCase("create")) 
            buffer.append(">" + mn8Interpreter.EOL);
        else
            buffer.append("returns=\"" + getReturnType().toString() + "\">" + mn8Interpreter.EOL);

        for( int i = 0 ; i < _paramTypes.size(); i++){
            buffer.append("<parameter name=\"\" type=\"" + ((StringConcept)_paramTypes.elementAt(i)).getValue() + "\"/>" + mn8Interpreter.EOL);
        }

        buffer.append("</Method>");
        return new StringConcept(buffer.toString());
    }


    public ConceptDefinition getConceptDefinition() {
        return MethodDefinition.getReference();
    }
    


    public static mn8Method getMethodUsingInheritance( String signature, Vector methods ) {
        mn8Method result = null;        
        Vector parametersList = new Vector(); //will contain the inherited concepts as series
        String methodName = signature;
        StringTokenizer paramTokenizer;

        if( signature.indexOf(":") != -1 ) {
            methodName = signature.substring(0, signature.indexOf(":"));
            paramTokenizer = new StringTokenizer(signature.substring(signature.indexOf(":") + 1), ",");
        } else {
            paramTokenizer = new StringTokenizer("");
        }


        // we get for each parameter the inherited concepts
        SeriesConcept tmpVect = null;
        String cc = null;
        while( paramTokenizer.hasMoreTokens() ) {
            cc= paramTokenizer.nextToken();
            tmpVect = ((ConceptDefinition)mn8Loader.getConceptDefinition(cc)).getAllInheritedConcepts();
            tmpVect.add(mn8Loader.getConceptDefinition(cc).getConceptType());
            parametersList.add(tmpVect);
        }

        Vector concMetParam = null;            //each method paramaters
        concMethodsCicle: 
        for(int methodIdx = 0; methodIdx < methods.size(); methodIdx++) {

            // if the name of the method is not the same, see next method
            if ( !((mn8Method)methods.elementAt( methodIdx )).getName().getValue().equalsIgnoreCase(methodName) )
                continue concMethodsCicle;

            // if the number of parameters are not the same, see next method
            concMetParam = ((mn8Method)methods.elementAt( methodIdx )).getParameterTypes().getVector();
            if( concMetParam.size() != parametersList.size() )
                continue concMethodsCicle;

            for( int paramIdx = 0; paramIdx < concMetParam.size(); paramIdx++) {               
                Vector paramAvailable = ((SeriesConcept)parametersList.elementAt( paramIdx )).getValue();

                if ( !paramAvailable.contains((StringConcept)concMetParam.elementAt( paramIdx ))) {
                    continue concMethodsCicle;
                }
            }

            return (mn8Method)methods.elementAt( methodIdx );
        }
        return null;
    }

    
    Method getMethod() {
        return _method;
    }


    public boolean equals(Object obj) {
        if( !(obj instanceof mn8Method) )
            return false;
        
        if( getSignature().equals( ((mn8Method)obj).getSignature()) )
            return true;
        else 
            return false;
    }


    public int hashCode() {
        return getSignature().hashCode();
    }


    protected Method findDeepMethod( Concept on, String signature ) {
        Class conceptClass = on.getClass();
        Method method = null;
        String methodName = null;
        Class[] methodParams = null;

        if( _javaSignature == null ) return null;
        
        methodParams =getMethodParamsFromJavaSig( _javaSignature );
        methodName = getMethodNameFromJavaSig( _javaSignature );

        while (conceptClass != null ) {
            try {
                method = conceptClass.getMethod( methodName, methodParams );
            } catch ( NoSuchMethodException ex ) {
                ;
            }
            if( method != null ) return method;
            conceptClass = conceptClass.getSuperclass();
        }
        return null;
    }


    protected String getMethodNameFromJavaSig( String javaSig ) {
        if( javaSig != null ){
            return javaSig.substring(0, javaSig.indexOf(":"));
        }
        else 
            return null;
    }


    protected Class[] getMethodParamsFromJavaSig( String javaSig ) {
        if( javaSig != null ){
            Class[] _params = null;
            Vector params = new Vector();

            StringTokenizer javaParam = new StringTokenizer(javaSig.substring(javaSig.indexOf(":")+1), ",");
            try {
                while( javaParam.hasMoreTokens() )
                    params.add( Class.forName( javaParam.nextToken() ));                  

                _params = new Class[ params.size() ];
                for( int i = 0; i < params.size(); i++) 
                    _params[i] = (Class) params.elementAt(i);
                
                return _params;
            }catch (ClassNotFoundException ex) {
                return null;
            }
        }
        else
            return null;        
    }


    public Object clone() {

        return new mn8JavaMethod ( getName().toString(),
                                   getMethod(),
                                   isStatic().getValue(),
                                   getParameterTypes().getVector(),
                                   getDeclaringConcept().toString(),
                                   hasReturn().getValue() ? getReturnType().toString() : null );
            
    }


    public int getLineNr() {
        return lineNr;
    }


    public void setLineNr(int lineNr) {
        this.lineNr = lineNr;
    }
}

