/* 
 * $COPYRIGHT$
 * $Id: mn8Loader.java,v 1.26 2002/06/05 14:56:14 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.concepts.StringConcept;
import org.media.mn8.concepts.SeriesConcept;
import org.media.mn8.event.*;
import org.media.mn8.parser.*;
import org.media.mn8.conf.SearchPathConf;

import antlr.*;
import antlr.collections.*;
import antlr.debug.misc.*;

import java.io.BufferedReader;
import java.io.Reader;
import java.io.StringReader;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.InvocationTargetException;
import javax.naming.NamingException;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.26 $ $Date: 2002/06/05 14:56:14 $
 */
public class mn8Loader {

    
    private static ConceptRegister _register = ConceptRegister.getReference();

    private static NamingManager _namingManager = NamingManager.getReference();

    private static String _scriptName = "";


    public static void setScriptName( String name ) {
        _scriptName = name;
    }

    
    public static String getScriptName() {
        return _scriptName;
    }


    public static synchronized Concept instantiateConceptFromURL( String conceptURL )
        throws ConceptNotFoundException {
            mn8Loader loader = new mn8Loader();
            return loader.getConceptInstance( loader.loadConceptFromURL( conceptURL, null ) );
    }


    public static synchronized JavaConcept instantiateConcept( String name ) 
        throws ConceptNotFoundException {
            mn8Loader loader = new mn8Loader();
            return loader.getConceptInstance( name );
    }


    public static synchronized void loadConcept( String name ) 
        throws ConceptNotFoundException {

        if( !mn8Loader.isConceptAvailable( name )){
            mn8Loader loader = new mn8Loader();
            if( ! loader.loadJavaConcept(name) ){
                String url = loader.findConcept(name);
                if( mn8RuntimeFlags.isDebug()) {
                    System.err.println("loading concept: " + name + " from url: " + url);
                }
                loader.loadConceptFromURL( url, name );
            }
        }
    }

    
    public static ConceptDefinition getConceptDefinition(String concept) {
        if( !isConceptAvailable( concept.toLowerCase() ) )
            loadConcept( concept );

        Object conceptSomething = _register.lookup(concept);
        if ( conceptSomething instanceof ConceptDefinition )
            return (ConceptDefinition) conceptSomething;
        else return null;        
    }


    /**
     * Tries to find if the Concepted specified by the concept name
     * is available to mn8. This is done, by looking first in the
     * concept register, if it is not there it tries to use find
     * in order to see if it is there somewhere and not loaded yet.
     * <p> Just not to forget. All the JavaConcepts will be specified 
     * in a configuration file, so, searching for them should not be
     * too dificult, if the concept is not there and it's not in 
     * the register either, that means that it might be an mn8 script
     * and is probably located in the script path somewere.
     * @param conceptName The name of the concept;
     * @return it's obvious ;)
     */
    public static boolean isConceptAvailable( String conceptName ) {
        if( _register.lookup(conceptName) != null ) 
            return true;
        else 
            return false;
    }


    /**
     * Returns the content of resource specified by the URL given
     * as parameter.
     * @param resourceURL The URL of the resource
     * @return The content of the resource
     */
    public static synchronized String getContent( String resourceURL ){
        try {
            StringBuffer buffer = new StringBuffer();
            
            InputStream is = null;
            try {
                URL url = new URL( resourceURL );
                URLConnection uc = url.openConnection();
                is = uc.getInputStream();
            } catch ( MalformedURLException mex) {
                // Maybe is a file
                File resFile = new File( resourceURL );
                if( resFile.exists() && resFile.isFile()) {
                    is = new FileInputStream(resFile);
                } else {
                    throw mex;
                }
            }
            BufferedReader reader = new BufferedReader( 
                                    new InputStreamReader( is ));
            int chr;
            while ( (chr = reader.read()) != -1) {
                buffer.append((char)chr);
            }        
            return buffer.toString();
        } catch (MalformedURLException ex) {
            String message = MessageFormatter.getReference().getMessage("warning",
                                                                        "net.malformed_url",
                                                                        resourceURL);
            LogManager.message( message );
            throw new mn8ResourceLoadingException(message);
        } catch (IOException ex) {
            String message = MessageFormatter.getReference().getMessage("warning",
                                                                        "net.connection_error",
                                                                        resourceURL,
                                                                        ex.getMessage());
            LogManager.message( message );
            throw new mn8ResourceLoadingException(message);

        } catch (Exception ex) {
            String message = MessageFormatter.getReference().getMessage("warning",
                                                                        "net.unexpected_error",
                                                                        resourceURL,
                                                                        ex.getMessage());
            LogManager.message( message );
            throw new mn8ResourceLoadingException(message);
        }
    }

    
    public static synchronized String resolveClassName( String className ) {
        return _register.resolveClass( className );
    }


    private JavaConcept getConceptInstance( String name ) 
        throws ConceptNotFoundException {
        JavaConcept result = null;
        
        if( !isConceptAvailable( name ) ) {
            loadConcept( name );
        }

        Object conceptDef = _register.lookup( name );

        if( conceptDef != null ) {
            
            if( conceptDef instanceof ConceptDefinition ) { 
                result = (JavaConcept) mn8TreeWalker.executeStaticMethod("create:", 
                                                                           ((ConceptDefinition) conceptDef).getConceptType().toString(),
                                                                           new SeriesConcept(), -1 );
            }
        } else {
            throw new RuntimeException("Unable to find concept: " + name);
        }
        return result;
    }

    
    private boolean loadJavaConcept( String name ) {
        try {
            String className = ((StringConcept) _namingManager.lookup("env://concept/javaConcepts/" + name.toLowerCase())).toString();
            if( className != null ) {
                Class conceptClass = Class.forName( className );
                if( ConceptDefinition.class.isAssignableFrom(conceptClass)) {
                    Method refMethod = conceptClass.getMethod("getReference", null);
                    if( refMethod != null && Modifier.isStatic(refMethod.getModifiers())) {
                        ConceptDefinition conceptDef = (ConceptDefinition) refMethod.invoke(this, null);
                        _register.register(name, conceptDef);
                        return true;
                    }
                }
            }
            return false;
        } catch (NamingException nex){
            return false;
        } catch (ClassNotFoundException cnfe) {
            System.err.println( cnfe.getMessage());
            return false;
        } catch (NoSuchMethodException cnfe) {
            System.err.println( cnfe.getMessage());
            return false;
        } catch (IllegalAccessException cnfe) {
            System.err.println( cnfe.getMessage());
            return false;
        } catch (InvocationTargetException cnfe) {
            System.err.println( cnfe.getMessage());
            return false;
        }
    }

    
    /**
     * Loads a concept into the {@link org.media.mn8.ConceptRegister}.
     * Because the Java concepts are accessed through the registry where 
     * it has a name asociated, through this method only the mn8 concepts 
     * will be loaded.
     * @param location The location of the conept to be loaded.
     * @return The name of type of the concept desctribed by conceptURL.
     * @exception ConceptNotFoundException If the concept is not found at the 
     * specified location.
     */
    public static synchronized String  loadConceptFromURL( String conceptURL, String name ) 
        throws ConceptNotFoundException {

        try {
            mn8Loader.setScriptName( conceptURL );
            String conceptName = "String";
            if( conceptURL.indexOf("/") == -1 && conceptURL.indexOf("\\") == -1  && conceptURL.indexOf(".") == -1 ) {
                conceptName = conceptURL;
                conceptURL = (new mn8Loader()).findConcept( conceptName );
            }
            String conceptContent = getContent( conceptURL );
            StringReader sreader = new StringReader( conceptContent );
            return (new mn8Loader()).loadConceptFromReader( sreader, name);
        } catch (Exception ex) {
            ex.printStackTrace();
            String message = MessageFormatter.getReference().getMessage("error",
                                                                        "concept.instantiation_location_error",
                                                                        conceptURL);
            LogManager.message( message );
            throw new mn8ConceptInstantiationException(message);
        }
    }


    public String loadConceptFromReader( Reader reader, String name) {   
        try {

            TokenStream scriptTokens = mn8Interpreter.getLexer( reader );
            mn8Parser parser = mn8Interpreter.parseTokenStream( scriptTokens, name );
            
            if( name == null ) {
                // the first concept   
                String conceptName = mn8Interpreter.processTree( parser.getAST(), parser.getTokenNames() );
                mn8Interpreter.preparingTreeAction( parser.getAST(), parser.getTokenNames() );
                return conceptName;                
            }

            if( _register.isConceptNamed(name) ) {
                return name;   
            } else {
                String conceptName = null;
                if( name != null ) {
                    conceptName = mn8Interpreter.processTree( parser.getAST(), parser.getTokenNames() );                                        
                }
                if( !_register.isConceptNamed(conceptName) ) {
                    throw new ConceptNotFoundException(name);                                  
                }
                return conceptName;
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            String message = MessageFormatter.getReference().getMessage("error",
                                                                        "concept.instantiation_location_error",
                                                                        "");
            LogManager.message( message );
            throw new mn8ConceptInstantiationException(message);
        }
    }


    /**
     * Tries to find the concept by looking first in the mem://registry/concepts/,
     * then in env://registry/concepts/, then in the directories or URL specified in,
     * the mem://registry/conceptspath/ and at the end env://registry/conceptspath.
     * @param name The name of the concept (Concept Type)
     * @return the exact URL of the location of the concept
     * @exception ConceptNotFoundException if the concept was not found
     */
    public String findConcept( String name ) throws ConceptNotFoundException {
        SearchPathConf searchPathConf = SearchPathConf.getRef();
        Iterator searchPaths = searchPathConf.iterator();
        String path = null;
        File currentPath = null;
        File[] resultPaths = null;

        if( name.trim().indexOf(":") != -1 )
            name = name.trim().substring(0, name.trim().indexOf(":"));

        if ( !name.trim().toLowerCase().endsWith(".mn8") )
            name = name.trim().toLowerCase() + ".mn8";
        try {
            mn8FileFilter fileFilter = new mn8FileFilter( name);
            while( searchPaths.hasNext() ) {
                path = ((StringConcept)searchPaths.next()).toString();
                currentPath = new File( path );
                resultPaths = currentPath.listFiles(  fileFilter );
                if( resultPaths != null && resultPaths.length > 0 )
                    return resultPaths[0].toURL().toString();
            }
            throw new ConceptNotFoundException(name);
        } catch (MalformedURLException mex) {
            if( mn8RuntimeFlags.isDebug() )
                mex.printStackTrace();
            else if (mn8RuntimeFlags.isVerbose()) {
                System.err.println(mex.getMessage());
            }
            throw new ConceptNotFoundException(name);
        }
    }


    public static void main (String args[]) {
        try {
            /*
            System.out.println( "String->" 
                                + mn8Loader.instantiateConcept("String").getConceptType());
            System.out.println( "String->" 
                                + mn8Loader.instantiateConcept("Test").getConceptType());
            */
            System.out.println( "Content of: " + args[0] + " is \n" 
                                + mn8Loader.instantiateConceptFromURL( args[0] ));

        } catch (Exception ex) {
            System.out.println( 
                    "/---: The stack trace is :----------------------------------\\");
            ex.printStackTrace();
            System.out.println( 
                    "\\------------------------------------------------------------/");
        }
    }


}
