/* 
 * $COPYRIGHT$
 * $Id: mn8ScriptAttribute.java,v 1.7 2002/06/05 14:56:23 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.event.*;
import org.media.mn8.concepts.*;
import org.media.mn8.parser.mn8TreeWalker;
import antlr.collections.AST;
import antlr.RecognitionException;
/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.7 $ $Date: 2002/06/05 14:56:23 $
 */
public class mn8ScriptAttribute extends FieldConcept {

    // These are inherited from FieldConcept
    // protected String _name = null;
    // protected boolean _isStatic = false;
    // protected String _label = null;
    // protected String _conceptType = null;
    // protected Concept _initialValue = null;
    // protected Field _field = null;

    protected String _declaringConcept = null;
    protected AttributeConcept _field = null;
    protected AST _initialValue = null;

    public mn8ScriptAttribute() {
        _showEmpty=false;
    }


    public mn8ScriptAttribute ( String name,
                                AttributeConcept field,
                                boolean isStatic,
                                String label,
                                String type,
                                String declaringConcept,
                                AST initial ) {
        _name = name;
        _field = field;
        _isStatic = isStatic;
        _label = label;
        _conceptType = type;
        _declaringConcept = declaringConcept;
        _initialValue = initial;
        _showEmpty = false;
    }


    public Concept newInstance() {
        AttributeConcept attr = new AttributeConcept(_name, _label, null);
        if( _initialValue != null ) {
            initializeField(attr);
        }
        attr.setHidden( isHidden());
        attr.setShowEmpty( showEmpty());
        return attr;
    }


    public boolean isAttribute() {
        return true;
    }
    

    public boolean isElement() {
        return false;
    }


    public Concept getValue( Concept on ) {
        if( isStatic() ) {
            return GlobalScope.getReference().getInstance( _declaringConcept + ":@" + _name);
        }
        
        if( on instanceof mn8Concept ) {
            ScopeStack conceptStack = ((mn8Concept)on).getScopeStack();
            if( conceptStack.isInstance( "@" + _name) )
                return conceptStack.getInstance( "@" + _name);
            else {
                throw new RuntimeException("Unamble to find atribute: " +  _name +
                                           " in concept" + on.getConceptType());                
            }
        }
        return new NilConcept();
    }


    public void setStatic( boolean isStatic ) {
        _isStatic = isStatic;
    }


     public void setValue( Concept on, Concept value) {
        if( isStatic() ) {
            GlobalScope.getReference().insertGlobalInstance( _declaringConcept, "@" + _name, value);
            return;
        }

        if( on instanceof mn8Concept ) {
            ScopeStack conceptStack = ((mn8Concept)on).getScopeStack();
            conceptStack.insertInstance( "@" + _name, value);
        }
     }
    

    /**
     * Performs initialization of the _field with the value specified as
     * the initial value of the field. Assumes that the field is not null initially.
     * @return a <code>boolean</code> value of true if the field was initialized
     * and false other-way.
     */
    protected boolean initializeField(AttributeConcept field) {
        try {        
            if ( _initialValue != null ) {
                field.setAttributeValue((new mn8TreeWalker()).identOrConstant( _initialValue ));
                return true;
            }
            else
                return false;                             
        } catch (RecognitionException e) {
            e.printStackTrace();            
            return false;
        } // end of try-catch
    }

}
