/* 
 * $COPYRIGHT$
 * $Id: mn8ScriptElement.java,v 1.7 2002/06/05 14:57:16 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.event.*;
import org.media.mn8.concepts.*;
import org.media.mn8.parser.mn8TreeWalker;
import antlr.collections.AST;
import antlr.RecognitionException;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.7 $ $Date: 2002/06/05 14:57:16 $
 */
public class mn8ScriptElement extends FieldConcept {

    // These are inherited from FieldConcept
    // protected String _name = null;
    // protected boolean _isStatic = false;
    // protected String _label = null;
    // protected String _conceptType = null;
    // protected Concept _initialValue = null;
    // protected Field _field = null;

    protected String _declaringConcept = null;
    protected ElementConcept _field = null;
    protected AST _initialValue = null;
    protected boolean _multi = false;

    public mn8ScriptElement() {}


    public mn8ScriptElement ( String name,
                              ElementConcept field,
                              boolean isStatic,
                              String label,
                              String type,
                              String declaringConcept,
                              AST initial ) {
        _name = name;
        _declaringConcept = declaringConcept;
        _field = field;
        _isStatic = isStatic;
        _label = label;
        _conceptType = type;
        _initialValue = initial;
    }


    public mn8ScriptElement ( String name,
                              ElementConcept field,
                              boolean isStatic,
                              String label,
                              String type,
                              String declaringConcept,
                              AST initial,
                              boolean multi
                              ) {
        _name = name;
        _declaringConcept = declaringConcept;
        _field = field;
        _isStatic = isStatic;
        _label = label;
        _conceptType = type;
        _initialValue = initial;
        _multi = multi;
    }


    public boolean isMulti() {
        return _multi;
    }


    public Concept newInstance() {
        Concept result = null;
        if( _conceptType != null 
            && !Helper.isElement(mn8Loader.getConceptDefinition(_conceptType))) {
            ConceptDefinition concDef = mn8Loader.getConceptDefinition(_conceptType);
            if( _multi ) {
                result = new TransparentElement(_name, 
                                                _label, 
                                                null,
                                                null,
                                                _multi,
                                                _conceptType);                                                  
            } else {
                Concept tmpVal = mn8Loader.instantiateConcept(_conceptType);
                tmpVal.setConceptLabel( new StringConcept(_label));
                result = new TransparentElement(_name, 
                                                _label, 
                                                tmpVal,
                                                null,
                                                _multi,
                                                _conceptType);
            }
        } else {
            if( !_multi) {
                result=  new ElementConcept(_name, _label, null, null, _multi, _conceptType);
            } else {
                result=  new TransparentElement(_name, _label, null, null, _multi, _conceptType);
            }
        }
        result.setConceptLabel(new StringConcept(_label));
        result.setHidden( isHidden());
        result.setShowEmpty( showEmpty());
        initializeField( (ElementConcept) result );
        return result;
    }


    public boolean isAttribute() {
        return false;
    }
    

    public boolean isElement() {
        return true;
    }


    public Concept getValue( Concept on ) {
        if( isStatic() ) {
            return GlobalScope.getReference().getInstance( _declaringConcept + ":" + _name);
        }

        if( on instanceof mn8Concept ) {
            ScopeStack conceptStack = ((mn8Concept)on).getScopeStack();
            if( conceptStack.isInstance( _name) )
                return conceptStack.getInstance(  _name);
            else {
                throw new RuntimeException("Unamble to find element: " +  _name +
                                           " in concept" + on.getConceptType());                
            }
        }
        return new NilConcept();
    }


    public void setStatic( boolean isStatic ) {
        _isStatic = isStatic;
    }


    public void setValue( Concept on, Concept value) {
        if( isStatic() ) {
            GlobalScope.getReference().insertGlobalInstance( _declaringConcept,  _name, value);
            return;
        }
        if( on instanceof mn8Concept ) {
            ScopeStack conceptStack = ((mn8Concept)on).getScopeStack();
            conceptStack.insertInstance( _name, value);
        }
    }
    

    /**
     * Performs initialization of the _field with the value specified as
     * the initial value of the field. Assumes that the field is not null initially.
     * @return a <code>boolean</code> value of true if the field was initialized
     * and false other-way.
     */
    protected boolean initializeField() {
        return initializeField( _field );
    }


    protected boolean initializeField(ElementConcept field) {
        try {        
            if ( _initialValue != null ) {
                if( field.isMulti().getValue() ) {
                    field.addValue( (new mn8TreeWalker()).identOrConstant( _initialValue ));
                } else {
                    field.setElementValue((new mn8TreeWalker()).identOrConstant( _initialValue ));
                }
                return true;
            }
            return false;                             
        } catch (RecognitionException e) {
            e.printStackTrace();            
            return false;
        } // end of try-catch
    }

}
