/* 
 * $COPYRIGHT$
 * $Id: mn8ScriptMethod.java,v 1.33 2002/06/05 14:56:42 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8;


import org.media.mn8.event.*;
import org.media.mn8.concepts.*;
import org.media.mn8.parser.*;

import antlr.*;
import antlr.collections.*;
import antlr.debug.misc.*;

import java.util.Vector;
import java.util.StringTokenizer;

public class mn8ScriptMethod extends mn8JavaMethod {

    private mn8ConceptDefinition _declaringConcept = null;

    private AST _codeAST = null;

    private Vector _paramNames = null;

    public static ScopeStack _cmStack = null;

    public mn8ScriptMethod() {
        new ErrorConcept("error", "MethodCreationNotAllowed",
                         "Sorry you are not allowed to create a new method.");        
    }


    public mn8ScriptMethod ( String name,
                             boolean isStatic,
                             Vector paramTypes,
                             Vector paramNames,
                             mn8ConceptDefinition declaringConcept,
                             String returnType ) {
        this.name.setAttributeValue(new StringConcept( name ));
        _isStatic = isStatic;
        _paramTypes = paramTypes;
        _paramNames = paramNames;
        _declaringConcept = declaringConcept;
        super._declaringConcept = declaringConcept.getConceptType().toString();
        _returnType = returnType;
    }

    
    /**
     * Responsible of generating the mn8ScriptMethod from the script method def.
     * @param codeASTNode AST Node, probably if it is a concept def, will be the root node.
     * @param tokenNames the token names as returned by the parser.
     * @see antlr.Parser
     * @see antlr.collections.AST
     */
    public mn8ScriptMethod ( AST codeASTNode, String[] tokenNames ) {

    }


    public static void cleanAll() {
        _cmStack.clear();
    }


    public static boolean ismn8SrcriptMethod( AST codeASTNode, String[] tokenNames){
         return tokenNames[codeASTNode.getType()].equals("METHOD_DEFINITION");
    }


    public StringConcept getDeclaringConcept() {
        return _declaringConcept.getConceptType();
    }


    public Concept invoke( Concept on, SeriesConcept parameters ){
        ScopeStack _stack = ((mn8Concept) on).getScopeStack();
        Scope currentScope = new Scope( _stack.getConceptGlobalScope() );

        if ( ( parameters == null && _paramNames.size() > 0 ) || 
             ( parameters != null && _paramNames.size() != parameters.getVector().size() )) {
            throw new RuntimeException("Ilegal number of parameters method:" 
                                       + name.getAttributeValue());
        }

        if ( parameters != null )
            for (int i = 0 ; i < parameters.getVector().size() ; i ++) {

                currentScope.insertInstance( ((StringConcept) _paramNames.elementAt(i)).toString(), 
                                             (Concept) parameters.getVector().elementAt(i));
                
            }
        _stack.pushScope( currentScope );
        ExecutionStack.getReference().push( on.getConceptType().getValue(), getSignature(), on, lineNr );

        mn8InterpretThread interpr = mn8Interpreter.getInterpreter(Thread.currentThread());

        Concept result = null;

        if( interpr == null ) {
            result = mn8Interpreter.interpretAST( _codeAST, (mn8Concept)on, this, _stack );
        } else {
            result = interpr.interpretAST( _codeAST, (mn8Concept)on, this, _stack );
        }
        ExecutionStack.getReference().pop();
        _stack.popScope();
        
        // we have an constructor
        if( getName().toString().equalsIgnoreCase("create")) {
            return on;
        }

        if( hasReturn().getValue()) {
            return result;
        } 
        else 
            return new NilConcept();
    }


    public Concept invoke( String conceptType, SeriesConcept parameters ){	
        Concept result;
        if ( !conceptType.equals("_current_") ) {
            ScopeStack _stack =
                new ScopeStack( (mn8ConceptDefinition)mn8Loader.getConceptDefinition(conceptType) );
            Scope currentScope = _stack.getCurrentScope();
            
            if ( ( parameters == null && _paramNames.size() > 0 ) || 
                 ( parameters != null && _paramNames.size() != parameters.getVector().size() )) {
                throw new RuntimeException("Ilegal number of parameters method:" 
                                           + name.getAttributeValue());
            }
            if ( parameters != null )
                for (int i = 0 ; i < parameters.getVector().size() ; i ++) {
                    currentScope.insertInstance( ((StringConcept) _paramNames.elementAt(i)).toString(), 
                                                 (Concept) parameters.getVector().elementAt(i));
                    
                }
            ExecutionStack.getReference().push( conceptType, getSignature(), null, lineNr  );

            mn8InterpretThread interpr = mn8Interpreter.getInterpreter(Thread.currentThread());

            if( interpr == null ) {
                result = mn8Interpreter.interpretAST( _codeAST, null, this, _stack);                
            } else {
                result = interpr.interpretAST( _codeAST, null, this, _stack);
            }

            ExecutionStack.getReference().pop();
            _stack.popScope();
        }
        else {
            if ( mn8ScriptMethod._cmStack == null ) {
                mn8ScriptMethod._cmStack =
                    new ScopeStack( (mn8ConceptDefinition)mn8Loader.getConceptDefinition(conceptType) );
            }
            Scope currentScope = mn8ScriptMethod._cmStack.getCurrentScope();
            
            if ( ( parameters == null && _paramNames.size() > 0 ) || 
                 ( parameters != null && _paramNames.size() != parameters.getVector().size() )) {
                throw new RuntimeException("Ilegal number of parameters method:" 
                                           + name.getAttributeValue());
            }
            if ( parameters != null )
                for (int i = 0 ; i < parameters.getVector().size() ; i ++) {
                    currentScope.insertInstance( ((StringConcept) _paramNames.elementAt(i)).toString(), 
                                                 (Concept) parameters.getVector().elementAt(i));
                    
                }

            mn8InterpretThread interpr = mn8Interpreter.getInterpreter(Thread.currentThread());
            if( interpr == null ) {
                result = mn8Interpreter.interpretAST( _codeAST, null, this, mn8ScriptMethod._cmStack );
            } else {
                result = interpr.interpretAST( _codeAST, null, this, mn8ScriptMethod._cmStack);
            }
        }
        
        if( hasReturn().getValue() ) {
            return result;
        } 
        else 
            return new NilConcept();
    }



    public Concept invokeErrorHandler( String setplace, SeriesConcept parameters ) {     
        ScopeStack _stack = new ScopeStack( (mn8ConceptDefinition)mn8Loader.getConceptDefinition(setplace) );
        Scope currentScope = _stack.getCurrentScope();
        String ehName = ((StringConcept) _paramNames.elementAt(0)).toString(); 
        ErrorConcept errCon = (ErrorConcept) parameters.getVector().elementAt(0);
        currentScope.insertInstance( ehName, errCon );
        Concept result = mn8Interpreter.interpretAST( _codeAST, null, this, _stack );
        _stack.popScope();
        if( hasReturn().getValue()) {
            return result;
        } else {
            return new NilConcept();
        }
    }


    public Concept invokeErrorHandler( Concept setplace, SeriesConcept parameters ) { 
        ScopeStack _stack = ((mn8Concept) setplace).getScopeStack();
        Scope currentScope = new Scope( _stack.getCurrentScope() );
        String ehName = ((StringConcept) _paramNames.elementAt(0)).toString(); 
        ErrorConcept errCon = (ErrorConcept) parameters.getVector().elementAt(0);
        currentScope.insertInstance( ehName, errCon );
        _stack.pushScope( currentScope );
        Concept result = mn8Interpreter.interpretAST( _codeAST, (mn8Concept)setplace, this, _stack );
        _stack.popScope();
        if( hasReturn().getValue()) {
            return result;
        } else {
            return new NilConcept();
        }
    }


    public Concept getConceptInstance() {
        return new mn8ScriptMethod();
    }

    
    public static String getSignature(String methodName, Object[] paramTypes) {
        String result = methodName + ":";
        if( paramTypes != null )
            for( int i=0; i < paramTypes.length; i++) {
                if( i > 0 )             
                    result +=",";
                result += ((Concept)paramTypes[i]).toTXT().getValue();
            }

        return result;
    }

    
    public static String getSignatureFromVect(String methodName, Vector paramTypes) {
        String result = methodName + ":";
        if( paramTypes != null )
            for( int i=0; i < paramTypes.size(); i++) {
                if( i > 0 )             
                    result +=",";
                result += ((StringConcept)paramTypes.elementAt(i)).getValue();
            }

        return result;
    }
    

    public String getSignature() {
        String  _name = ((StringConcept)name.getAttributeValue()).getValue();
        if( _paramTypes != null )
            return mn8ScriptMethod.getSignature( _name, _paramTypes.toArray() );
        else 
            return mn8ScriptMethod.getSignature( _name, null );
    }

    
    public int hashCode() {
        return getSignature().hashCode();
    }


    public boolean equals( Object obj ) {
        if( obj instanceof mn8Method ) 
            return  getSignature().equals( ((mn8Method)obj).getSignature() );        
        else 
            return false;
    }


    public void setCodeAST( AST codeAST ) {
        _codeAST = codeAST;
    }


    public void addCodeASTPiece( AST pieceAST ) {
        if( _codeAST == null )
            _codeAST = pieceAST;
        else {           
            AST node = _codeAST;
            while ( node.getNextSibling() != null ){
                node = node.getNextSibling();
            } 
            node.setNextSibling( pieceAST ); 
        }
    }


    /**
     * The scripts made ad-hoc, without encapsulating any concept
     * definition we create a dummy concept <i>_current_</i>, and 
     * the code goes to a main dummy method, this one.
     * @return a value of type 'mn8ScriptMethod'
     */
    static mn8ScriptMethod makeMainMethod(mn8ConceptDefinition definition) {
        Vector paramTypes = new Vector();
        Vector paramNames = new Vector();
        paramTypes.add(new StringConcept("Series"));
        paramNames.add(new StringConcept("$args"));
        return new mn8ScriptMethod( "main",
                                    true,
                                    paramTypes,
                                    paramNames,
                                    definition,
                                    null );
    }
    

    public StringConcept toTXT() {
        StringBuffer buffer = new StringBuffer();
        if( _isStatic ) 
            buffer.append("STATIC ");
        else
            buffer.append("       ");
        
        buffer.append(name.getValue().toTXT().getValue());
        buffer.append(":");

        for( int i = 0 ; i < _paramTypes.size(); i++){
            if( i > 0 )             
                buffer.append(",");

            buffer.append(((StringConcept)_paramTypes.elementAt(i)).getValue());
        }
        if( _returnType != null || getName().toString().equalsIgnoreCase("create")) {
            buffer.append(" TYPEOF ");
            buffer.append(getReturnType().toString());
        }
        return new StringConcept(buffer.toString());
    }


    public StringConcept toXML() {
        StringBuffer buffer = new StringBuffer("<Method name=\"");

        buffer.append(name.toString());
        buffer.append("\" declaringConcept=\"" + _declaringConcept.getConceptType().toString() + "\" ");
    
        if( _isStatic ) 
            buffer.append("isStatic=\"true\" ");
        else
            buffer.append("isStatic=\"false\" ");

        if( _returnType == null || getName().toString().equalsIgnoreCase("create") ) 
            buffer.append(">" + mn8Interpreter.EOL);
        else
            buffer.append("returns=\"" + getReturnType().toString() + "\">" + mn8Interpreter.EOL);

        for( int i = 0 ; i < _paramTypes.size(); i++){
            buffer.append("<parameter name=\"" + _paramNames.elementAt(i) + "\" type=\"" + ((StringConcept)_paramTypes.elementAt(i)).getValue() + "\"/>" + mn8Interpreter.EOL);
        }

        buffer.append("</Method>");
        return new StringConcept(buffer.toString());
    }


    public Object clone() {
        return this;
    }
}
