/* 
 * $COPYRIGHT$
 * $Id: mn8XMLReader.java,v 1.5 2002/06/05 14:56:50 neuro Exp $
 *
 * Date        Author            Changes 
 * May 18 2001 Remus Pereni      Created
 */
package org.media.mn8;



import org.media.mn8.concepts.*;
import org.media.mn8.event.*;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.StringReader;
import java.io.IOException;
import java.util.Stack;
import java.util.EmptyStackException;

/**
 * Class that reads an String containing a well formated XML
 * document and creates an Concept from it's content.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.5 $ $Date: 2002/06/05 14:56:50 $
 */
public class mn8XMLReader extends DefaultHandler {

    private XMLReader _xmlReader = null;
    private ElementConcept _flexConcept = new ElementConcept();
    private Stack _elStack = new Stack();

    protected mn8XMLReader() {
        try {
            SAXParserFactory sxParserFactory = SAXParserFactory.newInstance();
            SAXParser sxParser = sxParserFactory.newSAXParser();
            _xmlReader = sxParser.getXMLReader();
            _xmlReader.setContentHandler(this);
        } catch ( ParserConfigurationException pce ) {
            new ErrorConcept( "warning", "XMLParseException", pce.getMessage() );
            if( mn8RuntimeFlags.isVerbose () ) {
                System.err.println(pce.getMessage());
            }
            if( mn8RuntimeFlags.isDebug () ) {
                pce.printStackTrace();
            }

        } catch ( SAXException saxe ) {
            new ErrorConcept( "warning", "XMLParseException", saxe.getMessage() );
            if( mn8RuntimeFlags.isVerbose () ) {
                System.err.println(saxe.getMessage());
            }
            if( mn8RuntimeFlags.isDebug () ) {
                saxe.printStackTrace();
            }
        }
    }

    public void parse(String value) throws SAXException, IOException {
        URL url = null;
        try {
            url = new URL(value);
        } catch (MalformedURLException mex) {
            ;
        }
        if( url != null ) {
            _xmlReader.parse((new InputSource( url.toString())));
        } else {
            _xmlReader.parse((new InputSource( new StringReader(value))));
        }
    }


    public ElementConcept getFlexFromXML(StringConcept xmlDocument) {
        try {            
            parse( xmlDocument.toString() );
            return _flexConcept;
        } catch ( SAXException saxe ) {
            new ErrorConcept( "warning", "XMLParseException", saxe.getMessage() );
            if( mn8RuntimeFlags.isVerbose () ) {
                System.err.println(saxe.getMessage());
            }
            if( mn8RuntimeFlags.isDebug () ) {
                saxe.printStackTrace();
            }            
        } catch ( IOException iox ) {
            if( mn8RuntimeFlags.isVerbose () ) {
                System.err.println(iox.getMessage());
            }
            if( mn8RuntimeFlags.isDebug () ) {
                iox.printStackTrace();
            }
        }
        return null;
    }


    public void startDocument() throws SAXException {
    }


    public void endDocument() throws SAXException {
    }


    public void startElement(String uri, String localname, String qname, Attributes attributes) throws SAXException {        
        AttributeConcept attr = null;
        ElementConcept element = null;
        SeriesConcept attribs = new SeriesConcept();

        for( int i = 0 ; i < attributes.getLength(); i++) {
            attr = new AttributeConcept();
            attr.setAttributeName( attributes.getQName(i));
            attr.setAttributeLabel( attributes.getQName(i));
            attr.setAttributeValue(new StringConcept(attributes.getValue(i)));
            attribs.add( attr );
        }
        
        try {
            if( _elStack.peek() instanceof ElementConcept ) {
                element = new ElementConcept( qname, qname, null, attribs);
                ((ElementConcept)_elStack.peek()).addValue(element);
            } 

            _elStack.push( element );                
        } catch ( EmptyStackException see ) {
            _flexConcept.setLabel( new StringConcept( qname ));
            for( int i = 0 ; i < attributes.getLength(); i++) {
                _flexConcept.addAttribute( new StringConcept(attributes.getQName(i)),
                                           new StringConcept(attributes.getQName(i)),
                                           new StringConcept(attributes.getValue(i)));
            }
            _elStack.push( _flexConcept );
        }
            
    }


    public void endElement(String uri, String localname, String qname) throws SAXException {
        _elStack.pop();
    }


    public void characters(char[] ch, int start, int length ) throws SAXException {
        StringBuffer value = new StringBuffer(new String( ch, start, length));
        if( value.toString().trim().length() == 0) return;

        if( !mn8Interpreter.EOL.equals("\n") ) {
            int idx = value.toString().indexOf('\n');
            while( idx != -1 && idx < value.length()) {
                value.replace(idx, idx+1, mn8Interpreter.EOL);
                idx = value.toString().indexOf('\n', idx+ mn8Interpreter.EOL.length());
            }
        }

        if( _elStack.peek() instanceof ElementConcept ) {
            ElementConcept currElem = (ElementConcept)_elStack.peek();
            if( currElem.isMulti().getValue() ) {
                Concept lastElem = (Concept) ((SeriesConcept)currElem.getValue()).getVector().lastElement();
                
                if( lastElem instanceof StringConcept) {                        
                    ((SeriesConcept)currElem.getValue()).getVector().set( ((SeriesConcept)currElem.getValue()).getVector().size(),
                                                                          new StringConcept( ((StringConcept)lastElem).toString() +
                                                                                             value.toString()));
                } else {
                    appendString(currElem, value.toString());
                }                
            } else {
                appendString( currElem, value.toString());
            }
        }            
    }


    public void ignorableWhitespace (char[] ch, int start, int length ) throws SAXException {
    }

    public void processChar(String val) {
        for( int i = 0 ; i < val.length(); i++ ) {
            if( val.charAt(i) == '\r' ) {
                System.err.print("\\r");
            }else if( val.charAt(i) == '\n' ) {
                    System.err.print("\\n");
            } else {
                    System.err.print("" + val.charAt(i));
            }
        }
    }


    void appendString(ElementConcept toElem, String value) {
        if( toElem.getValueType().toString().equalsIgnoreCase("string") ) {
            toElem.setValue( new StringConcept( ((StringConcept)toElem.getValue()).toString() + value));
        } else {
            toElem.addValue(new StringConcept(value));
        }
    }
}
