/*
 * $COPYRIGHT$
 * $Id: BeepURLConnection.java,v 1.2 2002/01/24 17:57:07 crow Exp $
 *
 * Date           Author         Changes
 * sept 28 2001  Antal Attila    Created
 * sept 28 2001  Szabo Csaba     Created
 */

package org.media.mn8.protocol.beep;

import java.io.*;
import java.net.*;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.security.Security; 

import org.beepcore.beep.util.*;
import org.beepcore.beep.core.*;
import org.beepcore.beep.lib.Reply; 
import org.beepcore.beep.transport.tcp.*;
import org.beepcore.beep.profile.ProfileConfiguration;
import org.beepcore.beep.profile.tls.TLSProfile;
import org.beepcore.beep.profile.echo.EchoProfile;
import org.beepcore.beep.profile.sasl.otp.SASLOTPProfile;
import org.beepcore.beep.profile.sasl.otp.algorithm.md5.*;
import org.beepcore.beep.profile.sasl.otp.algorithm.sha1.*;
import org.beepcore.beep.profile.sasl.otp.database.UserDatabasePool; 

 /**
 * This class implements a connection to the new "beep"
 * URL scheme.
 */

class BeepURLConnection extends URLConnection {
    private static final String DEF_HOST     = "localhost";
    private static final    int DEF_PORT     = 10288;
    private static final String DEF_USER     = "guest";
    private static final String DEF_PASSWORD = "guest";
    private static String URI_PROPERTY = "mn8.protocol.beep.uri";
    private static String SECURITY = "none";
    
    private final String  hashMethod = "otp-md5";
    private final String        seed = "xyz";
    private final int            seq = 1;

    private Channel     channel = null;    
    private BEEPOutputStream os = null;

    private URL url;


    /**
    * Given a URL will instantiate a BeepURLConnection
    * object.
    * @param getURL The URL to contact.
    */
    BeepURLConnection( URL getURL ) {
	super( getURL );
	try {
	    url = new URL( URLDecoder.decode( getURL.toString() ) );
	}
	catch ( Exception e) {
	    System.out.println("Can't decode the URL: " + getURL.toString() );
	}
    }


    public void connect() throws IOException {
	String host    = host();
	int    port    = port();
	String user    = user();
	String pwd     = pwd();
	String file    = file();
	String profile = profile( file );
	String path    = path( file );
	String dbName  = dbName();
	String URI     = loadURI( profile );
	setVariablesFromQuery();

 	if ( URI == null ) throw new IOException("\nProfile: " + profile +
		            " without URI specification!\n"+
			    "To set the URI use the following system property:\n\t"+ 
			    URI_PROPERTY + "=\"profile1=URI1|profile2=URI2| ...\"\n");
	else getConnection( host, port, user, pwd, profile,
			    dbName, path, URI );
    }


    private String host() {
        if ( url.getHost().length() == 0 ) return DEF_HOST;
        else return url.getHost();
    }


    private int port() {
        if ( url.getPort() < 1 ) return DEF_PORT;
        else return url.getPort();

    }


    private String file() throws IOException {
	String file = url.getFile();
        if ( file.equals("/") || file.trim().length() < 1 )
	     throw new IOException( "Missing profile parameter from URL" );
        else {
	    if ( file.indexOf("?") < 0 ) file = file.substring(1);
	    else file = file.substring( 1, file.indexOf("?") );
	}
	return file;
    }


    private String profile( String file ) {
	if ( file.indexOf("/") > 0 ) return file.substring( 0, file.indexOf("/") );
	else return file;
    }


    private String path( String file ) {
	if ( file.indexOf("/") > 0 ) return file.substring( file.indexOf("/")+1 );
	else return null;
    }


    private String user() {
	String user;
	String ui = url.getUserInfo();
        if ( ui == null || ui.trim().length() < 1 ) user = DEF_USER;
	else {
	    if ( ui.indexOf(":") != -1 ) user = ui.substring( 0, ui.indexOf(":") );
	    else user = ui;
	    if ( user.trim().length() < 1 ) user = DEF_USER;
        }
	return user;
    }


    private String pwd() {
	String password;
	String ui = url.getUserInfo();
        if ( ui == null || ui.trim().length() < 1 ) password = DEF_PASSWORD;
	else {
	    if ( ui.indexOf(":") != -1 ) password = ui.substring( ui.indexOf(":")+1 );
	    else password = DEF_PASSWORD;
	    if ( password.trim().length() < 1 ) password = DEF_PASSWORD;
        }
	return password;
    }


    private String dbName() {
        if ( url.getRef() == null || url.getRef().trim().length() == 0 ) return null;
        else return url.getRef();
    }


    private void setVariablesFromQuery() throws IOException {
	String qu = url.getQuery();
        if ( qu != null) if (qu.trim().length() != 0 ) {
	    StringTokenizer pt = new StringTokenizer( qu, "&" );
	    String t, t_name, t_val;
	    try {
		while (pt.hasMoreTokens()) {
		    t = pt.nextToken();
		    t_name = t.substring( 0, t.indexOf("=") );
		    t_val  = t.substring( t.indexOf("=")+1 );
		    if ( t_name.equalsIgnoreCase("security") ) SECURITY = t_val;
		}
	    }
	    catch ( Exception e ) {
		throw new IOException( "\n\tInvalid query attributes in: " + qu );
	    }
	}
    }


    private String loadURI( String profile ) throws IOException {
	Hashtable ut = BeepURITable.profiles();
	String pURI = System.getProperty( URI_PROPERTY );
	if ( pURI != null ) {
	    StringTokenizer st = new StringTokenizer( pURI, "|" );
	    String t, t_name, t_uri;
	    try {
		while (st.hasMoreTokens()) {
		    t = st.nextToken();
		    t_name = t.substring( 0, t.indexOf("=") );
		    t_uri  = t.substring( t.indexOf("=")+1 );
		    ut.put( t_name, t_uri );
		}
	    }
	    catch ( Exception e ) {
		throw new IOException( "\n\tInvalid property keys in: " + 
				       URI_PROPERTY );
	    }
	}
	return (String) ut.get( profile ); 
    }


    public synchronized InputStream getInputStream() throws IOException {
	if ( channel == null ) connect();
	return os.getInputStream();
    }


    public synchronized OutputStream getOutputStream() throws IOException {
	if ( channel == null ) connect();
	if (channel != null) {
	    os = new BEEPOutputStream( channel );
	    return os;
	}
	return null;
    }

    
    
    private void getConnection( String host, int port,
				String user, String password,
				String profile, String dbName,
				String path, String URI ) throws IOException {

	channel = openChannel( host, port, user, password, profile, dbName, URI );
	if ( channel == null )
            throw new IOException("Can't initialize the profile: " + profile);
	else if ( profile.equals("sep") && dbName != null ) {
	    try {
		Reply reply= new Reply();
		Message result  = null;
		channel.sendMSG( new StringDataStream("SEP:" + dbName), reply );
		result = reply.getNextReply();
		if ( result != null && result.getMessageType() == Message.MESSAGE_TYPE_ERR )
	            throw new RuntimeException( "Connection failed to " + dbName );
	    }
	    catch ( Exception e ) {
		throw new IOException("Connection failed to " + dbName);
	    }
	    if (path != null) answerForURLSubtreePath( path );
	}
    }

    private void answerForURLSubtreePath( String path ) throws IOException {
	String fetch   = "<request reqno='1'><fetch><union><intersect>" +
	                 "<compare subtree='" + path + "'><path/>" +
    	                 "<value></value></compare>" +
                         "</intersect></union></fetch></request>";

	String pathType = decideThePathType( path );
	try {
	    PrintStream pstream = new PrintStream( getOutputStream() );
	    if ( pathType.equals("document") ) pstream.println( fetch );
	    if ( pathType.equals("tree") ) pstream.println( listTree( path, "false") );
	    pstream.flush();
	}
	catch ( Exception e) {
	    throw new IOException( e.getMessage() ); 
	}
    }


    private String decideThePathType( String path ) throws IOException {
	sendRequest( listTree( path, "true" ) );
	String pathType = readResponse();
	StringTokenizer pt = new StringTokenizer( pathType, ">" );
	String t = pt.nextToken(); t = pt.nextToken(); t = pt.nextToken(); t = pt.nextToken();
	t = t.substring( 0, 1 );
	if ( t.equals("-") ) return "document";
	if ( t.equals("t") ) return "tree";
	throw new IOException ( "Invalid subtree or document name: " + path );
    }

    
    private void sendRequest( String sep ) throws IOException {
	try {
	    PrintStream pstream = new PrintStream( getOutputStream() );
	    pstream.println( sep );
	    pstream.flush();
	}
	catch ( Exception e) {
	    throw new IOException( "Error while deciding subtree type: " + e.getMessage() ); 
	}
    }


    private String readResponse() throws IOException {
	String resp = "";
	String nextLine;
	try {
	    BufferedReader input = new BufferedReader( new InputStreamReader( getInputStream() ) );
	    while( ( nextLine = input.readLine() ) != null)
		resp += nextLine.trim();
	    input.close();
	}
	catch ( Exception e) {
	    throw new IOException( "Error while reading response: " + e.getMessage() ); 
	}
	return resp;
    }
    

    private String listTree( String path, String isTree ) {
	String ret = "<request reqno='1'><store action='update'>" +
	    "<block name='_admin.trees'>" +
	    "<tree action='list' long='true' tree='" + isTree + "'>" +
	    "<subtree>" + path + "</subtree>" +
	    "</tree></block></store></request>";
	return ret;
    }



    private Channel openChannel( String host, int port,
                        	 String user, String password,
			         String profile, String dbName,
                                 String URI ) throws IOException {
	
	if ( profile.equals("sep") && dbName == null )
	    throw new IOException("Can't initialize the session: Database name missing." );
										 
	try {
	    return initChannel(
                      initSession( host, port, user, password, dbName ), URI );
	}
	catch ( Exception e ) {
	    throw new IOException("Can't initialize the session: " + e.getMessage() );
	}
    }


    private Channel initChannel( Session ses, String URI ) throws Exception {
	Channel chan = null;
	  
	if ( ses == null ) return null;
	try {
	    chan = ses.startChannel( URI );
	}
	catch ( Exception e ) {
	    throw e;
         }
         return chan;
    }
    
    
    private Session initSession( String host, int port,
                                 String user, String password,
                                 String dbName ) throws Exception {
	Session ses;
	try {
	    ses = AutomatedTCPSessionCreator.initiate(host, port, new ProfileRegistry());
	    ses = tls( ses, SECURITY );
	    ses = authenticate( ses, user, password, dbName );
	}
	catch ( Exception e ) {
	    throw e;
	}
												     
        return ses;
    }


    private String preDigest( String pwd ) throws Exception {
	return SASLOTPProfile.convertBytesToHex(
	    ( new MD5() ).generateHash( pwd.getBytes() ) );
    }

										       
    private String getPasswdHash( String pwd ) throws Exception {
	byte[] pw = ( seed + pwd ).getBytes();
					   
	try {
	    for ( int cx = 0; cx <= seq; cx++ ) {
		if ( hashMethod.equals( "otp-md5" ) )
		    pw = ( new MD5() ).generateHash( pw );
		else if ( hashMethod.equals( "otp-sha1" ) )
		    pw = ( new SHA1() ).generateHash( pw );
		else
		    throw new Exception( "\nInvalid algorithm method:"+ hashMethod + "\n");
	    }
	}
	catch ( Exception e ) {
	    throw e;
	}
	
	return SASLOTPProfile.convertBytesToHex( pw );
    }

    private Session authenticate ( Session sess, String user,
                                   String  pwd, String dbName ) throws Exception {
	SASLOTPProfile otp = new SASLOTPProfile();
	if (dbName != null) {
	    try {
		String newHash = getPasswdHash( pwd = preDigest( pwd ) );
		otp.init( SASLOTPProfile.URI, new ProfileConfiguration() );
		( new UserDatabasePool() ).addUser( user, hashMethod, newHash, seed, seq + "" );
		sess = otp.AuthenticateSASLOTPWithInit(
		       sess, dbName + "_", user, pwd , hashMethod, newHash, seed, seq + "" );
	    }
	    catch ( Exception e ) {
		sess = null;
		throw e;
	    }
        }
	return sess;
    }    
    
    
    private Session tls( Session tlsSes, String privacy) throws Exception {
        /*
	if ( privacy.trim().length() > 0 ) {
	    Security.addProvider( new com.sun.net.ssl.internal.ssl.Provider() );
	    if ( "jsse".equals( privacy.toLowerCase() ) ) {
		try {
		    TLSProfile tlsProfile =
			TLSProfile.getInstance("org.beepcore.beep." +
			                       "profile.tls.jsse.TLSProfileJSSE");
			tlsSes = tlsProfile.startTLS( (TCPSession)tlsSes );
		}
		catch ( Exception e ) {
		    throw new Exception( "JSSE TLS connection not supported by server!" );
		}
	    }
	    else {
		if ( "ptls".equals(privacy.toLowerCase()) ) {
		    try {
			TLSProfile tlsProfile = TLSProfile.getDefaultInstance();
			tlsSes = tlsProfile.startTLS( (TCPSession)tlsSes );
		    }
		    catch ( Exception e1 ) {
			throw new Exception( "Pure TLS connection not supported by server!" );
		    }
		}
	    }
	}
	*/
        
	return tlsSes;
    }
    
    
    private class BEEPOutputStream extends OutputStream {
	private CharInputStream  cis;
	private Channel          channel;
	private InputStream      istream;	

	public BEEPOutputStream( Channel ch ) {
	    channel = ch;
	    cis     = new CharInputStream();
	    istream = null;
    	}
	

	public void write( int b ) throws IOException {
	    cis.write( b );
	}
	

	public void flush() throws IOException {
	    Reply reply = new Reply();
	    
	    try {
		channel.sendMSG( new InputStreamDataStream( cis.getInputStream(),
							    cis.getInputStream().available()),
				 reply);
		istream = reply.getNextReply().getDataStream().getInputStream(); 
		cis.flush();
	    }
	    catch ( Exception e ) {
		throw new IOException( e.getMessage() );
	    }
	}

	
	public InputStream getInputStream() {
	    return istream;
	}
    }


    private class CharInputStream {
  
	private StringWriter  writer;
      
      
        public CharInputStream() {
	    writer = new StringWriter();
	}
		      
		      
	public void write( int ch ) {
	    writer.write( ch );
	}
				      
			      
	public void flush() {
	    close();
	    writer = new StringWriter();
	}
							      
							      
	public InputStream getInputStream() {
	    close();
	    return new ByteArrayInputStream( writer.toString().getBytes() );
	}										      
										      
	
	private void close() {
	    try {
		writer.close();
	    }
	    catch ( Exception e ) {
	    }
	}
    }
}
