/*
 * $COPYRIGHT$
 * $Id: EnvURLConnection.java,v 1.1 2002/02/07 10:24:54 crow Exp $
 *
 * Date           Author          Changes
 * FEB 08 2002    Szabo Csaba     Created
 */

package org.media.mn8.protocol.env;

import java.io.*;
import java.net.*;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.security.Security; 

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.NameClassPair;

import org.media.mn8.NamingManager;
import org.media.mn8.concepts.StringConcept; 

 /**
 * This class implements a connection to the new "env"
 * URL scheme.
 */

class EnvURLConnection extends URLConnection {

    private URL url;
    private String path;
    private StringBuffer result = new StringBuffer();

    /**
    * Given a URL will instantiate a EnvURLConnection object.
    * @param getURL The URL to contact.
    */
    EnvURLConnection( URL getURL ) {
	super( getURL );
	try {
	    url = new URL( URLDecoder.decode( getURL.toString() ) );
	}
	catch ( Exception e) {
	    System.out.println("Can't decode the URL: " + getURL.toString() );
	}
    }


    public void connect() throws IOException {
	getConnection( path = file() );
    }


    private String file() throws IOException {
	String file = url.getFile();

        if ( file.equals("/") || file.trim().length() < 1 )
	     throw new IOException( "Missing path parameter from URL" );
        else {
	    if ( file.indexOf("?") < 0 ) file = file.substring(1);
	    else file = file.substring( 1, file.indexOf("?") );
	}

	return file;
    }


    public synchronized InputStream getInputStream() throws IOException {
        if ( !connected ) connect();
        return new ByteArrayInputStream( result.toString().getBytes() );
    }


    private void getConnection( String path ) throws IOException {
	try {
	    String value = ((StringConcept)NamingManager.getReference().lookup( path )).toString();
	    result.append( value );
	} 
	catch ( Exception ex ) {	
	    try {
		for ( NamingEnumeration ne = NamingManager.getReference().list( path ); ne.hasMore(); ) {
		    Object obj = ne.next();

		    if ( obj instanceof NameClassPair ) {
			String _path = ( path.endsWith("/") ? path : path + "/" ) + 
			               ((NameClassPair)obj).getName() + "/";
			result.append( _path + "\n" );
		    }
		    else if ( obj instanceof String )
			result.append ( (String)obj + "\n" ); 
		}
	    }
	    catch ( NamingException nex ) {
		throw new IOException ( nex.getMessage() );
	    }
	}
    }
}

