/* 
 * $COPYRIGHT$
 * $Id: Cookie.java,v 1.5 2002/06/03 10:55:17 atech Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.protocol.http;

import java.util.StringTokenizer;
import java.util.Date;
import java.util.Locale;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.sql.Timestamp;
import java.io.File;
import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.net.URL;
import java.util.Vector;
import java.util.Enumeration;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.5 $ $Date: 2002/06/03 10:55:17 $
 */
public class Cookie {
    private String name;
    private String value;
    private String domain;
    private String path;
    private String expires;
    private String secure;


    public Cookie() {
        this.name    = "";
        this.value   = "";
        this.domain  = "";
        this.path    = "";
        this.expires = "";
        this.secure  = "";
    }


    public Cookie ( String acookie , String _domain ) {
        if ( acookie == null || acookie.trim().length() < 1 ) 
            throw new IllegalArgumentException("Cookie string is not valid!!!");
	
        StringTokenizer st = new StringTokenizer( acookie , ";" );
        boolean   firstArg = true;
        Cookie      cookie = new Cookie();
	
        domain  = _domain;
        path    =  "";
        expires = System.currentTimeMillis() + "";
        secure  = "FALSE";
	
        while ( st.hasMoreTokens() ) {
            String token = st.nextToken(";");
	    
	    
            if ( firstArg ) {
                int poz = token.indexOf("=");
		
                if ( poz != -1 ) {
                    name  = token.substring(0, poz );
                    value = token.substring(poz + 1);
                }
                else {
                    name = token; value = "";
                }
                firstArg = false;
            }
            else {
                int poz = token.indexOf("=");
		
                if ( poz == -1 ) secure = token;
                else {
                    String key = token.substring(0 , poz).trim();
                    String val = token.substring(poz + 1).trim();
		
                    if ( key.equalsIgnoreCase("domain") )  domain = val;
                    if ( key.equalsIgnoreCase("path") )    path = val;
                    if ( key.equalsIgnoreCase("expires") ) expires = makeTimeStamp( val ) + "";
                } 
            }
        }
	
        if ( domain == null ) 
            throw new IllegalArgumentException("Domain argument is not valid!!!");
    }    


    public void setName( String name ) {
        this.name = name;
    }


    public void setValue( String value ) {
        this.value = value;
    }


    public void setDomain( String domain ) {
        this.domain = domain;
    }


    public void setPath( String path ) {
        this.path = path;
    }


    public void setExpires( String expires ) {
        this.expires = expires;
    }


    public void setSecure( String secure ) {
        this.secure = secure;
    }


    public String getName() {
        return name;
    }


    public String getValue() {
        return value;
    }


    public String getDomain() {
        return domain;
    }


    public String getPath() {
        return path;
    }


    public String getExpires() {
        return expires;
    }


    public String getSecure() {
        return secure;
    }


    public String toXMLString() {
        return  "<?xml version=\"1.0\"?>\n" +
            "<cookie version=\"0\">\n" +
            "    <domain>"  + normalize( domain )  + "</domain>\n"  +
            "    <path>"    + normalize( path )    + "</path>\n"    +
            "    <secure>"  + normalize( secure )  + "</secure>\n" +
            "    <expires>" + normalize( expires ) + "</expires>\n" +
            "    <name>"    + normalize( name )    + "</name>\n"    +
            "    <value>"   + normalize( value )   + "</value>\n"   +
            "</cookie>";
    }
    

    public String toString() {
        return  "Name   : " + name + "\n" +
            "Value  : " + value + "\n" +
            "Domain : " + domain + "\n" +
            "Path   : " + path + "\n" +
            "Expires: " + expires + "\n" +
            "Secure : " + secure + "\n";
    }
    
    
    private String normalize( String str ) {
        StringBuffer sb = new StringBuffer();
        int len = ( str != null )  ? str.length() : 0;
	
        for ( int i = 0; i < len; i++ ) {
            char ch = str.charAt(i);
	    
            switch ( ch ) {
            case  '<': sb.append("&lt;");
                break;
            case  '>': sb.append("&gt;");
                break;
            case  '&': sb.append("&amp;");
                break;
            case  '"': sb.append("&quot;");
                break;
            case '\'': sb.append("&#");
                sb.append(Integer.toString("'".charAt(0)));
                sb.append(";");
                break;
            default  : sb.append(ch);
            }
        }

        return sb.toString();
    }
	
    
    private long makeTimeStamp( String val ) {

        try {
            DateFormat df = new SimpleDateFormat( "EEE, dd-MMM-yyyy HH:mm:ss zzz" , Locale.US );
            return df.parse( val ).getTime();
        }
        catch ( ParseException pe ) {
        }

        try {
            DateFormat df = new SimpleDateFormat( "EEE, dd MMM yyyy HH:mm:ss zzz" , Locale.US );
            return df.parse( val ).getTime();
        }
        catch ( ParseException pe ) {
        }
	
        System.err.println("Unparseable date: " + val );
        return System.currentTimeMillis();
    }


    public String getCookieString( String uri, String xmlDir, boolean validate ) {
        String cookieString = "";
        Vector cookies = readCookiesFromXML( uri , xmlDir, validate );
        if ( cookies != null ) {
            for ( Enumeration en = cookies.elements(); en.hasMoreElements(); ) {
                Cookie cookie = (Cookie)en.nextElement();
                cookieString += ";" + cookie.getName() + "=" + cookie.getValue();
            }
            if ( cookieString.length() > 0 ) cookieString = cookieString.substring(1);
        }
	
        return cookieString;
    }


    public Vector readCookiesFromXML( String uri, String xmlDir, boolean validate ) {
        Vector      result = new Vector();
        File       xmlPath = null;
        File[] cookieFiles = null;
        String      secure = "FALSE";
        String  timeString = Long.toString( System.currentTimeMillis() );
        String      domain = "";
        String        path = "";
		
	
        try {
            URL url = new URL( uri );
            domain = url.getHost();
            path   = url.getPath();
            if ( url.getProtocol().equalsIgnoreCase("https") ) secure = "TRUE"; 
            xmlPath = new File( xmlDir );
            if ( !xmlPath.exists() ) return null;

            Vector domains = new Vector();
            domains.addElement(domain);

            StringTokenizer st = new StringTokenizer(domain, ".");
            int dots = st.countTokens()-1;
            if (!uri.toLowerCase().startsWith("www") && dots == 2) {
                domains.addElement( domain.substring(domain.indexOf('.')+1) );
            }
            for (int cx=0; cx <domains.size(); cx++) {
                cookieFiles = 
                    xmlPath.listFiles( new CookieFileFilter((String)domains.elementAt(cx)));
                while ( true ) {
                    if ( !path.endsWith("/") ) path += "/";
                    for ( int i = 0; i < cookieFiles.length; i++ ) {
                        Cookie cookie =
                            ( new CookieLoader() ).getCookie(
                                 new FileInputStream( cookieFiles[i] ), new Cookie() ); 

                        if ( cookie.getSecure().equalsIgnoreCase( secure ) ) {
                            String _path =
                                ( cookie.getPath() != null ) ? cookie.getPath() : "/";
			
                            if ( !_path.endsWith("/") ) _path += "/";
                            if ( path.equalsIgnoreCase( _path ) ) {
                                if ( !validate ) result.addElement( cookie );
                                else {
                                    String expires = cookie.getExpires();
                                    if ( new Long( timeString ).longValue() < new Long( expires ).longValue() )
                                        result.addElement( cookie );
                                }
                            }
                        }
                    }
                    if ( path.length() < 2 ) break;
                    path = path.substring(0, path.length() - 2 );
                    path = path.substring(0, path.lastIndexOf("/") + 1 );
                }
            }
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return null;
        }    
	
        return result;
    }


    public void writeCookiesToXML( String domain, Vector cookies, String dest ) {
        String fName = dest + File.separator + domain;
	
        if ( !( new File( dest ) ).exists() ) new File( dest ).mkdirs();
	
        for ( Enumeration e = cookies.elements(); e.hasMoreElements(); ) {
            Cookie cookie = (Cookie)e.nextElement();
            int  cookieNr = getCookieNr( dest, domain );
	    
            try {
                PrintWriter writer = new PrintWriter( new BufferedWriter (
                                                                          new FileWriter( fName + "[" + cookieNr + "].xml") ) );
                writer.println( cookie.toXMLString() );
                writer.close();
            }
            catch ( Exception ex ) {
                ex.printStackTrace();
            }	    
        }
    }


    public javax.servlet.http.Cookie getJavaCookie() {
        javax.servlet.http.Cookie cook =  new javax.servlet.http.Cookie( getName(), "");
        if ( ! getExpires().equals("") ) {
            try {
                cook.setMaxAge( (new Integer(getExpires())).intValue());
            } catch (NumberFormatException ex) {
                System.err.println("Warning! Cookie: expires must me a numerical value, ignoring");
            }
        }

        if ( ! getDomain().equals("") ) {
            cook.setDomain( getDomain());
        }

        if ( ! getPath().equals("") ) {
            cook.setPath( getPath());
        }


        if ( ! getValue().equals("") ) {
            cook.setValue( getValue());
        }

        if ( ! getSecure().equals("") ) {
            cook.setSecure( (new Boolean(getSecure())).booleanValue());
        }

        return cook;
    }

    
    private int getCookieNr( String dest, String domain ) {
        int       nr = 1;
        String fName = dest + File.separator + domain;
	
        while ( new File( fName + "[" + nr + "].xml").exists() ) nr++;
	
        return nr;
    }


    private class CookieFileFilter implements FilenameFilter {
	
        private String _filterName = null;
	
	
        public CookieFileFilter( String fName ) {
            _filterName = fName.toLowerCase();
        }  
	
	
        public boolean accept( File dir, String name ) {
            if ( name != null ) {
                String _name = name.trim().toLowerCase();
                int      poz = -1;
		
                if ( ( poz = _name.indexOf("[") ) != -1 ) _name = _name.substring( 0, poz );
                if ( _name.startsWith(".") ) _name = _name.substring(1);

                if ( _filterName.equals( _name ) ) return true; else return false;
            }
            else return false;
        }
    } 

    public boolean equals (Object obj) {
        if( obj instanceof Cookie ) {
            Cookie cook = (Cookie)obj;
            if ( cook.getName().equalsIgnoreCase(name)       &&
                 cook.getValue().equalsIgnoreCase(value)     &&
                 cook.getDomain().equalsIgnoreCase(domain)   &&
                 cook.getPath().equalsIgnoreCase(path)       &&
                 cook.getExpires().equalsIgnoreCase(expires) &&
                 cook.getSecure().equalsIgnoreCase(secure) ) return true;
        }
        return false;
    }


}
