/* 
 * $COPYRIGHT$
 * $Id: DefaultAttributes.java,v 1.1 2002/01/31 20:34:35 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 29 2002 Szabo Csaba       Created
 */

package org.media.mn8.protocol.http;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/01/31 20:34:35 $
 */

public class DefaultAttributes {
    
    private static final String attrNames[] = {
	     "id", "class", "style", "title", //core attribute name
	     "lang", "dir", //i18n name
	     "onclick", "ondblclick", "onmousedown", "onmouseup",
	     "onmouseover", "onmosemove", "onmouseout",
	     "onkeypress", "onkeydown", "onkeyup" //events name
	    };
				    
    private String attrList = "";


    public void parse( String tag ) {
	if ( tag == null || tag.trim().length() < 1 ) return;
	
	String attrs = getAttrsFromTag( tag );
	
	if ( attrs.trim().length() < 1 ) return;
	
	for ( int i = 0; i < attrNames.length; i++ ) {
	    String value = getValueOf( attrs, attrNames[i] );
	    
	    if ( value != null ) attrList += getAttrPair ( attrNames[i], value ); 
	} 
    }


    public String getAttrList() {
	return attrList;
    }
    
    
    protected String getAttrsFromTag( String tag ) {
	String attrs = tag.replace('<',' ').replace('>',' ').trim();
	
	if ( attrs.endsWith("/") ) attrs = attrs.substring(0, attrs.length() - 2 );
	attrs = attrs.substring( attrs.indexOf( " " ) != -1 ?  attrs.indexOf( " " ) : attrs.length() );
	
	return attrs + " ";
    }


    protected String getValueOf( String tag, String attrName ) {
	int       poz = 0;
	String result = null;
	
	while ( ( poz = tag.toLowerCase().indexOf( attrName, poz ) ) != -1 ) {
	    if ( isAttrName( tag, poz - 1, attrName ) ) {
		result = getAttrValue( tag, poz + attrName.length() );
		break;
	    } 
	    poz += attrName.length();
	}
	
	return result == null ? null : result.trim();
    }
    
    
    protected String getAttrPair( String aname, String attr ) {
	String ret = "";
	
	if ( attr != null ) ret = aname + "=\"" + normalize( attr ) + "\" ";
	
	return ret;
    }
    
    
    private boolean isWord( String tag, int start, int end ) {
	if ( !Character.isWhitespace( tag.charAt( start ) ) ) return false;
	if ( !Character.isWhitespace( tag.charAt( end ) ) && tag.charAt( end ) != '=' ) return false;
	return true;
    }
    
    
    private boolean isAttrName( String tag, int poz , String attrName ) {
	boolean foundWord = false;
	
	if ( poz > 0 && !isWord( tag, poz, poz + attrName.length() + 1 ) ) return false;
	
	while ( poz > 0 && tag.charAt( poz ) != '"' && tag.charAt( poz ) != '\'' && tag.charAt( poz ) != '=' )
	    if ( !Character.isWhitespace( tag.charAt( poz-- ) ) ) foundWord = true;
	
	if ( poz > 0 && tag.charAt( poz ) == '=' && !foundWord ) return false;
	
	if ( poz > 0 && tag.charAt( poz ) == '"' || tag.charAt( poz ) == '\'' ) {
	    while ( poz > 0 && Character.isWhitespace( tag.charAt( poz ) ) ) poz--;
	    if ( tag.charAt( poz ) == '=' ) return false;
	} 

	return true;
    }
    
    
    private String getAttrValue( String tag, int poz ) {
	while ( poz < tag.length() && Character.isWhitespace( tag.charAt( poz ) ) ) poz++;	
	if ( poz >= tag.length() || tag.charAt( poz ) != '=' ) return "TRUE"; poz++;
	while ( poz < tag.length() && Character.isWhitespace( tag.charAt( poz ) ) ) poz++;    
	
	String sChar = " ";
	
	if ( tag.charAt( poz ) == '"' )  { sChar = "\""; poz++; }
	if ( tag.charAt( poz ) == '\'' ) { sChar = "'"; poz++; }
	
	return tag.substring(poz, tag.indexOf(sChar, poz) != -1 ? tag.indexOf(sChar, poz) : tag.length() - 1);
    }
    
    
    private String normalize( String str ) {
	StringBuffer sb = new StringBuffer();
	int len = ( str != null )  ? str.length() : 0;
	
	for ( int i = 0; i < len; i++ ) {
	    char ch = str.charAt(i);
	    
	    switch ( ch ) {
		case  '<': sb.append("&lt;");
			   break;
		case  '>': sb.append("&gt;");
			   break;
                case  '&': sb.append("&amp;");
			   break;
		case  '"': sb.append("&quot;");
                           break;
		case '\'': sb.append("&#");
			   sb.append(Integer.toString("'".charAt(0)));
			   sb.append(";");
			   break;
		default  : sb.append(ch);
	    }
	}

	return sb.toString();
    }
}
