/* 
 * $COPYRIGHT$
 * $Id: DefaultCookie.java,v 1.1 2001/12/24 19:12:41 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Szabo Csaba      Created
 */
package org.media.mn8.protocol.http;

import java.io.File;
import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.net.URL;
import java.util.Vector;
import java.util.Enumeration;
import java.sql.Timestamp;

/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.1 $ $Date: 2001/12/24 19:12:41 $
 */
public abstract class DefaultCookie {

    public abstract void writeCookiesToXML( String src, String dest );
    
    
    public String getCookieString( String uri, String xmlDir, boolean validate ) {
        String cookieString = "";
        Vector cookies = readCookiesFromXML( uri , xmlDir, validate );
	
        if ( cookies != null ) {
            for ( Enumeration en = cookies.elements(); en.hasMoreElements(); ) {
                Cookie cookie = (Cookie)en.nextElement();
                cookieString += ";" + cookie.getName() + "=" + cookie.getValue();
            }
            if ( cookieString.length() > 0 ) cookieString = cookieString.substring(1);
        }
	
        return cookieString;
    }
    								    
							    
    public Vector readCookiesFromXML( String uri, String xmlDir, boolean validate ) {
        Vector      result = new Vector();
        File       xmlPath = null;
        File[] cookieFiles = null;
        String      secure = "FALSE";
        String  timeString = Long.toString( System.currentTimeMillis() );
        String      domain = "";
        String        path = "";
		
	
        try {
            URL url = new URL( uri );
	    
            domain = url.getHost();
            path   = url.getPath();
            if ( url.getProtocol().equalsIgnoreCase("https") ) secure = "TRUE"; 
            xmlPath = new File( xmlDir );
            if ( !xmlPath.exists() ) return null;
            cookieFiles = xmlPath.listFiles( new CookieFileFilter( domain ) );

            while ( true ) {
                if ( !path.endsWith("/") ) path += "/";
                for ( int i = 0; i < cookieFiles.length; i++ ) {
                    Cookie cookie = ( new CookieLoader() ).getCookie( new FileInputStream( cookieFiles[i] ), new Cookie() ); 
		
                    if ( cookie.getSecure().equalsIgnoreCase( secure ) ) {
                        String _path = ( cookie.getPath() != null ) ? cookie.getPath() : "/";
			
                        if ( !_path.endsWith("/") ) _path += "/";
                        if ( path.equalsIgnoreCase( _path ) ) {
                            if ( !validate ) result.addElement( cookie );
                            else {
                                String expires = cookie.getExpires();
                                if ( new Long( timeString ).longValue() < new Long( expires ).longValue() )
                                    result.addElement( cookie );
                            }
                        }
                    }
                }
                if ( path.length() < 2 ) break;
                path = path.substring(0, path.length() - 2 );
                path = path.substring(0, path.lastIndexOf("/") + 1 );
            }
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return null;
        }    
	
        return result;
    }


    protected void writeCookiesToXML( String domain, Vector cookies, String dest ) {
        String fName = dest + File.separator + domain;
	
        if ( !( new File( dest ) ).exists() ) new File( dest ).mkdirs();
	
        for ( Enumeration e = cookies.elements(); e.hasMoreElements(); ) {
            Cookie cookie = (Cookie)e.nextElement();
            int  cookieNr = getCookieNr( dest, domain );
	    
            try {
                PrintWriter writer = new PrintWriter( new BufferedWriter (
                                                                          new FileWriter( fName + "[" + cookieNr + "].xml") ) );
                writer.println( cookie.toXMLString() );
                writer.close();
            }
            catch ( Exception ex ) {
                ex.printStackTrace();
            }	    
        }
    }

    
    private int getCookieNr( String dest, String domain ) {
        int       nr = 1;
        String fName = dest + File.separator + domain;
	
        while ( new File( fName + "[" + nr + "].xml").exists() ) nr++;
	
        return nr;
    }


    private class CookieFileFilter implements FilenameFilter {
	
        private String _filterName = null;
	
	
        public CookieFileFilter( String fName ) {
            _filterName = fName.toLowerCase();
        }  
	
	
        public boolean accept( File dir, String name ) {
            if ( name != null ) {
                String _name = name.trim().toLowerCase();
                int      poz = -1;
		
                if ( ( poz = _name.indexOf("[") ) != -1 ) _name = _name.substring( 0, poz );
                if ( _name.startsWith(".") ) _name = _name.substring(1);
                if ( _filterName.endsWith( _name ) ) return true; else return false;
            }
            else return false;
        }
    }    
}
