/* 
 * $COPYRIGHT$
 * $Id: Form.java,v 1.5 2002/05/28 15:58:06 atech Exp $
 *
 * Date        Author            Changes 
 * IAN 29 2002 Szabo Csaba       Created
 * May 27 2002 Antal Attila      removed writeToXML(), toURLString()
 */

package org.media.mn8.protocol.http;

import java.io.File;
import java.io.FilenameFilter;
import java.io.FileInputStream;
import java.util.Vector;
import java.util.Enumeration;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.5 $ $Date: 2002/05/28 15:58:06 $
 */

public class Form extends DefaultAttributes 
	          implements DefaultControl {
    
    private static final String attrNames[] = {
        "action", "method", "name", "target", 
        "enctype", "accept", "onsubmit", "onreset", "accept-charset"
    };
    
    //form attibutes
    private String action;
    private String method;
    private String name;
    private String target;
    private String encoding;

    //container of controls
    private Vector controls;
    
    private String attrList = "";
    
    private String _host = "";
    private String _path = "";

    
    public Form( String host, String path ) {
        controls = new Vector();
        _path    = path;
        _host    = host;
    }
    
    
    public void setAction( String action ) {
        this.action = action;
        attrList += getAttrPair ( "action", action );
    }


    public void setMethod( String method ) {
        this.method = method;
        attrList += getAttrPair ( "method", method );
    }


    public void setName( String name ) {
        this.name = name;
        attrList += getAttrPair ( "name", name );
    }


    public void setTarget( String target ) {
        this.target = target;
        attrList += getAttrPair ( "target", target );
    }


    public void setEncoding( String enc ) {
        this.encoding = enc;
        attrList += getAttrPair ( "enctype", enc );
    }
    
    
    public boolean isSendable() {
        return true;
    }


    public String getAction() {
        return action;
    }


    public String getMethod() {
        return method;
    }

    
    public String getName() {
        return name;
    }

    
    public Vector getValues() {
        return null;
    }


    public String getTarget() {
        return target;
    }


    public String getEncoding() {
        return encoding;
    }


    public Vector getControls() {
        return controls;
    }

    
    public Form getSavedForm( String domain, String destDir ) {
        if ( !( new File( destDir ) ).exists() ) return null;
        File formFiles[] = ( new File( destDir ) ).listFiles( new DomainNameFilter ( domain ) ); 
	
        for ( int i = 0; i < formFiles.length; i++ ) {
            try {
                Form form = ( new  FormLoader() ).getForm( new FileInputStream( formFiles[ i ] ) , new Form( _host, _path ) );
                if ( equals( (DefaultControl)form ) ) return form;
            }
            catch ( Exception e ) {
                return null;
            } 
        }
	
        return null;
    }
    

	public String toXMLString() {                                                  
		StringBuffer sb = new StringBuffer("<?xml version=\"1.0\"?>\n<FORM ");     
                                                                                
		sb.append( getAttrList().trim() +">" );                                    
                                                                                
		for ( Enumeration e = controls.elements(); e.hasMoreElements(); )          
			sb.append("\n    " + ((DefaultControl)e.nextElement()).toXMLString() );
		
		sb.append("\n</FORM>");                                                    
                                                                                  
		return sb.toString();                                                      
	}


    public String toString() {
        StringBuffer ret = new StringBuffer( "FORM : \n" + getAttrList().replace(' ', '\n') );
	
        for ( Enumeration e = controls.elements(); e.hasMoreElements(); ) 
            ret.append( "\n\n" + ((DefaultControl)e.nextElement()).toString() );
	 
        return  ret.toString();
    }


    public void parse( String tag ) {
        super.parse( tag );
	
        if ( tag == null || tag.trim().length() < 1 ) return;
	
        String attrs = getAttrsFromTag( tag ); 
	
        if ( attrs.trim().length() > 0 ) {
            for ( int i = 0; i < attrNames.length; i++ ) {
                String value = super.getValueOf( attrs, attrNames[i] );
	
                if ( value != null ) {
                    if ( attrNames[i].equals("action") ){action = value;continue;}
                    if ( attrNames[i].equals("name") )     name = value;
                    if ( attrNames[i].equals("method") ) method = value;
                    if ( attrNames[i].equals("target") ) target = value;
		
                    attrList += getAttrPair ( attrNames[i], value );
                }
            }
        }
	
        if ( action == null ) action = _host + _path;
        else if ( action.indexOf(":/") == -1 ) {
            if ( action.startsWith("/") ) 
                action = _host + action;
            else 
                action = _host + ( _path.endsWith("/") ? _path : _path + "/" ) + action;
        } 
	
        attrList += getAttrPair ( "action", action );
        if ( method == null ) { attrList += getAttrPair ( "method", "GET" );method = "GET";}                                                                          
    }
    
    
    public String getAttrList() {
        return attrList + super.getAttrList();
    }

    
    public void addControl( DefaultControl control ) {
        controls.addElement( control );
    }
    
    
    public String getControlType() {
        return "FORM";
    }


    public boolean equals( DefaultControl dcontrol ) {
        Form form = (Form)dcontrol;
	
        if ( form == null ) return false;
        if ( form.action != null && !form.action.equalsIgnoreCase( action ) ) return false;
        if ( form.method != null && !form.method.equalsIgnoreCase( method ) ) return false;
        if ( form.target != null && !form.target.equalsIgnoreCase( target ) ) return false;
        if ( form.name   != null && !form.name.equalsIgnoreCase( name ) ) return false;
        if ( form.getControlNr() != getControlNr() ) return false;
        
        for ( Enumeration e = form.controls.elements(); e.hasMoreElements(); ) {
            DefaultControl control = (DefaultControl)e.nextElement();
            if ( !foundControl( control ) ) return false;
        }
	
        return true;
    }
    
    
    private boolean foundControl( DefaultControl control ) {
        for ( Enumeration e = controls.elements(); e.hasMoreElements(); ) {
            DefaultControl lcontrol = (DefaultControl)e.nextElement();
	
            if ( control.getControlType().equals( lcontrol.getControlType() ) && 
                 control.equals( lcontrol ) ) return true;
        }
	
        return false;
    }
    
    
    private int getControlNr() {
        return controls.size();
    }
    
}
