/* 
 * $COPYRIGHT$
 * $Id: FormExtractor.java,v 1.6 2002/06/04 19:05:03 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 30 2002 Szabo Csaba       Created
 */
package org.media.mn8.protocol.http;

import java.util.Vector;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;
import java.io.FilenameFilter;

import org.media.mn8.concepts.FileConcept;
import org.media.mn8.concepts.SeriesConcept;
import org.media.mn8.concepts.ElementConcept;
import org.media.mn8.Concept;
import org.media.mn8.ConceptConceptDefinition;

import org.media.mn8.concepts.AttributeSeries;
import org.media.mn8.concepts.AttributeConcept;
import org.media.mn8.concepts.StringConcept;
import org.media.mn8.concepts.HTMLFormConcept;
import org.media.mn8.mn8RuntimeFlags;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.6 $ $Date: 2002/06/04 19:05:03 $
 */
public class FormExtractor extends LinkExtractor {

    Vector            forms = new Vector();
    boolean          inForm = false;
    boolean      inTextArea = false;
    boolean        inSelect = false;
    boolean        inOption = false;
    boolean       formError = false;
    Form              _form = null;
    StringBuffer     _value = null;
    DefaultControl _control = null;
    OptGroupControl _optgrp = null;
    OptionControl      _opt = null;
    HTMLFormConcept    form = null;  
    boolean  isSendableForm = true; 
    int              linkNr = 0;


    public Vector getForms( String pageContent ){
        parsePage( pageContent );
        return forms;
    }


    public void tagFound( String tag ) {
        super.tagFound( tag );

        if ( links.size() > linkNr ) { linkNr++;return; }

        if ( formError || baseURL == null ) return;
	
        if ( inForm && _form != null ) {
	
            if ( inTextArea && !tag.toLowerCase().startsWith("/textarea") ) {
                _value.append( "<" + tag + ">" );
                return;
            }

            if ( inOption && !tag.toLowerCase().startsWith("/option" ) ) {
                _value.append( "<" + tag + ">" );
                return;
            } 
            
            if ( tag.toLowerCase().startsWith("input ") ) {
                _control = new InputControl();
                ((InputControl)_control).parse( tag );
                _form.addControl( _control ); 
                _control = null;
            }
            
            if ( tag.toLowerCase().startsWith("textarea " ) ) {
                _control = new TextAreaControl();
                _value   = new StringBuffer();

                inTextArea = true;
                ((TextAreaControl)_control).parse( tag );
            }
	    
            if ( tag.toLowerCase().startsWith("/textarea") ) {
                inTextArea = false;
                ((TextAreaControl)_control).setValue( _value.toString() );
                _form.addControl( _control );
                _control = null;
            }

            if ( tag.toLowerCase().startsWith("select ") ) {
                _control = new SelectControl();
                ((SelectControl)_control).parse( tag );
                inSelect = true;
            }

            if ( tag.toLowerCase().startsWith("optgroup ") && inSelect ) {
                if ( _opt != null ) ((SelectControl)_control).addOption( _opt );
                _opt = null;
                _optgrp = new OptGroupControl();
                _optgrp.parse( tag );
            }

            if ( tag.toLowerCase().startsWith("/optgroup") && inSelect ) {
                if ( _opt != null ) _optgrp.addOption( _opt );
                ((SelectControl)_control).addOptionGroup ( _optgrp );
                _optgrp = null; _opt = null;
            }
	    
            if ( (tag + " ").toLowerCase().startsWith("option ") && inSelect ) {
                if ( _opt != null && _optgrp != null ) _optgrp.addOption( _opt );
                else if ( _opt != null && _control != null ) ((SelectControl)_control).addOption( _opt );
		
                _opt = new OptionControl();
                _value = new StringBuffer();
                _opt.parse( tag ); inOption = true;
            }

            if ( tag.toLowerCase().startsWith("/option") && inSelect ) {
                _opt.setOptValue( _value.toString() ); inOption = false;
            }

            if ( tag.toLowerCase().startsWith("/select") ) {
                if ( _opt != null ) ((SelectControl)_control).addOption( _opt );
                _form.addControl( _control );
            }
        }

        if ( tag.trim().toLowerCase().startsWith("form ") ) {
            inForm = true;
            formError = false;
            try {
                String _host = baseURL.getProtocol() + "://" + baseURL.getHost() +
                    ( baseURL.getPort() == -1 ? "" : ":" + baseURL.getPort() );
                String _path = baseURL.getPath();
		    
                if ( !_path.endsWith("/") ) 
                    if ( _path.indexOf("/", _path.indexOf(".") == -1 ? _path.length() - 1 : _path.indexOf(".") ) == -1 )
                        _path = _path.substring( 0, _path.lastIndexOf("/") + 1 );
		    
                _form = new Form( _host, _path );
                _form.parse( tag );
            } catch ( Exception ex ) {
                formError = true;
                if( mn8RuntimeFlags.isVerbose() || mn8RuntimeFlags.isDebug() )
                    System.err.println( ex.getMessage() );
            }
        }
	
        if ( tag.trim().toLowerCase().startsWith("/form") ||  
             tag.trim().toLowerCase().startsWith("/html") ) {
            inForm = false;
            try {
                String    _host = baseURL.getHost().trim().length() > 0 ? baseURL.getHost() : "localhost";
                String _destDir = System.getProperty("user.home");
                
                if( !_destDir.endsWith( File.separator) ) _destDir += File.separator;
                _destDir += HTMLFormConcept.FORM_DB;

                HTMLFormConcept loaded = new HTMLFormConcept( _form );
                HTMLFormConcept saved = loadForm( _destDir, _host,  loaded.getID() );
                if ( saved != null ) {
                    forms.add( saved );
                    isSendableForm = true;
                }
                else {
                    forms.add( loaded );
                    isSendableForm = false;
                }

            } catch ( Exception ex ) {
                ex.printStackTrace();
                if( mn8RuntimeFlags.isVerbose() || mn8RuntimeFlags.isDebug() )
                    System.err.println( ex.getMessage() );
            } 
        }
    }


    private HTMLFormConcept loadForm( String destDir, String host, String id ) {
        if ( !( new File( destDir ) ).exists() ) return null;
        String formFiles[] = ( new File( destDir ) ).list( new DomainNameFilter ( host ) ); 
        for ( int i = 0; i < formFiles.length; i++ ) {
            try {
                FileConcept xmlfile = new FileConcept( destDir + File.separator + formFiles[i] );
                StringConcept cont = (StringConcept) xmlfile.getContent();
                HTMLFormConcept xmlForm = (HTMLFormConcept) HTMLFormConcept.fromXML( cont );
                if ( id.equals(xmlForm.getID()) ) return xmlForm;
            }
            catch ( Exception e ) {
                return null;
            } 

        }
        return null;
    }


    public void textFound( String text ) {
        super.textFound( text );

        if ( inTextArea ) _value.append( text );
        if ( inOption ) _value.append( text );
    }
}
