/* 
 * $COPYRIGHT$
 * $Id: FormLoader.java,v 1.1 2002/01/31 20:35:16 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 29 2002 Szabo Csaba       Created
 */

package org.media.mn8.protocol.http;

import java.io.InputStream;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/01/31 20:35:16 $
 */

public class FormLoader extends DefaultHandler {
    private Form         form;
    private String       errorMsg;
    private StringBuffer parents;
    
    private InputControl    icontrol;
    private TextAreaControl tacontrol;
    private SelectControl   scontrol;
    private OptionControl   ocontrol;
    private OptGroupControl ogcontrol;

    private String          tempValue;
    
    public FormLoader() {
    }
    
    
    protected FormLoader( Form form ) {
	this.form      = form;
	this.errorMsg  = null;
	this.parents   = new StringBuffer();
	this.icontrol  = null;
	this.tacontrol = null;
	this.scontrol  = null;
	this.ocontrol  = null;
	this.ogcontrol = null;
	this.tempValue = "";
    }
    
    
    public String getErrorMsg() {
	return errorMsg;
    }
    
    
    public Form getForm( InputStream src, Form _form ) {
	SAXParserFactory factory = SAXParserFactory.newInstance();
	DefaultHandler   handler = new FormLoader( _form );
	
	try {
	    SAXParser parser = factory.newSAXParser();
	    parser.parse( src, handler );
	    return _form;
	}
	catch ( Exception e ) {
	    errorMsg = "Unable to parse XML.\n " + e.getMessage();
	    return null;
	}
    }
    
    
    public void startDocument() throws SAXException {
	parents = new StringBuffer();
    }
    
    
    public void startElement( String uri, String name, String qname, Attributes attrs ) throws SAXException {
	String pName = getParentName();
	
	parents.append( "," + qname );
	
	if ( pName.equalsIgnoreCase("form") && qname.equalsIgnoreCase("input") ) 	
	    icontrol = new InputControl();
	
	if ( pName.equalsIgnoreCase("form") && qname.equalsIgnoreCase("select") ) 
	    scontrol = new SelectControl();

	if ( pName.equalsIgnoreCase("select") && qname.equalsIgnoreCase("optgroup") ) 
	    ogcontrol = new OptGroupControl();

	if ( pName.equalsIgnoreCase("form") && qname.equalsIgnoreCase("textarea") ) {
	    tacontrol = new TextAreaControl();
	    tempValue = "";
	}
	
	if ( pName.equalsIgnoreCase("select") && qname.equalsIgnoreCase("option") ) {
	    ocontrol = new OptionControl();
	    tempValue = "";
	}

	if ( pName.equalsIgnoreCase("optgroup") && qname.equalsIgnoreCase("option") ) {
	    ocontrol = new OptionControl();
	    tempValue = "";
	}
	
	if ( attrs != null ) {
	    for ( int i = 0; i < attrs.getLength(); i++ ) {
		if ( qname.equalsIgnoreCase("form") ) {
		    if ( attrs.getQName(i).equalsIgnoreCase("action") ) form.setAction( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("method") ) form.setMethod( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("name") )   form.setName  ( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("target") ) form.setTarget( attrs.getValue(i) );
		}
		else if ( pName.equalsIgnoreCase("form") && qname.equalsIgnoreCase("input") ) {
		    if ( attrs.getQName(i).equalsIgnoreCase("type") )    icontrol.setType   ( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("name") )    icontrol.setName   ( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("value") )   icontrol.setValue  ( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("checked") ) icontrol.setChecked( attrs.getValue(i) );
		}
		else if ( pName.equalsIgnoreCase("form") && qname.equalsIgnoreCase("textarea") ) {
		    if ( attrs.getQName(i).equalsIgnoreCase("name") ) tacontrol.setName( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("rows") ) tacontrol.setRows( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("cols") ) tacontrol.setCols( attrs.getValue(i) );
		}
		else if ( pName.equalsIgnoreCase("form") && qname.equalsIgnoreCase("select") ) {
		    if ( attrs.getQName(i).equalsIgnoreCase("name") )     scontrol.setName( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("size") )     scontrol.setSize( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("multiple") ) scontrol.setMultiple( attrs.getValue(i) );
		}
		else if ( pName.equalsIgnoreCase("select") && qname.equalsIgnoreCase("option") ) {
		    if ( attrs.getQName(i).equalsIgnoreCase("value") )    ocontrol.setValue( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("selected") ) ocontrol.setSelected( attrs.getValue(i) );
		}
		else if ( pName.equalsIgnoreCase("select") && qname.equalsIgnoreCase("optgroup") ) {
		    if ( attrs.getQName(i).equalsIgnoreCase("label") ) ogcontrol.setLabel( attrs.getValue(i) );
		}
		else if ( pName.equalsIgnoreCase("optgroup") && qname.equalsIgnoreCase("option") ) {
		    if ( attrs.getQName(i).equalsIgnoreCase("value") )    ocontrol.setValue( attrs.getValue(i) );
		    if ( attrs.getQName(i).equalsIgnoreCase("selected") ) ocontrol.setSelected( attrs.getValue(i) );
		}
	    }
	}
    }
    
    
    public void endElement( String uri, String name, String qname ) throws SAXException {
	String pName = parents.toString().trim();
	int      poz = pName.lastIndexOf(",");
	
	if ( poz != -1 ) pName = pName.substring( poz + 1 ).trim();
	if ( poz < 0 ) poz = 0;
	if ( !qname.equals( pName ) ) throw new SAXException( "Missing end element : '" + qname + "'" );
	
	if ( qname.equalsIgnoreCase("input") && icontrol != null ) { 
	    form.addControl( icontrol );
	    icontrol = null; 
	}

	if ( qname.equalsIgnoreCase("select") && scontrol != null ) { 
	    form.addControl( scontrol );
	    scontrol = null; 
	}

	if ( qname.equalsIgnoreCase("optgroup") && ogcontrol != null && scontrol != null ) { 
	    scontrol.addOptionGroup( ogcontrol );
	    ogcontrol = null; 
	}

	if ( qname.equalsIgnoreCase("textarea") && tacontrol != null ) { 
	    if ( tempValue.trim().length() > 0 ) tacontrol.setValue( tempValue );
	    form.addControl( tacontrol);
	    tacontrol = null; 
	} 

	if ( qname.equalsIgnoreCase("option") && ocontrol != null && scontrol != null ) {
	    if ( tempValue.trim().length() > 0 ) ocontrol.setOptValue( tempValue );
	    if ( ogcontrol != null ) ogcontrol.addOption( ocontrol );else scontrol.addOption( ocontrol );
	    ocontrol = null;
	}
	
	parents.delete( poz, parents.length() );
    }
    
    
    public void characters( char ch[], int start, int length ) throws SAXException {
	String pName = getParentName();
	
	if ( pName.equalsIgnoreCase("textarea") )
	    tempValue += new String(ch, start, length ).trim();
	if ( pName.equalsIgnoreCase("option") )
	    tempValue += new String(ch, start, length ).trim();
    }
    
    
    public void ignorableWhitespace( char ch[], int start, int length ) throws SAXException {
	characters( ch, start, length );
    } 
    
    
    public void comment( char ch[], int start, int length ) throws SAXException {
    }
	  
	  
    public void startCDATA() throws SAXException {
    }
		  
		  
    public void endCDATA() throws SAXException {
    }
			  
 
    public void startDTD( String name, String pID, String sId ) throws SAXException {
    }
				  
				  
    public void endDTD() throws SAXException {
    }
					  
					  
    public void startEntity( String name ) throws SAXException {
    }
						  
						  
    public void endEntity( String name ) throws SAXException {
    }
    
    
    private String getParentName( ) {
	String pName = parents.toString().trim();
	int     _poz = pName.lastIndexOf( "," );
	
	if ( _poz != -1 ) pName = pName.substring( _poz + 1 ).trim();
	
	return pName;
    }
}
