/* 
 * $COPYRIGHT$
 * $Id: FormTest.java,v 1.1 2002/01/31 20:35:24 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 29 2002 Szabo Csaba       Created
 */

package org.media.mn8.protocol.http;

import java.net.*;
import java.util.*;
import java.io.*;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/01/31 20:35:24 $
 */

public class FormTest {
    
    public static void main( String args[] ) {
	if( args.length != 2 ) {
	    System.out.println( "Usage: java FormTest url formDir" );
	}
	else {
	    try {
		FormTest ct = new FormTest();
		URL u = new URL( args[0] );
		ct.connectToURL( u , args[1] );
	    }
	    catch( Exception ex ) {
		System.err.println( "Malformed url: " + args[0] );
	    }
	}
    }


    public void connectToURL( URL u, String destDir ) {
	boolean         inForm = false;
	Form              form = null;
	
	try {
	    URLConnection uconn = u.openConnection();
	    uconn.connect();

	    InputStream is = uconn.getInputStream();
	    
	    for (;;) {
		boolean openTag = true;
		String      tag = nextTag( is );
		
		if ( tag == null ) break;
		
		tag = tag.substring( tag.lastIndexOf('<') != -1 ? tag.lastIndexOf('<') : 0 );
		if ( tag.trim().startsWith("</") ) openTag = false;
		String _tag = tag.replace('<', ' ').replace('>', ' ').trim() + " ";
		if ( _tag.startsWith("/") ) _tag = _tag.substring(1).trim() + " ";
		
		if ( inForm ) {
		    
		    if ( _tag.toLowerCase().startsWith("input ") ) { 
			InputControl icontrol = new InputControl();
			icontrol.parse( _tag );
			form.addControl( icontrol );
		    }
		    
		    if ( _tag.toLowerCase().startsWith("textarea ") ) {
			TextAreaControl tacontrol = new TextAreaControl();
			String ta_value = "";
	
			tacontrol.parse( _tag );
			while ( true ) {
			    _tag = nextTag( is );
			    if ( _tag == null ) break;
			    
			    ta_value += _tag;
			    String end_tag = _tag.substring( _tag.lastIndexOf('<') != -1 ? _tag.lastIndexOf('<') : 0 );
			    if ( end_tag.trim().startsWith("</") ) {
				end_tag = end_tag.replace('<', ' ').replace('>', ' ').replace('/', ' ').trim() + " ";
				if ( end_tag.toLowerCase().startsWith("textarea ") ) break;
			    }
			}
			tacontrol.setValue( ta_value.lastIndexOf("</") != -1 ? ta_value.substring(0, ta_value.lastIndexOf("</") ) : ta_value );
			form.addControl( tacontrol );
			continue;
		    }
		    
		    if ( _tag.toLowerCase().startsWith("select ") ) {
			SelectControl    scontrol = new SelectControl();
			String           optValue = "";
			OptionControl    ocontrol = null;
			OptGroupControl ogcontrol = null;
			
			scontrol.parse( _tag );
			
			while (true) {
			    String innerTag = nextTag( is ).trim();
			    if ( innerTag == null ) break;
			
			    String eName = innerTag.substring( innerTag.lastIndexOf('<') != -1 ? innerTag.lastIndexOf('<') : 0 );
			    
			    if ( eName.startsWith("</") ) {
				String end_tag = eName.substring(2).replace('>', ' ').trim() + " ";
				if ( end_tag.toLowerCase().startsWith("select ") ) break;
			    }
			    
			    eName = eName.replace('<', ' ').replace('>', ' ').trim() + " ";
			    
			    if ( eName.toLowerCase().startsWith("optgroup ") ) {
			    	if ( ocontrol != null ) scontrol.addOption( ocontrol );ocontrol = null;
				ogcontrol = new OptGroupControl();
				ogcontrol.parse( innerTag );
			    }
			    else if (eName.startsWith("/") && ( eName.substring(1).trim() + " ").toLowerCase().startsWith("optgroup ")) {
				if ( ocontrol != null ) ogcontrol.addOption( ocontrol );
				scontrol.addOptionGroup( ogcontrol );
				ogcontrol = null;
				ocontrol  = null;
			    }
			    else if ( eName.toLowerCase().startsWith("option ") ) {
				if ( ocontrol != null && ogcontrol != null ) ogcontrol.addOption( ocontrol );
				else if ( ocontrol != null && scontrol != null  ) scontrol.addOption( ocontrol );
				
				ocontrol = new OptionControl();
				ocontrol.parse( innerTag );
				optValue = "";
			    }
			    else if ( eName.startsWith("/") && (eName.substring(1).trim() + " ").toLowerCase().startsWith("option ") ) {
				optValue += innerTag.substring(0, innerTag.lastIndexOf('<') != -1 ? innerTag.lastIndexOf('<') : innerTag.length() - 1);
				ocontrol.setOptValue( optValue );
			    }
			    else optValue += innerTag;
			}
			if ( ocontrol != null ) scontrol.addOption( ocontrol );
		
			form.addControl( scontrol );
			continue;
		    } 
		}

		if ( _tag.toLowerCase().startsWith("form ") ) { 
		    if ( openTag ) {
			inForm = true;
			String host = u.getProtocol() + "://" + u.getHost() + ( u.getPort() == -1 ? "" : ":" + u.getPort() );
			String path = u.getPath();
			if ( !path.endsWith("/") )
			    if ( path.indexOf("/", path.indexOf(".") == -1 ? path.length() - 1 : path.indexOf(".") ) == -1 )
				path = path.substring( 0, path.lastIndexOf("/") + 1 );
			
			form = new Form( host, path );
			form.parse( tag );
		    } 
		    else { 
			inForm = false;
			String _host = u.getHost().trim().length() > 0 ? u.getHost() : "localhost";
			Form   sform = form.getSavedForm( _host, destDir );
			
			if ( sform != null ) 
			    System.out.println( sform.toXMLString() );
			else
			    System.out.println( "Saved form not found !!!" );        
			//form.writeToXML( _host, destDir );
		    }
		}
	    }
	}
	catch( IOException ex ) {
	    System.err.println( "Couldn't connect to " + u.toString() );
	}
	catch( Exception e ) {
	    e.printStackTrace();
	}
    }
    
    private String nextTag( InputStream is ) {
	StringBuffer sb = new StringBuffer();
	
	try {
	    int ch = 0;
	    
	    while ( ( ch = is.read() ) != -1 ) {
		sb.append( (char)ch );
		if ( ch == '>' ) {
		    if ( sb.toString().trim().startsWith("<!--") ) {
			if ( sb.toString().trim().endsWith("-->") )  
			    return sb.toString();
			else 
			    continue;
		    }
		    else return sb.toString();
		}
	    } 
	    return null;
	}
	catch ( IOException ex ) {
	    System.err.println( "Couldn't connect to URL " );
	    return null;
	}
    }

} 
