/* 
 * $COPYRIGHT$
 * $Id: InputControl.java,v 1.1 2002/01/31 20:35:35 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 29 2002 Szabo Csaba       Created
 */

package org.media.mn8.protocol.http;

import java.util.Vector;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/01/31 20:35:35 $
 */

public class InputControl extends DefaultAttributes 
			  implements DefaultControl {

    private static final String attrNames[] = {
             "type", "value", "name", "checked", "disabled", "readonly",
	     "size", "maxlength", "src", "alt", "usemap", "ismap", "tabindex",
	     "accesskey", "onfocus", "onblur", "onselect", "onchange", "accept", "align" , "alt"
	     };

    // input attibutes
    private String type;
    private String name;
    private String value;
    private String checked;

    private String attrList = ""; 
    

    public InputControl() {
    }


    public void setType( String type ) {
	this.type = type;
	attrList += getAttrPair ( "type", type );
    }
    

    public void setName( String name ) {
	this.name = name;
	attrList += getAttrPair ( "name", name );
    }
    

    public void setValue( String value ) {
	this.value = value;
	attrList += getAttrPair ( "value", value );
    }
    

    public void setChecked( String chk ) {
	this.checked = chk;
	attrList += getAttrPair ( "checked", chk );
    }
    

    public boolean isSendable() {

	if ( type.equalsIgnoreCase("submit") &&
	     ( name == null || name.trim().length() < 1 ) ) name = type;

	if ( ( type.equalsIgnoreCase("text") ||
	       type.equalsIgnoreCase("password") ||
	       type.equalsIgnoreCase("file") ||
	       type.equalsIgnoreCase("submit") ) && 
	     name != null && name.trim().length() > 0 ) return true;
	else if ( ( type.equalsIgnoreCase("radio") ||
	       type.equalsIgnoreCase("checkbox") ||
	       type.equalsIgnoreCase("hidden") ) &&
	       name != null && name.trim().length() > 0 &&
	     value != null && value.trim().length() > 0 ) return true;
	
	return false;
    }
    

    public String getName() {
	return name;
    }


    public Vector getValues() {
	Vector result = new Vector();
	result.add( value );
	return result;
    }

    
    public String getType() {
	return type;
    }


    public String toXMLString() {
	return "<INPUT " + getAttrList().trim() + "/>";
    }


    public String toString() {
	return  "INPUT : \n" + getAttrList().replace(' ', '\n');
    }

    
    public void parse( String tag ) {
	super.parse( tag );

        if ( tag == null || tag.trim().length() < 1 ) return; 

	String attrs = getAttrsFromTag( tag ); 
        
	if ( attrs.trim().length() > 0 ) {
    	    for ( int i = 0; i < attrNames.length; i++ ) {    
        	String value = super.getValueOf( attrs, attrNames[i] );
        	if ( value != null ) {
		    if ( attrNames[i].equals("type") )         type = value;
		    if ( attrNames[i].equals("name") )         name = value;
		    if ( attrNames[i].equals("value") )  this.value = value;
		    if ( attrNames[i].equals("checked") )   checked = value;

		    attrList += getAttrPair ( attrNames[i], value );
		}
	    }
	}
	
	if ( this.value == null ) { attrList += getAttrPair ( "value", "" );value = "";}    
	if ( this.type  == null ) { attrList += getAttrPair ( "type", "text" );type = "text";}
	if ( checked == null && ( type.equalsIgnoreCase("radio") ||
	     type.equalsIgnoreCase("checkbox") ) ) attrList += getAttrPair ( "checked", "false" );

    }


    public String getAttrList() {
	return attrList + super.getAttrList();
    } 
    
    
    public String getControlType() {
	return "INPUT";
    }

    
    public boolean equals( DefaultControl control ) {
	InputControl icontrol = (InputControl)control;
	
	if ( !icontrol.type.equalsIgnoreCase( type ) ) return false;
	if ( icontrol.name != null && !icontrol.name.equalsIgnoreCase( name ) ) return false;
	
	return true;
    }
}
