/* 
 * $COPYRIGHT$
 * $Id: OperaCookie.java,v 1.1 2001/12/24 19:13:18 neuro Exp $
 *
 * Date        Author            Changes 
 * Nov 02 2001 Antal Attila      Created
 */
package org.media.mn8.protocol.http;

import java.io.File;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.EOFException;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Enumeration;

/** 
 * @author <a href="mailto:atech@nolimits.ro">Antal Attila</a>
 * @version $Revision: 1.1 $ $Date: 2001/12/24 19:13:18 $
 */
public class OperaCookie extends DefaultCookie {

    private String name, value, path, secure, dest;
    private int expires, lastused;
    private Vector domains = new Vector();


    public void writeCookiesToXML( String  src, String dest ) {
        DataInputStream is = null;
        this.dest = dest;
        try {
            is =new DataInputStream(  new FileInputStream( src ) );
            readFileVersionNumber( is );
            readAppVersionNumber( is );
            int id_length = readIdtagLength( is );
            int length_length = readLengthLength( is );
            readStructRecordItems( is, id_length, length_length );
        }
        catch (FileNotFoundException e) {
            System.out.println("ERROR:" + e.getMessage() );
        }
        catch (IOException e) {
            System.out.println("ERROR:" + e.getMessage() );
        }
    }


    private int readSequence ( DataInputStream is, int type ) {
        int ret = 100000; // 0x85
        try {
            switch (type) {
            case 1: return is.readByte();
            case 2: return is.readShort();
            case 4: return is.readInt();
            }
        }
        catch ( EOFException e ) {
            return ret;
        }
        catch ( IOException e ) {
            System.out.println("ERROR:" + e.getMessage() );
        }
        return ret;
    }


    private DataInputStream readPayload ( DataInputStream is, int length ) {
        DataInputStream bais = null;
        try {
            byte[] arry = new byte[length];
            is.read( arry );
            bais = new DataInputStream( new ByteArrayInputStream(arry) );
        }
        catch ( IOException e ) {
            System.out.println("ERROR:" + e.getMessage() );
        }
        return bais;
    }


    private String readString ( DataInputStream is) {
        int b;
        String ret = "";
        try {
            b = is.readShort();
            for ( int x = 1; x <= b; x++) {
                ret += (char) is.readByte();
            }
        }
        catch ( IOException e ) {
            System.out.println("ERROR:" + e.getMessage() );
        }
        return ret;
    }


    private void readFileVersionNumber ( DataInputStream is) {
        readSequence( is, 4 ); // uint32
    }

    private void readAppVersionNumber ( DataInputStream is) {
        readSequence( is, 4 ); // uint32
    }

    private int readIdtagLength ( DataInputStream is) {
        return readSequence( is, 2 ); // uint16
    }

    private int readLengthLength ( DataInputStream is) {
        return readSequence( is, 2 ); // uint16
    }


    private void readDomainRecord ( DataInputStream di ) {
        boolean isend = false;
        int id = 0;
        while ( !isend ) {
            id = readSequence( di, 1 );
            switch(id) {
            case -101: break;   // 0x9B
            case -123: break;   // 0x85
            case -124: break;   // 0x84
            case 100000:        // 0x84
                isend = true;
                break;
            case 30:            // 0x001E - domain name
                domains.insertElementAt( readString( di ), 0 );
                break;
            case 31:            // 0x001F can be: 1, 2, 3, 4
                int accepted  = readSequence( di, readSequence( di, 2 ) );
                //System.out.println( "Accepted:" + accepted ); 
                break;
            case 33:            // 0x0021 can be: 1, 2
                int explicit  = readSequence( di, readSequence( di, 2 ) );
                //System.out.println( "Explicit:" + explicit ); 
                break;
            case 37:            // 0x0025 can be: 1, 2, 3, 4
                int third  = readSequence( di, readSequence( di, 2 ) );
                //System.out.println( "Third:" + third ); 
                break;
            default:
                System.out.println("Invalid ID in domain record: " + id ); 
            }
        }
    }


    private void  readPathRecord ( DataInputStream di ) {
        int id = readSequence( di, 1 );
        switch(id) {
        case -123: break;  // 0x85
        case -124: break;  // 0x84
        case 29:           // 0x001D - path name
            path += "/" + readString( di );
            break;
        default:
            System.out.println("Invalid ID in path record: " + id ); 
        }
    }


    private void readCookieRecord ( DataInputStream di ) {
        boolean end_di = false;
        int id;
        while ( !end_di ) {
            id = readSequence( di, 1 );
            switch(id) {
            case -101: break;              // 0x9B
            case -123: break;              // 0x85
            case -124: break;              // 0x84
            case -96: break;               // 0xA0
            case 100000:                   // 0x84
                end_di = true;
                break; 
            case 16:                       // 0x0010 - name
                name = readString( di );
                break;
            case 17:                       // 0x0011 - value
                value = readString( di );
                break;
            case 18:    // 0x0012 - expiry date: 00:00 Jan 1, 1970 GMT ->
                expires = readSequence( di, readSequence( di, 2 ) );
                break;
            case 19:   // 0x0013 - last used
                lastused = readSequence( di, readSequence( di, 2 ) );
                break;
            case 20:   // 0x0014 - Comment/Description of use (RFC 2965)
                String descr = readString( di );
                break;
            case 21:   // 0x0015 - URL for Comment/Description of use (RFC 2965)
                String urldescr = readString( di );
                break;
            case 22:    // 0x0016 - The domain received with version=1 cookies (RFC 2965)
                String domain_ver1 = readString( di );
                break;
            case 23:    // 0x0017 - The path received with version=1 cookies (RFC 2965)
                String path_ver1 = readString( di );
                break;
            case 24:    // 0x0018 - The port limitations received with version=1 cookies (RFC 2965)
                String portlim = readString( di );
                break;
            case 25:   // 0x0019 - if present SECURE = true
                secure = "TRUE"; 
                break;
            case 26:   // 0x001A - COOKIE VERSION NUMBER
                int ver = readSequence( di, 1 );
                break;
            case 27:    // 0x001B - This cookie will only be sent to the server that sent it
                break;
            case 28:    // 0x001C - Reserved for delete protection: Not yet implemented
                break;
            case 32:    // 0x0020 - This cookie will not be sent if the path is only a prefix of the URL
                break;
            case 34:   // 0x0022 - if true, this cookie was set as the result of a password login form
                break;
            case 35:   // 0x0023 - If true, this cookie was set as the result of a HTTP authentication login
                break;
            case 36:   // 0x0024 - set if the cookie was set by a third party server, and only these cookies will be sent
                break;
            default:
                System.out.println("Invalid ID in cookie record: " + id ); 
            }
        }
	
    }


    private void readStructRecordItems ( DataInputStream is, int id_length, int length_length  ) {
        int id = 0, length;
        boolean end_is = false;
        DataInputStream di;

        while ( !end_is ) {
            id = readSequence( is, id_length );
            switch(id) {
            case 100000: end_is = true; break;
            case -123:  // 0x85
                break;
            case -124:  // 0x84
                if ( domains.size() > 0 ) domains.remove(0);
                break;
            case 1:  // 0x01
                path = "";
                di = readPayload( is, readSequence( is, length_length ) ); 
                readDomainRecord( di );
                break;
            case 2:  // 0x0002
                di = readPayload( is, readSequence( is, length_length ) ); 
                readPathRecord( di );
                break;
            case 3:  // 0x0003
                expires = 0;
                lastused = 0;
                name = "";
                value = "";
                secure = "FALSE";

                di = readPayload( is, readSequence( is, length_length ) ); 
                readCookieRecord( di );
		
                String domain = "";
                for ( Enumeration e = domains.elements(); e.hasMoreElements(); ) {
                    domain +=  "." + (String) e.nextElement();
                }
                domain = domain.substring(1);

                //System.out.println("domain=" + domain + "; path=" + path + "; " + name + "=" + value +
                //		   "; ED: " + expires + "; LD: " + lastused ); 

                Cookie cook = new Cookie();
                String _expires = expires + "000";
                cook.setDomain(domain);
                cook.setPath(path);
                cook.setSecure(secure);
                cook.setExpires(_expires);
                cook.setName(name);
                cook.setValue(value);
                Vector cookies = new Vector();
                cookies.addElement( cook );
                //cook.listCookie( true );

                writeCookiesToXML(domain, cookies, dest);

                path = "";
                break;
            default:
                System.out.println("Invalid ID in record: " + id ); 

            }
        }
    }
}
