/* 
 * $COPYRIGHT$
 * $Id: SelectControl.java,v 1.1 2002/01/31 20:36:03 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 29 2002 Szabo Csaba       Created
 */

package org.media.mn8.protocol.http;

import java.util.Vector;
import java.util.Enumeration;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/01/31 20:36:03 $
 */

public class SelectControl extends DefaultAttributes 
			   implements DefaultControl {

    private static final String attrNames[] = {
	     "name", "size", "multiple", "disabled",
	     "tabindex", "onfocus", "onblur", "onchange"
	    };
	    
    // select attlist
    private String name;
    private String size;
    private String multiple;

    private Vector options;

    private String attrList = "";
    

    public SelectControl() {
	options  = new Vector();
    }
    
    
    public void setName( String name ) {
	this.name = name;
	attrList += getAttrPair ( "name", name );
    }
   

    public boolean isSendable() {
	return true;
    }


    public String getName() {
	return name;
    }

    
    public Vector getValues() {
	Vector result = new Vector();
	Vector values = null;

	for ( Enumeration e = options.elements(); e.hasMoreElements(); ) {
	    Object obj = e.nextElement();
	    
	    if ( obj instanceof OptGroupControl ) 
		values = ((OptGroupControl)obj).getValues();
	    else if ( obj instanceof OptionControl )
		values = ((OptionControl)obj).getValues();

	    for ( Enumeration e1 = values.elements(); e1.hasMoreElements(); ) {
		String value = (String)e1.nextElement();
		result.add( value );
	    }
	}

	return result;
    }

    
    public void setSize( String size ) {
	this.size = size;
	attrList += getAttrPair ( "size", size );
    }
    
    
    public void setMultiple( String mtp ) {
	this.multiple = mtp;
	attrList += getAttrPair ( "multiple", mtp );
    }
    
    
    public String toXMLString() {
	StringBuffer sb = new StringBuffer("<SELECT " + getAttrList().trim() + ">");
	
	for ( Enumeration e = options.elements(); e.hasMoreElements(); ) {
	    Object obj = e.nextElement();
	    
	    if ( obj instanceof OptGroupControl ) 
		sb.append("\n        " + ((OptGroupControl)obj).toXMLString());
	    if ( obj instanceof OptionControl ) 
		sb.append("\n        " + ((OptionControl)obj).toXMLString());
	}	
	sb.append("\n    </SELECT>");
	
	return sb.toString();
    }
    

    public String toString() {
	StringBuffer ret = new StringBuffer( "SELECT : \n" + getAttrList().replace(' ', '\n') );
	   
	for ( Enumeration e = options.elements(); e.hasMoreElements(); ) {
	    Object obj = e.nextElement();
	
	    if ( obj instanceof OptGroupControl ) 
		ret.append( "\n\n" + ((OptGroupControl)obj).toString() );
	    if ( obj instanceof OptionControl ) 
		ret.append( "\n\n" + ((OptionControl)obj).toString() );	
	}		       
	
	return  ret.toString();
    }
    
    
    public void parse( String tag ) {
	super.parse( tag );
	
	if ( tag == null || tag.trim().length() < 1 ) return;
		 
	String attrs = getAttrsFromTag( tag );
	
	if ( attrs.trim().length() > 0 ) {
	    for ( int i = 0; i < attrNames.length; i++ ) {
		String value = super.getValueOf( attrs, attrNames[i] );
	    
		if ( value != null ) {
		    if ( attrNames[i].equals("name") )     name = value;
		    if ( attrNames[i].equals("size") )     size = value;
		    if ( attrNames[i].equals("multiple") ) multiple = value;
		
		    attrList += getAttrPair( attrNames[i], value );
		}
	    }
	}
	
	if ( multiple == null ) attrList += getAttrPair ( "multiple", "false" ); 
    }
    
    
    public String getAttrList() {
	return attrList + super.getAttrList();
    }
    
    
    public void addOption( OptionControl opt ) {
	options.addElement( opt );
    }
    
    
    public void addOptionGroup( OptGroupControl optGrp ) {
	options.addElement( optGrp );
    }
     
     
    public String getControlType() {
	return "SELECT";
    }
    
    
    public boolean equals( DefaultControl control ) {
	SelectControl scontrol = (SelectControl)control;
	
	if ( scontrol.name != null && scontrol.name.equalsIgnoreCase( name ) ) return true; 
	if ( scontrol.multiple != null && scontrol.multiple.equalsIgnoreCase( multiple ) ) return true;
	
	return false;
    }
}
