/* 
 * $COPYRIGHT$
 * $Id: TagExtractor.java,v 1.1 2001/12/24 19:13:24 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Remus Pereni      Created
 */
package org.media.mn8.protocol.http;

import org.media.mn8.concepts.AttributeSeries;
import org.media.mn8.concepts.StringConcept;
import org.media.mn8.concepts.AttributeConcept;
import org.media.mn8.concepts.Tag;
import java.util.Vector;
import java.util.StringTokenizer;
/** 
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.1 $ $Date: 2001/12/24 19:13:24 $
 */
public class TagExtractor {
    String input = null;
    boolean inTag = false;
    char curCh;
    int idx = 0;
    Vector tagList = new Vector();
    boolean withText = false;


    public TagExtractor() {

    }


    public TagExtractor(String input, boolean withText) {
        this.input = input;
        this.withText = withText;
    }
    

    public Vector parsePage(String input) {
        this.input = input;
        curCh = input.charAt(0);
        String text = null;    
        while( idx < input.length() ) {
            if( startTag() ) { 
                inTag= true;
                nextChar();
                tagFound( getTag() );
                inTag=false;
            }else {
                text= getText();
                if( !text.trim().equals(""))
                    textFound(text);
            }
            nextChar();
        }
        return tagList;
    }


    public void nextChar() {
        idx++;
        if( idx < input.length() -1 )
            curCh = input.charAt(idx);
    }


    public void previousChar() {
        if( idx > 0) {
            idx--;
            curCh = input.charAt(idx);
        }
    }
  
    public boolean startTag() {
        return !inTag && curCh  == '<';
    }


    public boolean endTag() {
        return inTag && curCh == '>';
    }


    public String getTag() {
        StringBuffer buffer = new StringBuffer();
        while( !endTag() && idx < input.length() - 1) {
            buffer.append( curCh );
            nextChar();
        }
        return buffer.toString();
    }


    public String getText() {
        StringBuffer buffer = new StringBuffer();
        while( !startTag() && idx < input.length()) {
            buffer.append( curCh );
            nextChar();
        }
        if( startTag() )
            previousChar();
        return buffer.toString();
    }


    public void tagFound(String tag) {
        String tagName  = getTagName(tag);
        if( tagName.trim().startsWith("?") ) {
            tagList.add( new Tag(tag.trim(), true, null));
        }else if( tagName.trim().startsWith("!--")) {
            tagList.add( new Tag(tag.trim(), true, null));
        }else if( tagName.trim().startsWith("!")) {
            tagList.add( new Tag(tag.trim(), true, null));
        }else {
            AttributeSeries attr = getAttributes(tagName, tag);
            tagList.add( new Tag( tagName, 
                                  tag.trim().lastIndexOf("/")== tag.trim().length()-1 || tag.trim().startsWith("/"), 
                                  attr.getVector()));
            //System.err.println("-- attribs:" +  attr.toTXT().toString());
        }
    }


    public void textFound(String text) {
        if( withText ) tagList.add(new StringConcept(text));
    }



    /**
     * The expected tag can be of the next form :
     * <code> (<)?(WS)*(/)?name (WS)* (/)? (WS)* (>)?  </code>
     * @param tag a <code>String</code> value
     * @return a <code>String</code> value
     */
    public static String getTagName(String tag) {
        //System.err.println("--: " + tag); 
        String result = "";
        int start = 0;
        tag= tag.trim();

        if( tag.startsWith("/") )
            start = 1;
        
        // I don't know what the hell I was thinking :), but who knows
        // int idx = eatWS(tag, start+1);
        // if( tag.charAt(idx) == '.' ){
        //    idx++;
        //    idx = eatWS(tag, idx);
        //}
        
        int idx = start;
        if( Character.isLetter(tag.charAt(idx))) {
            int endidx = tag.indexOf(" ", idx);
            if( endidx == -1 ) {
                endidx = tag.indexOf("\t", idx);
            }
            if( endidx == -1 ) {
                endidx = tag.indexOf("\r\n", idx);
            }
            if( endidx == -1 ) {
                endidx = tag.indexOf("\r", idx);
            }
            if( endidx == -1 ) {
                endidx = tag.indexOf("\n", idx);
            }
            if( endidx == -1 ) {
                endidx = tag.indexOf("/", idx);
            }
            if( endidx == -1 ) {
                endidx = tag.indexOf(">", idx);
            }
            if ( endidx == -1 ) {
                endidx = tag.length();
            }
            //System.err.println("--: "  + idx + "-" + endidx + "-" + tag.substring(idx, endidx)) ; 
            result = tag.substring(idx, endidx);
        } else {
            result = tag.trim();
        }
        return result;
    }


    public static AttributeSeries getAttributes(String tagName, String tag) {
        StringTokenizer tokenizer = new StringTokenizer(tag, " \t\r\n><");
        String currentToken = null;
        String key;
        String value;
        Vector result = new Vector();

        while( tokenizer.hasMoreTokens() ) {
            key=null;
            value = null;
                
            currentToken = tokenizer.nextToken();
            if( currentToken.startsWith("/") )
                currentToken = currentToken.substring(1);

            if( currentToken.toLowerCase().equals(tagName.toLowerCase()))
                continue;

            if( currentToken.equals("/") ||
                currentToken.equals("/>") ||
                currentToken.equals(">"))
                continue;

            if( currentToken.indexOf("=") != -1) {
                key = currentToken.substring(0, currentToken.indexOf("="));
                value = (currentToken.substring(currentToken.indexOf("=") + 1));
                if( value.indexOf("\"") != -1 && value.lastIndexOf("\"") ==  value.indexOf("\"")) {
                    currentToken = null;

                    while( tokenizer.hasMoreTokens() && (currentToken = tokenizer.nextToken()).indexOf("\"") == -1) {
                        value = value + " " + currentToken;
                        currentToken = null;
                    }
                    if( currentToken != null )
                        value = value + " " + currentToken;
                }
                value = stripQuotes(value);
                result.add( new AttributeConcept(key.toLowerCase(), null, new StringConcept(value)));
            }
            else {
                result.add( new AttributeConcept( currentToken.toLowerCase(), null, null));
            }
        }
        return new AttributeSeries( result);
    }


    public static int eatWS(String text, int start) {
        if( start >= text.length() ) return start -1 ;
        char ch = text.charAt(start);
        int result = start;
        while( Character.isWhitespace(ch) && result < text.length() -1) {
            result++;
            ch = text.charAt(result);
        }
        if( result == start) 
            return start -1;
        else 
            return result;
    }


	public static String stripQuotes(String src) {
		int h = src.indexOf('"');
		if ( h==-1 ) h = src.indexOf('\'');
		int t = src.lastIndexOf('"');
		if ( t==-1 ) t = src.lastIndexOf('\'');
		if ( h==-1 || t==-1 ) return src;
    if ( h+1 >= src.length()) return src;
		return src.substring(h+1,t);
	}
}
