/* 
 * $COPYRIGHT$
 * $Id: TextAreaControl.java,v 1.1 2002/01/31 20:36:17 crow Exp $
 *
 * Date        Author            Changes 
 * IAN 29 2002 Szabo Csaba       Created
 */

package org.media.mn8.protocol.http;

import java.util.Vector;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/01/31 20:36:17 $
 */

public class TextAreaControl extends DefaultAttributes 
			  implements DefaultControl {

    private static final String attrNames[] = {
             "name", "rows", "cols", "disabled", "readonly",
	     "tabindex", "accesskey", "onfocus", "onblur", "onselect", "onchange"
	     };

    // text area attibutes
    private String name;
    private String rows;
    private String cols;
    private String value;

    private String attrList = ""; 
    

    public TextAreaControl() {
    }

    
    public void setName( String name ) {
	this.name = name;
	attrList += getAttrPair ( "name", name ); 
    }
    
    
    public void setRows( String row ) {
	this.rows = row;
	attrList += getAttrPair ( "rows", row );
    }
    
    
    public void setCols( String col ) {
	this.cols = col;
	attrList += getAttrPair ( "cols", col );
    }
    
    
    public void setValue( String value ) {
	this.value = value;
    }


    public boolean isSendable() {
	if ( name != null && name.trim().length() > 0 &&
	     value != null && value.trim().length() > 0 ) return true;
	return false;
    }

    
    public String getName() {
	return name;
    }


    public Vector getValues() {
	Vector result = new Vector();
	result.add(value);
	return result;
    }


    public String toXMLString() {
	String ret = "<TEXTAREA " + getAttrList().trim() + ">\n"; 
	
	if ( value != null && value.trim().length() > 0 ) 
	    ret += "        <![CDATA[ " + value.trim() + " ]]>\n";
	ret += "    </TEXTAREA>";
	
	return ret;
    }


    public String toString() {
	return  "TEXTAREA : \n" + getAttrList().replace(' ', '\n') + 
	        "\nVALUE: " + value;
    }


    public void parse( String tag ) {
	super.parse( tag );

        if ( tag == null || tag.trim().length() < 1 ) return; 

	String attrs = getAttrsFromTag( tag ); 
        
	if ( attrs.trim().length() < 1 ) return;
	
        for ( int i = 0; i < attrNames.length; i++ ) {    
            String value = super.getValueOf( attrs, attrNames[i] );
    
            if ( value != null ) {
		if ( attrNames[i].equals("name") ) name = value;
		if ( attrNames[i].equals("rows") ) rows = value;
		if ( attrNames[i].equals("cols") ) cols = value;

		attrList += getAttrPair ( attrNames[i], value );
	    }
	}
    }


    public String getAttrList() {
	return attrList + super.getAttrList();
    }
     
     
    public String getControlType() {
        return "TEXTAREA";
    }
    
    
    public boolean equals( DefaultControl control ) {
	TextAreaControl tacontrol = (TextAreaControl)control;
	
	if ( tacontrol.name != null && tacontrol.name.equalsIgnoreCase( name ) ) return true;
	
	return false;
    } 
}
