/* 
 * $COPYRIGHT$
 * $Id: JabberDataBlockDispatcher.java,v 1.1 2002/04/09 16:47:46 crow Exp $
 *
 * Date        Author            Changes 
 * APR 08 2002 Szabo Csaba       Created
 */
package org.media.mn8.protocol.jabber;

import java.io.*;
import java.util.*;

/**
 * The dispatcher for blocks that have arrived. Adds new blocks to the
 * dispatch queue, and then dispatches waiting blocks in their own thread to
 * avoid holding up the stream reader.
 */
public class JabberDataBlockDispatcher extends Thread {
    /**
     * The recipient waiting on this stream
     */
    private JabberListener listener = null;

    /**
     * The list of messages waiting to be dispatched
     */
    private Vector waitingQueue = new Vector();
    
    /**
     * Flag to watch the dispatching loop
     */
    private boolean dispatcherActive;
    

    /**
     * Constructor to start the dispatcher in a thread.
     */
    public JabberDataBlockDispatcher() {
        start();
    }


    /**
     * Set the listener that we are dispatching to. Allows for switching
     * of clients in mid stream.
     *
     * @param _listener The listener to dispatch to.
     */
    public void setJabberListener( JabberListener _listener ) {
        listener = _listener;
    }


    /**
     * Method to add a datablock to the dispatch queue
     *
     * @param datablock The block to add
     */
    public void broadcastJabberDataBlock( JabberDataBlock dataBlock ) {
        waitingQueue.addElement( dataBlock );
    }


    /**
     * The thread loop that handles dispatching any waiting datablocks
     */
    public void run() {
        dispatcherActive = true;
        while( dispatcherActive ) {
            while( waitingQueue.size() == 0 ) {
                try {
                    Thread.sleep( 100L );
                } catch( InterruptedException e ) { }
            }
            
            JabberDataBlock dataBlock = (JabberDataBlock)waitingQueue.elementAt(0);
            waitingQueue.removeElementAt( 0 );
            if( listener != null ) listener.blockArrived( dataBlock );
        }
    }


    /**
     * Method to stop the dispatcher
     */
    public void halt() {
        dispatcherActive = false;
    }


    /**
     * Method to tell the listener the connection has been terminated
     *
     * @param exception The exception that caused the termination. This may be
     * null for the situtations where the connection has terminated without an
     * exception.
     */
    public void broadcastTerminatedConnection( Exception exception ) {
        halt();
        if( listener != null ) listener.connectionTerminated( exception );
    }


    /**
     * Method to tell the listener the stream is ready for talking to.
     */
    public void broadcastBeginConversation( ) {
        if( listener != null ) listener.beginConversation();
    }
}
