/* 
 * $COPYRIGHT$
 * $Id: Login.java,v 1.1 2002/04/09 16:47:46 crow Exp $
 *
 * Date        Author            Changes 
 * APR 08 2002 Szabo Csaba       Created
 */
package org.media.mn8.protocol.jabber.datablocks;

import org.media.mn8.protocol.jabber.*;

/**
 * The class representing the login message.
 * 
 * This has no incomming constructor as login messages are never
 * received from the server.
 */
public class Login extends JabberDataBlock {

    /**
     * Constructor. Builds the string ready for sending to the server.
     *
     * @param username The username to log in with
     * @param password The password to log in with
     * @param resource The resource name to use
     */
    public Login( String username, String password, String resource ) {
        super( );
     
        setAttribute( "id", "logon" );
        setAttribute( "type", "set" );

        JabberDataBlock queryBlock = new JabberDataBlock( "query", null, null );
        queryBlock.setAttribute( "xmlns", "jabber:iq:auth" );
        
        addChild(queryBlock);
        if( username != null ) {
            JabberDataBlock usernameBlock = new JabberDataBlock( "username", null, null );
            usernameBlock.addText( username );
            queryBlock.addChild( usernameBlock );
        }

        if( password != null ) {
            JabberDataBlock passwordBlock = new JabberDataBlock( "password", null, null );
            passwordBlock.addText( password );
            queryBlock.addChild( passwordBlock );
        }

        if( resource != null ) {
            JabberDataBlock resourceBlock = new JabberDataBlock( "resource", null, null );
            resourceBlock.addText( resource );
            queryBlock.addChild( resourceBlock );
        }
    }


    /**
     * Method to return the tag name
     *
     * @return Always the string "iq".
     */
    public String getTagName() {
        return "iq";
    }
}
