/* 
 * $COPYRIGHT$
 * $Id: Presence.java,v 1.1 2002/04/09 16:47:46 crow Exp $
 *
 * Date        Author            Changes 
 * APR 08 2002 Szabo Csaba       Created
 */
package org.media.mn8.protocol.jabber.datablocks;

import org.media.mn8.protocol.jabber.*;
import java.util.*;

/**
 * Class representing the presence message block.
 */
public class Presence extends JabberDataBlock {
    
    /**
     * Constructor.
     *
     * @param _parent The parent of this datablock
     * @param _attributes The list of element attributes
     */

    public Presence( JabberDataBlock _parent, Hashtable _attributes ) {
        super( _parent, _attributes );
    }


    /**
     * Default constructor for outgoing presence messages.
     */
    public Presence() {
        super( null, null );
    }


    /**
     * Method to fill this presence object with the information neccessary
     * to indicate the client is available.
     */
    public void setAvailable() {
        JabberDataBlock status = new JabberDataBlock( "status", this, null );
        status.addText("Online");
        addChild( status );
        
        JabberDataBlock priority = new JabberDataBlock( "priority", this, null );
        priority.addText("1");
        addChild( priority );
    }


    /**
     * Method to fill this presence object with the information neccessary
     * to indicate the client is available.
     */
    public void setUnavailable() {
        setAttribute( "type", "unavailable" );
        
        JabberDataBlock status = new JabberDataBlock( "status", this, null );
        status.addText("Unavailable");
        addChild( status );
        
        JabberDataBlock priority = new JabberDataBlock( "priority", this, null );
        priority.addText("1");
        addChild( priority );
    }
    

    /**
     * Method to fill this presence object with the information neccessary to do a probe.
     */
    public void setProbe() {
        setAttribute( "type", "probe" );
    }


    /**
     * Method to fill this presence object with the information neccessary to do a probe.
     */
    public void setSubscribe() {
        setAttribute( "type", "subscribe" );
    }


    /**
     * Method to set the presence type
     */
    public void setType( String type ) {
        setAttribute("type", type);
    }


    /**
     * Method to get the name of the tag
     */
    public String getTagName() {
        return "presence";
    }


    /**
     * Construct a reply message
     *
     * @return A message object destined for the sender of this message.
     */
    public Presence constructReply() {
        if( attributes == null ) return null;
         Presence reply = new Presence( );
        super.constructReply( reply );
        return reply;
    }
}
