/*
 * $COPYRIGHT$
 * $Id: POP3URLConnection.java,v 1.2 2002/02/01 09:19:06 neuro Exp $
 *
 * Date           Author         Changes
 * IAN 25 2002    Szabo Csaba    Created
 */

package org.media.mn8.protocol.pop3;

import java.io.*;
import java.net.*;
import java.util.Hashtable;
import java.util.Vector;
import java.util.StringTokenizer;

import java.net.MalformedURLException;

import com.oroinc.net.pop3.*;

/**
 * This class implements a connection to the new "pop3"
 * URL scheme.
 */

class POP3URLConnection extends URLConnection {
    private static final String DEF_HOST     = "localhost";
    private static final    int DEF_PORT     = 110;
    private static final String DEF_USER     = "guest";
    private static final String DEF_PASSWORD = "guest";

    private String host;
    private int    port;
    private String user;
    private String password;
    private String command;
    private String messID = null;

    private StringBuffer result = new StringBuffer();

    private boolean connected = false;
    private URL url;

    /**
     * Given a URL will instantiate a POP3URLConnection
     * object.
     * @param getURL The URL to contact.
     */
    POP3URLConnection( URL getURL ) {
        super( getURL );
        try {
            url = new URL( URLDecoder.decode( getURL.toString() ) );
        }
        catch ( Exception e) {
            System.out.println("Can't decode the URL: " + getURL.toString() );
        }
    }


    public void connect() throws IOException {

        if ( url.getHost().length() == 0 ) host = DEF_HOST;
        else host = url.getHost();

        if ( url.getPort() < 1 ) port = DEF_PORT;
        else port = url.getPort();

        String file = url.getFile();
        if ( file.equals("/") || file.trim().length() < 1 )
            command = "retrieve";
        else {
            if ( file.indexOf("?") < 0 ) command = file.substring(1);
            else command = file.substring( 1, file.indexOf("?") );
        }

        String ui = url.getUserInfo();
        if ( ui == null || ui.trim().length() < 1 ) {
            user = DEF_USER;
            password = DEF_PASSWORD;
        } 
        else {
            if ( ui.indexOf(":") != -1 ) {
                user = ui.substring( 0, ui.indexOf(":") );
                password = ui.substring( ui.indexOf(":")+1 );
            }
            else {
                user = ui;
                password = DEF_PASSWORD;
            }
            if ( user.trim().length() < 1 ) user = DEF_USER;
            if ( password.trim().length() < 1 ) password = DEF_PASSWORD;
        }

        String qu = url.getQuery();

        if ( qu != null && qu.indexOf("=") < 0 )
            throw new IOException("\nThe query structure is: ?id=messID\n");
        if ( qu.trim().length() != 0 ) messID =  qu.substring( qu.indexOf("=") + 1 );
 
        getConnection( host, port, user, password, command, messID );
    }


    public synchronized InputStream getInputStream() throws IOException {
        if ( !connected ) connect();
        return new ByteArrayInputStream( result.toString().getBytes() );
    }


    public String getCommand() {
        return command;
    }

    
    public String getMessageID() {
        return messID;
    }


    private void getConnection( String host, int port, String user, String password,
                                String command, String messID ) throws IOException {
           
        POP3Client client = new POP3Client( );
    
        try {                                                                    
            client.connect( host, port );
            if ( !client.login( user, password ) ) 
                throw new IOException( "Authentification failed." );
            connected = true;
        
            if ( command.equalsIgnoreCase("list") ) {
                if ( messID != null ) {
                    POP3MessageInfo info = client.listMessage( new Integer( messID ).intValue() );
                    if ( info != null ) result.append( info.number + " (" + info.size + ")\n" );
                }
                else {
                    POP3MessageInfo info[] = client.listMessages();
                    if ( info != null )
                        for ( int i = 0; i < info.length; i++ )
                            result.append( info[i].number + " (" + info[i].size + ")\n" );
                }

                return;
            }

            if ( command.equalsIgnoreCase("delete") ) {
                if ( messID != null ) {
                    if ( client.deleteMessage( new Integer(messID).intValue() ) ) 
                        result.append( messID + " message is deleted.\n");
                    else 
                        result.append( "Unable to delete message "  + messID + ".\n" );
                }
                else {
                    POP3MessageInfo info[] = client.listMessages();
                    boolean     allDeleted = true;
        
                    if ( info == null ) return;

                    for ( int i = 0; i < info.length; i++ )
                        if ( !client.deleteMessage( info[i].number ) ) allDeleted = false;
        
                    if ( allDeleted ) 
                        result.append("All messages is deleted.\n");
                    else
                        result.append("Can't delete all messages.\n");
                }

                return;
            }

            if ( command.equalsIgnoreCase("retrieve") ) {
                if ( messID != null ) { 
                    try {
                        result.append( getMessage( client.retrieveMessage ( new Integer( messID ).intValue( ) ) ) );
                    } 
                    catch ( Exception ex ) {
                        result.append( "Unable to read message " + messID + ".\n" );  
                    } 
                }
                else { 
                    POP3MessageInfo info[] = client.listMessages();
        
                    if ( info != null ) 
                        for ( int i = 0; i < info.length; i++ )
                            try {
                                result.append( getMessage( client.retrieveMessage ( info[i].number ) ) + "\n" );
                            } catch ( Exception ex ) {
                                result.append( "Unable to read message " + info[i].number + ".\n" );
                            }
                }

                return;
            }

            throw new IOException("Unknown command: " + command + ". Command must be: list, delete or retrieve.");
        }
        catch (MalformedURLException e) { throw new IOException( e.getMessage() ); }
        catch (IOException e)           { throw new IOException( e.getMessage() ); }
        catch (Exception e)             { throw new IOException( e.getMessage() ); }
    }

    
    private String getMessage( Reader rd ) throws Exception {
        BufferedReader reader = new BufferedReader( rd );
        StringBuffer       sb = new StringBuffer();
        String           line = null;

        while ( ( line = reader.readLine() ) != null ) sb.append ( line + "\n" );
        reader.close();
  
        return sb.toString();
    }
}
