/*
 * $COPYRIGHT$
 * $Id: XmlrpcURLConnection.java,v 1.1 2001/12/14 00:12:33 neuro Exp $
 *
 * Date           Author         Changes
 * sept 28 2001  Antal Attila    Created
 */

package org.media.mn8.protocol.xmlrpc;

import java.io.*;
import java.net.*;
import java.util.Hashtable;
import java.util.Vector;
import java.util.StringTokenizer;

import java.net.MalformedURLException;

import org.apache.xmlrpc.XmlRpc;                                                     
import org.apache.xmlrpc.XmlRpcClient;                                               
import org.apache.xmlrpc.XmlRpcException;

/**
 * This class implements a connection to the new "beep"
 * URL scheme.
 */
class XmlrpcURLConnection extends URLConnection {
    private static final String DEF_HOST     = "localhost";
    private static final    int DEF_PORT     = 10300;
    private static final String DEF_USER     = "guest";
    private static final String DEF_PASSWORD = "guest";

    private String host;
    private int    port;
    private String methodName;
    private String user;
    private String password;
    private Vector params = new Vector();

    private String paramNames = "";

    private String result = "";
    private boolean connected = false;
    private URL url;

    /**
     * Given a URL will instantiate a BeepURLConnection
     * object.
     * @param getURL The URL to contact.
     */
    XmlrpcURLConnection( URL getURL ) {
        super( getURL );
        try {
            url = new URL( URLDecoder.decode( getURL.toString() ) );
        }
        catch ( Exception e) {
            System.out.println("Can't decode the URL: " + getURL.toString() );
        }
    }


    public void connect() throws IOException {

        if ( url.getHost().length() == 0 ) host = DEF_HOST;
        else host = url.getHost();

        if ( url.getPort() < 1 ) port = DEF_PORT;
        else port = url.getPort();

        String file = url.getFile();
        if ( file.equals("/") || file.trim().length() < 1 )
            throw new IOException( "Missing method parameter from URL" );
        else {
            if ( file.indexOf("?") < 0 ) methodName = file.substring(1);
            else methodName = file.substring( 1, file.indexOf("?") );
        }

        if ( methodName.indexOf(".") < 0 )
            throw new IOException("\nThe method name must contain the handler eg.:\n" +
                                  "xmlrpc://user:password@host:port/handler.method?param1=...&param2=...&...");
  
        String ui = url.getUserInfo();
        if ( ui == null || ui.trim().length() < 1 ) {
            user = DEF_USER;
            password = DEF_PASSWORD;
        } 
        else {
            if ( ui.indexOf(":") != -1 ) {
                user = ui.substring( 0, ui.indexOf(":") );
                password = ui.substring( ui.indexOf(":")+1 );
            }
            else {
                user = ui;
                password = DEF_PASSWORD;
            }
            if ( user.trim().length() < 1 ) user = DEF_USER;
            if ( password.trim().length() < 1 ) password = DEF_PASSWORD;
        }


        String qu = url.getQuery();
        if ( qu == null)
            throw new IOException("\nYou must have query element in URL, to post parameters for handler\n");
        if ( qu.indexOf("=") < 0 )
            throw new IOException("\nThe query structure is: ?name1=parameter1&name2=parameter2&...\n");
        if ( qu.trim().length() != 0 ) {
            StringTokenizer pt = new StringTokenizer( qu, "&" );
            String t;
            try {
                while (pt.hasMoreTokens()) {
                    t = pt.nextToken();
                    if (paramNames.length() < 1)
                        paramNames = t.substring( 0, t.indexOf("=") );
                    else
                        paramNames += ", " + t.substring( 0, t.indexOf("=") );
                    params.addElement( t.substring( t.indexOf("=")+1 ) );
                }
            }
            catch ( Exception e ) {
                throw new IOException( "\nInvalid query attributes: " + qu );
            }
        }
        else throw new IOException( "\nQuery attributes in URL is missing\n" );

        getConnection( host, port, user, password, methodName, params );
    }


    public synchronized InputStream getInputStream() throws IOException {
        if ( !connected ) connect();
        return new ByteArrayInputStream( result.getBytes() );

    }


    public String getMethodName() {
        return methodName;
    }


    private void getConnection( String host, int port,
                                String user, String password,
                                String method, Vector methodParams ) throws IOException {

        String xURL = "http://" + host + ":" + port + "/";
        try {                                                                    
            XmlRpcClient client = new XmlRpcClient( xURL );   
            client.setBasicAuthentication( user, password );
            result = (String) client.execute( method, methodParams );      
            connected = true;
        }
        catch (MalformedURLException e) { throw new IOException( e.getMessage() ); }
        catch (XmlRpcException e)       { throw new IOException( e.getMessage()+"( "+paramNames+" )" ); }
        catch (IOException e)           { throw new IOException( e.getMessage() ); }
    }

}
