/* 
 * $COPYRIGHT$
 * $Id: ServletRequestConcept.java,v 1.3 2002/06/03 11:37:51 neuro Exp $
 *
 * Date        Author            Changes 
 * May 16 2002 Remus Pereni      Created
 */
package org.media.mn8.servlet;


import org.media.mn8.concepts.*;
import org.media.mn8.event.*;
import org.media.mn8.*;

import javax.servlet.http.*;
import javax.servlet.*;
import javax.naming.NamingException;
import java.util.Enumeration;
import java.util.Locale;
import java.io.PrintWriter;
import java.io.PrintStream;
import java.io.IOException;
import java.io.File;

/**
 * The servlet request implementation .
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.3 $ $Date: 2002/06/03 11:37:51 $
 */
public class ServletRequestConcept extends JavaConcept {

    protected HttpServletRequest _servletRequest = null;
    private MapConcept _headers = null;
    private MapConcept _parameters = null;
    private SeriesConcept _cookies = null;
    private SeriesConcept _locales = null;
    private ServletContext _servletContext = null;

    public ServletRequestConcept() {

    }


    public ServletRequestConcept( HttpServletRequest request, ServletContext context) {
        _servletRequest = request;
        _servletContext = context;
    }


    public StringConcept getAuthType() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getAuthType());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getContextPath() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getContextPath());
        } else {
            return new StringConcept();
        }
    }


    public ServletSession getSession() {
        if( _servletRequest != null ) {
            return new ServletSession( _servletRequest.getSession());
        } else {
            return new ServletSession();
        }
    }


    public ServletSession getSession(LogicalConcept create) {
        if( _servletRequest != null ) {
            return new ServletSession( _servletRequest.getSession( create.getValue()));
        } else {
            return new ServletSession();
        }
    }

    
    public SeriesConcept getCookies() {
        if( _cookies != null )
            return _cookies;

        SeriesConcept result = new SeriesConcept();
        if( _servletRequest != null ) {
            Cookie[] cookies = _servletRequest.getCookies();
            if( cookies != null ) {
                for( int i = 0 ; i < cookies.length; i++ ) {
                    CookieConcept cc = new CookieConcept(new StringConcept(cookies[i].getName()), new StringConcept(cookies[i].getDomain()));
                    if( cookies[i].getDomain() != null ) {
                        cc.setDomain( new StringConcept("" + cookies[i].getDomain()));
                    }
                    cc.setExpires( new StringConcept("" + cookies[i].getMaxAge()));

                    if( cookies[i].getPath() != null ) {
                        cc.setPath( new StringConcept("" + cookies[i].getPath()));
                    }
                    cc.setSecure( new StringConcept("" + cookies[i].getSecure()));
                    cc.setValue( new StringConcept("" + cookies[i].getValue()));
                    result.add(cc);
                }
            }
            _cookies = result;
        } 
        return result;
    }


    public SeriesConcept getLocales() {
        if( _locales != null )
            return _locales;

        SeriesConcept result = new SeriesConcept();
        Locale loc = null;
        if( _servletRequest != null ) {
            Enumeration locales = _servletRequest.getLocales();
            while( locales != null && locales.hasMoreElements() ) {
                loc = (Locale) locales.nextElement();
                if( loc != null ) 
                    result.add( new LocaleConcept(loc));
            }
            _locales = result;
        } 
        return result;
    }



    public StringConcept getMethod() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getMethod());
        } else {
            return new StringConcept();
        }
    }


    public MapConcept getHeaders() {
        if( _servletRequest != null ) {
            if( _headers != null ) {
                return _headers;
            }
            MapConcept result = new MapConcept();
            Enumeration headNames = _servletRequest.getHeaderNames();
            String name;
            while ( headNames != null && headNames.hasMoreElements() ) {
                name = (String) headNames.nextElement();
                result.add( new StringConcept(name), new StringConcept( _servletRequest.getHeader(name)));
            }
            _headers = result;
            return result ;
        } else {
            return new MapConcept();
        }
    }


    public MapConcept getParameters() {
        if( _servletRequest != null ) {
            if( _parameters != null ) {
                return _parameters;
            }
            MapConcept result = new MapConcept();
            Enumeration paramNames = _servletRequest.getParameterNames();
            String name;
            String[] values;

            while ( paramNames != null && paramNames.hasMoreElements() ) {
                name = (String) paramNames.nextElement();
                values = _servletRequest.getParameterValues(name);
                if ( values.length > 0 && values.length < 2 ) {
                    result.add( new StringConcept(name), new StringConcept( values[0]));
                } else {
                    SeriesConcept vals = new SeriesConcept();
                    for( int i =0 ; i < values.length ; i ++ ) {
                        vals.add( new StringConcept( values[i]));
                    }
                    result.add( new StringConcept(name), vals);                    
                }
            }
            _parameters = result;
            return result ;
        } else {
            return new MapConcept();
        }
    }


    public StringConcept getPathInfo() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getPathInfo());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getPathTranslated() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getPathTranslated());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getQueryString() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getQueryString());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getRemoteUser() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getRemoteUser());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getRequestedSessionId() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getRequestedSessionId());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getRequestURI() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getRequestURI());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getServletPath() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getServletPath());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getCharacterEncoding() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getCharacterEncoding());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getContentType() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getContentType());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getProtocol() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getProtocol());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getRemoteAddr() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getRemoteAddr());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getRemoteHost() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getRemoteHost());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getScheme() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getScheme());
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getServerName() {
        if( _servletRequest != null ) {
            return new StringConcept( _servletRequest.getServerName());
        } else {
            return new StringConcept();
        }
    }


    public IntegerConcept getContentLength() {
        if( _servletRequest != null ) {
            return new IntegerConcept( _servletRequest.getContentLength());
        } else {
            return new IntegerConcept(-1);
        }
    }


    public IntegerConcept getServerPort() {
        if( _servletRequest != null ) {
            return new IntegerConcept( _servletRequest.getServerPort());
        } else {
            return new IntegerConcept(-1);
        }
    }


    public StringConcept getUserPrincipalName() {
        if( _servletRequest != null && _servletRequest.getUserPrincipal() != null ) {
            return new StringConcept( _servletRequest.getUserPrincipal().getName());
        } else {
            return new StringConcept();
        }
    }



    public StringConcept getRealPath(StringConcept path) {
        if( _servletContext != null ) {
            return new StringConcept( _servletContext.getRealPath(path.toString()));
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getMimeType(StringConcept path) {
        if( _servletContext != null ) {
            return new StringConcept( _servletContext.getMimeType(path.toString()));
        } else {
            return new StringConcept();
        }
    }


    public StringConcept getResourceURL(StringConcept path) {
        if( _servletContext != null ) {
            try {
                return new StringConcept( _servletContext.getResource(path.toString()).toString());
            } catch (Exception ex) {
                return new StringConcept();                
            }
        } else {
            return new StringConcept();
        }
    }


    public LogicalConcept isRequestedSessionIdFromCookie() {
        if( _servletRequest != null ) {
            return new LogicalConcept( _servletRequest.isRequestedSessionIdFromCookie());
        } else {
            return new LogicalConcept(false);
        }
    }


    public LogicalConcept isSecure() {
        if( _servletRequest != null ) {
            return new LogicalConcept( _servletRequest.isSecure());
        } else {
            return new LogicalConcept(false);
        }
    }


    public LogicalConcept isRequestedSessionIdFromURL() {
        if( _servletRequest != null ) {
            return new LogicalConcept( _servletRequest.isRequestedSessionIdFromURL());
        } else {
            return new LogicalConcept(false);
        }
    }


    public LogicalConcept isRequestedSessionIdValid() {
        if( _servletRequest != null ) {
            return new LogicalConcept( _servletRequest.isRequestedSessionIdValid());
        } else {
            return new LogicalConcept(false);
        }
    }


    public LogicalConcept isUserInRole(StringConcept roleName) {
        if( _servletRequest != null ) {
            return new LogicalConcept( _servletRequest.isUserInRole(roleName.toString()));
        } else {
            return new LogicalConcept(false);
        }
    }


    public ConceptDefinition getConceptDefinition() {
        return ServletRequestDefinition.getReference();
    }

    
    public void setRequest( HttpServletRequest request ) {
        _servletRequest = request;
    }


    public HttpServletRequest getReuqest() {
        return _servletRequest;
    }


    public Object clone() {
        return this;
    }
}
