/* 
 * $COPYRIGHT$
 * $Id: ServletResponseConcept.java,v 1.2 2002/05/21 14:07:39 neuro Exp $
 *
 * Date        Author            Changes 
 * May 16 2002 Remus Pereni      Created
 */
package org.media.mn8.servlet;


import org.media.mn8.concepts.*;
import org.media.mn8.event.*;
import org.media.mn8.*;

import javax.servlet.http.*;
import javax.servlet.*;
import javax.naming.NamingException;
import java.util.Enumeration;
import java.io.PrintWriter;
import java.io.PrintStream;
import java.io.IOException;
import java.io.File;

/**
 * The servlet session implementation .
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.2 $ $Date: 2002/05/21 14:07:39 $
 */
public class ServletResponseConcept extends JavaConcept {
    private HttpServletResponse _response = null;
    

    public ServletResponseConcept() {

    }


    public ServletResponseConcept( HttpServletResponse response) {
        _response = response;
    }


    public Concept create() {
        return this;
    }


    public void setContentType( StringConcept contentType ) {
        if ( _response !=  null ) {
            _response.setContentType(contentType.toString());
        }
    }


    public void addCookie( CookieConcept cookie ) {
        if ( _response !=  null ) {
            _response.addCookie(cookie.getCookie().getJavaCookie());
        }
    }


    public StringConcept encodeURL( StringConcept url ) {
        if ( _response !=  null ) {
            return new StringConcept(_response.encodeURL(url.toString()));
        }
        return new StringConcept();
    }


    public StringConcept encodeRedirectURL( StringConcept url ) {
        if ( _response !=  null ) {
            return new StringConcept(_response.encodeRedirectURL(url.toString()));
        }
        return new StringConcept();
    }


    public void setContentLength( IntegerConcept length ) {
        if ( _response !=  null ) {
            _response.setContentLength(length.getInt());
        }
    }


    public void setBufferSize( IntegerConcept size ) {
        if ( _response !=  null ) {
            _response.setBufferSize(size.getInt());
        }
    }


    public void setStatus( IntegerConcept statusCode ) {
        if ( _response !=  null ) {
            _response.setStatus(statusCode.getInt());
        }
    }


    public void sendError( IntegerConcept statusCode, StringConcept message ) {
        if ( _response !=  null ) {
            _response.setStatus(statusCode.getInt(), message.toString());
        }
    }


    public void sendError( IntegerConcept statusCode) {
        if ( _response !=  null ) {
            _response.setStatus(statusCode.getInt());
        }
    }


    public void setHeader( StringConcept name, StringConcept value ) {
        if ( _response !=  null ) {
            _response.setHeader(name.toString(), value.toString());
        }
    }


    public void addHeader( StringConcept name, StringConcept value ) {
        if ( _response !=  null ) {
            _response.addHeader(name.toString(), value.toString());
        }
    }


    public void sendRedirect( StringConcept location ) {
        if ( _response !=  null ) {
            try {
                _response.sendRedirect(location.toString());
            } catch (IOException ex) {
                if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose()) {
                    System.err.println("ServletResponse:sendRedirect: " + ex.getMessage());
                }
            }

        }
    }


    public IntegerConcept getBufferSize( ) {
        if ( _response !=  null ) {
           return new IntegerConcept( _response.getBufferSize() );
        }
        return new IntegerConcept(-1);
    }


    public StringConcept getCharacterEncoding( ) {
        if ( _response !=  null ) {
           return new StringConcept( _response.getCharacterEncoding() );
        }
        return new StringConcept();
    }


    public void flushBuffer( ) {
        if ( _response !=  null ) {
            try {
                _response.flushBuffer();
            } catch (IOException ex) {
                if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose()) {
                    System.err.println("ServletResponse:flushBuffer: " + ex.getMessage());
                }
            }
        }
    }


    public void reset( ) {
        if ( _response !=  null ) {
            _response.reset();
        }
    }


    public LogicalConcept isCommitted( ) {
        if ( _response !=  null ) {
           return new LogicalConcept( _response.isCommitted());
        }
        return new LogicalConcept(false);
    }


    public LogicalConcept containsHeader( StringConcept name) {
        if ( _response !=  null ) {
           return new LogicalConcept( _response.containsHeader( name.toString()));
        }
        return new LogicalConcept(false);
    }



    public ConceptDefinition getConceptDefinition() {
        return ServletResponseDefinition.getReference();
    }


    public Object clone() {
        return this;
    }
}
