/* 
 * $COPYRIGHT$
 * $Id: ServletSession.java,v 1.2 2002/05/21 14:07:15 neuro Exp $
 *
 * Date        Author            Changes 
 * May 16 2002 Remus Pereni      Created
 */
package org.media.mn8.servlet;


import org.media.mn8.concepts.*;
import org.media.mn8.event.*;
import org.media.mn8.*;

import javax.servlet.http.*;
import javax.servlet.*;
import javax.naming.NamingException;
import java.util.Enumeration;
import java.io.PrintWriter;
import java.io.PrintStream;
import java.io.IOException;
import java.io.File;

/**
 * The servlet session implementation .
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.2 $ $Date: 2002/05/21 14:07:15 $
 */
public class ServletSession extends JavaConcept {
    private HttpSession _session = null;
    private MapConcept _attributes = null;

    public ServletSession() {
    }


    public ServletSession( HttpSession session) {
        _session = session;
    }


    public Concept create() {
        return this;
    }



    public IntegerConcept getCreationTime() {
        if( _session != null ) {
            return new IntegerConcept( _session.getCreationTime() );
        }
        return new IntegerConcept(-1);
    }


    public IntegerConcept getLastAccessedTime() {
        if( _session != null ) {
            return new IntegerConcept( _session.getLastAccessedTime() );
        }
        return new IntegerConcept(-1);
    }


    public IntegerConcept getMaxInactiveInterval() {
        if( _session != null ) {
            return new IntegerConcept( _session.getMaxInactiveInterval() );
        }
        return new IntegerConcept(-1);
    }


    public void setMaxInactiveInterval(IntegerConcept interval) {
        if( _session != null ) {
            _session.setMaxInactiveInterval(interval.getInt());
        }
    }


    public MapConcept getAttributes() {
        if( _attributes != null ) {
            return _attributes;
        }

        MapConcept result = new MapConcept();
        if( _session != null ) {
            Enumeration attrNames = _session.getAttributeNames();
            if( attrNames  == null ) { return result; }
            
            String name;
            while (attrNames.hasMoreElements()) {
                name = (String) attrNames.nextElement();
                result.add( new StringConcept(name), (Concept) _session.getAttribute(name));
            }            
        }
        _attributes = result;
        return result;
    }


    public void setAttributes(MapConcept attr) {

        if( _session != null ) {
            for( int i = 0 ; i < attr.getKeyVector().size();  i++ ) {
                _session.setAttribute( ((Concept)attr.getKeyVector().elementAt(i)).toTXT().toString(),
                                       ((Concept)attr.getValuesVector().elementAt(i)));

            }
        }
        _attributes = attr;
    }


    public void invalidate() {
        if( _session != null ) {
            _session.invalidate();
        }
    }


    public LogicalConcept isNew() {
        if( _session != null ) {
            return new LogicalConcept( _session.isNew() );
        }
        return new LogicalConcept(false);
    }


    public StringConcept getId() {
        if( _session != null ) {
            return new StringConcept( _session.getId() );
        }
        return new StringConcept();
    }


    public void setSession(HttpSession session) {
        _session = session;
    }


    public HttpSession getSession() {
        return _session;
    }


    public ConceptDefinition getConceptDefinition() {
        return ServletSessionDefinition.getReference();
    }


    public Object clone() {
        return this;
    }
}
