/* 
 * $COPYRIGHT$
 * $Id: mn8RequestMapper.java,v 1.2 2002/06/05 00:19:05 neuro Exp $
 *
 * Date        Author            Changes 
 * May 16 2002 Remus Pereni      Created
 */
package org.media.mn8.servlet;


import org.media.mn8.concepts.*;
import org.media.mn8.event.*;
import org.media.mn8.*;

import org.apache.oro.text.regex.*;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

import javax.naming.NamingException;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;

/**
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.2 $ $Date: 2002/06/05 00:19:05 $
 */
public class mn8RequestMapper {
    protected Pattern[]  _pathExpressions = null;
    protected String[] _conceptNames = null;
    protected SeriesConcept[] _conceptParams = null;
    protected Perl5Matcher _matcher = new Perl5Matcher();

    public mn8RequestMapper() {
    }
    

    public void parseMapInfo( File mapInfo ) {
        MapInfoParser mapParser = new MapInfoParser( mapInfo);

        if( mapParser.conceptName != null && 
            mapParser.conceptParams != null && 
            mapParser.pathInfo != null &&
            mapParser.conceptParams.size() == mapParser.conceptName.size() &&
            mapParser.conceptName.size() == mapParser.pathInfo.size() ) {
            _pathExpressions = new Pattern[ mapParser.pathInfo.size()];
            _conceptNames = new String[ mapParser.pathInfo.size()];
            _conceptParams = new SeriesConcept[ mapParser.pathInfo.size()];
            Perl5Compiler compiler = new Perl5Compiler();
            for( int i = 0 ; i < mapParser.pathInfo.size(); i++) {
                try {
                    _pathExpressions[i] = compiler.compile( (String)mapParser.pathInfo.elementAt(i));
                    _conceptNames[i] = (String) mapParser.conceptName.elementAt(i);
                    _conceptParams[i] = (SeriesConcept) mapParser.conceptParams.elementAt(i);
                } catch ( MalformedPatternException mex) {
                    mex.printStackTrace();
                }
            }
        }
    }


    public MapInfoResult getMatch( String uri ) {
        for( int i=0 ; i < _pathExpressions.length ; i++ ) {
            if( _matcher.matches( uri.trim(), _pathExpressions[i])) {
                MapInfoResult result = new MapInfoResult();
                result.mappedPath = _pathExpressions[i].getPattern();
                result.conceptName = _conceptNames[i];
                result.conceptParams = _conceptParams[i];
                return result;
            }
        }
        return null;
    }


    public void printDebugInfo() {
        System.err.println("MN8 Servlet : Map Info ");
        if( _pathExpressions != null && _conceptNames != null && _conceptParams != null ) {
            for( int i = 0 ; i < _pathExpressions.length; i ++ ) {
                System.err.println("\t" + i + ".) " + _pathExpressions[i].getPattern() + " -> " + _conceptNames[i]);
                Vector params = _conceptParams[i].getVector();
                System.err.println("\t\t With the parameters:");
                for( int j = 0; j < params.size() ; j++ ) {
                    System.err.println("\t\t" + ((StringConcept)params.elementAt(j)).toString());
                }
            }
        }
    }


    class MapInfoParser extends DefaultHandler {
        public Vector pathInfo;
        public Vector conceptName;
        public Vector conceptParams;

        private String _conceptName;
        private SeriesConcept _conceptParams;
        private boolean _isPath = false;
        private boolean _isParam = false;
        private StringBuffer _value = null;
        
        public MapInfoParser( File mapInfoFile ) {
            pathInfo = new Vector();
            conceptName = new Vector();
            conceptParams = new Vector();
            try {
                SAXParserFactory sxParserFactory = SAXParserFactory.newInstance();
                SAXParser sxParser = sxParserFactory.newSAXParser();
                XMLReader sxReader = sxParser.getXMLReader();
                sxReader.setContentHandler(this);
                sxReader.parse((new InputSource( new FileReader(mapInfoFile))));
            } catch ( ParserConfigurationException pce ) {
                pce.printStackTrace();
            } catch ( SAXException saxe ) {
                saxe.printStackTrace();
            } catch ( IOException iox ) {
                iox.printStackTrace();
            }            

        }

        public void startElement ( String uri, String qname, String name, Attributes attr) 
            throws SAXException {

            if ( name.equalsIgnoreCase("path") ) {
                _isPath = true;
            }


            if ( name.equalsIgnoreCase("parameters") ) {
                _conceptParams = new SeriesConcept();
            }


            if ( name.equalsIgnoreCase("concept")) {                
                if ( attr.getLength() >= 1 &&
                     attr.getQName(0).toLowerCase().equals("name") &&
                     ( attr.getValue(0) != null || !attr.getValue(0).equals(""))) {
                     _conceptName = attr.getValue(0);
                } else if ( attr.getLength() >= 1 &&
                     attr.getQName(0).toLowerCase().equals("path") &&
                     ( attr.getValue(0) != null || !attr.getValue(0).equals(""))) {
                     _conceptName = attr.getValue(0);
                }
            }
        }


        public void characters(char[] ch, int start, int length ) throws SAXException {
            if( _value == null ) {
                _value = new StringBuffer();
            }
            String str = new String( ch, start, length);
            if( str.trim().length() > 0 ) {
                _value.append(str);
            }
        }
    

        public void endElement ( String uri, String name, String qname ) 
            throws SAXException {

            if( name.equalsIgnoreCase("path") || qname.equalsIgnoreCase("path")) {
                if( _value.toString() != null ) {
                    pathInfo.add( _value.toString().trim());
                    _value = null;
                }
            }

            if( name.equalsIgnoreCase("param") || qname.equalsIgnoreCase("param")) {
                if( _value.toString() != null ) {
                    _conceptParams.add( new StringConcept(Helper.fillWithValues(_value.toString())));
                    _value = null;
                }
            }

            if( name.equalsIgnoreCase("concept") || qname.equalsIgnoreCase("concept")) {
                if( _conceptName != null ) {
                    if( _conceptParams == null ) {
                        _conceptParams = new SeriesConcept();
                    }
                    conceptName.add( _conceptName);
                    conceptParams.add(_conceptParams);
                    _conceptParams = null;
                    _conceptName = null;
                }
            }

        }


        public void error ( SAXParseException spe) 
            throws SAXParseException {
            throw spe;
        }

    }
}
