/* 
 * $COPYRIGHT$
 * $Id: mn8Runner.java,v 1.4 2002/06/05 00:19:45 neuro Exp $
 *
 * Date        Author            Changes 
 * May 16 2002 Remus Pereni      Created
 */
package org.media.mn8.servlet;


import org.media.mn8.concepts.*;
import org.media.mn8.event.*;
import org.media.mn8.*;

import javax.servlet.http.*;
import javax.servlet.*;
import javax.naming.NamingException;
import java.util.Enumeration;
import java.net.MalformedURLException;
import java.io.PrintWriter;
import java.io.PrintStream;
import java.io.IOException;
import java.io.File;

/**
 * The servlet implementation which will run mn8 scripts or concepts.
 * from any servlet aware web server (Apache, IIS, ...).
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.4 $ $Date: 2002/06/05 00:19:45 $
 */
public class mn8Runner extends HttpServlet {
    static final String ENV_PATH = "env:/servlet/";
    private ServletContext _servletContext = null;
    private ServletConfig _servletConfig = null;
    private mn8RequestMapper _requestMapper = new mn8RequestMapper();
    private boolean _enableConceptLookup = false;
    private boolean _enableMapInfo       = false;

    public void init(ServletConfig config ) throws ServletException {
        System.setProperty("java.protocol.handler.pkgs", "org.media.mn8.protocol");
        _servletConfig = config;
        mn8Interpreter.initialize();
        try {
            _servletContext = config.getServletContext();
            NamingManager.envBind(ENV_PATH + "server.info", new StringConcept(_servletContext.getServerInfo()));
            NamingManager.envBind(ENV_PATH + "servlet.name", new StringConcept(config.getServletName()));
            NamingManager.envBind(ENV_PATH + "servlet.api.version", new StringConcept("" + _servletContext.getMajorVersion() + "." +_servletContext.getMinorVersion()));

            Enumeration paramNames = config.getInitParameterNames();
            String name;
            while( paramNames.hasMoreElements() ) {
                name = (String)paramNames.nextElement();
                NamingManager.envBind(ENV_PATH + "/parameters/" + name,
                                      new StringConcept( Helper.fillWithValues(config.getInitParameter(name))));
            }
            if( config.getInitParameter("debug") != null && config.getInitParameter("debug").equalsIgnoreCase("true")) {
                mn8RuntimeFlags.setDebug(true);
            }

            if( config.getInitParameter("verbose") != null && config.getInitParameter("verbose").equalsIgnoreCase("true")) {
                mn8RuntimeFlags.setVerbose(true);
            }

            if( config.getInitParameter("EnableConceptLookup") != null && config.getInitParameter("EnableConceptLookup").equalsIgnoreCase("true")) {
                _enableConceptLookup = true;
            }

            if( config.getInitParameter("MapInfo") != null) {
                String path = config.getInitParameter("MapInfo");
                File fpath;                
                path = Helper.fillWithValues( path);
                if( path.trim().equals("")) {
                    try {
                        if( _servletContext.getResource("/mn8MapInfo.xml") != null ) {
                            path = _servletContext.getResource("/mn8MapInfo.xml").toString();
                        }
                    } catch ( MalformedURLException mex) {
                        System.err.println(mex.getMessage());
                    }
                }
                if( path != null && !path.trim().equals("")) {
                    fpath = new File(path);
                    try {
                        System.err.println("Loading MN8 Mapping Info from: " +  fpath.toURL().toString());
                    } catch ( MalformedURLException mex) {
                        System.err.println(mex.getMessage());
                    }
                    _enableMapInfo = true;
                    _requestMapper.parseMapInfo( fpath );
                    if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose() ) {
                        _requestMapper.printDebugInfo();
                    }
                }
            }

        } catch (NamingException nex) {
            throw new ServletException( nex.getMessage());
        }
    }

    

    public String getServletInfo() {
        return "mn8Runner is a Java servlet which allows MN8 based scripts or concepts"
            + " to be run server side, for more information on MN8 please"
            + " visit: http://mappa.mundi.net/spacemapper";
    }
 


    /**
     * Handle the HTTP GET method by building a simple web page.
     */
    public void service (HttpServletRequest request, HttpServletResponse response) 
        throws ServletException, IOException {

        ServletRequestConcept requestConc = new ServletRequestConcept(request, _servletContext);
        ServletResponseConcept responseConc = new ServletResponseConcept(response);

        mn8Interpreter.setOut(new PrintStream(response.getOutputStream()) , Thread.currentThread());
        mn8Interpreter.setErr(System.err , Thread.currentThread());
        try {
            if( request.getPathTranslated() != null ) {
                File path = new File(request.getPathTranslated());
                SeriesConcept paramVal = new SeriesConcept();
                paramVal.add( requestConc );
                paramVal.add( responseConc );

                if( path.exists() && path.isFile() && path.getName().toLowerCase().endsWith(".mn8")) {
                    SeriesConcept param = new SeriesConcept();
                    param.add( paramVal );
                    mn8Interpreter.threadExecuteURLRun(path.toURL().toString(), null, param);
                } else {
                    if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose()) {
                        System.err.println("MN8 Servlet: not direct call");
                    }

                    MapInfoResult result = _requestMapper.getMatch( request.getPathInfo());

                    if( _enableMapInfo && result != null ) {

                        if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose()) {
                            System.err.println("MN8 Servlet: map enabled");
                        }
                        
                        if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose()) {
                            System.err.println("MN8 Servlet: mapped to : " + result.conceptName);
                        }
                        
                        if( ! result.conceptName.equalsIgnoreCase("_SYSTEM_")) {
                            paramVal.addSeries( result.conceptParams );
                            SeriesConcept param = new SeriesConcept();
                            param.add(paramVal);
                            mn8Interpreter.threadExecuteURLRun(result.conceptName, null, param);
                        } else {
                            (new mn8SystemServlet(_servletConfig, request, response)).serve();
                        }

                    } else if ( _enableConceptLookup ) {
                        String conceptName = request.getPathInfo();
                        if( conceptName.startsWith("/")) {
                            conceptName = conceptName.substring(1);
                        }
                        
                        if( conceptName.indexOf("/") != -1 ) {
                            conceptName = conceptName.substring(0, conceptName.indexOf("/"));
                        }

                        if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose()) {
                            System.err.println("MN8 Servlet: Concept Lookup : " + conceptName );
                        }

                        try {
                            String fileName = (new mn8Loader()).findConcept(conceptName);

                            if( mn8RuntimeFlags.isDebug() || mn8RuntimeFlags.isVerbose()) {
                                System.err.println("MN8 Servlet: Concept Lookup found: " + fileName );
                            }
                            SeriesConcept param = new SeriesConcept();
                            param.add(paramVal);
                            mn8Interpreter.threadExecuteURLRun(fileName, null, param);
                        } catch ( ConceptNotFoundException cex) {
                            response.sendError(404, cex.getMessage());
                            cex.printStackTrace();
                        }
                    } else if( path.exists() && path.isFile() ) {
                        response.sendError(501);               
                    } else if( path.exists() && path.isDirectory() ) {
                        response.sendError(403, "Directory listing forbidden on this server.");
                    } else {
                        response.sendError(404, "The requested URL was not found on this server.");
                    }
                }
            } 
        } catch (RuntimeException rex) {
            response.sendError(500, rex.getMessage());
            throw rex;
        } finally {
            PrintStream out = mn8Interpreter.getOut(Thread.currentThread());
            PrintStream err = mn8Interpreter.getErr(Thread.currentThread());
            if( out != System.out ) {
                out.flush();
                out.close();
            }
            if( err != System.err ) {
                err.flush();
                err.close();
            }
            mn8Interpreter.cleanStreams( Thread.currentThread());
        }
    }
}
