/* 
 * $COPYRIGHT$
 * $Id: CronMultiplier.java,v 1.1 2002/05/18 22:20:42 atech Exp $
 *
 * Date        Author            Changes 
 * May 16 2002 Antal Attila      Created
 */

package org.media.mn8.util;

import java.util.Vector;
import java.util.StringTokenizer;

/**
 * This is the multiplier for crons to make crons with (*), intervals (3-12),
 * selectors (2-8/2) and lists (3,4,8).
 *
 * @author <a href="mailto:atech@nolimits.ro">Antal Attila</a>
 */
public class CronMultiplier {
                                                                                
	CronElement ce = new CronElement();
    private Vector avalonCrons = new Vector();
	private Vector repMins = new Vector();
	private Vector repHours = new Vector();
	private Vector repDays = new Vector();
	private Vector repMonths = new Vector();
	private Vector repWeekDays = new Vector();
	private boolean weekDaysFound = true;
	private boolean mnthDaysFound = true;
	private byte paramNr = 1;
	private String errMsg;
	public boolean ignored = false;
                                                                                
	public CronMultiplier() {}
	
	public void setMaker( String cron ) {                                   
		StringTokenizer st = new StringTokenizer( cron, " " );              
		while (st.hasMoreTokens() && paramNr <= 5 ) {                       
			comaParser( st.nextToken() );                                   
			paramNr++;                                                      
		}                                                                   
		wormCollector();                                                    
	}
	
	public void wormCollector() {                                           
		if ( weekDaysFound ) makeCronElements( repWeekDays, true );         
		if ( mnthDaysFound ) makeCronElements( repDays, false );            
		if ( !weekDaysFound && !mnthDaysFound )                             
			makeCronElements( repDays, false );                             
	}

	public void makeCronElements( Vector days, boolean week) {              
		byte min, hour, day, month;                                         
		for ( int i = 0; i < repMins.size(); i++ ) {                        
			min = ((Integer)repMins.get(i)).byteValue();                    
			for ( int j = 0; j < repHours.size(); j++ ) {                   
				hour = ((Integer)repHours.get(j)).byteValue();              
				for ( int k = 0; k < days.size(); k++ ) {                   
					day = ((Integer)days.get(k)).byteValue();               
					for ( int l = 0; l < repMonths.size(); l++ ) {          
						month = ((Integer)repMonths.get(l)).byteValue();    
						CronElement newCE = new CronElement();              
						newCE.min = min;                                    
						newCE.hour = hour;                                  
						newCE.day = day;                                    
						newCE.month = month;                                
						newCE.week = week;                                  
						avalonCrons.addElement( newCE );                    
					}                                                       
				}                                                           
			}                                                               
		}                                                                   
	}
	
	public void setCurrent( int current ) {                                 
		ce = ( CronElement )avalonCrons.elementAt( current );               
	}                                                                       
	
	
	public int getSize() {                                                  
		return avalonCrons.size();                                          
	}                                                                       
                                                                                
	
	public int getMin() {                                                   
		return ce.min;                                                      
	}                                                                       
	
                                                                                
	public int getHour() {                                                  
		return ce.hour;                                                     
	}
	

	public int getDay() {                                                   
		return ce.day;                                                      
	}                                                                       

	
	public int getMonth() {                                                 
		return ce.month;                                                    
	}                                                                       
                                                                                
                                                                                
	public boolean getIsDayOfWeek() {                                       
		return ce.week;                                                     
	}                                                                       


	public String getErrMsg() {
		return errMsg;
	}
	
	
	private byte paramLow( byte par ) {                                     
		if ( par == 1 || par == 2 || par == 5 ) return 0;                   
		else return 1;                                                      
	}
	

	private byte paramHigh( byte par ) {                                    
		if ( par == 1 ) return 59;                                          
		else if ( par == 2 ) return 23;                                     
		else if ( par == 3 ) return 31;                                     
		else if ( par == 4 ) return 12;                                     
		else return 7;                                                      
	}
	

	private void comaParser( String worm ) {                                
		byte avalonTypeOfAll = -1;                                          
		StringTokenizer coma = new StringTokenizer( worm, ",");             
		while ( coma.hasMoreTokens() ) {                                    
			String _coma = coma.nextToken();                                
			if ( _coma.equals("*") ) {                                      
				wormMaker( avalonTypeOfAll );                               
				if ( paramNr == 5 ) weekDaysFound = false;                  
				if ( paramNr == 3 ) mnthDaysFound = false;                  
				break;                                                      
			}                                                               
			else if ( convert(_coma) >= 0 ) {                               
				wormMaker( convert(_coma) );                                
			}                                                               
			else {                                                          
				slashParser(_coma);                                         
			}                                                               
		}                                                                   
	}
	

	private void slashParser ( String slash ) {                             
		StringTokenizer _slash =                                            
			new StringTokenizer( slash, "/");                               
		String interval = "";                                               
		byte count = 1;                                                     
		int step = 1;                                                       
		while ( _slash.hasMoreTokens() ) {                                  
			if ( count == 1 ) interval = _slash.nextToken();                
			if ( count == 2 ) {                                             
				step = convert(_slash.nextToken());                         
				break;                                                      
			}                                                               
			count++;                                                        
		}                                                                   
		if ( interval.equals("*") ) {                                       
			for ( byte cx = paramLow(paramNr);                              
				  cx <= paramHigh(paramNr);                                 
				  cx += step ) {                                            
				wormMaker( cx );                                            
			}                                                               
		}
		else if ( convert( interval ) >= 0 ) {                              
			wormMaker( convert(interval) );                                 
		}                                                                   
		else {                                                              
			minusParser ( interval, step );                                 
		}                                                                   
	}


	private void minusParser ( String interval, int step ) {                
		StringTokenizer minus =                                             
			new StringTokenizer( interval, "-");                            
		int first = 0, last = 0;                                            
		byte count = 1;                                                     
		while ( minus.hasMoreTokens() ) {                                   
			if ( count == 1 ) first = convert( minus.nextToken() );         
			if ( count == 2 ) {                                             
				last = convert( minus.nextToken() );                        
				break;                                                      
			}                                                               
			count++;                                                        
		}                                                                   
		for ( int cx = first; cx <= last; cx += step ) {                    
			wormMaker( cx );                                                
		}                                                                   
	}


	private void wormMaker( int worm ) {                                    
		if (!ignored) {                                                     
			switch ( paramNr ) {                                            
			case 1 : repMins.add( new Integer(worm) ); break;               
			case 2 : repHours.add( new Integer(worm) ); break;              
			case 3 : repDays.add( new Integer(worm) ); break;               
			case 4 : repMonths.add( new Integer(worm) ); break;             
			case 5 : repWeekDays.add( new Integer(worm) ); break;           
			}                                                               
		}                                                                   
	}


	private int convert( String conv ) {                                    
		int retValue = -1;                                                  
		try {                                                               
			retValue = new Integer( conv ).intValue();                      
			if ( (retValue < paramLow(paramNr) ||                           
				  retValue > paramHigh(paramNr)) && !ignored ) {            
				ignoreCron( conv );                                         
			}                                                               
		}                                                                   
		catch ( Exception e ) {                                             
			if ( conv.indexOf(',') == -1 &&                                 
				 conv.indexOf('-') == -1 &&                                 
				 conv.indexOf('/') == -1 && !ignored ) {                    
				ignoreCron( conv );                                         
			}                                                               
		}                                                                   
		return retValue;                                                    
	}

	
	private void ignoreCron( String ignoredParam ) {                        
		String paramWord = "";                                              
		switch(paramNr) {                                                   
		case 1: paramWord = "MINUTE"; break;                                
		case 2: paramWord = "HOUR"; break;                                  
		case 3: paramWord = "DAY"; break;                                   
		case 4: paramWord = "MONTH"; break;                                 
		case 5: paramWord = "DAY OF WEEK"; break;                           
		}                                                                   
		errMsg =  "Bad " +paramWord+ " parameter:" + ignoredParam;
		ignored = true;                                                     
	}


	class CronElement {                                                         
        public int min;                                                         
        public int hour;                                                        
        public int day;                                                         
        public int month;                                                       
        public boolean week;                                                    
        private CronElement() {                                                 
        }                                                                       
    }                                                                           
                                                                                
                                                                                
    class CronCollector {                                                       
        public String targetName  = "";                                         
        public String cronString  = "";                                         
        public String targetClass = "";                                         
        private CronCollector() {                                               
        }                                                                       
    }
}
