/* 
 * $COPYRIGHT$
 * $Id: CronScheduledEntry.java,v 1.1 2002/05/18 22:20:42 atech Exp $
 *
 * Date        Author            Changes 
 * May 16 2002 Antal Attila      Created
 */


package org.media.mn8.util;

/**
 * @author <a href="mailto:atech@nolimits.ro">Antal Attila</a>
 * @version $Revision: 1.1 $ $Date: 2002/05/18 22:20:42 $
 */
final class CronScheduledEntry implements Comparable {
    protected final String   m_name;
    protected final Cron     m_trigger;
    protected final String   m_target;

    //cached version of time from TimeTrigger class
    protected long               m_time;
    protected boolean            m_isValid;

    public CronScheduledEntry( String name, Cron trigger, String target ) {
        m_name = name;
        m_trigger = trigger;
        m_target = target;
        //m_time = m_trigger.getTimeAfter( System.currentTimeMillis() );
        m_isValid = true;
    }

    /**
     * Return name of trigger.
     *
     * @return the name of trigger
     */
    public String getName() {
        return m_name;
    }

    public String getTarget() {
        return m_target;
    }

    public Cron getTimeTrigger() {
        return m_trigger;
    }

    /**
     * Determine if this entry is valid
     *
     * @return true if trigger is valid, false otherwise
     */
    public boolean isValid()
    {
        return m_isValid;
    }

    /**
     * Invalidate trigger
     */
    public void invalidate() {
        m_isValid = false;
    }

    /**
     * Retrieve cached time when trigger should run next.
     *
     * @return the time in milliseconds when trigger should run
     */
    public long getNextTime() {
        return m_time;
    }

    /**
     * Set cached time in milliseconds when trigger should run
     *
     * @param time the time
     */
    public void setNextTime( long time ) {
        m_time = time;
    }

    /**
     * Implement comparable interface used to help sort triggers.
     * Triggers are compared based on next time to run
     *
     * @param object the other trigger
     * @return -'ve value if other trigger occurs before this trigger
     */
    public int compareTo( final Object object ) {
        final CronScheduledEntry other = (CronScheduledEntry)object;
        return (int)-(other.m_time - m_time);
    }

    public String toString() {
        return "TimeEntry[ name=" + m_name + " valid=" + m_isValid + " time=" + m_time;
    }
}

