
/* 
 * $COPYRIGHT$
 * $Id: LineWrapper.java,v 1.4 2002/04/28 13:10:10 neuro Exp $
 *
 * Date        Author            Changes 
 * Jul 03 2001 Szabo Csaba       Created
 * Dec 02 2001 Remus Pereni      Imported in mn8
 */
package org.media.mn8.util;

import org.media.mn8.mn8Interpreter;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;

/** 
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.4 $ $Date: 2002/04/28 13:10:10 $
 */
public class LineWrapper {
    private static final int BLOCK_SIZE = 8096;  
    private static final int   TAB_SIZE = 8;  

    private static int  MAX_COLUMN = 72;
    private static String    ALIGN = "LEFT";
    private static String     TEXT = "";
    
    public static void main( String args[] ) {
        
        if ( args.length < 1 ) {
            System.out.println("Usage : java LineWrapper filename [ column [ justify ] ]");
            System.out.println("   - filename, filename for normalizing");
            System.out.println("   - column, number of columns for result string ( default 80 )");
            System.out.println("   - justify, is text alignment : left(default), center, right, justified \n");
            System.exit( 1 );
        }
  
        File f = new File( args[ 0 ] );
        if ( !f.exists() || !f.canRead() ) {
            System.out.println("File: " + args [ 0 ] + " doesn't exists or haven't read permissions.");
            System.exit( 1 );
        }
  
        if ( ( TEXT = readFile( f ) ) == null ) {
            System.out.println("I/O error: " + args [ 0 ] + " can't be readed.");
            System.exit( 1 );
        }
  
        if ( args.length > 1 ) {
            try {
                MAX_COLUMN = Integer.parseInt( args[ 1 ] );
            } 
            catch ( NumberFormatException e ) {
                System.out.println("Column argument: '" + args [ 1 ] + "' isn't a number.");
                System.exit( 1 );
            }
      
            if ( args.length > 2 ) {
                String align = args[ 2 ].toUpperCase().trim();
                if ( !align.equals("LEFT") && !align.equals("CENTER") && !align.equals("RIGHT") && !align.equals("JUSTIFIED")) {
                    System.out.println("Justify argument can be : left, center, right or justified.");
                    System.exit( 1 );
                }
                ALIGN = align;
            }
        }
  
        System.out.println( alignText( TEXT, MAX_COLUMN, ALIGN ) );

    }
    
    
    private static String readFile ( File f ) {
        StringBuffer sb = new StringBuffer();
        char[]       buffer = new char[ BLOCK_SIZE ];
        int          size = 0;

        try {
            BufferedReader reader = new BufferedReader( new FileReader( f ) );
          
            while ( true ) {
                if ( ( size = reader.read( buffer, 0, BLOCK_SIZE ) ) == -1 ) break;
                sb.append( new String( buffer, 0, size ) );
            }
        }
        catch ( Exception e ) {
            return null;
        }
  
        return sb.toString();
    } 
    
    
    private static String removeLF( String str ) {
        StringBuffer sb = new StringBuffer();
        int        LFNr = 0;  
        int     spaceNr = 0; 
    
        for ( int i = 0; i < str.length(); i++ ) {
            char ch = str.charAt( i );
      
            if ( ch == '\t' ) spaceNr += TAB_SIZE ;
            else if ( ch == '\n' ) {
                LFNr++;
                if ( (i + 1) < str.length() && LFNr < 2 && 
                     !Character.isWhitespace( str.charAt( i + 1 ) ) ) 
                    spaceNr = 1;
                else spaceNr = 0;
            }
            else if ( !Character.isWhitespace( ch ) ) {
                if ( LFNr > 1 ) 
                    for ( int j = 0; j < LFNr; j++ ) sb.append( mn8Interpreter.EOL );
                sb.append( space( spaceNr ) + ch );LFNr = 0;spaceNr = 0;
            }
            else if ( ch == ' ' ) spaceNr++;
        }
  
        return sb.toString();
    }
    
    
    public static String alignText( String str, int column, String align ) {
        String     _str = removeLF( str );
        StringBuffer sb = new StringBuffer( !( _str.endsWith("\n") || _str.endsWith("\r\n") || _str.endsWith("\r")) ?  _str + mn8Interpreter.EOL: _str );
        int      wordNr = 0;
        int      curPoz = 0;
        int     prevPoz = 0;
        int       start = 0;
        
        while ( curPoz < sb.length() ) {
            char ch = sb.charAt( curPoz );
            
            if ( Character.isWhitespace( ch ) &&  ch != '\n' && ch != '\r') {
                if ( ( curPoz - start ) > column ) {
                    if ( wordNr == 0 ) {
                        sb.insert( curPoz , mn8Interpreter.EOL);
                        curPoz += 1+ mn8Interpreter.EOL.length();
                        prevPoz = curPoz; 
                        start   = curPoz;
                    }
                    else {
                        sb.insert( prevPoz , mn8Interpreter.EOL );
                        curPoz = prevPoz;
                        start  = curPoz;
                        wordNr = 0;
                    }
                }
                else {
                    wordNr++;
                    prevPoz = ++curPoz;
                }
            }
            else {
                if ( ch == '\n' ) {
                    if ( ( curPoz  - start ) > column ) {
                        sb.insert( prevPoz , mn8Interpreter.EOL );
                    }
                    curPoz += 2;
                    start   = curPoz;
                    prevPoz = start;
                    wordNr  = 0;
                }
                else {
                    curPoz ++;
                }
            }
        }

        String result = sb.toString();

        if ( align.trim().equalsIgnoreCase( "left" ) )      result = alignLeft  ( sb.toString() , column );
        if ( align.trim().equalsIgnoreCase( "right" ) )     result = alignRight ( sb.toString() , column );
        if ( align.trim().equalsIgnoreCase( "center" ) )    result = alignCenter( sb.toString() , column );
        if ( align.trim().equalsIgnoreCase( "justified" ) ) result = justified  ( sb.toString() , column );
        if ( !( str.endsWith("\r\n") ||
                str.endsWith("\n") || 
                str.endsWith("\r")) && 
             result.endsWith(mn8Interpreter.EOL) ) {
            result = ("x" + result).trim().substring(1);
        }
        
        return result;
    }
    

    public static String alignLeft( String str , int column ) {
        StringBuffer sb = new StringBuffer( );
        int start = 0;
        int poz;
  
        while ( ( poz = str.indexOf( '\n', start ) ) != -1 ) {
            sb.append( str.substring( start, poz ).trim() + mn8Interpreter.EOL );
            start = poz + 1;
        }
  
        return sb.toString();
    }
    
    
    public static String alignRight( String str , int column ) {
        StringBuffer sb = new StringBuffer( );
        int start = 0;
        int poz;
  
        while ( ( poz = str.indexOf( '\n', start ) ) != -1 ) {
            String _str = str.substring( start, poz ).trim();
            int nrSpace = column - _str.length();
      
            sb.append( space( nrSpace ) + _str + mn8Interpreter.EOL );
            start = poz + 1;
        }
  
        return sb.toString();
    }


    public static String alignCenter( String str , int column ) {
        StringBuffer sb = new StringBuffer( );
        int start = 0;
        int poz;
  
        while ( ( poz = str.indexOf( '\n', start ) ) != -1 ) {
            String _str = str.substring( start, poz ).trim();
            int nrSpace = column - _str.length();
      
            sb.append( space( nrSpace / 2 ) + _str + mn8Interpreter.EOL );
            start = poz + 1;
        }
  
        return sb.toString();
    }

    
    public static String justified( String str, int column ) {
        StringBuffer sb = new StringBuffer();
        int start = 0;
        int poz;
  
        while ( ( poz = str.indexOf( '\n', start ) ) != -1 ) {
            String _str = ("_" + str.substring( start, poz )).trim();
            _str = _str.substring(1);
            int nrSpace = column - _str.length();
            int nrWord  = getWordNr( _str );
      
            if ( nrWord > 1 ) {
                if ( poz + 1 < str.length() && str.charAt( poz + 1 ) != '\n' ) 
                    _str = justifyLine( _str, nrWord , nrSpace ); 
            }
            sb.append( _str + mn8Interpreter.EOL );
      
            start = poz + 1;
        }
  
        return sb.toString();
    }


    private static String justifyLine(String line, int wordNr, int spaceNr ) {
        StringBuffer   sb = new StringBuffer();
        boolean   isSpace = false;
        boolean firstWord = false;
        int     plusSpace = spaceNr % ( wordNr - 1 );
        int     nrSpace   = spaceNr / ( wordNr - 1 );
  
        for ( int i = 0; i < line.length(); i++ ) {
            if ( Character.isWhitespace( line.charAt( i ) ) && !isSpace && firstWord ) {
                isSpace = true;
            }
            else if ( !Character.isWhitespace( line.charAt( i ) ) ) {
                firstWord = true;
                if ( isSpace ) {
                    isSpace = false;
                    sb.append( space( nrSpace ) );
                    if ( plusSpace-- > 0 ) sb.append(' ');
                }
            }
            sb.append( line.charAt( i ) );  
        }    
  
        return sb.toString();
    }
    
    
    private static String space ( int nrSpace ) {
        StringBuffer sb = new StringBuffer();
        for ( int i = 0; i < nrSpace; i++ ) sb.append(" ");
        return sb.toString();
    }
    
    
    private static int getWordNr( String str ) {
        boolean isSpace = false;
        int         wNr = 1;
  
        for ( int i = 0; i < str.length(); i++ ) {
            if ( Character.isWhitespace( str.charAt( i ) ) && !isSpace ) {
                isSpace = true;
                continue;
            }
            if ( !Character.isWhitespace( str.charAt( i ) ) && isSpace ) {
                isSpace = false;
                wNr ++;
            } 
        }
  
        return wNr;
    }
    
}
