/* 
 * $COPYRIGHT$
 * $Id: SynchronizedHeap.java,v 1.1 2002/05/18 22:20:42 atech Exp $
 *
 * Date        Author            Changes 
 * May 16 2002 Antal Attila      Created
 */

package org.media.mn8.util;

import java.util.NoSuchElementException;


/**
 * Use synchronized the BinaryHeap.
 *
 * @author <a href="mailto:atech@nolimits.ro">Antal Attila</a>
 */

public final class SynchronizedHeap {
    private final BinaryHeap m_heap;

    public SynchronizedHeap( final BinaryHeap heap ) {
        m_heap = heap;
    }

    /**
     * Clear all elements from queue.
     */
    public void clear() {
        synchronized( m_heap ) {
            m_heap.clear();
        }
    }

    /**
     * Test if queue is empty.
     *
     * @return true if queue is empty else false.
     */
    public boolean isEmpty() {
        synchronized( m_heap ) {
            return m_heap.isEmpty();
        }
    }

    /**
     * Insert an element into queue.
     *
     * @param element the element to be inserted
     */
    public void insert( final Object element ) {
        synchronized( m_heap ) {
            m_heap.insert( element );
        }
    }

    /**
     * Return element on top of heap but don't remove it.
     *
     * @return the element at top of heap
     * @throws NoSuchElementException if isEmpty() == true
     */
    public Object peek() throws NoSuchElementException {
        synchronized( m_heap ) {
            return m_heap.peek();
        }
    }

    /**
     * Return element on top of heap and remove it.
     *
     * @return the element at top of heap
     * @throws NoSuchElementException if isEmpty() == true
     */
    public Object pop() throws NoSuchElementException {
        synchronized( m_heap ) {
            return m_heap.pop();
        }
    }

    public String toString() {
        synchronized( m_heap ) {
            return m_heap.toString();
        }
    }
}

