/* 
 * $COPYRIGHT$
 * $Id: TarBuffer.java,v 1.2 2002/02/27 19:21:18 crow Exp $
 *
 * Date        Author            Changes 
 * FEB 27 2002 Szabo Csaba       Created
 */

package org.media.mn8.util.tar;

import java.io.*;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.2 $ $Date: 2002/02/27 19:21:18 $
 */

public class TarBuffer extends Object {

    public static final int DEFAULT_RCDSIZE = 512;
    public static final int DEFAULT_BLKSIZE = 20 * DEFAULT_RCDSIZE;

    private InputStream  is;
    private OutputStream os;
    
    private byte[] blkBuffer;
    private int    currBlkIdx;
    private int    currRecIdx;
    private int    blkSize;
    private int    recSize;
    private int    recperblk;

    
    public TarBuffer( InputStream is ) {
        this.is = is;
        this.os = null;
        this.initialize( TarBuffer.DEFAULT_BLKSIZE, TarBuffer.DEFAULT_RCDSIZE );
    }

    
    public TarBuffer( OutputStream os ) {
        this.is = null;
        this.os = os;
        this.initialize( TarBuffer.DEFAULT_BLKSIZE, TarBuffer.DEFAULT_RCDSIZE );
    }

    public int getRecordSize() {
        return recSize;
    }


    public int getBlockSize() {
        return blkSize;
    }


    public boolean isEOFRecord( byte[] rec ) {
        for ( int i = 0, s = getRecordSize(); i < s ; ++i )
            if ( rec[i] != 0 ) return false;
        return true;
    }
    
    
    public byte[] readRecord() throws IOException {
        if ( is == null ) throw new IOException("InputStream is null");
        if ( currRecIdx >= recperblk ) if ( !readBlock() ) return null;
        byte[] result = new byte[ recSize ];
        System.arraycopy( blkBuffer, currRecIdx*recSize, result, 0, recSize );
        currRecIdx++;
        return result;
    }

    
    public void writeRecord( byte[] buffer ) throws IOException {
        if ( os == null ) throw new IOException("OutputStream is null");
        if ( buffer.length != recSize ) throw new IOException("Buffer length must be: " + recSize );
        if ( currRecIdx >= recperblk ) writeBlock();
        System.arraycopy( buffer, 0, blkBuffer, currRecIdx*recSize, recSize );
        currRecIdx++;
    }


    public void writeRecord( byte[] buf, int offset ) throws IOException {
        if ( os == null ) throw new IOException("OutputStream is null");
        if ( ( offset + recSize ) > buf.length ) throw new IOException("Buffer length must be: " + recSize ); 
        if ( currRecIdx >= recperblk ) writeBlock();
        System.arraycopy( buf, offset, blkBuffer, currRecIdx*recSize, recSize );
        currRecIdx++;
   }


    public void close() throws IOException {
        if ( os != null ) {
            if ( currRecIdx > 0 ) writeBlock();
            os.close();
            os = null;
        }
        if ( is != null ) {
            is.close();
            is = null;
        }
    }

    
    private boolean readBlock() throws IOException {
        if ( is == null ) throw new IOException("InputStream is null");
        currRecIdx = 0;
        
        int offset = 0;
        int     bn = blkSize;
        
        while ( bn > 0 ) {
            long nb = is.read( blkBuffer, offset, bn );
            
            if ( nb == -1 ) break;
            offset += nb;
            bn -= nb;
        }
        for ( int i = bn; i > 0; i-- ) blkBuffer[blkSize - i] = 0;
        currBlkIdx++;

        return true;
    }


    private void writeBlock() throws IOException {
        if ( os == null ) throw new IOException("OutputStream is null");
        os.write( blkBuffer, 0, blkSize );
        os.flush();
        currRecIdx = 0;
        currBlkIdx++;
    }


    private void initialize( int bsize, int rsize ) {
        blkSize   = bsize;
        recSize   = rsize;
        recperblk = blkSize/recSize;
        blkBuffer = new byte[ blkSize ];

        if ( is != null ) {
            currBlkIdx = -1;
            currRecIdx = recperblk;
        }
        else {
            currBlkIdx = 0;
            currRecIdx = 0;
        }
    }
}
