/* 
 * $COPYRIGHT$
 * $Id: TarEntry.java,v 1.1 2002/02/27 19:05:53 crow Exp $
 *
 * Date        Author            Changes 
 * FEB 27 2002 Szabo Csaba       Created
 */

package org.media.mn8.util.tar;

import java.io.*;
import java.util.Date;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/02/27 19:05:53 $
 */

public class TarEntry extends Object {
    
    //This is the entry's header information.                                                                                                     
    protected TarHeader header;
    
    //The file reprezentation of entry
    protected File      file;


    public TarEntry( byte[] header ) throws Exception {
        initialize();
        parseTarHeader( this.header, header);
    }


    public TarEntry( String name , int length ) {
        initialize();
        nameTarHeader( header, name, length );
    }

    
    public String getName() {
        return header.name.toString();
    }


    public long getSize() {
        return header.size;
    }


    public void parseTarHeader( TarHeader hdr, byte[] header ) throws Exception {
        int offset = 0;

        hdr.name = TarHeader.parseName( header, offset, TarHeader.NAMELEN );
        offset += TarHeader.NAMELEN;

        hdr.mode = (int)TarHeader.parseOctal( header, offset, TarHeader.MODELEN );
        offset += TarHeader.MODELEN;

        hdr.userId = (int)TarHeader.parseOctal( header, offset, TarHeader.UIDLEN );
        offset += TarHeader.UIDLEN;
   
        hdr.groupId = (int)TarHeader.parseOctal( header, offset, TarHeader.GIDLEN );
        offset += TarHeader.GIDLEN;
 
        hdr.size = TarHeader.parseOctal( header, offset, TarHeader.SIZELEN );
        offset += TarHeader.SIZELEN;
        
        hdr.modTime = TarHeader.parseOctal( header, offset, TarHeader.MODTIMELEN );
        offset += TarHeader.MODTIMELEN;
 
        hdr.checkSum = (int)TarHeader.parseOctal( header, offset, TarHeader.CHKSUMLEN );
        offset += TarHeader.CHKSUMLEN;
 
        hdr.linkFlag = header[ offset++ ];

        hdr.linkName = TarHeader.parseName( header, offset, TarHeader.NAMELEN );
        offset += TarHeader.NAMELEN;
        
        hdr.magic = TarHeader.parseName( header, offset, TarHeader.MAGICLEN );
        offset += TarHeader.MAGICLEN;
        
        hdr.userName = TarHeader.parseName( header, offset, TarHeader.UNAMELEN );
        offset += TarHeader.UNAMELEN;
 
        hdr.groupName = TarHeader.parseName( header, offset, TarHeader.GNAMELEN );
        offset += TarHeader.GNAMELEN;

        hdr.devMajor = (int)TarHeader.parseOctal( header, offset, TarHeader.DEVLEN );
        offset += TarHeader.DEVLEN;
 
        hdr.devMinor = (int)TarHeader.parseOctal( header, offset, TarHeader.DEVLEN );
    }
    

    public void nameTarHeader( TarHeader hdr, String name, int length ) {
        boolean isDir = name.endsWith( File.separator );
        
        hdr.checkSum  = 0;
        hdr.devMajor  = 0;
        hdr.devMinor  = 0;
        hdr.name      = new StringBuffer( name );
        hdr.mode      = isDir ? 040755 : 0100644;
        hdr.userId    = 0;
        hdr.groupId   = 0;
        hdr.size      = length;
        hdr.modTime   = ( new java.util.Date()).getTime() / 1000;
        hdr.linkFlag  = isDir ? TarHeader.LF_DIR : TarHeader.LF_NORMAL;
        hdr.linkName  = new StringBuffer("");
        hdr.userName  = new StringBuffer("");
        hdr.groupName = new StringBuffer("");
        hdr.magic     = new StringBuffer("ustar");
    }


    public void writeEntryHeader( byte[] buf ) {
        int offset = 0;
        long  size = header.size;

        offset = TarHeader.getNameBytes     ( header.name,      buf, offset, TarHeader.NAMELEN );
        offset = TarHeader.getOctalBytes    ( header.mode,      buf, offset, TarHeader.MODELEN );
        offset = TarHeader.getOctalBytes    ( header.userId,    buf, offset, TarHeader.UIDLEN );
        offset = TarHeader.getOctalBytes    ( header.groupId,   buf, offset, TarHeader.GIDLEN );
        offset = TarHeader.getLongOctalBytes( size,             buf, offset, TarHeader.SIZELEN );
        offset = TarHeader.getLongOctalBytes( header.modTime,   buf, offset, TarHeader.MODTIMELEN );
        int csOffset = offset;
        for ( int c = 0 ; c < TarHeader.CHKSUMLEN ; ++c ) buf[ offset++ ] = (byte)' ';
        buf[ offset++ ] = header.linkFlag;
        offset = TarHeader.getNameBytes    ( header.linkName,  buf, offset, TarHeader.NAMELEN );
        offset = TarHeader.getNameBytes    ( header.magic,     buf, offset, TarHeader.MAGICLEN );
        offset = TarHeader.getNameBytes    ( header.userName,  buf, offset, TarHeader.UNAMELEN );
        offset = TarHeader.getNameBytes    ( header.groupName, buf, offset, TarHeader.GNAMELEN );
        offset = TarHeader.getOctalBytes   ( header.devMajor,  buf, offset, TarHeader.DEVLEN );
        offset = TarHeader.getOctalBytes   ( header.devMinor,  buf, offset, TarHeader.DEVLEN );
        for ( ; offset < buf.length ; ) buf[ offset++ ] = 0;
        long checkSum = this.computeCheckSum( buf );
        TarHeader.getCheckSumOctalBytes    ( checkSum,         buf, csOffset, TarHeader.CHKSUMLEN );
    }

    
    public boolean isDirectory() {
        if ( file != null ) return file.isDirectory();
        if ( header != null ) {
            if ( header.linkFlag == TarHeader.LF_DIR ) return true;
            if ( header.name.toString().endsWith( File.separator ) ) return true;
        }
        return false;
    }


    private long computeCheckSum( byte[] buf ) {
        long sum = 0;
        for ( int i = 0 ; i < buf.length ; ++i ) sum += 255 & buf[ i ];
        return sum;
    }
    

    private void initialize() {
        file   = null;
        header = new TarHeader();
    }
}
