/* 
 * $COPYRIGHT$
 * $Id: TarHeader.java,v 1.1 2002/02/27 19:07:34 crow Exp $
 *
 * Date        Author            Changes 
 * FEB 27 2002 Szabo Csaba       Created
 */

package org.media.mn8.util.tar;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/02/27 19:07:34 $
 */

public class TarHeader extends Object {

    //The length of the name field in a header buffer.
    public static final int         NAMELEN = 100;

    //The length of the mode field in a header buffer.
    public static final int         MODELEN = 8;

    //The length of the user id field in a header buffer.
    public static final int         UIDLEN = 8;

    //The length of the group id field in a header buffer.
    public static final int         GIDLEN = 8;

    //The length of the checksum field in a header buffer.
    public static final int         CHKSUMLEN = 8;

    //The length of the size field in a header buffer.
    public static final int         SIZELEN = 12;

    //The length of the magic field in a header buffer.
    public static final int         MAGICLEN = 8;

    //The length of the modification time field in a header buffer.
    public static final int         MODTIMELEN = 12;

    //The length of the user name field in a header buffer.
    public static final int         UNAMELEN = 32;

    //The length of the group name field in a header buffer.
    public static final int         GNAMELEN = 32; 

    //The length of the devices field in a header buffer. 
    public static final int         DEVLEN = 8; 

    /**
     * LF_ constants represent the "link flag" of an entry, or more commonly,
     * the "entry type". This is the "old way" of indicating a normal file.
     */
    public static final byte        LF_OLDNORM      = 0;
    public static final byte        LF_NORMAL       = (byte) '0';
    public static final byte        LF_LINK         = (byte) '1';
    public static final byte        LF_SYMLINK      = (byte) '2';
    public static final byte        LF_CHR          = (byte) '3';
    public static final byte        LF_BLK          = (byte) '4';
    public static final byte        LF_DIR          = (byte) '5';
    public static final byte        LF_FIFO         = (byte) '6';
    public static final byte        LF_CONTIG       = (byte) '7';
    
    //The magic tag representing a POSIX tar archive.
    public static final String      TMAGIC          = "ustar"; 
    
    //The magic tag representing a GNU tar archive.
    public static final String      GNU_TMAGIC      = "ustar  ";  

    //The entry's name.
    public StringBuffer             name;

    //The entry's permission mode.
    public int                      mode;
    
    //The entry's user id.
    public int                      userId;

    //The entry's group id.
    public int                      groupId;

    //The entry's size.
    public long                     size;

    //The entry's modification time.
    public long                     modTime;
    
    //The entry's checksum.
    public int                      checkSum;

    //The entry's link flag.
    public byte                     linkFlag;
    
    //The entry's link name.
    public StringBuffer             linkName;

    //The entry's magic tag.
    public StringBuffer             magic;

    //The entry's user name.
    public StringBuffer             userName;

    //The entry's group name.
    public StringBuffer             groupName;

    //The entry's major device number.
    public int                      devMajor;
    
    //The entry's minor device number.
    public int                      devMinor;
    
    
    public static StringBuffer parseName( byte[] header, int offset, int length ) throws Exception {
        StringBuffer result = new StringBuffer( length );
 
        int end = offset + length;
        for ( int i = offset ; i < end ; ++i ) {
            if ( header[i] == 0 ) break;
            result.append( (char)header[i] );
        }
        
        return result;
    }


    public static long parseOctal( byte[] header, int offset, int length ) throws Exception {
        long     result = 0;
        boolean padding = true;
        int         end = offset + length;
        
        for ( int i = offset; i < end; ++i ) {
            if ( header[i] == 0 ) break;
            if ( header[i] == (byte)' ' || header[i] == '0' ) {
                if ( padding ) continue;
                if ( header[i] == (byte)' ' ) break;
            }
            padding = false;
            result = (result << 3) + (header[i] - '0');
        }
 
        return result;
    }

    
    public static int getNameBytes( StringBuffer _name, byte[] buf, int offset, int length ) {
        int       i;
        byte[] _buf = _name.toString().getBytes();

        for ( i = 0; i < length && i < _buf.length; i++ ) buf[ offset + i ] = _buf[i];
        for ( ; i < length; i++ ) buf[ offset + i ] = 0;

        return offset + length;
    } 


    public static int getOctalBytes( long value, byte[] buf, int offset, int length ) {
        byte[] result = new byte[ length ];
        int       idx = length - 1;
 
        buf[ offset + idx ] = 0; --idx;
        buf[ offset + idx ] = (byte)' '; --idx;
        if ( value == 0 ) {
            buf[ offset + idx ] = (byte)'0'; --idx;
        }
        else {
            for ( long val = value ; idx >= 0 && val > 0 ; --idx ) {
                buf[ offset + idx ] = (byte)( (byte) '0' + (byte) (val & 7) );
                val = val >> 3;
            }
        }
        for ( ; idx >= 0 ; --idx ) buf[ offset + idx ] = (byte) ' ';
 
        return offset + length;
    }

    
    public static int getLongOctalBytes( long value, byte[] buf, int offset, int length ) {
        byte[] temp = new byte[ length + 1 ];
        TarHeader.getOctalBytes( value, temp, 0, length + 1 );
        System.arraycopy( temp, 0, buf, offset, length );
        return offset + length;
    }        


    public static int getCheckSumOctalBytes( long value, byte[] buf, int offset, int length ) {
        TarHeader.getOctalBytes( value, buf, offset, length );
        buf[ offset + length - 1 ] = (byte) ' ';
        buf[ offset + length - 2 ] = 0;
        return offset + length;
    }
}
