/* 
 * $COPYRIGHT$
 * $Id: TarInputStream.java,v 1.1 2002/02/27 19:09:47 crow Exp $
 *
 * Date        Author            Changes 
 * FEB 27 2002 Szabo Csaba       Created
 */

package org.media.mn8.util.tar;

import java.io.*;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/02/27 19:09:47 $
 */

public class TarInputStream extends FilterInputStream {
    
    protected TarBuffer    buffer;
    protected byte[]       readBuf;
    protected byte[]       oneBuf;
    protected boolean      hasEOF;
    protected TarEntry     currEntry;
    protected int          entrySize;
    protected int          entryOffset;


    public TarInputStream( InputStream is ) {
        super( is );
        this.buffer   = new TarBuffer( is );
        this.readBuf  = null;
        this.oneBuf   = new byte[1];
        this.hasEOF   = false;
    }


    public int getRecordSize() {
        return buffer.getRecordSize();
    }

    
    public int available() throws IOException {
        return entrySize - entryOffset;
    }


    public void close() throws IOException {
        buffer.close();
    }
    
    public void skip( int nSkip ) throws IOException {
        byte[] skipBuf = new byte[ 8 * 1024 ];

        for ( int i = nSkip; i > 0; ) {
            int n = read( skipBuf, 0, ( i > skipBuf.length ? skipBuf.length : i ) );
            if ( n == -1 ) break;
            i -= n;
        }
    }


    public int read() throws IOException {
        int num = read( oneBuf, 0, 1 );
        if ( num == -1 ) return num;
        else return (int)oneBuf[0];
    }

    
    public int read( byte[] buf ) throws IOException {
        return read( buf, 0, buf.length );
    }


    public int read( byte[] buf, int offset, int length ) throws IOException {
        int readed = 0;

        if ( entryOffset >= entrySize ) return -1;
        if ( ( length + entryOffset ) > entrySize ) length = entrySize - entryOffset;
        if ( readBuf != null ) {
            int s = ( length > readBuf.length ) ? readBuf.length : length;

            System.arraycopy( readBuf, 0, buf, offset, s );
            if ( s >= readBuf.length ) readBuf = null;
            else {
                int    _len = readBuf.length - s;
                byte[] _buf = new byte[ _len ];

                System.arraycopy( readBuf, s, _buf, 0, _len );
                readBuf = _buf;
            }
            readed += s;
            length -= s;
            offset += s;
        }

        while ( length > 0 ) {
            byte[] rec = buffer.readRecord();
            
            if ( rec == null ) throw new IOException("unexcepted EOF");
            int    s = length;
            int _len = rec.length;

            if ( _len > s ) {
                System.arraycopy( rec, 0, buf, offset, s );
                readBuf = new byte[ _len - s ];
                System.arraycopy( rec, s, readBuf, 0, _len - s );
            }
            else {
                s = _len;
                System.arraycopy( rec, 0, buf, offset, _len );
            }

            readed += s;
            length -= s;
            offset += s;
        }
        
        entryOffset += readed;
        
        return readed;
    }


    public TarEntry getNextEntry() throws IOException {
        if ( hasEOF ) return null;
        if ( currEntry != null ) {
            int nSkip = entrySize + entryOffset;
            if ( nSkip > 0 ) skip( nSkip );
            readBuf = null;
        }

        byte[] header = buffer.readRecord();
        
        if ( header == null ) hasEOF = true;
        else if ( buffer.isEOFRecord( header ) ) hasEOF = true;

        if ( hasEOF ) currEntry = null;
        else {
            try {
                currEntry = new TarEntry( header );
                if ( !( header[257]=='u' && header[258]=='s' && header[259]=='t' && header[260]=='a' && header[261]=='r' ) )
                    throw new IOException("header magic is not 'ustar'");
                entryOffset = 0;
                entrySize   = (int)currEntry.getSize(); 
            } 
            catch ( Exception ex ) {
                entrySize = 0;
                entryOffset = 0;
                currEntry = null;
                //throw new IOException("bad header, " + ex.getMessage() );
            }
        }
        return currEntry;
    }
}
