/* 
 * $COPYRIGHT$
 * $Id: TarOutputStream.java,v 1.1 2002/02/27 19:14:23 crow Exp $
 *
 * Date        Author            Changes 
 * FEB 27 2002 Szabo Csaba       Created
 */

package org.media.mn8.util.tar;

import java.io.*;

/**
 * @author <a href="mailto:crow@nolimits.ro">Szabo Csaba</a>
 * @version $Revision: 1.1 $ $Date: 2002/02/27 19:14:23 $
 */

public class TarOutputStream extends FilterOutputStream {
   
    protected TarBuffer buffer;
    protected byte[]    recordBuf;
    protected byte[]    oneBuf;
    protected int       currByte;
    protected int       currSize;
    protected int       tempLen;
    protected byte[]    tempBuf;


    public TarOutputStream( OutputStream os ) {
        super( os );
        buffer    = new TarBuffer( os );
        oneBuf    = new byte[1];
        recordBuf = new byte[ TarBuffer.DEFAULT_RCDSIZE ];
        tempBuf   = new byte[ TarBuffer.DEFAULT_RCDSIZE ];
        tempLen   = 0;
    }

    public int getRecordSize() {
        return buffer.getRecordSize();
    } 

    
    public void close() throws IOException {
        writeEOFRecord();
        buffer.close();
    }

    
    public void write( int b ) throws IOException {
        oneBuf[0] = (byte)b;
        write( oneBuf, 0, 1 );
    }


    public void write( byte buf[] ) throws IOException {
        write( buf, 0, buf.length );
    }

    
    public void write( byte buf[], int offset, int length ) throws IOException {
        if ( ( currByte + length ) > currSize )
            throw new IOException("You overstep the header size.");

        if ( tempLen > 0 ) {
            if ( ( tempLen + length ) >= recordBuf.length ) {
                int _len = recordBuf.length - tempLen;

                System.arraycopy( tempBuf, 0, recordBuf, 0, tempLen );
                System.arraycopy( buf, offset, recordBuf, tempLen, _len );
                buffer.writeRecord( recordBuf );
                currByte += recordBuf.length;
                offset += _len;
                length -= _len;
                tempLen = 0;
            } 
            else {
                System.arraycopy( buf, offset, tempBuf, tempLen, length );
                offset  += length;
                tempLen += length;
                length  -= length;
            }
        }
        
        while ( length > 0 ) {
            if ( length < recordBuf.length ) {
                System.arraycopy(buf, offset, tempBuf, tempLen, length);
                tempLen += length;
                break;
            }
            buffer.writeRecord( buf, offset );

            currByte += recordBuf.length;
            length   -= recordBuf.length;
            offset   += recordBuf.length;
        }
    }


    public void putNextEntry( TarEntry te ) throws IOException {
        if ( te.getName().length() > TarHeader.NAMELEN )
            throw new IOException( "File name '" + te.getName() + "' is too long ( > " +
                                  TarHeader.NAMELEN + " bytes )" );
        te.writeEntryHeader( recordBuf );
        buffer.writeRecord( recordBuf );
        currByte = 0;
        if ( te.isDirectory() ) 
            currSize = 0;
        else 
            currSize = (int)te.getSize();
        
    }


    public void closeEntry() throws IOException {
        if ( tempLen > 0 ) {
            for ( int i = tempLen; i < tempBuf.length; tempBuf[ i++ ] = 0 );
            buffer.writeRecord( tempBuf );
            currByte += tempLen;
            tempLen = 0;
        }
        if ( currByte < currSize ) 
            throw new IOException("Error durring when close entry.");
    }

    
    private void writeEOFRecord() throws IOException {
        for( int i = 0; i < recordBuf.length; recordBuf[ i++ ] = 0 );
        buffer.writeRecord( recordBuf );
    }
}
