/* 
 * $COPYRIGHT$
 * $Id: NamingManagerTestCase.java,v 1.7 2002/03/25 11:18:16 atech Exp $
 *
 * Date        Author            Changes 
 * May 04 2001 Remus Pereni      Created
 */
package org.media.mn8.test;

import junit.framework.*;
import org.media.mn8.NamingManager;
import org.media.mn8.concepts.StringConcept;
import java.util.Hashtable;
import javax.naming.NamingException;
import javax.naming.NamingEnumeration;
import javax.naming.NameClassPair;
import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.OperationNotSupportedException;
import javax.naming.NameNotFoundException;

/**
 * Class meant to run all the Test Cases related to mn8 naming manager.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.7 $ $Date: 2002/03/25 11:18:16 $
 */
public class NamingManagerTestCase extends TestCase {
    
    private static NamingManager _namingManager;
    

    public NamingManagerTestCase (String name){
        super (name);
    }


    public static Test suite () {
        TestSuite suite = new TestSuite();
        suite.addTest(new NamingManagerTestCase("namingManagerCreate"));      
        suite.addTest(new NamingManagerTestCase("namingManagerEnvBind"));
        suite.addTest(new NamingManagerTestCase("namingManagerEnvIsThere"));
        suite.addTest(new NamingManagerTestCase("namingManagerBind"));
        suite.addTest(new NamingManagerTestCase("namingManagerRebind"));
        suite.addTest(new NamingManagerTestCase("namingManagerMemList"));
        suite.addTest(new NamingManagerTestCase("namingManagerMemListBindings"));
        suite.addTest(new NamingManagerTestCase("namingManagerEnvList"));
        suite.addTest(new NamingManagerTestCase("namingManagerEnvListBindings"));
        suite.addTest(new NamingManagerTestCase("namingManagerUnbind"));
        suite.addTest(new NamingManagerTestCase("namingManagerRename"));
        suite.addTest(new NamingManagerTestCase("namingManagerCreateSubcontext"));
        suite.addTest(new NamingManagerTestCase("namingManagerDestroySubcontext"));
        suite.addTest(new NamingManagerTestCase("namingManagerClose"));
        return suite;
    }
    

    public void namingManagerCreate () {
        try {
            _namingManager = NamingManager.getReference();
            if (_namingManager != null) assertTrue(true);
            else assertTrue(false);
        } catch (RuntimeException ex){
            ex.printStackTrace();
            assertTrue (false);
        }
    }
   

    public void namingManagerEnvIsThere () {
        try {
            String value = ((StringConcept)_namingManager.lookup("env:/system/out")).toString();
            if ( !value.equals("System.out") )
                assertTrue ("env://system/out", false);

            value = ((StringConcept)_namingManager.lookup("env:/system/err")).toString();
            if ( !value.equals("System.err") )
                assertTrue ("env://system/err", false);

            value = ((StringConcept)_namingManager.lookup("env:/system/in")).toString();
            if ( !value.equals("System.in") )
                assertTrue ("env://system/in", false);        

            assertTrue (true); //we made it :)
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerEnvBind () {
        try {

            _namingManager.envBind("/hello", "hello");
            String value = (String)_namingManager.lookup("env:/hello");
            if ( !value.equals("hello") )
                assertTrue ("/hello", false);
        
            _namingManager.envBind("/test/hello", "hello");
            value = (String)_namingManager.lookup("env:/test/hello");
            if ( !value.equals("hello") )
                assertTrue ("/test/hello/hello", false);

            _namingManager.envBind("/test/remus/hello", "hello");
            value = (String)_namingManager.lookup("env:/test/remus/hello");
            if ( !value.equals("hello") )
                assertTrue ("/test/remus/hello", false);

            _namingManager.envBind("/test/remus/dir/oh", "hello");
            value = (String)_namingManager.lookup("env:/test/remus/dir/oh");
            if ( !value.equals("hello") )
                assertTrue ("/test/remus/dir/oh", false);

            _namingManager.envBind("env:/name", "Remus Pereni");
            value = (String)_namingManager.lookup("env:/name");
            if ( !value.equals("Remus Pereni") )
                assertTrue ("env:/name", false);

            _namingManager.envBind("env:/users/neuro/name", "Remus Pereni");
            value = (String)_namingManager.lookup("env:/users/neuro/name");
            if ( !value.equals("Remus Pereni") )
                assertTrue ("env:/users/neuro/name", false);

            try {
                _namingManager.envBind("mem:/users/", "");
                assertTrue("mem: -> envBinding", false);
            }catch (OperationNotSupportedException opex) {
                assertTrue("mem: -> envBinding", true);
            }

            assertTrue (true); //we made it :)
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerBind () {
        try {
            _namingManager.bind("mem:/hello", "hello");
            String value = (String)_namingManager.lookup("mem:/hello");
            if ( !value.equals("hello") )
                assertTrue ("mem:/hello", false);
        
            _namingManager.bind("mem:/test/name", "Remus Pereni");
            value = (String)_namingManager.lookup("mem:/test/name");
            if ( !value.equals("Remus Pereni") )
                assertTrue ("mem:/test/name", false);


            _namingManager.bind("mem:/users/neuro/name", "Remus Pereni");
            value = (String)_namingManager.lookup("mem:/users/neuro/name");
            if ( !value.equals("Remus Pereni") )
                assertTrue ("mem:/users/neuro/name", false);

            _namingManager.bind("mem:/users/neuro/city", "Satu Mare");
            value = (String)_namingManager.lookup("mem:/users/neuro/city");
            if ( !value.equals("Satu Mare") )
                assertTrue ("mem:/users/neuro/city", false);

            try {
                _namingManager.bind("env:/users/aaa", "");
                assertTrue("env: -> memBinding", false);
            }catch (OperationNotSupportedException opex) {
                assertTrue("env: -> memBinding", true);
            }

            assertTrue (true); //we made it :)
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }

    public void namingManagerMemList () {
        NamingEnumeration en;
        NameClassPair ncp;
        int cnt= 0;
        try {
            en = _namingManager.list("mem:/");
            
            while( en.hasMore() ){
                ncp = (NameClassPair) en.next();
                cnt++;
                if( ncp.getName() == null ||
                    ncp.getName().equals("") )
                    assertTrue( false );
            }
            
            if( cnt > 0 )
                assertTrue( true ); //we made it :)
            else 
                assertTrue( false );
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerEnvList () {
        NamingEnumeration en;
        NameClassPair ncp;
        int cnt= 0;
        try {
            en = _namingManager.list("env:/");
            
            while( en.hasMore() ){
                ncp = (NameClassPair) en.next();
                cnt++;
                if( ncp.getName() == null ||
                    ncp.getName().equals("") )
                    assertTrue( false );
            }
            
            if( cnt > 0 )
                assertTrue( true ); //we made it :)
            else 
                assertTrue( false );
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }



    public void namingManagerEnvListBindings () {
        NamingEnumeration en;
        Binding ncp;
        int cnt= 0;
        try {
            en = _namingManager.listBindings("env:/");
            
            while( en.hasMore() ){
                ncp = (Binding) en.next();
                cnt++;
                if( ncp.getName() == null ||
                    ncp.getName().equals("") )
                    assertTrue( false );
            }
            
            if( cnt > 0 )
                assertTrue( true ); //we made it :)
            else 
                assertTrue( false );
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerMemListBindings () {
        NamingEnumeration en;
        Binding ncp;
        int cnt= 0;
        try {
            en = _namingManager.listBindings("mem:/");
            
            while( en.hasMore() ){
                ncp = (Binding) en.next();
                cnt++;
                if( ncp.getName() == null ||
                    ncp.getName().equals("") )
                    assertTrue( false );
            }
            
            if( cnt > 0 )
                assertTrue( true ); //we made it :)
            else 
                assertTrue( false );
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerCreateSubcontext () {
        try {
            _namingManager.createSubcontext("mem:/cookies");
            Context context = (Context)_namingManager.lookup("mem:/cookies");
            if ( context == null )
                assertTrue ("mem:/cookies", false);

            _namingManager.createSubcontext("mem:/cookies/test");
            context = (Context)_namingManager.lookup("mem:/cookies/test");
            if ( context == null )
                assertTrue ("mem:/cookies/test", false);

            _namingManager.createSubcontext("mem:/forms/test/");
            context = (Context)_namingManager.lookup("mem:/forms/test/");
            if ( context == null )
                assertTrue ("mem:/forms/test/", false);
        
            try {
                _namingManager.createSubcontext("env:/users/aaa");
                assertTrue("env: -> memBinding", false);
            }catch (OperationNotSupportedException opex) {
                assertTrue("env: -> memBinding", true);
            }           
            assertTrue (true); //we made it :)
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerDestroySubcontext () {
        try {
            _namingManager.destroySubcontext("mem:/cookies/test");
            try {
                Context context = (Context)_namingManager.lookup("mem:/cookies/test");
                assertTrue( false );
            } catch( NameNotFoundException nex) { }

            _namingManager.destroySubcontext("mem:/cookies");
            try {
                Context context = (Context)_namingManager.lookup("mem:/cookies");
                assertTrue( false );
            } catch( NameNotFoundException nex) { }


            _namingManager.destroySubcontext("mem:/forms/");
            try {
                Context context = (Context)_namingManager.lookup("mem:/forms");
            } catch( NameNotFoundException nex) {
                assertTrue (false);
            }
            
            try {
                _namingManager.createSubcontext("env:/users/aaa");
                assertTrue( false );
            }catch (OperationNotSupportedException opex) {
                assertTrue( true );
            }           
            assertTrue (true); //we made it :)
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerRebind () {
        try {
            _namingManager.rebind("mem:/hello", "new hello");
            String value = (String)_namingManager.lookup("mem:/hello");
            if ( !value.equals("new hello") )
                assertTrue ("mem:/hello", false);
        
            _namingManager.rebind("mem:/test/name", "mn8");
            value = (String)_namingManager.lookup("mem:/test/name");
            if ( !value.equals("mn8") )
                assertTrue ("mem:/test/name", false);


            _namingManager.rebind("mem:/users/neuro/name", "mn8");
            value = (String)_namingManager.lookup("mem:/users/neuro/name");
            if ( !value.equals("mn8") )
                assertTrue ("mem:/users/neuro/name", false);

            _namingManager.rebind("mem:/users/neuro/country", "Romania");
            value = (String)_namingManager.lookup("mem:/users/neuro/country");
            if ( !value.equals("Romania") )
                assertTrue ("mem:/users/neuro/country", false);

            try {
                _namingManager.rebind("env:/users/aaa", "");
                assertTrue("env: -> memBinding", false);
            }catch (OperationNotSupportedException opex) {
                assertTrue("env: -> memBinding", true);
            }

            assertTrue (true); //we made it :)
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerUnbind () {
        Object obj;
        String value;
        try {
            _namingManager.bind("mem:/unbind", "unbind");
            value = (String)_namingManager.lookup("mem:/unbind");
            if ( value.equals("unbind") ) {
                _namingManager.unbind("mem:/unbind");
                try {
                    obj = _namingManager.lookup("mem:/unbind");
                    assertTrue( false );
                } catch( NameNotFoundException nex) { }

            }

            _namingManager.bind("mem:/unbindir/1/unbind", "unbind");
            value = (String)_namingManager.lookup("mem:/unbindir/1/unbind");
            if ( value.equals("unbind") ) {
                _namingManager.unbind("mem:/unbindir/1/unbind");
                try {
                    obj = _namingManager.lookup("mem:/unbindir/1/unbind");
                    assertTrue( false );
                } catch( NameNotFoundException nex) { }

            }

           
            try {
                _namingManager.unbind("env:/users/aaa");
                assertTrue( false );
            }catch (OperationNotSupportedException opex) {
                assertTrue( true );
            }           
            assertTrue (true); //we made it :)
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerRename () {
        String value;
        try {
            _namingManager.bind("mem:/rename", "rename");
            value = (String)_namingManager.lookup("mem:/rename");
            if ( value.equals("rename") ) {
                _namingManager.rename("mem:/rename", "mem:/new_rename");
                value = (String)_namingManager.lookup("mem:/new_rename");

                if ( value.equals( "rename" ) ) {
                    try {
                        value = (String)_namingManager.lookup("mem:/rename");
                        assertTrue( false );
                    } catch( NameNotFoundException nex) { }
                }
                else { 
                    assertTrue( false );
                }
            }

            _namingManager.bind("mem:/rename/1/oldname", "rename");
            value = (String)_namingManager.lookup("mem:/rename/1/oldname");
            if ( value.equals("rename") ) {
                _namingManager.rename("mem:/rename/1/oldname", "mem:/rename/1/newname");
                value = (String)_namingManager.lookup("mem:/rename/1/newname");

                if ( value.equals( "rename" ) ) {
                    try {
                        value = (String)_namingManager.lookup("mem:/rename/1/oldaname");
                        assertTrue( false );
                    } catch( NameNotFoundException nex) { }
                }
                else { 
                    assertTrue( false );
                }
            }

            _namingManager.bind("mem:/rename/1/oldname", "rename");
            value = (String)_namingManager.lookup("mem:/rename/1/oldname");
            if ( value.equals("rename") ) {
                _namingManager.rename("mem:/rename/1/oldname", "mem:/rename/2/newname");
                value = (String)_namingManager.lookup("mem:/rename/2/newname");

                if ( value.equals( "rename" ) ) {
                    try {
                        value = (String)_namingManager.lookup("mem:/rename/1/oldaname");
                        assertTrue( false );
                    } catch( NameNotFoundException nex) { }
                }
                else { 
                    assertTrue( false );
                }
            }


            try {
                _namingManager.rename("env:/users/aaa", "mem:/users/x");
                assertTrue( false );
            }catch (OperationNotSupportedException opex) {
                assertTrue( true );
            }           

            try {
                _namingManager.rename("mem:/users/aaa", "env:/users/x");
                assertTrue( false );
            }catch (OperationNotSupportedException opex) {
                assertTrue( true );
            }           

            assertTrue (true); //we made it :)
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


    public void namingManagerClose () {
        try {
            _namingManager.close();
            assertTrue (true);        
        }catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (nex.getMessage(), false);
        }
    }


}
