/* 
 * $COPYRIGHT$
 * $Id: CharacterTestCase.java,v 1.3 2002/03/25 11:18:16 atech Exp $
 *
 * Date        Author            Changes 
 * May 10 2001 Remus Pereni      Created
 */
package org.media.mn8.test.concepts;

import junit.framework.*;
import org.media.mn8.*;
import org.media.mn8.concepts.*;


/**
 * Class implementing a Test Case for the Character concept.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.3 $ $Date: 2002/03/25 11:18:16 $
 */
public class CharacterTestCase extends TestCase {

    private static ConceptDefinition _conceptDefinition =
                                            mn8Loader.getConceptDefinition("Character"); 
    private static Concept _concept = null;


    public CharacterTestCase (String name){
        super (name);
    }


    public static Test suite () {
        TestSuite suite = new TestSuite();
        suite.addTest(new CharacterTestCase("Character_conceptDefinitionRetrieval"));
        suite.addTest(new CharacterTestCase("Character_getConceptInstance"));
        suite.addTest(new CharacterTestCase("Character_getConceptType"));
        suite.addTest(new CharacterTestCase("Character_getConceptLabel"));
        suite.addTest(new CharacterTestCase("Character_getInheritedConcepts"));
        suite.addTest(new CharacterTestCase("Character_extendsConcept"));
        suite.addTest(new CharacterTestCase("Character_isDigit"));
        suite.addTest(new CharacterTestCase("Character_toUpperCase"));
        suite.addTest(new CharacterTestCase("Character_toLowerCase"));
        return suite;
    }


    public void Character_conceptDefinitionRetrieval() {
        assertNotNull( _conceptDefinition );
    }


    public void Character_getConceptInstance() {
        _concept = _conceptDefinition.getConceptInstance();
        assertNotNull( _concept );
        _concept = _concept.getConceptInstance();
        assertNotNull( _concept );
    }


    public void Character_getConceptType() {
        assertTrue( _concept.getConceptType().getValue().equals("Character"));
    }


    public void Character_getConceptLabel() {
        assertTrue( _concept.getConceptLabel().getValue().equals("Character"));
    }


    public void Character_getInheritedConcepts() {
        assertEquals( _concept.getInheritedConcepts().getValue().size(), 1);
    }


    public void Character_extendsConcept() {
        assertEquals( _concept.extendsConcept(new StringConcept("Concept")).getValue()
                     , true);
        assertEquals( _concept.extendsConcept(new StringConcept("Character")).getValue()
                     , true );
        assertEquals( _concept.extendsConcept(new StringConcept("Integer")).getValue()
                     , false );
    }


    public void Character_isDigit() {        
        assertEquals( (new CharacterConcept('1')).isDigit().getValue(), true);
        assertEquals( (new CharacterConcept('a')).isDigit().getValue(), false);
    }


    public void Character_toUpperCase() {
        assertEquals( (new CharacterConcept('a')).toUpperCase().getValue(), 'A');
        assertEquals( (new CharacterConcept('B')).toUpperCase().getValue(), 'B');
    }


    public void Character_toLowerCase() {
        assertEquals( (new CharacterConcept('a')).toLowerCase().getValue(), 'a');
        assertEquals( (new CharacterConcept('B')).toLowerCase().getValue(), 'b');
    }

    
    public static void main (String[] args) {
        junit.textui.TestRunner.run (suite());
    }
}
