/* 
 * $COPYRIGHT$
 * $Id: LexerTestCase.java,v 1.9 2002/03/25 11:18:16 atech Exp $
 *
 * Date        Author            Changes 
 * Apr 12 2001 Remus Pereni      Created
 */
package org.media.mn8.test.parser;

import junit.framework.*;
import antlr.*;
import org.media.mn8.parser.*;
import java.util.Vector;
import java.io.*;


/**
 * Class implementing a Test Case for the mn8 Lexer class.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.9 $ $Date: 2002/03/25 11:18:16 $
 */
public class LexerTestCase extends TestCase {

    private String _test_file_1 = "TestLexer.txt.tcf";
    private String _test_file_2 = "TestScript1.mn8.tcf";

    public LexerTestCase (String name){
        super (name);
    }


    public static Test suite () {
        return new TestSuite (LexerTestCase.class);
    }


    /**
     * Simple test method which gives one particular token at a time from TestLexer.txt.tcf
     * file to the lexer, and verify's if the Lexer has produced the proper Token.
     */
    public void testTokensInmn8Lexer() {
        try {
            mn8Lexer lexer = new mn8Lexer( getClass().getResourceAsStream(_test_file_1) );
            Token token;
             while ( (token = lexer.nextToken()).getText() != null ) {
                 // System.out.println("->" + token.getText() + "/" + token.getType() + " at line:" + token.getLine());
                if ( token.getType() != mn8Lexer.WS &&
                     token.getType() != mn8Lexer.EOL)
                    switch (token.getLine()) {
                    case  1: {
                        assertTrue ("mn8Lexer.CHAR_LITERAL, line 1", 
                                token.getType() == mn8Lexer.CHAR_LITERAL);
                        break;                
                    }
                    case  2: {
                        assertTrue ("mn8Lexer.STRING_LITERAL, line 2", 
                                token.getType() == mn8Lexer.STRING_LITERAL);

                        break;                
                    }
                    case  3: {
                        assertTrue ("mn8Lexer.IDENT, line 3", 
                                token.getType() == mn8Lexer.IDENT);
                        break;                
                    }
                    case  4: {
                        assertTrue ("mn8Lexer.CHAR_LITERAL, line 4", 
                                token.getType() == mn8Lexer.CHAR_LITERAL);
                        break;                
                    }
                    case  5: {
                        assertTrue ("mn8Lexer.NUM_INT, line 5", 
                                token.getType() == mn8Lexer.NUM_INT);
                        break;                
                    }
                    case  6: {
                        assertTrue ("mn8Lexer.NUM_INT, line 6", 
                                token.getType() == mn8Lexer.NUM_INT);
                        break;                
                    }
                    case  7: {
                        assertTrue ("mn8Lexer.NUM_REAL, line 7", 
                                token.getType() == mn8Lexer.NUM_REAL);
                        break;                
                    }
                    case  8: {
                        assertTrue ("mn8Lexer.NUM_REAL, line 8", 
                                token.getType() == mn8Lexer.NUM_REAL);
                        break;                
                    }
                    case  9: {
                        assertTrue ("mn8Lexer.IDENT, line 9", 
                                token.getType() == mn8Lexer.IDENT);
                        break;                
                    }
                    case 10: {
                        assertTrue ("mn8Lexer.IDENT, line 10", 
                                token.getType() == mn8Lexer.IDENT);
                        break;                
                    }
                }
            }
        }catch (Exception ex){
            ex.printStackTrace();
        }
    }


    /**
     * Simple test method which gives to the lexer a whole script file "TestScript1.mn8.tcf.
     * If there are problems with the lexer it will fail at lexing.
     */
    public void testmn8ScriptLexer() {
        try {
            mn8Lexer lexer = new mn8Lexer( getClass().getResourceAsStream(_test_file_2) );
            Token token;

            // if there are lexing problems, it will simply crash at the respective
            // token, but we have to make sure, every token is worked.
            while ( (token = lexer.nextToken()).getText() != null );
            assertTrue ("mn8Lexer parsed the sample script file:" + 
                    _test_file_2, true );

        } catch (Exception ex) {            
            ex.printStackTrace();
            assertTrue ("mn8Lexer failed to parse the sample script file:" + 
                    _test_file_2, false );
        }
    }

    public static void main (String[] args) {
        junit.textui.TestRunner.run (suite());
    }
}
