/* 
 * $COPYRIGHT$
 * $Id: ConceptNavPatternsTestCase.java,v 1.1 2001/10/22 20:25:16 neuro Exp $
 *
 * Date        Author            Changes 
 * Apr 08 2001 Remus Pereni      Created
 */
package org.media.mn8.test.util;

import junit.framework.*;
import org.media.mn8.util.ConceptNavPatterns;

/**
 * Class implementing a Test Case for the SimpleExpression class.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.1 $ $Date: 2001/10/22 20:25:16 $
 */
public class ConceptNavPatternsTestCase extends TestCase {

    public ConceptNavPatternsTestCase (String name){
        super (name);
    }


    public static Test suite () {
        return new TestSuite (ConceptNavPatternsTestCase.class);
    }

    
    public void testIsAbsolutePath() {
        assertEquals(  ConceptNavPatterns.isAbsolutePath("/test/"), true);
        assertEquals(  ConceptNavPatterns.isAbsolutePath("test/"), false);
    }
    
    
    public void testIsRelativePath() {
        assertEquals(  ConceptNavPatterns.isRelativePath("/test/"), false);
        assertEquals(  ConceptNavPatterns.isRelativePath("test/"), true);
    }
 
    
    public void testgetFirstItem() {
        assertEquals(  ConceptNavPatterns.getFirstItem("/test/"), "/test");
        assertEquals(  ConceptNavPatterns.getFirstItem("/test@elem"), "/test");
        assertEquals(  ConceptNavPatterns.getFirstItem("test@elem"), "test");
        assertEquals(  ConceptNavPatterns.getFirstItem("test"), "test");
        assertEquals(  ConceptNavPatterns.getFirstItem("@test"), "@test");        
        assertEquals(  ConceptNavPatterns.getFirstItem("/@test"), "/@test");
        // invalid but it should work anyway
        assertEquals(  ConceptNavPatterns.getFirstItem("/@test/test"), "/@test");
        assertEquals(  ConceptNavPatterns.getFirstItem("/@test@test"), "/@test");
    }
    
    
    public void testgetRemainingItems() {
        assertEquals(  ConceptNavPatterns.getRemainingItems("/test/elem"), "/elem");
        assertEquals(ConceptNavPatterns.getRemainingItems("/test/elem@attribute"), "/elem@attribute");
    }
    
    
    public void testIsAttribute() {
        assertEquals(  ConceptNavPatterns.isAttribute("@attribute"), true);
        assertEquals(  ConceptNavPatterns.isAttribute("/@attribute"), true);
        assertEquals(  ConceptNavPatterns.isAttribute("/attribute"), false);
        assertEquals(  ConceptNavPatterns.isAttribute("//attribute"), false);
    }
    
    
    public void testGetAttributeName() {
        assertEquals(  ConceptNavPatterns.getAttributeName("@attribute"), "attribute");
        assertEquals(  ConceptNavPatterns.getAttributeName("/@attribute"), "attribute");
        assertEquals(  ConceptNavPatterns.getAttributeName("/attribute"), null);
        assertEquals(  ConceptNavPatterns.getAttributeName("attribute"), null);
    }

    public void testGetElementName() {
        assertEquals(  ConceptNavPatterns.getElementName("/elem"), "elem");
        assertEquals(  ConceptNavPatterns.getElementName("elem"), "elem");
    }
}
