/* 
 * $COPYRIGHT$
 * $Id: SimpleExpressionTestCase.java,v 1.15 2002/03/25 11:18:16 atech Exp $
 *
 * Date        Author            Changes 
 * Apr 08 2001 Remus Pereni      Created
 */
package org.media.mn8.test.util;

import junit.framework.*;
import org.media.mn8.util.SimpleExpression;
import java.util.Vector;
import java.io.*;

/**
 * Class implementing a Test Case for the SimpleExpression class.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.15 $ $Date: 2002/03/25 11:18:16 $
 */
public class SimpleExpressionTestCase extends TestCase {
    private String _simpletarget1 = "Hello \n I'm a test target";
    private String _simplepattern1 = "Hello*a # *";

    private String _test_file_1 = "testfile1.xml.tcf";
    private String _test_file_2 = "testfile2.xml.tcf";

    public SimpleExpressionTestCase (String name){
        super (name);
    }


    public static Test suite () {
        return new TestSuite (SimpleExpressionTestCase.class);
    }


    /**
     * Tests if the operatorOccurences method returns the right number of 
     * operators.
     * see org.media.mn8.util.SimpleExpression#operatorOccurences
     */
    public void testSimpleExpressionOperatorOccurences () {
        assert( SimpleExpressionTCP.testOperatorsOccurences ( "hello this # should be 1", "#" ) == 1);
        assert( SimpleExpressionTCP.testOperatorsOccurences ( "hello this should be 0", "#" ) == 0);
        assert( SimpleExpressionTCP.testOperatorsOccurences ( "##hello this ## should# be 5", "#" ) == 5);
    }
 
   
    /**
     * Tests if the makeTokens method returns the proper number of 
     * tokens, if it does then probably we have the right tokens
     * see org.media.mn8.util.SimpleExpression#makeTokens
     */
    public void testSimpleExpressionMakeTokens () {
        assert( SimpleExpressionTCP.testMakeTokens ( "hello this should be 1" ).length == 1);
        assertTrue( SimpleExpressionTCP.testMakeTokens ( "hello this # should be 3" ).length == 3);
        assertTrue( SimpleExpressionTCP.testMakeTokens ( "* #" ).length == 3);
        assertTrue( SimpleExpressionTCP.testMakeTokens ( "hello this * should be 3" ).length == 3);
        assertTrue( SimpleExpressionTCP.testMakeTokens ( "hello this * should# be 5" ).length == 5);
        assertTrue( SimpleExpressionTCP.testMakeTokens ( "*#hello this *# should# be 8" ).length == 8);
        assertTrue( SimpleExpressionTCP.testMakeTokens ( "*#hello this *# should# be 9*" ).length == 9);
    }


    public void testSimpleExpressionMatch () {
        assertTrue( !SimpleExpression.match ( "simple test",  "xxx" ));
        assertTrue( SimpleExpression.match ( "test",  "test" ));
        assertTrue( SimpleExpression.match ( "main:Series",  "main:Series" ));
        assertTrue("SimpleExpression->Match \"simple test\" with  \"test\"", 
               !SimpleExpression.match ( "simple test",  "test" ));
        assertTrue("SimpleExpression->Match \"simple test\" with  \"test*simple\"", 
               !SimpleExpression.match ( "simple test",  "test*simple" ));
        assertTrue("SimpleExpression->Match \"simple test\" with  \"*\"", 
               SimpleExpression.match ( "simple test",  "*" ));
        assertTrue("SimpleExpression->Match \"simple test\" with  \"#\"", 
               SimpleExpression.match ( "simple test",  "#" ));
        assertTrue("SimpleExpression->Match \"simple test\" with  \"* #\"",
               SimpleExpression.match ( "simple test",  "* #" ));
        assertTrue("SimpleExpression->Match \"simple_test\" with  \"* #\"",
               !SimpleExpression.match ( "simple_test",  "* #" ));
        assertTrue("SimpleExpression->Match " + _simpletarget1+ " with " + _simplepattern1,
               SimpleExpression.match ( _simpletarget1,  _simplepattern1 ));        
    }


    public void testSimpleExpressionSelect () {
        Vector test_return;

        try {
              SimpleExpression.select ( "simple test",  "test**" );
              assertTrue( "SimpleExpression->Select \"simple test\" with  \"test**\"", false);
        } catch ( RuntimeException ex) {
              assertTrue( "SimpleExpression->Select \"simple test\" with  \"test**\"", true);
        }

        try {
              SimpleExpression.select ( "simple test",  "test##" );
              assertTrue( "SimpleExpression->Select \"simple test\" with  \"test##\"", false);
         } catch ( RuntimeException ex) {
             assertTrue( "SimpleExpression->Select \"simple test\" with  \"test##\"", true);
         }

        try {
              SimpleExpression.select ( "simple test",  "test#*" );
              assertTrue( "SimpleExpression->Select \"simple test\" with  \"test#*\"", false);
         } catch ( RuntimeException ex) {
             assertTrue( "SimpleExpression->Select \"simple test\" with  \"test#*\"", true);
         }

        try {
              SimpleExpression.select ( "simple test",  "test*#" );
              assertTrue( "SimpleExpression->Select \"simple test\" with  \"test*#\"", false);
         } catch ( RuntimeException ex) {
             assertTrue( "SimpleExpression->Select \"simple test\" with  \"test*#\"", true);
         }
        assertTrue("SimpleExpression->Select \"simple test\" with  \"test\"", 
               SimpleExpression.select ( "simple test",  "test" ) == null );
        assertTrue("SimpleExpression->Select \"simple test\" with  \"simple test\"", 
               SimpleExpression.select ( "simple test",  "simple test" ) == null );
        assertTrue("SimpleExpression->Select \"simple test\" with  \"* test\"", 
               SimpleExpression.select ( "simple test",  "* test" ) == null );
        assertTrue("SimpleExpression->Select \"simple test\" with  \"simple *\"", 
               SimpleExpression.select ( "simple test",  "simple *" ) == null );

        test_return = (Vector) SimpleExpression.select ( "simple test",  "# test" );
        assertTrue("SimpleExpression->Select \"simple test\" with  \"# test\"", 
               test_return.size() == 1 &&
               ((String[])test_return.elementAt(0)).length == 1 &&
               ((String[])test_return.elementAt(0))[0].equals ("simple") );

        test_return = (Vector) SimpleExpression.select ( "simple test",  "simple #" );
        assertTrue("SimpleExpression->Select \"simple test\" with  \"simple *\"", 
               test_return.size() == 1 &&
               ((String[])test_return.elementAt(0)).length == 1 &&
               ((String[])test_return.elementAt(0))[0].equals ("test") );

        test_return = (Vector) SimpleExpression.select ( "simple testcase",  "# test#" );
        assertTrue("SimpleExpression->Select \"simple testcase\" with  \"# test#\"", 
               test_return.size() == 1 &&
               ((String[])test_return.elementAt(0)).length == 2 &&
               ((String[])test_return.elementAt(0))[0].equals ("simple") &&
               ((String[])test_return.elementAt(0))[1].equals ("case"));

        test_return = (Vector) SimpleExpression.select ( "a simple testcase",  "a # test#" );
        assertTrue("SimpleExpression->Select \"a simple testcase\" with  \"a # test#\"", 
               test_return.size() == 1 &&
               ((String[])test_return.elementAt(0)).length == 2 &&
               ((String[])test_return.elementAt(0))[0].equals ("simple") &&
               ((String[])test_return.elementAt(0))[1].equals ("case"));

        test_return = (Vector) SimpleExpression.select ( "a very simple testcase",  "a * # test#" );
         assertTrue("SimpleExpression->Select \"a very simple testcase\" with  \"a * # test#\"", 
               test_return.size() == 1 &&
               ((String[])test_return.elementAt(0)).length == 2 &&
               ((String[])test_return.elementAt(0))[0].equals ("simple") &&
               ((String[])test_return.elementAt(0))[1].equals ("case"));

        test_return = (Vector) SimpleExpression.select ( "a very simple testcase",  "a * # test#a" );
        assertTrue("SimpleExpression->Select \"a very simple testcase\" with  \"a * # test#a\"", 
               test_return == null);

        test_return = (Vector) SimpleExpression.select ( "a very simple testcase",  "as a * # test#a" );
        assertTrue("SimpleExpression->Select \"a very simple testcase\" with  \"a * # test#a\"", 
               test_return == null);
        test_return = (Vector) SimpleExpression.select ( "a very simple testcase",  "as a * # test#z" );
        assertTrue("SimpleExpression->Select \"a very simple testcase\" with  \"a * # test#a\"", 
               test_return == null);

        test_return = (Vector) SimpleExpression.select ( "061-730641/122\n" 
                                                        +"061-735969/1246\n" 
                                                        +"062-429332/200\n", 
							 "06*-#/#\n" );
        assertTrue( test_return.size() == 3 &&
               ((String[])test_return.elementAt(0)).length == 2 &&
               ((String[])test_return.elementAt(0))[0].equals ("730641") &&
               ((String[])test_return.elementAt(0))[1].equals ("122")    &&
               ((String[])test_return.elementAt(1))[0].equals ("735969") &&
               ((String[])test_return.elementAt(1))[1].equals ("1246")   &&
               ((String[])test_return.elementAt(2))[0].equals ("429332") &&
               ((String[])test_return.elementAt(2))[1].equals ("200")    );

        BufferedReader reader= new BufferedReader(new InputStreamReader 
                                         (getClass().getResourceAsStream(_test_file_1)));
        StringBuffer buffer = new StringBuffer();
        String line;
        try {
            while ( (line = reader.readLine()) != null )
                buffer.append(line);

            String pattern = "*<title>#</title>*";
            test_return = (Vector) SimpleExpression.select ( buffer.toString(),  pattern );
            assertTrue( test_return.size() == 8 &&
                    ((String[])test_return.elementAt(0)).length == 1 &&
                    ((String[])test_return.elementAt(0))[0].equals ("WriteTheWeb") &&
                    ((String[])test_return.elementAt(5))[0].equals ("Syndication and Metadata") &&
                    ((String[])test_return.elementAt(7))[0].equals ("Yournamehere.com more important than anything"));

        }catch (Exception ex){
            ex.printStackTrace();
            assertTrue ("Exception related to test file:" + _test_file_1, false);
        }

        reader= new BufferedReader(new InputStreamReader 
                                  (getClass().getResourceAsStream(_test_file_2)));
        buffer = new StringBuffer();
        try {
            while ( (line = reader.readLine()) != null )
                buffer.append(line);

            String pattern = "*<U>#</U>*<T>#</T>*";
            test_return = (Vector) SimpleExpression.select ( buffer.toString(),  pattern );

            assertTrue( test_return.size() == 10 &&
                    ((String[])test_return.elementAt(0)).length == 2 &&
                    ((String[])test_return.elementAt(9))[0].equals 
                    ("http://ling.ohio-state.edu/~jansche/tetex/html/tex4ht/mn8.html") &&
                    ((String[])test_return.elementAt(9))[1].equals 
                    ("&lt;b&gt;mn8&lt;/b&gt;.html"));

        }catch (Exception ex){
            ex.printStackTrace();
            assertTrue ("Exception related to test file:" + _test_file_2, false);
        }
    }


    public static void main (String[] args) {
        junit.textui.TestRunner.run (suite());
    }

}

/**
 * Wrapper for the protected members which should have been private, but then we couldn't write
 * any tests for them, so they are protected.
 */
 class SimpleExpressionTCP extends SimpleExpression {

     public static int testOperatorsOccurences (String target, String operator) {
        return operatorOccurences (target, operator);
     }

     public static String[] testMakeTokens (String target) {
        return makeTokens (target);
     }


}
