/* 
 * $COPYRIGHT$
 * $Id: AllNamingTestCase.java,v 1.4 2002/03/25 11:18:16 atech Exp $
 *
 * Date        Author            Changes 
 * May 10 2001 Remus Pereni      Created
 */
package org.media.naming.test;

import junit.framework.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import java.util.Hashtable;
import org.media.naming.*;

/**
 * Class meant to run all the Test Cases related to mn8 naming package.
 * @author <a href="mailto:remus@nolimits.ro">Remus Pereni</a>
 * @version $Revision: 1.4 $ $Date: 2002/03/25 11:18:16 $
 */
public class AllNamingTestCase extends TestCase {
    
    private Context _initContext;
    

    public AllNamingTestCase (String name){
        super (name);
    }

    protected void setUp () {
        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY,
                "org.media.naming.MemoryContextFactory");
        env.put(Context.PROVIDER_URL,
                "env:/");

        try {
            _initContext = new InitialContext(env);        
        } catch (NamingException nex) {
            nex.printStackTrace();
        }

    }

    public static Test suite () {
        TestSuite suite = new TestSuite();
        suite.addTest(new AllNamingTestCase("memoryContextCreate"));
        suite.addTest(new AllNamingTestCase("memoryContextCreateSubcontext"));
        suite.addTest(new AllNamingTestCase("memoryContextDestroySubcontext"));
        suite.addTest(new AllNamingTestCase("memoryContextBind"));
        suite.addTest(new AllNamingTestCase("memoryContextLookup"));
        suite.addTest(new AllNamingTestCase("memoryContextRebind"));
        suite.addTest(new AllNamingTestCase("memoryContextRename"));
        suite.addTest(new AllNamingTestCase("memoryContextList"));
        suite.addTest(new AllNamingTestCase("memoryContextUnbind"));
        suite.addTest(new AllNamingTestCase("memoryContextClose"));
        return suite;
    }
    
    public void memoryContextCreate () {
        if (_initContext != null) assert(true);
        else assert(false);
    }


    public void memoryContextCreateSubcontext () {
        try {
            Context sctx = _initContext.createSubcontext("test");
            if ( sctx!= null ) assertTrue (true);
            else assertTrue (false);
        } catch (NamingException nex) {
            nex.printStackTrace();
            assertTrue (false);
        }
    }

    public void memoryContextDestroySubcontext () {
        try {
            _initContext.destroySubcontext("test");
            assertTrue (true);
        } catch (Exception ex) {
            ex.printStackTrace();
            assertTrue (false);
        }
    }

    public void memoryContextBind () {
        try {
            _initContext.bind("test", "TEST_VALUE");
            assertTrue (true);
        } catch (Exception ex) {
            ex.printStackTrace();
            assertTrue (false);
        }
    }

    public void memoryContextLookup () {
        try {
            String value = (String) _initContext.lookup("test");
            if (value.equals("TEST_VALUE"))
                assertTrue (true);
            else 
                assertTrue (false);
        } catch (Exception ex) {
            ex.printStackTrace();
            assertTrue (false);
        }
    }


    public void memoryContextRebind () {
        try {
            _initContext.rebind("test", "NEW_TEST_VALUE");
            String value = (String) _initContext.lookup("test");
            if (value.equals("NEW_TEST_VALUE"))
                assertTrue (true);
            else 
                assertTrue (false);

        } catch (Exception ex) {
            ex.printStackTrace();
            assertTrue (false);
        }
    }

    public void memoryContextRename () {
        try {
            _initContext.rename("test", "new_name");
            String value = (String) _initContext.lookup("new_name");
            if (value.equals("NEW_TEST_VALUE"))
                assertTrue (true);
            else 
                assertTrue (false);

        } catch (Exception ex) {
            ex.printStackTrace();
            assertTrue (false);
        }
    }


    public void memoryContextList () {
        try {
            NamingEnumeration nen = _initContext.list("");
            while ( nen.hasMore() ) {
                String name = ((NameClassPair) nen.next()).getName();
                if (name.equals("new_name"))
                    assertTrue (true);
                else 
                    assertTrue (false);                
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            assertTrue (false);
        }
    }


    public void memoryContextUnbind () {
        try {
            _initContext.unbind( "new_name");
            NamingEnumeration nen = _initContext.list("");
            while ( nen.hasMore() )
                assertTrue (false);                
            assertTrue (true);
        } catch (Exception ex) {
            ex.printStackTrace();
            assertTrue (false);
        }
    }



    public void memoryContextClose () {
        try {
            _initContext.close();            
            assertTrue (true);
        } catch (Exception ex) {
            ex.printStackTrace();
            assertTrue (false);
        }
    }




    public static void main (String[] args) {
        junit.textui.TestRunner.run (suite());
    }

}
